/* @copyright Itential, LLC 2019 (pre-modifications) */

/* eslint import/no-dynamic-require: warn */
/* eslint object-curly-newline: warn */
/* eslint no-underscore-dangle: warn  */
/* eslint camelcase: warn  */
/* eslint no-param-reassign: warn  */

// Set globals
/* global log */

/* Required libraries.  */
const path = require('path');
const aws4 = require('aws4');
const querystring = require('querystring');

/*
const deviceMap = {
  eos: 'arista-dcs',
  asa: 'cisco-asa',
  ios: 'cisco-ios',
  iosxr: 'cisco-iosxr',
  nxos: 'cisco-nx',
  bigip: 'f5-bigip',
  junos: 'juniper-junos'
};
*/

/* Fetch in the other needed components for the this Adaptor */
const AdapterBaseCl = require(path.join(__dirname, 'adapterBase.js'));

/*
 * INTERNAL FUNCTION: return stub results based on the path and method
 */
function processInArray(inArray, currentParams, key) {
  if (!inArray || !Array.isArray(inArray) || inArray.length === 0) {
    return;
  }

  // Go through the array to convert the items in it
  for (let i = 0; i < inArray.length; i += 1) {
    // are we dealing with just data
    if (typeof inArray[i] === 'string' || typeof inArray[i] === 'number' || typeof inArray[i] === 'boolean') {
      currentParams[`${key}.${i + 1}`] = inArray[i];
    } else if (typeof inArray[i] === 'object') {
      // got an object - need to add keys and values
      const objKeys = Object.keys(inArray[i]);

      for (let o = 0; o < objKeys.length; o += 1) {
        const thisKey = objKeys[o].substring(0, 1).toUpperCase() + objKeys[o].substring(1);
        const thisValue = inArray[i][objKeys[o]];

        if (Array.isArray(thisValue)) {
          processInArray(thisValue, currentParams, `${key}.${i + 1}.${thisKey}`);
        } else {
          currentParams[`${key}.${i + 1}.${thisKey}`] = thisValue;
        }
      }
    }
  }
}

/**
 * This is the adapter/interface into Awsec2
 */

/* GENERAL ADAPTER FUNCTIONS */
class Awsec2 extends AdapterBaseCl {
  /**
   * Awsec2 Adapter
   * @constructor
   */
  /* Working on changing the way we do Emit methods due to size and time constrainsts
  constructor(prongid, properties) {
    // Instantiate the AdapterBase super class
    super(prongid, properties);

    const restFunctionNames = this.getWorkflowFunctions();

    // Dynamically bind emit functions
    for (let i = 0; i < restFunctionNames.length; i += 1) {
      // Bind function to have name fnNameEmit for fnName
      const version = restFunctionNames[i].match(/__v[0-9]+/);
      const baseFnName = restFunctionNames[i].replace(/__v[0-9]+/, '');
      const fnNameEmit = version ? `${baseFnName}Emit${version}` : `${baseFnName}Emit`;
      this[fnNameEmit] = function (...args) {
        // extract the callback
        const callback = args[args.length - 1];
        // slice the callback from args so we can insert our own
        const functionArgs = args.slice(0, args.length - 1);
        // create a random name for the listener
        const eventName = `${restFunctionNames[i]}:${Math.random().toString(36)}`;
        // tell the calling class to start listening
        callback({ event: eventName, status: 'received' });
        // store parent for use of this context later
        const parent = this;
        // store emission function
        const func = function (val, err) {
          parent.removeListener(eventName, func);
          parent.emit(eventName, val, err);
        };
        // Use apply to call the function in a specific context
        this[restFunctionNames[i]].apply(this, functionArgs.concat([func])); // eslint-disable-line prefer-spread
      };
    }

    // Uncomment if you have things to add to the constructor like using your own properties.
    // Otherwise the constructor in the adapterBase will be used.
    // Capture my own properties - they need to be defined in propertiesSchema.json
    // if (this.allProps && this.allProps.myownproperty) {
    //   mypropvariable = this.allProps.myownproperty;
    // }
  }
  */

  /**
   * @callback healthCallback
   * @param {Object} reqObj - the request to send into the healthcheck
   * @param {Callback} callback - The results of the call
   */
  healthCheck(reqObj, callback) {
    // you can modify what is passed into the healthcheck by changing things in the newReq
    let newReq = null;
    if (reqObj) {
      newReq = Object.assign(...reqObj);
    }
    super.healthCheck(newReq, callback);
  }

  /**
   * updateAdapterConfiguration is used to update any of the adapter configuration files. This
   * allows customers to make changes to adapter configuration without having to be on the
   * file system.
   *
   * @function updateAdapterConfiguration
   * @param {string} configFile - the name of the file being updated (required)
   * @param {Object} changes - an object containing all of the changes = formatted like the configuration file (required)
   * @param {string} entity - the entity to be changed, if an action, schema or mock data file (optional)
   * @param {string} type - the type of entity file to change, (action, schema, mock) (optional)
   * @param {string} action - the action to be changed, if an action, schema or mock data file (optional)
   * @param {Callback} callback - The results of the call
   */
  updateAdapterConfiguration(configFile, changes, entity, type, action, callback) {
    super.updateAdapterConfiguration(configFile, changes, entity, type, action, callback);
  }

  /**
   * @callback healthCallback
   * @param {Object} result - the result of the get request (contains an id and a status)
   */
  /**
   * @callback getCallback
   * @param {Object} result - the result of the get request (entity/ies)
   * @param {String} error - any error that occurred
   */
  /**
   * @callback createCallback
   * @param {Object} item - the newly created entity
   * @param {String} error - any error that occurred
   */
  /**
   * @callback updateCallback
   * @param {String} status - the status of the update action
   * @param {String} error - any error that occurred
   */
  /**
   * @callback deleteCallback
   * @param {String} status - the status of the delete action
   * @param {String} error - any error that occurred
   */

  /**
   * @summary Determines if this adapter supports the specific entity
   *
   * @function hasEntity
   * @param {String} entityType - the entity type to check for
   * @param {String/Array} entityId - the specific entity we are looking for
   *
   * @param {Callback} callback - An array of whether the adapter can has the
   *                              desired capability or an error
   */
  hasEntity(entityType, entityId, callback) {
    const origin = `${this.id}-adapter-hasEntity`;
    log.trace(origin);

    // Make the call -
    // verifyCapability(entityType, actionType, entityId, callback)
    return this.verifyCapability(entityType, null, entityId, callback);
  }

  /**
   * @summary Determines if this adapter supports any in a list of entities
   *
   * @function hasEntities
   * @param {String} entityType - the entity type to check for
   * @param {Array} entityList - the list of entities we are looking for
   *
   * @param {Callback} callback - A map where the entity is the key and the
   *                              value is true or false
   */
  hasEntities(entityType, entityList, callback) {
    const origin = `${this.id}-adapter-hasEntities`;
    log.trace(origin);

    switch (entityType) {
      case 'Device':
        return this.hasDevices(entityList, callback);
      default:
        return callback(null, `${this.id} does not support entity ${entityType}`);
    }
  }

  /**
   * @summary Helper method for hasEntities for the specific device case
   *
   * @param {Array} deviceList - array of unique device identifiers
   * @param {Callback} callback - A map where the device is the key and the
   *                              value is true or false
   */
  hasDevices(deviceList, callback) {
    const origin = `${this.id}-adapter-hasDevices`;
    log.trace(origin);

    const findings = deviceList.reduce((map, device) => {
      // eslint-disable-next-line no-param-reassign
      map[device] = false;
      log.debug(`In reduce: ${JSON.stringify(map)}`);
      return map;
    }, {});
    const apiCalls = deviceList.map((device) => new Promise((resolve) => {
      this.describeVpcs(null, [device], null, null, null, (result, error) => {
        if (error) {
          log.debug(`In map error: ${JSON.stringify(device)}`);
          return resolve({ device, found: false });
        }
        log.debug(`In map: ${JSON.stringify(device)}`);
        return resolve({ name: device, found: true });
      });
    }));
    Promise.all(apiCalls).then((results) => {
      results.forEach((device) => {
        findings[device.name] = device.found;
      });
      log.debug(`FINDINGS: ${JSON.stringify(findings)}`);
      return callback(findings);
    }).catch((errors) => {
      log.error('Unable to do device lookup.');
      return callback(null, { code: 503, message: 'Unable to do device lookup.', error: errors });
    });
  }

  /**
   * @summary Provides a way for the adapter to tell north bound integrations
   * whether the adapter supports type, action and specific entity
   *
   * @function verifyCapability
   * @param {String} entityType - the entity type to check for
   * @param {String} actionType - the action type to check for
   * @param {String/Array} entityId - the specific entity we are looking for
   *
   * @param {Callback} callback - An array of whether the adapter can has the
   *                              desired capability or an error
   */
  verifyCapability(entityType, actionType, entityId, callback) {
    const meth = 'adapterBase-verifyCapability';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    // if caching
    if (this.caching) {
      // Make the call - verifyCapability(entityType, actionType, entityId, callback)
      return this.requestHandlerInst.verifyCapability(entityType, actionType, entityId, (results, error) => {
        if (error) {
          return callback(null, error);
        }

        // if the cache needs to be updated, update and try again
        if (results && results[0] === 'needupdate') {
          switch (entityType) {
            case 'template_entity': {
              // if the cache is invalid, update the cache
              return this.getEntities(null, null, null, null, (data, err) => {
                if (err) {
                  const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Could not update entity: $VARIABLE$, cache', [entityType], null, null, null);
                  log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
                  return callback(null, errorObj);
                }

                // need to check the cache again since it has been updated
                return this.requestHandlerInst.verifyCapability(entityType, actionType, entityId, (vcapable, verror) => {
                  if (verror) {
                    return callback(null, verror);
                  }

                  return this.capabilityResults(vcapable, callback);
                });
              });
            }
            default: {
              // unsupported entity type
              const result = [false];

              // put false in array for all entities
              if (Array.isArray(entityId)) {
                for (let e = 1; e < entityId.length; e += 1) {
                  result.push(false);
                }
              }

              return callback(result);
            }
          }
        }

        // return the results
        return this.capabilityResults(results, callback);
      });
    }

    // if no entity id
    if (!entityId) {
      // need to check the cache again since it has been updated
      return this.requestHandlerInst.verifyCapability(entityType, actionType, null, (vcapable, verror) => {
        if (verror) {
          return callback(null, verror);
        }

        return this.capabilityResults(vcapable, callback);
      });
    }

    // if not caching
    switch (entityType) {
      case 'template_entity': {
        // need to get the entities to check
        return this.getEntities(null, null, null, null, (data, err) => {
          if (err) {
            const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Could not update entity: $VARIABLE$, cache', [entityType], null, null, null);
            log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
            return callback(null, errorObj);
          }

          // need to check the cache again since it has been updated
          return this.requestHandlerInst.verifyCapability(entityType, actionType, null, (vcapable, verror) => {
            if (verror) {
              return callback(null, verror);
            }

            // is the entity in the list?
            const isEntity = this.entityInList(entityId, data.response, callback);
            const res = [];

            // not found
            for (let i = 0; i < isEntity.length; i += 1) {
              if (vcapable) {
                res.push(isEntity[i]);
              } else {
                res.push(false);
              }
            }

            return callback(res);
          });
        });
      }
      default: {
        // unsupported entity type
        const result = [false];

        // put false in array for all entities
        if (Array.isArray(entityId)) {
          for (let e = 1; e < entityId.length; e += 1) {
            result.push(false);
          }
        }

        return callback(result);
      }
    }
  }

  /**
   * @summary Updates the cache for all entities by call the get All entity method
   *
   * @function updateEntityCache
   *
   */
  updateEntityCache() {
    const origin = `${this.id}-adapter-updateEntityCache`;
    log.trace(origin);

    if (this.caching) {
      // if the cache is invalid, update the cache
      this.getEntities(null, null, null, null, (data, err) => {
        if (err) {
          log.trace(`${origin}: Could not load template_entity into cache - ${err}`);
        }
      });
    }
  }

  /**
   * @summary Gets the hcma authorization for the call
   *
   * @function getAuthorization
   * @param {String} method - the method for the action we are setting authorization for
   * @param {String} region - the region for the action we are setting authorization for
   * @param {String} pathBody - the path or body for the action we are setting authorization for
   *
   *  @return {Object} the headers to add to the request
   */
  getAuthorization(method, pathBody) {
    const origin = `${this.id}-adapter-getAuthorization`;
    log.trace(origin);

    // set up the options for the AWS header
    const options = {
      host: this.allProps.host,
      service: 'ec2',
      region: this.allProps.region
    };

    if (method === 'GET') {
      options.path = pathBody;
    } else {
      options.body = pathBody;
    }

    const authOpts = aws4.sign(options, { accessKeyId: this.allProps.authentication.aws_access_key, secretAccessKey: this.allProps.authentication.aws_secret_key });

    // return the headers
    return authOpts.headers;
  }

  /**
   * @summary Accepts the Convertible Reserved Instance exchange quote described in the  GetReservedInstancesExchangeQuote  call.
   *
   * @function acceptReservedInstancesExchangeQuote
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} reservedInstanceId - The IDs of the Convertible Reserved Instances to exchange for another Convertible Reserved Instance of the same or higher value.
   * @param {array} targetConfiguration - The configuration of the target Convertible Reserved Instance to exchange for your current Convertible Reserved Instances.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  acceptReservedInstancesExchangeQuote(dryRun, reservedInstanceId, targetConfiguration, callback) {
    const meth = 'adapter-acceptReservedInstancesExchangeQuote';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AcceptReservedInstancesExchangeQuote';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (reservedInstanceId === undefined || reservedInstanceId === null || reservedInstanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['reservedInstanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (reservedInstanceId && reservedInstanceId.length > 0) {
      processInArray(reservedInstanceId, queryParamsAvailable, 'ReservedInstanceId');
    }
    if (targetConfiguration && targetConfiguration.length > 0) {
      processInArray(targetConfiguration, queryParamsAvailable, 'TargetConfiguration');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'acceptReservedInstancesExchangeQuote', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['acceptReservedInstancesExchangeQuote'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Accepts a request to attach a VPC to a transit gateway.   The VPC attachment must be in the  pendingAcceptance  state. Use  DescribeTransitGatewayVpcAttachments  to view your pending VPC attachment requests. Use  RejectTransitGatewayVpcAttachment  to reject a VPC attachment request.
   *
   * @function acceptTransitGatewayVpcAttachment
   * @param {string} transitGatewayAttachmentId - The ID of the attachment.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  acceptTransitGatewayVpcAttachment(transitGatewayAttachmentId, dryRun, callback) {
    const meth = 'adapter-acceptTransitGatewayVpcAttachment';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AcceptTransitGatewayVpcAttachment';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayAttachmentId === undefined || transitGatewayAttachmentId === null || transitGatewayAttachmentId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayAttachmentId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      TransitGatewayAttachmentId: transitGatewayAttachmentId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'acceptTransitGatewayVpcAttachment', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['acceptTransitGatewayVpcAttachment'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Accepts one or more interface VPC endpoint connection requests to your VPC endpoint service.
   *
   * @function acceptVpcEndpointConnections
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} serviceId - The ID of the endpoint service.
   * @param {array} vpcEndpointId - The IDs of one or more interface VPC endpoints.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  acceptVpcEndpointConnections(dryRun, serviceId, vpcEndpointId, callback) {
    const meth = 'adapter-acceptVpcEndpointConnections';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AcceptVpcEndpointConnections';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (serviceId === undefined || serviceId === null || serviceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['serviceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (vpcEndpointId === undefined || vpcEndpointId === null || vpcEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ServiceId: serviceId,
      Action: action,
      Version: version
    };
    if (vpcEndpointId && vpcEndpointId.length > 0) {
      processInArray(vpcEndpointId, queryParamsAvailable, 'VpcEndpointId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'acceptVpcEndpointConnections', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['acceptVpcEndpointConnections'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Accept a VPC peering connection request. To accept a request, the VPC peering connection must be in the  pending-acceptance  state, and you must be the owner of the peer VPC. Use  DescribeVpcPeeringConnections  to view your outstanding VPC peering connection requests.   For an inter-Region VPC peering connection request, you must accept the VPC peering connection in the Region of the accepter VPC.
   *
   * @function acceptVpcPeeringConnection
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} vpcPeeringConnectionId - The ID of the VPC peering connection. You must specify this parameter in the request.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  acceptVpcPeeringConnection(dryRun, vpcPeeringConnectionId, callback) {
    const meth = 'adapter-acceptVpcPeeringConnection';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AcceptVpcPeeringConnection';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VpcPeeringConnectionId: vpcPeeringConnectionId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'acceptVpcPeeringConnection', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['acceptVpcPeeringConnection'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Advertises an IPv4 address range that is provisioned for use with your AWS resources through bring your own IP addresses (BYOIP).   You can perform this operation at most once every 10 seconds, even if you specify different address ranges each time.   We recommend that you stop advertising the BYOIP CIDR from other locations when you advertise it from AWS. To minimize down time, you can configure your AWS resources to use an address from a BYOIP CIDR before it is advertised, and then simultaneo...(description truncated)
   *
   * @function advertiseByoipCidr
   * @param {string} cidr - The IPv4 address range, in CIDR notation. This must be the exact range that you provisioned. You can't advertise only a portion of the provisioned range.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  advertiseByoipCidr(cidr, dryRun, callback) {
    const meth = 'adapter-advertiseByoipCidr';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AdvertiseByoipCidr';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (cidr === undefined || cidr === null || cidr === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['cidr'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Cidr: cidr,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'advertiseByoipCidr', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['advertiseByoipCidr'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Allocates an Elastic IP address to your AWS account. After you allocate the Elastic IP address you can associate it with an instance or network interface. After you release an Elastic IP address, it is released to the IP address pool and can be allocated to a different AWS account.   You can allocate an Elastic IP address from an address pool owned by AWS or from an address pool created from a public IPv4 address range that you have brought to AWS for use with your AWS resources using bring you...(description truncated)
   *
   * @function allocateAddress
   * @param {string} domain - Set to  vpc  to allocate the address for use with instances in a VPC.   Default: The address is for use with instances in EC2-Classic.
   * @param {string} address - [EC2-VPC] The Elastic IP address to recover or an IPv4 address from an address pool.
   * @param {string} publicIpv4Pool - The ID of an address pool that you own. Use this parameter to let Amazon EC2 select an address from the address pool. To specify a specific address from the address pool,...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  allocateAddress(domain = 'vpc', address, publicIpv4Pool, dryRun, callback) {
    const meth = 'adapter-allocateAddress';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AllocateAddress';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Domain: domain,
      Address: address,
      PublicIpv4Pool: publicIpv4Pool,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'allocateAddress', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['allocateAddress'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Allocates a Dedicated Host to your account. At a minimum, specify the instance size type, Availability Zone, and quantity of hosts to allocate.
   *
   * @function allocateHosts
   * @param {string} autoPlacement - Indicates whether the host accepts any untargeted instance launches that match its instance type configuration, or if it only accepts Host tenancy instance launches that...(description truncated)
   * @param {string} availabilityZone - The Availability Zone in which to allocate the Dedicated Host.
   * @param {string} clientToken - Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see  How to Ensure Idempotency  in the  Amazon Elastic ...(description truncated)
   * @param {string} instanceType - Specifies the instance type for which to configure your Dedicated Hosts. When you specify the instance type, that is the only instance type that you can launch onto that ...(description truncated)
   * @param {number} quantity - The number of Dedicated Hosts to allocate to your account with these parameters.
   * @param {array} tagSpecification - The tags to apply to the Dedicated Host during creation.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  allocateHosts(autoPlacement = 'on', availabilityZone, clientToken, instanceType, quantity, tagSpecification, callback) {
    const meth = 'adapter-allocateHosts';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AllocateHosts';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (availabilityZone === undefined || availabilityZone === null || availabilityZone === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['availabilityZone'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (instanceType === undefined || instanceType === null || instanceType === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceType'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (quantity === undefined || quantity === null || quantity === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['quantity'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      AutoPlacement: autoPlacement,
      AvailabilityZone: availabilityZone,
      ClientToken: clientToken,
      InstanceType: instanceType,
      Quantity: quantity,
      Action: action,
      Version: version
    };
    if (tagSpecification && tagSpecification.length > 0) {
      processInArray(tagSpecification, queryParamsAvailable, 'TagSpecification');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Host', 'allocateHosts', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['allocateHosts'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Applies a security group to the association between the target network and the Client VPN endpoint. This action replaces the existing security groups with the specified security groups.
   *
   * @function applySecurityGroupsToClientVpnTargetNetwork
   * @param {string} clientVpnEndpointId - The ID of the Client VPN endpoint.
   * @param {string} vpcId - The ID of the VPC in which the associated target network is located.
   * @param {array} securityGroupId - The IDs of the security groups to apply to the associated target network. Up to 5 security groups can be applied to an associated target network.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  applySecurityGroupsToClientVpnTargetNetwork(clientVpnEndpointId, vpcId, securityGroupId, dryRun, callback) {
    const meth = 'adapter-applySecurityGroupsToClientVpnTargetNetwork';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ApplySecurityGroupsToClientVpnTargetNetwork';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointId === undefined || clientVpnEndpointId === null || clientVpnEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (securityGroupId === undefined || securityGroupId === null || securityGroupId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['securityGroupId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientVpnEndpointId: clientVpnEndpointId,
      VpcId: vpcId,
      Action: action,
      Version: version
    };
    if (securityGroupId && securityGroupId.length > 0) {
      processInArray(securityGroupId, queryParamsAvailable, 'SecurityGroupId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('SecurityGroup', 'applySecurityGroupsToClientVpnTargetNetwork', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['applySecurityGroupsToClientVpnTargetNetwork'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Assigns one or more IPv6 addresses to the specified network interface. You can specify one or more specific IPv6 addresses, or you can specify the number of IPv6 addresses to be automatically assigned from within the subnet's IPv6 CIDR block range. You can assign as many IPv6 addresses to a network interface as you can assign private IPv4 addresses, and the limit varies per instance type. For information, see  IP Addresses Per Network Interface Per Instance Type  in the  Amazon Elastic Compute C...(description truncated)
   *
   * @function assignIpv6Addresses
   * @param {number} ipv6AddressCount - The number of IPv6 addresses to assign to the network interface. Amazon EC2 automatically selects the IPv6 addresses from the subnet range. You can't use this option if s...(description truncated)
   * @param {array} ipv6Addresses - One or more specific IPv6 addresses to be assigned to the network interface. You can't use this option if you're specifying a number of IPv6 addresses.
   * @param {string} networkInterfaceId - The ID of the network interface.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  assignIpv6Addresses(ipv6AddressCount, ipv6Addresses, networkInterfaceId, callback) {
    const meth = 'adapter-assignIpv6Addresses';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AssignIpv6Addresses';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (networkInterfaceId === undefined || networkInterfaceId === null || networkInterfaceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['networkInterfaceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      Ipv6AddressCount: ipv6AddressCount,
      NetworkInterfaceId: networkInterfaceId,
      Action: action,
      Version: version
    };
    if (ipv6Addresses && ipv6Addresses.length > 0) {
      processInArray(ipv6Addresses, queryParamsAvailable, 'Ipv6Addresses');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'assignIpv6Addresses', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['assignIpv6Addresses'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Assigns one or more secondary private IP addresses to the specified network interface.   You can specify one or more specific secondary IP addresses, or you can specify the number of secondary IP addresses to be automatically assigned within the subnet's CIDR block range. The number of secondary IP addresses that you can assign to an instance varies by instance type. For information about instance types, see  Instance Types  in the  Amazon Elastic Compute Cloud User Guide . For more information...(description truncated)
   *
   * @function assignPrivateIpAddresses
   * @param {boolean} allowReassignment - Indicates whether to allow an IP address that is already assigned to another network interface or instance to be reassigned to the specified network interface.
   * @param {string} networkInterfaceId - The ID of the network interface.
   * @param {array} privateIpAddress - One or more IP addresses to be assigned as a secondary private IP address to the network interface. You can't specify this parameter when also specifying a number of sec...(description truncated)
   * @param {number} secondaryPrivateIpAddressCount - The number of secondary IP addresses to assign to the network interface. You can't specify this parameter when also specifying private IP addresses.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  assignPrivateIpAddresses(allowReassignment, networkInterfaceId, privateIpAddress, secondaryPrivateIpAddressCount, callback) {
    const meth = 'adapter-assignPrivateIpAddresses';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AssignPrivateIpAddresses';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (networkInterfaceId === undefined || networkInterfaceId === null || networkInterfaceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['networkInterfaceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      AllowReassignment: allowReassignment,
      NetworkInterfaceId: networkInterfaceId,
      SecondaryPrivateIpAddressCount: secondaryPrivateIpAddressCount,
      Action: action,
      Version: version
    };
    if (privateIpAddress && privateIpAddress.length > 0) {
      processInArray(privateIpAddress, queryParamsAvailable, 'PrivateIpAddress');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'assignPrivateIpAddresses', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['assignPrivateIpAddresses'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Associates an Elastic IP address with an instance or a network interface. Before you can use an Elastic IP address, you must allocate it to your account.   An Elastic IP address is for use in either the EC2-Classic platform or in a VPC. For more information, see  Elastic IP Addresses  in the  Amazon Elastic Compute Cloud User Guide .   [EC2-Classic, VPC in an EC2-VPC-only account] If the Elastic IP address is already associated with a different instance, it is disassociated from that instance a...(description truncated)
   *
   * @function associateAddress
   * @param {string} allocationId - [EC2-VPC] The allocation ID. This is required for EC2-VPC.
   * @param {string} instanceId - The ID of the instance. This is required for EC2-Classic. For EC2-VPC, you can specify either the instance ID or the network interface ID, but not both. The operation fai...(description truncated)
   * @param {string} publicIp - The Elastic IP address to associate with the instance. This is required for EC2-Classic.
   * @param {boolean} allowReassociation - [EC2-VPC] For a VPC in an EC2-Classic account, specify true to allow an Elastic IP address that is already associated with an instance or network interface to be reassoci...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} networkInterfaceId - [EC2-VPC] The ID of the network interface. If the instance has more than one network interface, you must specify a network interface ID.   For EC2-VPC, you can specify e...(description truncated)
   * @param {string} privateIpAddress - [EC2-VPC] The primary or secondary private IP address to associate with the Elastic IP address. If no private IP address is specified, the Elastic IP address is associate...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  associateAddress(allocationId, instanceId, publicIp, allowReassociation, dryRun, networkInterfaceId, privateIpAddress, callback) {
    const meth = 'adapter-associateAddress';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AssociateAddress';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AllocationId: allocationId,
      InstanceId: instanceId,
      PublicIp: publicIp,
      AllowReassociation: allowReassociation,
      NetworkInterfaceId: networkInterfaceId,
      PrivateIpAddress: privateIpAddress,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'associateAddress', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['associateAddress'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Associates a target network with a Client VPN endpoint. A target network is a subnet in a VPC. You can associate multiple subnets from the same VPC with a Client VPN endpoint. You can associate only one subnet in each Availability Zone. We recommend that you associate at least two subnets to provide Availability Zone redundancy.
   *
   * @function associateClientVpnTargetNetwork
   * @param {string} clientVpnEndpointId - The ID of the Client VPN endpoint.
   * @param {string} subnetId - The ID of the subnet to associate with the Client VPN endpoint.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  associateClientVpnTargetNetwork(clientVpnEndpointId, subnetId, dryRun, callback) {
    const meth = 'adapter-associateClientVpnTargetNetwork';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AssociateClientVpnTargetNetwork';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointId === undefined || clientVpnEndpointId === null || clientVpnEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (subnetId === undefined || subnetId === null || subnetId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['subnetId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientVpnEndpointId: clientVpnEndpointId,
      SubnetId: subnetId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'associateClientVpnTargetNetwork', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['associateClientVpnTargetNetwork'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Associates a set of DHCP options (that you've previously created) with the specified VPC, or associates no DHCP options with the VPC.   After you associate the options with the VPC, any existing instances and all new instances that you launch in that VPC use the options. You don't need to restart or relaunch the instances. They automatically pick up the changes within a few hours, depending on how frequently the instance renews its DHCP lease. You can explicitly renew the lease using the operat...(description truncated)
   *
   * @function associateDhcpOptions
   * @param {string} dhcpOptionsId - The ID of the DHCP options set, or  default  to associate no DHCP options with the VPC.
   * @param {string} vpcId - The ID of the VPC.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  associateDhcpOptions(dhcpOptionsId, vpcId, dryRun, callback) {
    const meth = 'adapter-associateDhcpOptions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AssociateDhcpOptions';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (dhcpOptionsId === undefined || dhcpOptionsId === null || dhcpOptionsId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['dhcpOptionsId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      DhcpOptionsId: dhcpOptionsId,
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('DHCP', 'associateDhcpOptions', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['associateDhcpOptions'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Associates an IAM instance profile with a running or stopped instance. You cannot associate more than one IAM instance profile with an instance.
   *
   * @function associateIamInstanceProfile
   * @param {string} iamInstanceProfileArn - Describes an IAM instance profile.
The Amazon Resource Name (ARN) of the instance profile.
   * @param {string} iamInstanceProfileName - Describes an IAM instance profile.
The name of the instance profile.
   * @param {string} instanceId - The ID of the instance.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  associateIamInstanceProfile(iamInstanceProfileArn, iamInstanceProfileName, instanceId, callback) {
    const meth = 'adapter-associateIamInstanceProfile';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AssociateIamInstanceProfile';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      IamInstanceProfileArn: iamInstanceProfileArn,
      IamInstanceProfileName: iamInstanceProfileName,
      InstanceId: instanceId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'associateIamInstanceProfile', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['associateIamInstanceProfile'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Associates a subnet with a route table. The subnet and route table must be in the same VPC. This association causes traffic originating from the subnet to be routed according to the routes in the route table. The action returns an association ID, which you need in order to disassociate the route table from the subnet later. A route table can be associated with multiple subnets.   For more information, see  Route Tables  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function associateRouteTable
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} routeTableId - The ID of the route table.
   * @param {string} subnetId - The ID of the subnet.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  associateRouteTable(dryRun, routeTableId, subnetId, callback) {
    const meth = 'adapter-associateRouteTable';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AssociateRouteTable';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (routeTableId === undefined || routeTableId === null || routeTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['routeTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (subnetId === undefined || subnetId === null || subnetId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['subnetId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      RouteTableId: routeTableId,
      SubnetId: subnetId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Route', 'associateRouteTable', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['associateRouteTable'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Associates a CIDR block with your subnet. You can only associate a single IPv6 CIDR block with your subnet. An IPv6 CIDR block must have a prefix length of /64.
   *
   * @function associateSubnetCidrBlock
   * @param {string} ipv6CidrBlock - The IPv6 CIDR block for your subnet. The subnet must have a /64 prefix length.
   * @param {string} subnetId - The ID of your subnet.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  associateSubnetCidrBlock(ipv6CidrBlock, subnetId, callback) {
    const meth = 'adapter-associateSubnetCidrBlock';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AssociateSubnetCidrBlock';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (ipv6CidrBlock === undefined || ipv6CidrBlock === null || ipv6CidrBlock === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['ipv6CidrBlock'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (subnetId === undefined || subnetId === null || subnetId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['subnetId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      Ipv6CidrBlock: ipv6CidrBlock,
      SubnetId: subnetId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Subnet', 'associateSubnetCidrBlock', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['associateSubnetCidrBlock'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Associates the specified attachment with the specified transit gateway route table. You can associate only one route table with an attachment.
   *
   * @function associateTransitGatewayRouteTable
   * @param {string} transitGatewayRouteTableId - The ID of the transit gateway route table.
   * @param {string} transitGatewayAttachmentId - The ID of the attachment.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  associateTransitGatewayRouteTable(transitGatewayRouteTableId, transitGatewayAttachmentId, dryRun, callback) {
    const meth = 'adapter-associateTransitGatewayRouteTable';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AssociateTransitGatewayRouteTable';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayRouteTableId === undefined || transitGatewayRouteTableId === null || transitGatewayRouteTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayRouteTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (transitGatewayAttachmentId === undefined || transitGatewayAttachmentId === null || transitGatewayAttachmentId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayAttachmentId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      TransitGatewayRouteTableId: transitGatewayRouteTableId,
      TransitGatewayAttachmentId: transitGatewayAttachmentId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'associateTransitGatewayRouteTable', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['associateTransitGatewayRouteTable'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Associates a CIDR block with your VPC. You can associate a secondary IPv4 CIDR block, or you can associate an Amazon-provided IPv6 CIDR block. The IPv6 CIDR block size is fixed at /56.   For more information about associating CIDR blocks with your VPC and applicable restrictions, see  VPC and Subnet Sizing  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function associateVpcCidrBlock
   * @param {boolean} amazonProvidedIpv6CidrBlock - Requests an Amazon-provided IPv6 CIDR block with a /56 prefix length for the VPC. You cannot specify the range of IPv6 addresses, or the size of the CIDR block.
   * @param {string} cidrBlock - An IPv4 CIDR block to associate with the VPC.
   * @param {string} vpcId - The ID of the VPC.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  associateVpcCidrBlock(amazonProvidedIpv6CidrBlock, cidrBlock, vpcId, callback) {
    const meth = 'adapter-associateVpcCidrBlock';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AssociateVpcCidrBlock';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      AmazonProvidedIpv6CidrBlock: amazonProvidedIpv6CidrBlock,
      CidrBlock: cidrBlock,
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'associateVpcCidrBlock', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['associateVpcCidrBlock'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Links an EC2-Classic instance to a ClassicLink-enabled VPC through one or more of the VPC's security groups. You cannot link an EC2-Classic instance to more than one VPC at a time. You can only link an instance that's in the  running  state. An instance is automatically unlinked from a VPC when it's stopped - you can link it to the VPC again when you restart it.   After you've linked an instance, you cannot change the VPC security groups that are associated with it. To change the security group...(description truncated)
   *
   * @function attachClassicLinkVpc
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} securityGroupId - The ID of one or more of the VPC's security groups. You cannot specify security groups from a different VPC.
   * @param {string} instanceId - The ID of an EC2-Classic instance to link to the ClassicLink-enabled VPC.
   * @param {string} vpcId - The ID of a ClassicLink-enabled VPC.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  attachClassicLinkVpc(dryRun, securityGroupId, instanceId, vpcId, callback) {
    const meth = 'adapter-attachClassicLinkVpc';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AttachClassicLinkVpc';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (securityGroupId === undefined || securityGroupId === null || securityGroupId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['securityGroupId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      InstanceId: instanceId,
      VpcId: vpcId,
      Action: action,
      Version: version
    };
    if (securityGroupId && securityGroupId.length > 0) {
      processInArray(securityGroupId, queryParamsAvailable, 'SecurityGroupId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'attachClassicLinkVpc', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['attachClassicLinkVpc'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Attaches an internet gateway to a VPC, enabling connectivity between the internet and the VPC. For more information about your VPC and internet gateway, see the  Amazon Virtual Private Cloud User Guide .
   *
   * @function attachInternetGateway
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} internetGatewayId - The ID of the internet gateway.
   * @param {string} vpcId - The ID of the VPC.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  attachInternetGateway(dryRun, internetGatewayId, vpcId, callback) {
    const meth = 'adapter-attachInternetGateway';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AttachInternetGateway';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (internetGatewayId === undefined || internetGatewayId === null || internetGatewayId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['internetGatewayId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      InternetGatewayId: internetGatewayId,
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Gateway', 'attachInternetGateway', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['attachInternetGateway'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Attaches a network interface to an instance.
   *
   * @function attachNetworkInterface
   * @param {number} deviceIndex - The index of the device for the network interface attachment.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} instanceId - The ID of the instance.
   * @param {string} networkInterfaceId - The ID of the network interface.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  attachNetworkInterface(deviceIndex, dryRun, instanceId, networkInterfaceId, callback) {
    const meth = 'adapter-attachNetworkInterface';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AttachNetworkInterface';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (deviceIndex === undefined || deviceIndex === null || deviceIndex === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['deviceIndex'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (networkInterfaceId === undefined || networkInterfaceId === null || networkInterfaceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['networkInterfaceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      DeviceIndex: deviceIndex,
      InstanceId: instanceId,
      NetworkInterfaceId: networkInterfaceId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkInterface', 'attachNetworkInterface', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['attachNetworkInterface'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Attaches an EBS volume to a running or stopped instance and exposes it to the instance with the specified device name.   Encrypted EBS volumes may only be attached to instances that support Amazon EBS encryption. For more information, see  Amazon EBS Encryption  in the  Amazon Elastic Compute Cloud User Guide .   For a list of supported device names, see  Attaching an EBS Volume to an Instance . Any device names that aren't reserved for instance store volumes can be used for EBS volumes. For mo...(description truncated)
   *
   * @function attachVolume
   * @param {string} device - The device name (for example,  /dev/sdh  or  xvdh ).
   * @param {string} instanceId - The ID of the instance.
   * @param {string} volumeId - The ID of the EBS volume. The volume and instance must be within the same Availability Zone.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  attachVolume(device, instanceId, volumeId, dryRun, callback) {
    const meth = 'adapter-attachVolume';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AttachVolume';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (device === undefined || device === null || device === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['device'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (volumeId === undefined || volumeId === null || volumeId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['volumeId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Device: device,
      InstanceId: instanceId,
      VolumeId: volumeId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Volume', 'attachVolume', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['attachVolume'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Attaches a virtual private gateway to a VPC. You can attach one virtual private gateway to one VPC at a time.   For more information, see  AWS Site-to-Site VPN  in the  AWS Site-to-Site VPN User Guide .
   *
   * @function attachVpnGateway
   * @param {string} vpcId - The ID of the VPC.
   * @param {string} vpnGatewayId - The ID of the virtual private gateway.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  attachVpnGateway(vpcId, vpnGatewayId, dryRun, callback) {
    const meth = 'adapter-attachVpnGateway';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AttachVpnGateway';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (vpnGatewayId === undefined || vpnGatewayId === null || vpnGatewayId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpnGatewayId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VpcId: vpcId,
      VpnGatewayId: vpnGatewayId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'attachVpnGateway', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['attachVpnGateway'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Adds an ingress authorization rule to a Client VPN endpoint. Ingress authorization rules act as firewall rules that grant access to networks. You must configure ingress authorization rules to enable clients to access resources in AWS or on-premises networks.
   *
   * @function authorizeClientVpnIngress
   * @param {string} clientVpnEndpointId - The ID of the Client VPN endpoint.
   * @param {string} targetNetworkCidr - The IPv4 address range, in CIDR notation, of the network for which access is being authorized.
   * @param {string} accessGroupId - The ID of the Active Directory group to grant access.
   * @param {boolean} authorizeAllGroups - Indicates whether to grant access to all clients. Use  true  to grant all clients who successfully establish a VPN connection access to the network.
   * @param {string} description - A brief description of the authorization rule.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  authorizeClientVpnIngress(clientVpnEndpointId, targetNetworkCidr, accessGroupId, authorizeAllGroups, description, dryRun, callback) {
    const meth = 'adapter-authorizeClientVpnIngress';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AuthorizeClientVpnIngress';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointId === undefined || clientVpnEndpointId === null || clientVpnEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (targetNetworkCidr === undefined || targetNetworkCidr === null || targetNetworkCidr === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['targetNetworkCidr'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientVpnEndpointId: clientVpnEndpointId,
      TargetNetworkCidr: targetNetworkCidr,
      AccessGroupId: accessGroupId,
      AuthorizeAllGroups: authorizeAllGroups,
      Description: description,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'authorizeClientVpnIngress', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['authorizeClientVpnIngress'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary [VPC only] Adds the specified egress rules to a security group for use with a VPC.   An outbound rule permits instances to send traffic to the specified destination IPv4 or IPv6 CIDR address ranges, or to the specified destination security groups for the same VPC.   You specify a protocol for each rule (for example, TCP). For the TCP and UDP protocols, you must also specify the destination port or port range. For the ICMP protocol, you must also specify the ICMP type and code. You can use -1 fo...(description truncated)
   *
   * @function authorizeSecurityGroupEgress
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} groupId - The ID of the security group.
   * @param {array} ipPermissions - The sets of IP permissions. You can't specify a destination security group and a CIDR IP address range in the same set of permissions.
   * @param {string} cidrIp - Not supported. Use a set of IP permissions to specify the CIDR.
   * @param {number} fromPort - Not supported. Use a set of IP permissions to specify the port.
   * @param {string} ipProtocol - Not supported. Use a set of IP permissions to specify the protocol name or number.
   * @param {number} toPort - Not supported. Use a set of IP permissions to specify the port.
   * @param {string} sourceSecurityGroupName - Not supported. Use a set of IP permissions to specify a destination security group.
   * @param {string} sourceSecurityGroupOwnerId - Not supported. Use a set of IP permissions to specify a destination security group.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  authorizeSecurityGroupEgress(dryRun, groupId, ipPermissions, cidrIp, fromPort, ipProtocol, toPort, sourceSecurityGroupName, sourceSecurityGroupOwnerId, callback) {
    const meth = 'adapter-authorizeSecurityGroupEgress';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AuthorizeSecurityGroupEgress';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (groupId === undefined || groupId === null || groupId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['groupId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      GroupId: groupId,
      CidrIp: cidrIp,
      FromPort: fromPort,
      IpProtocol: ipProtocol,
      ToPort: toPort,
      SourceSecurityGroupName: sourceSecurityGroupName,
      SourceSecurityGroupOwnerId: sourceSecurityGroupOwnerId,
      Action: action,
      Version: version
    };
    if (ipPermissions && ipPermissions.length > 0) {
      processInArray(ipPermissions, queryParamsAvailable, 'IpPermissions');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('SecurityGroup', 'authorizeSecurityGroupEgress', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['authorizeSecurityGroupEgress'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Adds the specified ingress rules to a security group.   An inbound rule permits instances to receive traffic from the specified destination IPv4 or IPv6 CIDR address ranges, or from the specified destination security groups.   You specify a protocol for each rule (for example, TCP). For TCP and UDP, you must also specify the destination port or port range. For ICMP/ICMPv6, you must also specify the ICMP/ICMPv6 type and code. You can use -1 to mean all types or all codes.   Rule changes are prop...(description truncated)
   *
   * @function authorizeSecurityGroupIngress
   * @param {string} cidrIp - The IPv4 address range, in CIDR format. You can't specify this parameter when specifying a source security group. To specify an IPv6 address range, use a set of IP permi...(description truncated)
   * @param {number} fromPort - The start of port range for the TCP and UDP protocols, or an ICMP type number. For the ICMP type number, use  -1  to specify all types. If you specify all ICMP types, yo...(description truncated)
   * @param {string} groupId - The ID of the security group. You must specify either the security group ID or the security group name in the request. For security groups in a nondefault VPC, you must s...(description truncated)
   * @param {string} groupName - [EC2-Classic, default VPC] The name of the security group. You must specify either the security group ID or the security group name in the request.
   * @param {array} ipPermissions - The sets of IP permissions.
   * @param {string} ipProtocol - The IP protocol name ( tcp ,  udp ,  icmp ) or number (see  Protocol Numbers ). To specify  icmpv6 , use a set of IP permissions.   [VPC only] Use  -1  to specify all pr...(description truncated)
   * @param {string} sourceSecurityGroupName - [EC2-Classic, default VPC] The name of the source security group. You can't specify this parameter in combination with the following parameters: the CIDR IP address range...(description truncated)
   * @param {string} sourceSecurityGroupOwnerId - [nondefault VPC] The AWS account ID for the source security group, if the source security group is in a different account. You can't specify this parameter in combination...(description truncated)
   * @param {number} toPort - The end of port range for the TCP and UDP protocols, or an ICMP code number. For the ICMP code number, use  -1  to specify all codes. If you specify all ICMP types, you ...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  authorizeSecurityGroupIngress(cidrIp, fromPort, groupId, groupName, ipPermissions, ipProtocol, sourceSecurityGroupName, sourceSecurityGroupOwnerId, toPort, dryRun, callback) {
    const meth = 'adapter-authorizeSecurityGroupIngress';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'AuthorizeSecurityGroupIngress';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      CidrIp: cidrIp,
      FromPort: fromPort,
      GroupId: groupId,
      GroupName: groupName,
      IpProtocol: ipProtocol,
      SourceSecurityGroupName: sourceSecurityGroupName,
      SourceSecurityGroupOwnerId: sourceSecurityGroupOwnerId,
      ToPort: toPort,
      Action: action,
      Version: version
    };
    if (ipPermissions && ipPermissions.length > 0) {
      processInArray(ipPermissions, queryParamsAvailable, 'IpPermissions');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('SecurityGroup', 'authorizeSecurityGroupIngress', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['authorizeSecurityGroupIngress'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Bundles an Amazon instance store-backed Windows instance.   During bundling, only the root device volume (C:\) is bundled. Data on other instance store volumes is not preserved.     This action is not applicable for Linux/Unix instances or Windows instances that are backed by Amazon EBS.
   *
   * @function bundleInstance
   * @param {string} instanceId - The ID of the instance to bundle.   Type: String   Default: None   Required: Yes
   * @param {string} storageS3 - Describes the storage location for an instance store-backed AMI.
An Amazon S3 storage location.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  bundleInstance(instanceId, storageS3, dryRun, callback) {
    const meth = 'adapter-bundleInstance';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'BundleInstance';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      InstanceId: instanceId,
      StorageS3: storageS3,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'bundleInstance', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['bundleInstance'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Cancels a bundling operation for an instance store-backed Windows instance.
   *
   * @function cancelBundleTask
   * @param {string} bundleId - The ID of the bundle task.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  cancelBundleTask(bundleId, dryRun, callback) {
    const meth = 'adapter-cancelBundleTask';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CancelBundleTask';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (bundleId === undefined || bundleId === null || bundleId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['bundleId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      BundleId: bundleId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionCancelBundleTask', 'getCancelBundleTask', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['cancelBundleTask'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Cancels the specified Capacity Reservation, releases the reserved capacity, and changes the Capacity Reservation's state to  cancelled .   Instances running in the reserved capacity continue running until you stop them. Stopped instances that target the Capacity Reservation can no longer launch. Modify these instances to either target a different Capacity Reservation, launch On-Demand Instance capacity, or run in any open Capacity Reservation that has matching attributes and sufficient capacity...(description truncated)
   *
   * @function cancelCapacityReservation
   * @param {string} capacityReservationId - The ID of the Capacity Reservation to be cancelled.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  cancelCapacityReservation(capacityReservationId, dryRun, callback) {
    const meth = 'adapter-cancelCapacityReservation';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CancelCapacityReservation';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (capacityReservationId === undefined || capacityReservationId === null || capacityReservationId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['capacityReservationId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      CapacityReservationId: capacityReservationId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Capacity', 'cancelCapacityReservation', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['cancelCapacityReservation'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Cancels an active conversion task. The task can be the import of an instance or volume. The action removes all artifacts of the conversion, including a partially uploaded volume or instance. If the conversion is complete or is in the process of transferring the final disk image, the command fails and returns an exception.   For more information, see  Importing a Virtual Machine Using the Amazon EC2 CLI .
   *
   * @function cancelConversionTask
   * @param {string} conversionTaskId - The ID of the conversion task.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} reasonMessage - The reason for canceling the conversion task.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  cancelConversionTask(conversionTaskId, dryRun, reasonMessage, callback) {
    const meth = 'adapter-cancelConversionTask';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CancelConversionTask';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (conversionTaskId === undefined || conversionTaskId === null || conversionTaskId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['conversionTaskId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ConversionTaskId: conversionTaskId,
      ReasonMessage: reasonMessage,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionCancelConversionTask', 'getCancelConversionTask', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['cancelConversionTask'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Cancels an active export task. The request removes all artifacts of the export, including any partially-created Amazon S3 objects. If the export task is complete or is in the process of transferring the final disk image, the command fails and returns an error.
   *
   * @function cancelExportTask
   * @param {string} exportTaskId - The ID of the export task. This is the ID returned by  CreateInstanceExportTask .
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  cancelExportTask(exportTaskId, callback) {
    const meth = 'adapter-cancelExportTask';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CancelExportTask';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (exportTaskId === undefined || exportTaskId === null || exportTaskId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['exportTaskId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      ExportTaskId: exportTaskId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionCancelExportTask', 'getCancelExportTask', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['cancelExportTask'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Cancels an in-process import virtual machine or import snapshot task.
   *
   * @function cancelImportTask
   * @param {string} cancelReason - The reason for canceling the task.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} importTaskId - The ID of the import image or import snapshot task to be canceled.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  cancelImportTask(cancelReason, dryRun, importTaskId, callback) {
    const meth = 'adapter-cancelImportTask';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CancelImportTask';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      CancelReason: cancelReason,
      ImportTaskId: importTaskId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionCancelImportTask', 'getCancelImportTask', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['cancelImportTask'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Cancels the specified Reserved Instance listing in the Reserved Instance Marketplace.   For more information, see  Reserved Instance Marketplace  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function cancelReservedInstancesListing
   * @param {string} reservedInstancesListingId - The ID of the Reserved Instance listing.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  cancelReservedInstancesListing(reservedInstancesListingId, callback) {
    const meth = 'adapter-cancelReservedInstancesListing';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CancelReservedInstancesListing';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (reservedInstancesListingId === undefined || reservedInstancesListingId === null || reservedInstancesListingId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['reservedInstancesListingId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      ReservedInstancesListingId: reservedInstancesListingId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'cancelReservedInstancesListing', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['cancelReservedInstancesListing'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Cancels the specified Spot Fleet requests.   After you cancel a Spot Fleet request, the Spot Fleet launches no new Spot Instances. You must specify whether the Spot Fleet should also terminate its Spot Instances. If you terminate the instances, the Spot Fleet request enters the  cancelled_terminating  state. Otherwise, the Spot Fleet request enters the  cancelled_running  state and the instances continue to run until they are interrupted or you terminate them manually.
   *
   * @function cancelSpotFleetRequests
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} spotFleetRequestId - The IDs of the Spot Fleet requests.
   * @param {boolean} terminateInstances - Indicates whether to terminate instances for a Spot Fleet request if it is canceled successfully.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  cancelSpotFleetRequests(dryRun, spotFleetRequestId, terminateInstances, callback) {
    const meth = 'adapter-cancelSpotFleetRequests';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CancelSpotFleetRequests';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (spotFleetRequestId === undefined || spotFleetRequestId === null || spotFleetRequestId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['spotFleetRequestId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (terminateInstances === undefined || terminateInstances === null || terminateInstances === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['terminateInstances'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      TerminateInstances: terminateInstances,
      Action: action,
      Version: version
    };
    if (spotFleetRequestId && spotFleetRequestId.length > 0) {
      processInArray(spotFleetRequestId, queryParamsAvailable, 'SpotFleetRequestId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Fleet', 'cancelSpotFleetRequests', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['cancelSpotFleetRequests'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Cancels one or more Spot Instance requests.     Canceling a Spot Instance request does not terminate running Spot Instances associated with the request.
   *
   * @function cancelSpotInstanceRequests
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} spotInstanceRequestId - One or more Spot Instance request IDs.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  cancelSpotInstanceRequests(dryRun, spotInstanceRequestId, callback) {
    const meth = 'adapter-cancelSpotInstanceRequests';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CancelSpotInstanceRequests';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (spotInstanceRequestId === undefined || spotInstanceRequestId === null || spotInstanceRequestId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['spotInstanceRequestId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (spotInstanceRequestId && spotInstanceRequestId.length > 0) {
      processInArray(spotInstanceRequestId, queryParamsAvailable, 'SpotInstanceRequestId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'cancelSpotInstanceRequests', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['cancelSpotInstanceRequests'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Determines whether a product code is associated with an instance. This action can only be used by the owner of the product code. It is useful when a product code owner must verify whether another user's instance is eligible for support.
   *
   * @function confirmProductInstance
   * @param {string} instanceId - The ID of the instance.
   * @param {string} productCode - The product code. This must be a product code that you own.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  confirmProductInstance(instanceId, productCode, dryRun, callback) {
    const meth = 'adapter-confirmProductInstance';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ConfirmProductInstance';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (productCode === undefined || productCode === null || productCode === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['productCode'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      InstanceId: instanceId,
      ProductCode: productCode,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'confirmProductInstance', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['confirmProductInstance'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Copies the specified Amazon FPGA Image (AFI) to the current Region.
   *
   * @function copyFpgaImage
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} sourceFpgaImageId - The ID of the source AFI.
   * @param {string} description - The description for the new AFI.
   * @param {string} name - The name for the new AFI. The default is the name of the source AFI.
   * @param {string} sourceRegion - The Region that contains the source AFI.
   * @param {string} clientToken - Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see  Ensuring Idempotency .
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  copyFpgaImage(dryRun, sourceFpgaImageId, description, name, sourceRegion, clientToken, callback) {
    const meth = 'adapter-copyFpgaImage';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CopyFpgaImage';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (sourceFpgaImageId === undefined || sourceFpgaImageId === null || sourceFpgaImageId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['sourceFpgaImageId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (sourceRegion === undefined || sourceRegion === null || sourceRegion === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['sourceRegion'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      SourceFpgaImageId: sourceFpgaImageId,
      Description: description,
      Name: name,
      SourceRegion: sourceRegion,
      ClientToken: clientToken,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Image', 'copyFpgaImage', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['copyFpgaImage'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Initiates the copy of an AMI from the specified source Region to the current Region. You specify the destination Region by using its endpoint when making the request.   Copies of encrypted backing snapshots for the AMI are encrypted. Copies of unencrypted backing snapshots remain unencrypted, unless you set  Encrypted  during the copy operation. You cannot create an unencrypted copy of an encrypted backing snapshot.   For more information about the prerequisites and limits when copying an AMI, ...(description truncated)
   *
   * @function copyImage
   * @param {string} clientToken - Unique, case-sensitive identifier you provide to ensure idempotency of the request. For more information, see  How to Ensure Idempotency  in the  Amazon Elastic Compute C...(description truncated)
   * @param {string} description - A description for the new AMI in the destination Region.
   * @param {boolean} encrypted - Specifies whether the destination snapshots of the copied image should be encrypted. You can encrypt a copy of an unencrypted snapshot, but you cannot create an unencrypt...(description truncated)
   * @param {string} kmsKeyId - An identifier for the AWS Key Management Service (AWS KMS) customer master key (CMK) to use when creating the encrypted volume. This parameter is only required if you wa...(description truncated)
   * @param {string} name - The name of the new AMI in the destination Region.
   * @param {string} sourceImageId - The ID of the AMI to copy.
   * @param {string} sourceRegion - The name of the Region that contains the AMI to copy.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  copyImage(clientToken, description, encrypted, kmsKeyId, name, sourceImageId, sourceRegion, dryRun, callback) {
    const meth = 'adapter-copyImage';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CopyImage';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (name === undefined || name === null || name === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['name'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (sourceImageId === undefined || sourceImageId === null || sourceImageId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['sourceImageId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (sourceRegion === undefined || sourceRegion === null || sourceRegion === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['sourceRegion'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientToken: clientToken,
      Description: description,
      Encrypted: encrypted,
      KmsKeyId: kmsKeyId,
      Name: name,
      SourceImageId: sourceImageId,
      SourceRegion: sourceRegion,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Image', 'copyImage', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['copyImage'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Copies a point-in-time snapshot of an EBS volume and stores it in Amazon S3. You can copy the snapshot within the same Region or from one Region to another. You can use the snapshot to create EBS volumes or Amazon Machine Images (AMIs). The snapshot is copied to the regional endpoint that you send the HTTP request to.   Copies of encrypted EBS snapshots remain encrypted. Copies of unencrypted snapshots remain unencrypted, unless the  Encrypted  flag is specified during the snapshot copy operati...(description truncated)
   *
   * @function copySnapshot
   * @param {string} description - A description for the EBS snapshot.
   * @param {string} destinationRegion - The destination Region to use in the  PresignedUrl  parameter of a snapshot copy operation. This parameter is only valid for specifying the destination Region in a  Pres...(description truncated)
   * @param {boolean} encrypted - Specifies whether the destination snapshot should be encrypted. You can encrypt a copy of an unencrypted snapshot, but you cannot use it to create an unencrypted copy of ...(description truncated)
   * @param {string} kmsKeyId - An identifier for the AWS Key Management Service (AWS KMS) customer master key (CMK) to use when creating the encrypted volume. This parameter is only required if you wa...(description truncated)
   * @param {string} presignedUrl - When you copy an encrypted source snapshot using the Amazon EC2 Query API, you must supply a pre-signed URL. This parameter is optional for unencrypted snapshots. For mo...(description truncated)
   * @param {string} sourceRegion - The ID of the Region that contains the snapshot to be copied.
   * @param {string} sourceSnapshotId - The ID of the EBS snapshot to copy.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  copySnapshot(description, destinationRegion, encrypted, kmsKeyId, presignedUrl, sourceRegion, sourceSnapshotId, dryRun, callback) {
    const meth = 'adapter-copySnapshot';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CopySnapshot';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (sourceRegion === undefined || sourceRegion === null || sourceRegion === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['sourceRegion'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (sourceSnapshotId === undefined || sourceSnapshotId === null || sourceSnapshotId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['sourceSnapshotId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Description: description,
      DestinationRegion: destinationRegion,
      Encrypted: encrypted,
      KmsKeyId: kmsKeyId,
      PresignedUrl: presignedUrl,
      SourceRegion: sourceRegion,
      SourceSnapshotId: sourceSnapshotId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Snapshot', 'copySnapshot', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['copySnapshot'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a new Capacity Reservation with the specified attributes.   Capacity Reservations enable you to reserve capacity for your Amazon EC2 instances in a specific Availability Zone for any duration. This gives you the flexibility to selectively add capacity reservations and still get the Regional RI discounts for that usage. By creating Capacity Reservations, you ensure that you always have access to Amazon EC2 capacity when you need it, for as long as you need it. For more information, see  ...(description truncated)
   *
   * @function createCapacityReservation
   * @param {string} clientToken - Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see  How to Ensure Idempotency .   Constraint: Maximum...(description truncated)
   * @param {string} instanceType - The instance type for which to reserve capacity. For more information, see  Instance Types  in the  Amazon Elastic Compute Cloud User Guide .
   * @param {string} instancePlatform - The type of operating system for which to reserve capacity.
   * @param {string} availabilityZone - The Availability Zone in which to create the Capacity Reservation.
   * @param {string} tenancy - Indicates the tenancy of the Capacity Reservation. A Capacity Reservation can have one of the following tenancy settings:         default  - The Capacity Reservation is ...(description truncated)
   * @param {number} instanceCount - The number of instances for which to reserve capacity.
   * @param {boolean} ebsOptimized - Indicates whether the Capacity Reservation supports EBS-optimized instances. This optimization provides dedicated throughput to Amazon EBS and an optimized configuration ...(description truncated)
   * @param {boolean} ephemeralStorage - Indicates whether the Capacity Reservation supports instances with temporary, block-level storage.
   * @param {string} endDate - The date and time at which the Capacity Reservation expires. When a Capacity Reservation expires, the reserved capacity is released and you can no longer launch instance...(description truncated)
   * @param {string} endDateType - Indicates the way in which the Capacity Reservation ends. A Capacity Reservation can have one of the following end types:         unlimited  - The Capacity Reservation r...(description truncated)
   * @param {string} instanceMatchCriteria - Indicates the type of instance launches that the Capacity Reservation accepts. The options include:         open  - The Capacity Reservation automatically matches all in...(description truncated)
   * @param {array} tagSpecifications - The tags to apply to the Capacity Reservation during launch.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createCapacityReservation(clientToken, instanceType, instancePlatform = 'Linux/UNIX', availabilityZone, tenancy = 'default', instanceCount, ebsOptimized, ephemeralStorage, endDate, endDateType = 'unlimited', instanceMatchCriteria = 'open', tagSpecifications, dryRun, callback) {
    const meth = 'adapter-createCapacityReservation';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateCapacityReservation';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceType === undefined || instanceType === null || instanceType === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceType'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (instancePlatform === undefined || instancePlatform === null || instancePlatform === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instancePlatform'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (availabilityZone === undefined || availabilityZone === null || availabilityZone === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['availabilityZone'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (instanceCount === undefined || instanceCount === null || instanceCount === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceCount'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientToken: clientToken,
      InstanceType: instanceType,
      InstancePlatform: instancePlatform,
      AvailabilityZone: availabilityZone,
      Tenancy: tenancy,
      InstanceCount: instanceCount,
      EbsOptimized: ebsOptimized,
      EphemeralStorage: ephemeralStorage,
      EndDate: endDate,
      EndDateType: endDateType,
      InstanceMatchCriteria: instanceMatchCriteria,
      Action: action,
      Version: version
    };
    if (tagSpecifications && tagSpecifications.length > 0) {
      processInArray(tagSpecifications, queryParamsAvailable, 'TagSpecifications');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Capacity', 'createCapacityReservation', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createCapacityReservation'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple new Capacity Reservations with the specified attributes.   Capacity Reservations enable you to reserve capacity for your Amazon EC2 instances in a specific Availability Zone for any duration. This gives you the flexibility to selectively add capacity reservations and still get the Regional RI discounts for that usage. By creating Capacity Reservations, you ensure that you always have access to Amazon EC2 capacity when you need it, for as long as you need it. For more information, see  ...(description truncated)
   *
   * @function createCapacityReservations
   * @param {array} capacityReservationArray - An array of objects(capacity reservation parameters). clientToken, instanceType, instancePlatform, availabilityZone, tenancy, instanceCount, ebsOptimized, ephemeralStorage, endDate, endDateType, instanceMatchCriteria, tagSpecifications, dryRun.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createCapacityReservations(capacityReservationArray, callback) {
    const meth = 'adapter-createCapacityReservations';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (capacityReservationArray === undefined || capacityReservationArray === null || capacityReservationArray === '' || capacityReservationArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['capacityReservationArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      capacityReservationArray.forEach((capacityReservation) => {
        const { clientToken, instanceType, instancePlatform, availabilityZone, tenancy, instanceCount, ebsOptimized, ephemeralStorage, endDate, endDateType, instanceMatchCriteria, tagSpecifications, dryRun } = capacityReservation;
        returnArray.push(
          new Promise((resolve) => {
            this.createCapacityReservation(clientToken, instanceType, instancePlatform, availabilityZone, tenancy, instanceCount, ebsOptimized, ephemeralStorage, endDate, endDateType, instanceMatchCriteria, tagSpecifications, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a Client VPN endpoint. A Client VPN endpoint is the resource you create and configure to enable and manage client VPN sessions. It is the destination endpoint at which all client VPN sessions are terminated.
   *
   * @function createClientVpnEndpoint
   * @param {string} clientCidrBlock - The IPv4 address range, in CIDR notation, from which to assign client IP addresses. The address range cannot overlap with the local CIDR of the VPC in which the associate...(description truncated)
   * @param {string} serverCertificateArn - The ARN of the server certificate. For more information, see the  AWS Certificate Manager User Guide .
   * @param {array} authentication - Information about the authentication method to be used to authenticate clients.
   * @param {string} connectionLogOptionsEnabled - Describes the client connection logging options for the Client VPN endpoint.
Indicates whether connection logging is enabled.
   * @param {string} connectionLogOptionsCloudwatchLogGroup - Describes the client connection logging options for the Client VPN endpoint.
The name of the CloudWatch Logs log group.
   * @param {string} connectionLogOptionsCloudwatchLogStream - Describes the client connection logging options for the Client VPN endpoint.
The name of the CloudWatch Logs log stream to which the connection data is published.
   * @param {array} dnsServers - Information about the DNS servers to be used for DNS resolution. A Client VPN endpoint can have up to two DNS servers. If no DNS server is specified, the DNS address of t...(description truncated)
   * @param {string} transportProtocol - The transport protocol to be used by the VPN session.   Default value:  udp
   * @param {string} description - A brief description of the Client VPN endpoint.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} clientToken - Unique, case-sensitive identifier you provide to ensure the idempotency of the request. For more information, see   How to Ensure Idempotency .
   * @param {array} tagSpecification - The tags to apply to the Client VPN endpoint during creation.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createClientVpnEndpoint(clientCidrBlock, serverCertificateArn, authentication, connectionLogOptionsEnabled, connectionLogOptionsCloudwatchLogGroup, connectionLogOptionsCloudwatchLogStream, dnsServers, transportProtocol = 'tcp', description, dryRun, clientToken, tagSpecification, callback) {
    const meth = 'adapter-createClientVpnEndpoint';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateClientVpnEndpoint';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientCidrBlock === undefined || clientCidrBlock === null || clientCidrBlock === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientCidrBlock'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (serverCertificateArn === undefined || serverCertificateArn === null || serverCertificateArn === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['serverCertificateArn'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (authentication === undefined || authentication === null || authentication === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['authentication'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientCidrBlock: clientCidrBlock,
      ServerCertificateArn: serverCertificateArn,
      ConnectionLogOptionsEnabled: connectionLogOptionsEnabled,
      ConnectionLogOptionsCloudwatchLogGroup: connectionLogOptionsCloudwatchLogGroup,
      ConnectionLogOptionsCloudwatchLogStream: connectionLogOptionsCloudwatchLogStream,
      TransportProtocol: transportProtocol,
      Description: description,
      ClientToken: clientToken,
      Action: action,
      Version: version
    };
    if (authentication && authentication.length > 0) {
      processInArray(authentication, queryParamsAvailable, 'Authentication');
    }
    if (dnsServers && dnsServers.length > 0) {
      processInArray(dnsServers, queryParamsAvailable, 'DnsServers');
    }
    if (tagSpecification && tagSpecification.length > 0) {
      processInArray(tagSpecification, queryParamsAvailable, 'TagSpecification');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'createClientVpnEndpoint', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createClientVpnEndpoint'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple Client VPN endpoints. A Client VPN endpoint is the resource you create and configure to enable and manage client VPN sessions. It is the destination endpoint at which all client VPN sessions are terminated.
   *
   * @function createClientVpnEndpoints
   * @param {array} clientVpnEndpointArray - An array of objects(capacity reservation parameters). clientCidrBlock, serverCertificateArn, authentication, connectionLogOptionsEnabled, connectionLogOptionsCloudwatchLogGroup, connectionLogOptionsCloudwatchLogStream, dnsServers, transportProtocol, description, dryRun, clientToken, tagSpecification.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createClientVpnEndpoints(clientVpnEndpointArray, callback) {
    const meth = 'adapter-createClientVpnEndpoints';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointArray === undefined || clientVpnEndpointArray === null || clientVpnEndpointArray === '' || clientVpnEndpointArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      clientVpnEndpointArray.forEach((clientVpnEndpoint) => {
        const { clientCidrBlock, serverCertificateArn, authentication, connectionLogOptionsEnabled, connectionLogOptionsCloudwatchLogGroup, connectionLogOptionsCloudwatchLogStream, dnsServers, transportProtocol, description, dryRun, clientToken, tagSpecification } = clientVpnEndpoint;
        returnArray.push(
          new Promise((resolve) => {
            this.createClientVpnEndpoint(clientCidrBlock, serverCertificateArn, authentication, connectionLogOptionsEnabled, connectionLogOptionsCloudwatchLogGroup, connectionLogOptionsCloudwatchLogStream, dnsServers, transportProtocol, description, dryRun, clientToken, tagSpecification, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Adds a route to a network to a Client VPN endpoint. Each Client VPN endpoint has a route table that describes the available destination network routes. Each route in the route table specifies the path for traﬃc to speciﬁc resources or networks.
   *
   * @function createClientVpnRoute
   * @param {string} clientVpnEndpointId - The ID of the Client VPN endpoint to which to add the route.
   * @param {string} destinationCidrBlock - The IPv4 address range, in CIDR notation, of the route destination. For example:       To add a route for Internet access, enter  0.0.0.0/0         To add a route for a ...(description truncated)
   * @param {string} targetVpcSubnetId - The ID of the subnet through which you want to route traffic. The specified subnet must be an existing target network of the Client VPN endpoint.
   * @param {string} description - A brief description of the route.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createClientVpnRoute(clientVpnEndpointId, destinationCidrBlock, targetVpcSubnetId, description, dryRun, callback) {
    const meth = 'adapter-createClientVpnRoute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateClientVpnRoute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointId === undefined || clientVpnEndpointId === null || clientVpnEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (destinationCidrBlock === undefined || destinationCidrBlock === null || destinationCidrBlock === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['destinationCidrBlock'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (targetVpcSubnetId === undefined || targetVpcSubnetId === null || targetVpcSubnetId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['targetVpcSubnetId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientVpnEndpointId: clientVpnEndpointId,
      DestinationCidrBlock: destinationCidrBlock,
      TargetVpcSubnetId: targetVpcSubnetId,
      Description: description,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'createClientVpnRoute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createClientVpnRoute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Adds multiple routes to a network to a Client VPN endpoint. Each Client VPN endpoint has a route table that describes the available destination network routes. Each route in the route table specifies the path for traﬃc to speciﬁc resources or networks.
   *
   * @function createClientVpnRoutes
   * @param {array} createClientVpnRouteArray - An array of objects(Client Vpn Routes parameters). clientVpnEndpointId, destinationCidrBlock, targetVpcSubnetId, description, dryRun.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createClientVpnRoutes(createClientVpnRouteArray, callback) {
    const meth = 'adapter-createClientVpnRoutes';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (createClientVpnRouteArray === undefined || createClientVpnRouteArray === null || createClientVpnRouteArray === '' || createClientVpnRouteArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['createClientVpnRouteArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      createClientVpnRouteArray.forEach((item) => {
        const { clientVpnEndpointId, destinationCidrBlock, targetVpcSubnetId, description, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createClientVpnRoute(clientVpnEndpointId, destinationCidrBlock, targetVpcSubnetId, description, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Provides information to AWS about your VPN customer gateway device. The customer gateway is the appliance at your end of the VPN connection. (The device on the AWS side of the VPN connection is the virtual private gateway.) You must provide the Internet-routable IP address of the customer gateway's external interface. The IP address must be static and may be behind a device performing network address translation (NAT).   For devices that use Border Gateway Protocol (BGP), you can also provide t...(description truncated)
   *
   * @function createCustomerGateway
   * @param {number} bgpAsn - For devices that support BGP, the customer gateway's BGP ASN.   Default: 65000
   * @param {string} ipAddress - The Internet-routable IP address for the customer gateway's outside interface. The address must be static.
   * @param {string} type - The type of VPN connection that this customer gateway supports ( ipsec.1 ).
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createCustomerGateway(bgpAsn, ipAddress, type = 'ipsec.1', dryRun, callback) {
    const meth = 'adapter-createCustomerGateway';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateCustomerGateway';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (bgpAsn === undefined || bgpAsn === null || bgpAsn === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['bgpAsn'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (ipAddress === undefined || ipAddress === null || ipAddress === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['ipAddress'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (type === undefined || type === null || type === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['type'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      BgpAsn: bgpAsn,
      IpAddress: ipAddress,
      Type: type,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Gateway', 'createCustomerGateway', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createCustomerGateway'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Create multiple customer gateways. The customer gateway is the appliance at your end of the VPN connection. (The device on the AWS side of the VPN connection is the virtual private gateway.) You must provide the Internet-routable IP address of the customer gateway's external interface. The IP address must be static and may be behind a device performing network address translation (NAT).   For devices that use Border Gateway Protocol (BGP), you can also provide t...(description truncated)
   *
   * @function createCustomerGateways
   * @param {array} createCustomerGatewayArray - An array of objects(CustomerGateway parameters). bgpAsn, ipAddress, type, dryRun.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createCustomerGateways(createCustomerGatewayArray, callback) {
    const meth = 'adapter-createCustomerGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (createCustomerGatewayArray === undefined || createCustomerGatewayArray === null || createCustomerGatewayArray === '' || createCustomerGatewayArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['createCustomerGatewayArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      createCustomerGatewayArray.forEach((item) => {
        const { bgpAsn, ipAddress, type, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createCustomerGateway(bgpAsn, ipAddress, type, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a default subnet with a size  /20  IPv4 CIDR block in the specified Availability Zone in your default VPC. You can have only one default subnet per Availability Zone. For more information, see  Creating a Default Subnet  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function createDefaultSubnet
   * @param {string} availabilityZone - The Availability Zone in which to create the default subnet.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createDefaultSubnet(availabilityZone, dryRun, callback) {
    const meth = 'adapter-createDefaultSubnet';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateDefaultSubnet';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (availabilityZone === undefined || availabilityZone === null || availabilityZone === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['availabilityZone'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AvailabilityZone: availabilityZone,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Subnet', 'createDefaultSubnet', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createDefaultSubnet'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates mutiple default subnets with a size  /20  IPv4 CIDR block in the specified Availability Zone in your default VPC. You can have only one default subnet per Availability Zone. For more information, see  Creating a Default Subnet  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function createDefaultSubnets
   * @param {array} createDefaultSubnetArray - An array of objects(DefaultSubnets parameters). availabilityZone, dryRun.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createDefaultSubnets(createDefaultSubnetArray, callback) {
    const meth = 'adapter-createDefaultSubnets';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (createDefaultSubnetArray === undefined || createDefaultSubnetArray === null || createDefaultSubnetArray === '' || createDefaultSubnetArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['createDefaultSubnetArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      createDefaultSubnetArray.forEach((item) => {
        const { availabilityZone, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createDefaultSubnet(availabilityZone, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a default VPC with a size  /16  IPv4 CIDR block and a default subnet in each Availability Zone. For more information about the components of a default VPC, see  Default VPC and Default Subnets  in the  Amazon Virtual Private Cloud User Guide . You cannot specify the components of the default VPC yourself.   If you deleted your previous default VPC, you can create a default VPC. You cannot have more than one default VPC per Region.   If your account supports EC2-Classic, you cannot use t...(description truncated)
   *
   * @function createDefaultVpc
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createDefaultVpc(dryRun, callback) {
    const meth = 'adapter-createDefaultVpc';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateDefaultVpc';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'createDefaultVpc', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createDefaultVpc'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple default VPCs with a size  /16  IPv4 CIDR block and a default subnet in each Availability Zone. For more information about the components of a default VPC, see  Default VPC and Default Subnets  in the  Amazon Virtual Private Cloud User Guide . You cannot specify the components of the default VPC yourself.   If you deleted your previous default VPC, you can create a default VPC. You cannot have more than one default VPC per Region.   If your account supports EC2-Classic, you cannot use t...(description truncated)
   *
   * @function createDefaultVpcs
   * @param {array} createDefaultVpcArray - An array of objects(DefaultVpc parameters). dryRun.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createDefaultVpcs(createDefaultVpcArray, callback) {
    const meth = 'adapter-createDefaultVpcs';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (createDefaultVpcArray === undefined || createDefaultVpcArray === null || createDefaultVpcArray === '' || createDefaultVpcArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['createDefaultVpcArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      createDefaultVpcArray.forEach((item) => {
        const { dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createDefaultVpc(dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a set of DHCP options for your VPC. After creating the set, you must associate it with the VPC, causing all existing and new instances that you launch in the VPC to use this set of DHCP options. The following are the individual DHCP options you can specify. For more information about the options, see  RFC 2132 .         domain-name-servers  - The IP addresses of up to four domain name servers, or AmazonProvidedDNS. The default DHCP option set specifies AmazonProvidedDNS. If specifying m...(description truncated)
   *
   * @function createDhcpOptions
   * @param {array} dhcpConfiguration - A DHCP configuration option.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createDhcpOptions(dhcpConfiguration, dryRun, callback) {
    const meth = 'adapter-createDhcpOptions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateDhcpOptions';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (dhcpConfiguration === undefined || dhcpConfiguration === null || dhcpConfiguration === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['dhcpConfiguration'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (dhcpConfiguration && dhcpConfiguration.length > 0) {
      processInArray(dhcpConfiguration, queryParamsAvailable, 'DhcpConfiguration');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('DHCP', 'createDhcpOptions', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createDhcpOptions'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple DHCP options for your VPC. After creating the set, you must associate it with the VPC, causing all existing and new instances that you launch in the VPC to use this set of DHCP options. The following are the individual DHCP options you can specify. For more information about the options, see  RFC 2132 .         domain-name-servers  - The IP addresses of up to four domain name servers, or AmazonProvidedDNS. The default DHCP option set specifies AmazonProvidedDNS. If specifying m...(description truncated)
   *
   * @function createMultipleDhcpOptions
   * @param {array} dhcpOptionsArray - An array of objects(Dhcp Options parameters). dhcpConfiguration, dryRun.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createMultipleDhcpOptions(dhcpOptionsArray, callback) {
    const meth = 'adapter-createMultipleDhcpOptions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (dhcpOptionsArray === undefined || dhcpOptionsArray === null || dhcpOptionsArray === '' || dhcpOptionsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['dhcpOptionsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      dhcpOptionsArray.forEach((item) => {
        const { dhcpConfiguration, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createDhcpOptions(dhcpConfiguration, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary [IPv6 only] Creates an egress-only internet gateway for your VPC. An egress-only internet gateway is used to enable outbound communication over IPv6 from instances in your VPC to the internet, and prevents hosts outside of your VPC from initiating an IPv6 connection with your instance.
   *
   * @function createEgressOnlyInternetGateway
   * @param {string} clientToken - Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see  How to Ensure Idempotency .
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} vpcId - The ID of the VPC for which to create the egress-only internet gateway.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createEgressOnlyInternetGateway(clientToken, dryRun, vpcId, callback) {
    const meth = 'adapter-createEgressOnlyInternetGateway';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateEgressOnlyInternetGateway';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientToken: clientToken,
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Gateway', 'createEgressOnlyInternetGateway', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createEgressOnlyInternetGateway'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple egress-only internet gateways for your VPC.[IPv6 only]
   *
   * @function createEgressOnlyInternetGateways
   * @param {array} EgressOnlyInternetGatewaysArray - An array of objects(internet gateways parameters). clientToken, dryRun, vpcId.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createEgressOnlyInternetGateways(EgressOnlyInternetGatewaysArray, callback) {
    const meth = 'adapter-createEgressOnlyInternetGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (EgressOnlyInternetGatewaysArray === undefined || EgressOnlyInternetGatewaysArray === null || EgressOnlyInternetGatewaysArray === '' || EgressOnlyInternetGatewaysArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['EgressOnlyInternetGatewaysArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      EgressOnlyInternetGatewaysArray.forEach((item) => {
        const { clientToken, dryRun, vpcId } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createEgressOnlyInternetGateway(clientToken, dryRun, vpcId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Launches an EC2 Fleet.   You can create a single EC2 Fleet that includes multiple launch specifications that vary by instance type, AMI, Availability Zone, or subnet.   For more information, see  Launching an EC2 Fleet  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function createFleet
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} clientToken - Unique, case-sensitive identifier you provide to ensure the idempotency of the request. For more information, see  Ensuring Idempotency .
   * @param {string} spotOptionsAllocationStrategy - Describes the configuration of Spot Instances in an EC2 Fleet request.
Indicates how to allocate the target capacity across the Spot pools specified by the Spot Fleet req...(description truncated)
   * @param {string} spotOptionsInstanceInterruptionBehavior - Describes the configuration of Spot Instances in an EC2 Fleet request.
The behavior when a Spot Instance is interrupted. The default is  terminate .
   * @param {string} spotOptionsInstancePoolsToUseCount - Describes the configuration of Spot Instances in an EC2 Fleet request.
The number of Spot pools across which to allocate your target Spot capacity. Valid only when Spot  ...(description truncated)
   * @param {string} spotOptionsSingleInstanceType - Describes the configuration of Spot Instances in an EC2 Fleet request.
Indicates that the fleet uses a single instance type to launch all Spot Instances in the fleet.
   * @param {string} spotOptionsSingleAvailabilityZone - Describes the configuration of Spot Instances in an EC2 Fleet request.
Indicates that the fleet launches all Spot Instances into a single Availability Zone.
   * @param {string} spotOptionsMinTargetCapacity - Describes the configuration of Spot Instances in an EC2 Fleet request.
The minimum target capacity for Spot Instances in the fleet. If the minimum target capacity is not ...(description truncated)
   * @param {string} onDemandOptionsAllocationStrategy - The allocation strategy of On-Demand Instances in an EC2 Fleet.
The order of the launch template overrides to use in fulfilling On-Demand capacity. If you specify  lowest...(description truncated)
   * @param {string} onDemandOptionsSingleInstanceType - The allocation strategy of On-Demand Instances in an EC2 Fleet.
Indicates that the fleet uses a single instance type to launch all On-Demand Instances in the fleet.
   * @param {string} onDemandOptionsSingleAvailabilityZone - The allocation strategy of On-Demand Instances in an EC2 Fleet.
Indicates that the fleet launches all On-Demand Instances into a single Availability Zone.
   * @param {string} onDemandOptionsMinTargetCapacity - The allocation strategy of On-Demand Instances in an EC2 Fleet.
The minimum target capacity for On-Demand Instances in the fleet. If the minimum target capacity is not re...(description truncated)
   * @param {string} excessCapacityTerminationPolicy - Indicates whether running instances should be terminated if the total target capacity of the EC2 Fleet is decreased below the current size of the EC2 Fleet.
   * @param {array} launchTemplateConfigs - The configuration for the EC2 Fleet.
   * @param {string} targetCapacitySpecificationTotalTargetCapacity - The number of units to request. You can choose to set the target capacity in terms of instances or a performance characteristic that is important to your application work...(description truncated)
   * @param {string} targetCapacitySpecificationOnDemandTargetCapacity - The number of units to request. You can choose to set the target capacity in terms of instances or a performance characteristic that is important to your application work...(description truncated)
   * @param {string} targetCapacitySpecificationSpotTargetCapacity - The number of units to request. You can choose to set the target capacity in terms of instances or a performance characteristic that is important to your application work...(description truncated)
   * @param {string} targetCapacitySpecificationDefaultTargetCapacityType - The number of units to request. You can choose to set the target capacity in terms of instances or a performance characteristic that is important to your application work...(description truncated)
   * @param {boolean} terminateInstancesWithExpiration - Indicates whether running instances should be terminated when the EC2 Fleet expires.
   * @param {string} type - The type of the request. By default, the EC2 Fleet places an asynchronous request for your desired capacity, and maintains it by replenishing interrupted Spot Instances (...(description truncated)
   * @param {string} validFrom - The start date and time of the request, in UTC format (for example,  YYYY - MM - DD T HH : MM : SS Z). The default is to start fulfilling the request immediately.
   * @param {string} validUntil - The end date and time of the request, in UTC format (for example,  YYYY - MM - DD T HH : MM : SS Z). At this point, no new EC2 Fleet requests are placed or able to fulfil...(description truncated)
   * @param {boolean} replaceUnhealthyInstances - Indicates whether EC2 Fleet should replace unhealthy instances.
   * @param {array} tagSpecification - The key-value pair for tagging the EC2 Fleet request on creation. The value for  ResourceType  must be  fleet , otherwise the fleet request fails. To tag instances at lau...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createFleet(dryRun, clientToken, spotOptionsAllocationStrategy, spotOptionsInstanceInterruptionBehavior, spotOptionsInstancePoolsToUseCount, spotOptionsSingleInstanceType, spotOptionsSingleAvailabilityZone, spotOptionsMinTargetCapacity, onDemandOptionsAllocationStrategy, onDemandOptionsSingleInstanceType, onDemandOptionsSingleAvailabilityZone, onDemandOptionsMinTargetCapacity, excessCapacityTerminationPolicy = 'no-termination', launchTemplateConfigs, targetCapacitySpecificationTotalTargetCapacity, targetCapacitySpecificationOnDemandTargetCapacity, targetCapacitySpecificationSpotTargetCapacity, targetCapacitySpecificationDefaultTargetCapacityType, terminateInstancesWithExpiration, type = 'request', validFrom, validUntil, replaceUnhealthyInstances, tagSpecification, callback) {
    const meth = 'adapter-createFleet';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateFleet';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (launchTemplateConfigs === undefined || launchTemplateConfigs === null || launchTemplateConfigs === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['launchTemplateConfigs'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientToken: clientToken,
      SpotOptionsAllocationStrategy: spotOptionsAllocationStrategy,
      SpotOptionsInstanceInterruptionBehavior: spotOptionsInstanceInterruptionBehavior,
      SpotOptionsInstancePoolsToUseCount: spotOptionsInstancePoolsToUseCount,
      SpotOptionsSingleInstanceType: spotOptionsSingleInstanceType,
      SpotOptionsSingleAvailabilityZone: spotOptionsSingleAvailabilityZone,
      SpotOptionsMinTargetCapacity: spotOptionsMinTargetCapacity,
      OnDemandOptionsAllocationStrategy: onDemandOptionsAllocationStrategy,
      OnDemandOptionsSingleInstanceType: onDemandOptionsSingleInstanceType,
      OnDemandOptionsSingleAvailabilityZone: onDemandOptionsSingleAvailabilityZone,
      OnDemandOptionsMinTargetCapacity: onDemandOptionsMinTargetCapacity,
      ExcessCapacityTerminationPolicy: excessCapacityTerminationPolicy,
      TargetCapacitySpecificationTotalTargetCapacity: targetCapacitySpecificationTotalTargetCapacity,
      TargetCapacitySpecificationOnDemandTargetCapacity: targetCapacitySpecificationOnDemandTargetCapacity,
      TargetCapacitySpecificationSpotTargetCapacity: targetCapacitySpecificationSpotTargetCapacity,
      TargetCapacitySpecificationDefaultTargetCapacityType: targetCapacitySpecificationDefaultTargetCapacityType,
      TerminateInstancesWithExpiration: terminateInstancesWithExpiration,
      Type: type,
      ValidFrom: validFrom,
      ValidUntil: validUntil,
      ReplaceUnhealthyInstances: replaceUnhealthyInstances,
      Action: action,
      Version: version
    };
    if (launchTemplateConfigs && launchTemplateConfigs.length > 0) {
      processInArray(launchTemplateConfigs, queryParamsAvailable, 'LaunchTemplateConfigs');
    }
    if (tagSpecification && tagSpecification.length > 0) {
      processInArray(tagSpecification, queryParamsAvailable, 'TagSpecification');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Fleet', 'createFleet', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createFleet'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Launches multiple EC2 Fleets.
   *
   * @function createFleets
   * @param {array} fleetsArray - An array of objects(Fleets parameters). bilityZone, spotOptionsMinTargetCapacity, onDemandOptionsAllocationStrategy, onDemandOptionsSingleInstanceType, onDemandOptionsSingleAvailabilityZone, onDemandOptionsMinTargetCapacity, excessCapacityTerminationPolicy, launchTemplateConfigs, targetCapacitySpecificationTotalTargetCapacity, targetCapacitySpecificationOnDemandTargetCapacity, targetCapacitySpecificationSpotTargetCapacity, targetCapacitySpecificationDefaultTargetCapacityType, terminateInstancesWithExpiration, type, validFrom, validUntil, replaceUnhealthyInstances, tagSpecification
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createFleets(fleetsArray, callback) {
    const meth = 'adapter-createFleets';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (fleetsArray === undefined || fleetsArray === null || fleetsArray === '' || fleetsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['fleetsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      fleetsArray.forEach((item) => {
        const { dryRun, clientToken, spotOptionsAllocationStrategy, spotOptionsInstanceInterruptionBehavior, spotOptionsInstancePoolsToUseCount, spotOptionsSingleInstanceType, spotOptionsSingleAvailabilityZone, spotOptionsMinTargetCapacity, onDemandOptionsAllocationStrategy, onDemandOptionsSingleInstanceType, onDemandOptionsSingleAvailabilityZone, onDemandOptionsMinTargetCapacity, excessCapacityTerminationPolicy, launchTemplateConfigs, targetCapacitySpecificationTotalTargetCapacity, targetCapacitySpecificationOnDemandTargetCapacity, targetCapacitySpecificationSpotTargetCapacity, targetCapacitySpecificationDefaultTargetCapacityType, terminateInstancesWithExpiration, type, validFrom, validUntil, replaceUnhealthyInstances, tagSpecification } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createFleet(dryRun, clientToken, spotOptionsAllocationStrategy, spotOptionsInstanceInterruptionBehavior, spotOptionsInstancePoolsToUseCount, spotOptionsSingleInstanceType, spotOptionsSingleAvailabilityZone, spotOptionsMinTargetCapacity, onDemandOptionsAllocationStrategy, onDemandOptionsSingleInstanceType, onDemandOptionsSingleAvailabilityZone, onDemandOptionsMinTargetCapacity, excessCapacityTerminationPolicy, launchTemplateConfigs, targetCapacitySpecificationTotalTargetCapacity, targetCapacitySpecificationOnDemandTargetCapacity, targetCapacitySpecificationSpotTargetCapacity, targetCapacitySpecificationDefaultTargetCapacityType, terminateInstancesWithExpiration, type, validFrom, validUntil, replaceUnhealthyInstances, tagSpecification, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates one or more flow logs to capture information about IP traffic for a specific network interface, subnet, or VPC.    Flow log data for a monitored network interface is recorded as flow log records, which are log events consisting of fields that describe the traffic flow. For more information, see  Flow Log Records  in the  Amazon Virtual Private Cloud User Guide .   When publishing to CloudWatch Logs, flow log records are published to a log group, and each network interface has a unique l...(description truncated)
   *
   * @function createFlowLogs
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} clientToken - Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see  How to Ensure Idempotency .
   * @param {string} deliverLogsPermissionArn - The ARN for the IAM role that permits Amazon EC2 to publish flow logs to a CloudWatch Logs log group in your account.   If you specify  LogDestinationType  as  s3 , do n...(description truncated)
   * @param {string} logGroupName - The name of a new or existing CloudWatch Logs log group where Amazon EC2 publishes your flow logs.   If you specify  LogDestinationType  as  s3 , do not specify  Deliver...(description truncated)
   * @param {array} resourceId - The ID of the subnet, network interface, or VPC for which you want to create a flow log.   Constraints: Maximum of 1000 resources
   * @param {string} resourceType - The type of resource for which to create the flow log. For example, if you specified a VPC ID for the  ResourceId  property, specify  VPC  for this property.
   * @param {string} trafficType - The type of traffic to log. You can log traffic that the resource accepts or rejects, or all traffic.
   * @param {string} logDestinationType - Specifies the type of destination to which the flow log data is to be published. Flow log data can be published to CloudWatch Logs or Amazon S3. To publish flow log data...(description truncated)
   * @param {string} logDestination - Specifies the destination to which the flow log data is to be published. Flow log data can be published to a CloudWatch Logs log group or an Amazon S3 bucket. The value ...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createFlowLogs(dryRun, clientToken, deliverLogsPermissionArn, logGroupName, resourceId, resourceType = 'VPC', trafficType = 'ACCEPT', logDestinationType = 'cloud-watch-logs', logDestination, callback) {
    const meth = 'adapter-createFlowLogs';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateFlowLogs';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (resourceId === undefined || resourceId === null || resourceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['resourceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (resourceType === undefined || resourceType === null || resourceType === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['resourceType'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (trafficType === undefined || trafficType === null || trafficType === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['trafficType'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientToken: clientToken,
      DeliverLogsPermissionArn: deliverLogsPermissionArn,
      LogGroupName: logGroupName,
      ResourceType: resourceType,
      TrafficType: trafficType,
      LogDestinationType: logDestinationType,
      LogDestination: logDestination,
      Action: action,
      Version: version
    };
    if (resourceId && resourceId.length > 0) {
      processInArray(resourceId, queryParamsAvailable, 'ResourceId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionCreateFlowLogs', 'getCreateFlowLogs', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createFlowLogs'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple flow logs to capture information about IP traffic for a specific network interface, subnet, or VPC.
   *
   * @function createMultipleFlowLogs
   * @param {array} flowLogsArray - An array of objects(FlowLogs parameters). dryRun, clientToken, deliverLogsPermissionArn, logGroupName, resourceId, resourceType, trafficType, logDestinationType, logDestination
.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createMultipleFlowLogs(flowLogsArray, callback) {
    const meth = 'adapter-createMultipleFlowLogs';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (flowLogsArray === undefined || flowLogsArray === null || flowLogsArray === '' || flowLogsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['flowLogsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      flowLogsArray.forEach((item) => {
        const { dryRun, clientToken, deliverLogsPermissionArn, logGroupName, resourceId, resourceType, trafficType, logDestinationType, logDestination } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createFlowLogs(dryRun, clientToken, deliverLogsPermissionArn, logGroupName, resourceId, resourceType, trafficType, logDestinationType, logDestination, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates an Amazon FPGA Image (AFI) from the specified design checkpoint (DCP).   The create operation is asynchronous. To verify that the AFI is ready for use, check the output logs.   An AFI contains the FPGA bitstream that is ready to download to an FPGA. You can securely deploy an AFI on multiple FPGA-accelerated instances. For more information, see the  AWS FPGA Hardware Development Kit .
   *
   * @function createFpgaImage
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} inputStorageLocationBucket - Describes a storage location in Amazon S3.
The name of the S3 bucket.
   * @param {string} inputStorageLocationKey - Describes a storage location in Amazon S3.
The key.
   * @param {string} logsStorageLocationBucket - Describes a storage location in Amazon S3.
The name of the S3 bucket.
   * @param {string} logsStorageLocationKey - Describes a storage location in Amazon S3.
The key.
   * @param {string} description - A description for the AFI.
   * @param {string} name - A name for the AFI.
   * @param {string} clientToken - Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see  Ensuring Idempotency .
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createFpgaImage(dryRun, inputStorageLocationBucket, inputStorageLocationKey, logsStorageLocationBucket, logsStorageLocationKey, description, name, clientToken, callback) {
    const meth = 'adapter-createFpgaImage';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateFpgaImage';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      InputStorageLocationBucket: inputStorageLocationBucket,
      InputStorageLocationKey: inputStorageLocationKey,
      LogsStorageLocationBucket: logsStorageLocationBucket,
      LogsStorageLocationKey: logsStorageLocationKey,
      Description: description,
      Name: name,
      ClientToken: clientToken,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Image', 'createFpgaImage', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createFpgaImage'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple Amazon FPGA Image (AFI) from the specified design checkpoint (DCP).
   *
   * @function createFpgaImages
   * @param {array} FpgaImagesArray - An array of objects(FpgaImage parameters). dryRun, clientToken, deliverLogsPermissionArn, logGroupName, resourceId, resourceType, trafficType, logDestinationType, logDestination
.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createFpgaImages(FpgaImagesArray, callback) {
    const meth = 'adapter-createFpgaImages';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (FpgaImagesArray === undefined || FpgaImagesArray === null || FpgaImagesArray === '' || FpgaImagesArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['FpgaImagesArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      FpgaImagesArray.forEach((item) => {
        const { dryRun, inputStorageLocationBucket, inputStorageLocationKey, logsStorageLocationBucket, logsStorageLocationKey, description, name, clientToken } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createFpgaImage(dryRun, inputStorageLocationBucket, inputStorageLocationKey, logsStorageLocationBucket, logsStorageLocationKey, description, name, clientToken, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates an Amazon EBS-backed AMI from an Amazon EBS-backed instance that is either running or stopped.   If you customized your instance with instance store volumes or EBS volumes in addition to the root device volume, the new AMI contains block device mapping information for those volumes. When you launch an instance from this new AMI, the instance automatically launches with those additional volumes.   For more information, see  Creating Amazon EBS-Backed Linux AMIs  in the  Amazon Elastic Co...(description truncated)
   *
   * @function createImage
   * @param {array} blockDeviceMapping - Tthe block device mappings. This parameter cannot be used to modify the encryption status of existing volumes or snapshots. To create an AMI with encrypted snapshots, use...(description truncated)
   * @param {string} description - A description for the new image.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} instanceId - The ID of the instance.
   * @param {string} name - A name for the new image.   Constraints: 3-128 alphanumeric characters, parentheses (()), square brackets ([]), spaces ( ), periods (.), slashes (/), dashes (-), single ...(description truncated)
   * @param {boolean} noReboot - By default, Amazon EC2 attempts to shut down and reboot the instance before creating the image. If the 'No Reboot' option is set, Amazon EC2 doesn't shut down the instanc...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createImage(blockDeviceMapping, description, dryRun, instanceId, name, noReboot, callback) {
    const meth = 'adapter-createImage';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateImage';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (name === undefined || name === null || name === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['name'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Description: description,
      InstanceId: instanceId,
      Name: name,
      NoReboot: noReboot,
      Action: action,
      Version: version
    };
    if (blockDeviceMapping && blockDeviceMapping.length > 0) {
      processInArray(blockDeviceMapping, queryParamsAvailable, 'BlockDeviceMapping');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Image', 'createImage', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createImage'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple Amazon EBS-backed AMI from an Amazon EBS-backed instance that is either running or stopped.
   *
   * @function createImages
   * @param {array} ImagesArray - An array of objects(Image parameters). blockDeviceMapping, description, dryRun, instanceId, name, noReboot
.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createImages(ImagesArray, callback) {
    const meth = 'adapter-createImages';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (ImagesArray === undefined || ImagesArray === null || ImagesArray === '' || ImagesArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['ImagesArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      ImagesArray.forEach((item) => {
        const { blockDeviceMapping, description, dryRun, instanceId, name, noReboot } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createImage(blockDeviceMapping, description, dryRun, instanceId, name, noReboot, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Exports a running or stopped instance to an S3 bucket.   For information about the supported operating systems, image formats, and known limitations for the types of instances you can export, see  Exporting an Instance as a VM Using VM Import/Export  in the  VM Import/Export User Guide .
   *
   * @function createInstanceExportTask
   * @param {string} description - A description for the conversion task or the resource being exported. The maximum length is 255 bytes.
   * @param {string} exportToS3ContainerFormat - Describes an instance export task.
The container format used to combine disk images with metadata (such as OVF). If absent, only the disk image is exported.
   * @param {string} exportToS3DiskImageFormat - Describes an instance export task.
The format for the exported image.
   * @param {string} exportToS3S3Bucket - Describes an instance export task.
The S3 bucket for the destination image. The destination bucket must exist and grant WRITE and READ_ACP permissions to the AWS account ...(description truncated)
   * @param {string} exportToS3S3Prefix - Describes an instance export task.
The image is written to a single object in the S3 bucket at the S3 key s3prefix + exportTaskId + '.' + diskImageFormat.
   * @param {string} instanceId - The ID of the instance.
   * @param {string} targetEnvironment - The target virtualization environment.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createInstanceExportTask(description, exportToS3ContainerFormat, exportToS3DiskImageFormat, exportToS3S3Bucket, exportToS3S3Prefix, instanceId, targetEnvironment = 'citrix', callback) {
    const meth = 'adapter-createInstanceExportTask';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateInstanceExportTask';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      Description: description,
      ExportToS3ContainerFormat: exportToS3ContainerFormat,
      ExportToS3DiskImageFormat: exportToS3DiskImageFormat,
      ExportToS3S3Bucket: exportToS3S3Bucket,
      ExportToS3S3Prefix: exportToS3S3Prefix,
      InstanceId: instanceId,
      TargetEnvironment: targetEnvironment,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'createInstanceExportTask', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createInstanceExportTask'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Exports multiple running or stopped instance to an S3 bucket.
   *
   * @function createInstanceExportTasks
   * @param {array} InstanceExportTasksArray - An array of objects(InstanceExportTask parameters). description, exportToS3ContainerFormat, exportToS3DiskImageFormat, exportToS3S3Bucket, exportToS3S3Prefix, instanceId, targetEnvironment
.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createInstanceExportTasks(InstanceExportTasksArray, callback) {
    const meth = 'adapter-createInstanceExportTasks';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (InstanceExportTasksArray === undefined || InstanceExportTasksArray === null || InstanceExportTasksArray === '' || InstanceExportTasksArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['InstanceExportTasksArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      InstanceExportTasksArray.forEach((item) => {
        const { description, exportToS3ContainerFormat, exportToS3DiskImageFormat, exportToS3S3Bucket, exportToS3S3Prefix, instanceId, targetEnvironment } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createInstanceExportTask(description, exportToS3ContainerFormat, exportToS3DiskImageFormat, exportToS3S3Bucket, exportToS3S3Prefix, instanceId, targetEnvironment, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates an internet gateway for use with a VPC. After creating the internet gateway, you attach it to a VPC using  AttachInternetGateway .   For more information about your VPC and internet gateway, see the  Amazon Virtual Private Cloud User Guide .
   *
   * @function createInternetGateway
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createInternetGateway(dryRun, callback) {
    const meth = 'adapter-createInternetGateway';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateInternetGateway';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Gateway', 'createInternetGateway', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createInternetGateway'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple internet gateways for use with a VPC. After creating the internet gateway, you attach it to a VPC using  AttachInternetGateway .   For more information about your VPC and internet gateway, see the  Amazon Virtual Private Cloud User Guide .
   *
   * @function createInternetGateways
   * @param {array} internetGatewayArray - An array of objects(internet gateway parameters). dryRun.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createInternetGateways(internetGatewayArray, callback) {
    const meth = 'adapter-createInternetGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (internetGatewayArray === undefined || internetGatewayArray === null || internetGatewayArray === '' || internetGatewayArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['internetGatewayArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each internet gateway individually.
      internetGatewayArray.forEach((internetGateway) => {
        const { dryRun } = internetGateway;
        returnArray.push(
          new Promise((resolve) => {
            this.createInternetGateway(dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a 2048-bit RSA key pair with the specified name. Amazon EC2 stores the public key and displays the private key for you to save to a file. The private key is returned as an unencrypted PEM encoded PKCS#1 private key. If a key with the specified name already exists, Amazon EC2 returns an error.   You can have up to five thousand key pairs per Region.   The key pair returned to you is available only in the Region in which you create it. If you prefer, you can create your own key pair using...(description truncated)
   *
   * @function createKeyPair
   * @param {string} keyName - A unique name for the key pair.   Constraints: Up to 255 ASCII characters
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createKeyPair(keyName, dryRun, callback) {
    const meth = 'adapter-createKeyPair';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateKeyPair';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (keyName === undefined || keyName === null || keyName === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['keyName'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      KeyName: keyName,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('KeyPair', 'createKeyPair', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createKeyPair'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple 2048-bit RSA key pairs with the specified name.
   *
   * @function createKeyPairs
   * @param {array} KeyPairsArray - An array of objects(KeyPair parameters). keyName, dryRun.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createKeyPairs(KeyPairsArray, callback) {
    const meth = 'adapter-createKeyPairs';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (KeyPairsArray === undefined || KeyPairsArray === null || KeyPairsArray === '' || KeyPairsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['KeyPairsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      KeyPairsArray.forEach((item) => {
        const { keyName, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createKeyPair(keyName, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a launch template. A launch template contains the parameters to launch an instance. When you launch an instance using  RunInstances , you can specify a launch template instead of providing the launch parameters in the request.
   *
   * @function createLaunchTemplate
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} clientToken - Unique, case-sensitive identifier you provide to ensure the idempotency of the request. For more information, see  Ensuring Idempotency .   Constraint: Maximum 128 ASCII...(description truncated)
   * @param {string} launchTemplateName - A name for the launch template.
   * @param {string} versionDescription - A description for the first version of the launch template.
   * @param {string} launchTemplateDataKernelId - The information to include in the launch template.
 The ID of the kernel.     We recommend that you use PV-GRUB instead of kernels and RAM disks. For more information, se...(description truncated)
   * @param {string} launchTemplateDataEbsOptimized - The information to include in the launch template.
Indicates whether the instance is optimized for Amazon EBS I/O. This optimization provides dedicated throughput to Amaz...(description truncated)
   * @param {string} launchTemplateDataIamInstanceProfile - The information to include in the launch template.
The IAM instance profile.
   * @param {array} launchTemplateDataBlockDeviceMappings - The information to include in the launch template.
 The block device mapping.     Supplying both a snapshot ID and an encryption value as arguments for block-device mappi...(description truncated)
   * @param {array} launchTemplateDataNetworkInterfaces - The information to include in the launch template.
One or more network interfaces. If you specify a network interface, you must specify any security groups as part of the...(description truncated)
   * @param {string} launchTemplateDataImageId - The information to include in the launch template.
The ID of the AMI, which you can get by using  DescribeImages .
   * @param {string} launchTemplateDataInstanceType - The information to include in the launch template.
The instance type. For more information, see  Instance Types  in the  Amazon Elastic Compute Cloud User Guide .
   * @param {string} launchTemplateDataKeyName - The information to include in the launch template.
 The name of the key pair. You can create a key pair using  CreateKeyPair  or  ImportKeyPair .     If you do not specif...(description truncated)
   * @param {string} launchTemplateDataMonitoring - The information to include in the launch template.
The monitoring for the instance.
   * @param {string} launchTemplateDataPlacement - The information to include in the launch template.
The placement for the instance.
   * @param {string} launchTemplateDataRamDiskId - The information to include in the launch template.
 The ID of the RAM disk.     We recommend that you use PV-GRUB instead of kernels and RAM disks. For more information, ...(description truncated)
   * @param {string} launchTemplateDataDisableApiTermination - The information to include in the launch template.
If set to  true , you can't terminate the instance using the Amazon EC2 console, CLI, or API. To change this attribute ...(description truncated)
   * @param {string} launchTemplateDataInstanceInitiatedShutdownBehavior - The information to include in the launch template.
 Indicates whether an instance stops or terminates when you initiate shutdown from the instance (using the operating sy...(description truncated)
   * @param {string} launchTemplateDataUserData - The information to include in the launch template.
The Base64-encoded user data to make available to the instance. For more information, see  Running Commands on Your Lin...(description truncated)
   * @param {array} launchTemplateDataTagSpecifications - The information to include in the launch template.
The tags to apply to the resources during launch. You can only tag instances and volumes on launch. The specified tags ...(description truncated)
   * @param {array} launchTemplateDataElasticGpuSpecifications - The information to include in the launch template.
An elastic GPU to associate with the instance.
   * @param {array} launchTemplateDataElasticInferenceAccelerators - The information to include in the launch template.
 The elastic inference accelerator for the instance.
   * @param {array} launchTemplateDataSecurityGroupIds - The information to include in the launch template.
One or more security group IDs. You can create a security group using  CreateSecurityGroup . You cannot specify both a ...(description truncated)
   * @param {array} launchTemplateDataSecurityGroups - The information to include in the launch template.
[EC2-Classic, default VPC] One or more security group names. For a nondefault VPC, you must use security group IDs inst...(description truncated)
   * @param {string} launchTemplateDataInstanceMarketOptions - The information to include in the launch template.
The market (purchasing) option for the instances.
   * @param {string} launchTemplateDataCreditSpecification - The information to include in the launch template.
The credit option for CPU usage of the instance. Valid for T2 or T3 instances only.
   * @param {string} launchTemplateDataCpuOptions - The information to include in the launch template.
The CPU options for the instance. For more information, see  Optimizing CPU Options  in the  Amazon Elastic Compute Clo...(description truncated)
   * @param {string} launchTemplateDataCapacityReservationSpecification - The information to include in the launch template.
The Capacity Reservation targeting option. If you do not specify this parameter, the instance's Capacity Reservation pr...(description truncated)
   * @param {array} launchTemplateDataLicenseSpecifications - The information to include in the launch template.
The license configurations.
   * @param {string} launchTemplateDataHibernationOptions - The information to include in the launch template.
Indicates whether an instance is enabled for hibernation. This parameter is valid only if the instance meets the  hiber...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createLaunchTemplate(dryRun, clientToken, launchTemplateName, versionDescription, launchTemplateDataKernelId, launchTemplateDataEbsOptimized, launchTemplateDataIamInstanceProfile, launchTemplateDataBlockDeviceMappings, launchTemplateDataNetworkInterfaces, launchTemplateDataImageId, launchTemplateDataInstanceType, launchTemplateDataKeyName, launchTemplateDataMonitoring, launchTemplateDataPlacement, launchTemplateDataRamDiskId, launchTemplateDataDisableApiTermination, launchTemplateDataInstanceInitiatedShutdownBehavior, launchTemplateDataUserData, launchTemplateDataTagSpecifications, launchTemplateDataElasticGpuSpecifications, launchTemplateDataElasticInferenceAccelerators, launchTemplateDataSecurityGroupIds, launchTemplateDataSecurityGroups, launchTemplateDataInstanceMarketOptions, launchTemplateDataCreditSpecification, launchTemplateDataCpuOptions, launchTemplateDataCapacityReservationSpecification, launchTemplateDataLicenseSpecifications, launchTemplateDataHibernationOptions, callback) {
    const meth = 'adapter-createLaunchTemplate';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateLaunchTemplate';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (launchTemplateName === undefined || launchTemplateName === null || launchTemplateName === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['launchTemplateName'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientToken: clientToken,
      LaunchTemplateName: launchTemplateName,
      VersionDescription: versionDescription,
      LaunchTemplateDataKernelId: launchTemplateDataKernelId,
      LaunchTemplateDataEbsOptimized: launchTemplateDataEbsOptimized,
      LaunchTemplateDataIamInstanceProfile: launchTemplateDataIamInstanceProfile,
      LaunchTemplateDataImageId: launchTemplateDataImageId,
      LaunchTemplateDataInstanceType: launchTemplateDataInstanceType,
      LaunchTemplateDataKeyName: launchTemplateDataKeyName,
      LaunchTemplateDataMonitoring: launchTemplateDataMonitoring,
      LaunchTemplateDataPlacement: launchTemplateDataPlacement,
      LaunchTemplateDataRamDiskId: launchTemplateDataRamDiskId,
      LaunchTemplateDataDisableApiTermination: launchTemplateDataDisableApiTermination,
      LaunchTemplateDataInstanceInitiatedShutdownBehavior: launchTemplateDataInstanceInitiatedShutdownBehavior,
      LaunchTemplateDataUserData: launchTemplateDataUserData,
      LaunchTemplateDataInstanceMarketOptions: launchTemplateDataInstanceMarketOptions,
      LaunchTemplateDataCreditSpecification: launchTemplateDataCreditSpecification,
      LaunchTemplateDataCpuOptions: launchTemplateDataCpuOptions,
      LaunchTemplateDataCapacityReservationSpecification: launchTemplateDataCapacityReservationSpecification,
      LaunchTemplateDataHibernationOptions: launchTemplateDataHibernationOptions,
      Action: action,
      Version: version
    };
    if (launchTemplateDataBlockDeviceMappings && launchTemplateDataBlockDeviceMappings.length > 0) {
      processInArray(launchTemplateDataBlockDeviceMappings, queryParamsAvailable, 'LaunchTemplateDataBlockDeviceMappings');
    }
    if (launchTemplateDataNetworkInterfaces && launchTemplateDataNetworkInterfaces.length > 0) {
      processInArray(launchTemplateDataNetworkInterfaces, queryParamsAvailable, 'LaunchTemplateDataNetworkInterfaces');
    }
    if (launchTemplateDataTagSpecifications && launchTemplateDataTagSpecifications.length > 0) {
      processInArray(launchTemplateDataTagSpecifications, queryParamsAvailable, 'LaunchTemplateDataTagSpecifications');
    }
    if (launchTemplateDataElasticGpuSpecifications && launchTemplateDataElasticGpuSpecifications.length > 0) {
      processInArray(launchTemplateDataElasticGpuSpecifications, queryParamsAvailable, 'LaunchTemplateDataElasticGpuSpecifications');
    }
    if (launchTemplateDataElasticInferenceAccelerators && launchTemplateDataElasticInferenceAccelerators.length > 0) {
      processInArray(launchTemplateDataElasticInferenceAccelerators, queryParamsAvailable, 'LaunchTemplateDataElasticInferenceAccelerators');
    }
    if (launchTemplateDataSecurityGroupIds && launchTemplateDataSecurityGroupIds.length > 0) {
      processInArray(launchTemplateDataSecurityGroupIds, queryParamsAvailable, 'LaunchTemplateDataSecurityGroupIds');
    }
    if (launchTemplateDataSecurityGroups && launchTemplateDataSecurityGroups.length > 0) {
      processInArray(launchTemplateDataSecurityGroups, queryParamsAvailable, 'LaunchTemplateDataSecurityGroups');
    }
    if (launchTemplateDataLicenseSpecifications && launchTemplateDataLicenseSpecifications.length > 0) {
      processInArray(launchTemplateDataLicenseSpecifications, queryParamsAvailable, 'LaunchTemplateDataLicenseSpecifications');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('LaunchTemplate', 'createLaunchTemplate', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createLaunchTemplate'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple launch templates.
   *
   * @function createLaunchTemplates
   * @param {array} LaunchTemplatesArray - An array of objects(LaunchTemplates parameters). dryRun, clientToken, launchTemplateName, versionDescription, launchTemplateDataKernelId, launchTemplateDataEbsOptimized, launchTemplateDataIamInstanceProfile, launchTemplateDataBlockDeviceMappings, launchTemplateDataNetworkInterfaces, launchTemplateDataImageId, launchTemplateDataInstanceType, launchTemplateDataKeyName, launchTemplateDataMonitoring, launchTemplateDataPlacement, launchTemplateDataRamDiskId, launchTemplateDataDisableApiTermination, launchTemplateDataInstanceInitiatedShutdownBehavior, launchTemplateDataUserData, launchTemplateDataTagSpecifications, launchTemplateDataElasticGpuSpecifications, launchTemplateDataElasticInferenceAccelerators, launchTemplateDataSecurityGroupIds, launchTemplateDataSecurityGroups, launchTemplateDataInstanceMarketOptions, launchTemplateDataCreditSpecification, launchTemplateDataCpuOptions, launchTemplateDataCapacityReservationSpecification, launchTemplateDataLicenseSpecifications, launchTemplateDataHibernationOptions.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createLaunchTemplates(LaunchTemplatesArray, callback) {
    const meth = 'adapter-createLaunchTemplates';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (LaunchTemplatesArray === undefined || LaunchTemplatesArray === null || LaunchTemplatesArray === '' || LaunchTemplatesArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['LaunchTemplatesArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      LaunchTemplatesArray.forEach((item) => {
        const { dryRun, clientToken, launchTemplateName, versionDescription, launchTemplateDataKernelId, launchTemplateDataEbsOptimized, launchTemplateDataIamInstanceProfile, launchTemplateDataBlockDeviceMappings, launchTemplateDataNetworkInterfaces, launchTemplateDataImageId, launchTemplateDataInstanceType, launchTemplateDataKeyName, launchTemplateDataMonitoring, launchTemplateDataPlacement, launchTemplateDataRamDiskId, launchTemplateDataDisableApiTermination, launchTemplateDataInstanceInitiatedShutdownBehavior, launchTemplateDataUserData, launchTemplateDataTagSpecifications, launchTemplateDataElasticGpuSpecifications, launchTemplateDataElasticInferenceAccelerators, launchTemplateDataSecurityGroupIds, launchTemplateDataSecurityGroups, launchTemplateDataInstanceMarketOptions, launchTemplateDataCreditSpecification, launchTemplateDataCpuOptions, launchTemplateDataCapacityReservationSpecification, launchTemplateDataLicenseSpecifications, launchTemplateDataHibernationOptions } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createLaunchTemplate(dryRun, clientToken, launchTemplateName, versionDescription, launchTemplateDataKernelId, launchTemplateDataEbsOptimized, launchTemplateDataIamInstanceProfile, launchTemplateDataBlockDeviceMappings, launchTemplateDataNetworkInterfaces, launchTemplateDataImageId, launchTemplateDataInstanceType, launchTemplateDataKeyName, launchTemplateDataMonitoring, launchTemplateDataPlacement, launchTemplateDataRamDiskId, launchTemplateDataDisableApiTermination, launchTemplateDataInstanceInitiatedShutdownBehavior, launchTemplateDataUserData, launchTemplateDataTagSpecifications, launchTemplateDataElasticGpuSpecifications, launchTemplateDataElasticInferenceAccelerators, launchTemplateDataSecurityGroupIds, launchTemplateDataSecurityGroups, launchTemplateDataInstanceMarketOptions, launchTemplateDataCreditSpecification, launchTemplateDataCpuOptions, launchTemplateDataCapacityReservationSpecification, launchTemplateDataLicenseSpecifications, launchTemplateDataHibernationOptions, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a new version for a launch template. You can specify an existing version of launch template from which to base the new version.   Launch template versions are numbered in the order in which they are created. You cannot specify, change, or replace the numbering of launch template versions.
   *
   * @function createLaunchTemplateVersion
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} clientToken - Unique, case-sensitive identifier you provide to ensure the idempotency of the request. For more information, see  Ensuring Idempotency .   Constraint: Maximum 128 ASCII...(description truncated)
   * @param {string} launchTemplateId - The ID of the launch template. You must specify either the launch template ID or launch template name in the request.
   * @param {string} launchTemplateName - The name of the launch template. You must specify either the launch template ID or launch template name in the request.
   * @param {string} sourceVersion - The version number of the launch template version on which to base the new version. The new version inherits the same launch parameters as the source version, except for ...(description truncated)
   * @param {string} versionDescription - A description for the version of the launch template.
   * @param {string} launchTemplateDataKernelId - The information to include in the launch template.
 The ID of the kernel.     We recommend that you use PV-GRUB instead of kernels and RAM disks. For more information, se...(description truncated)
   * @param {string} launchTemplateDataEbsOptimized - The information to include in the launch template.
Indicates whether the instance is optimized for Amazon EBS I/O. This optimization provides dedicated throughput to Amaz...(description truncated)
   * @param {string} launchTemplateDataIamInstanceProfile - The information to include in the launch template.
The IAM instance profile.
   * @param {array} launchTemplateDataBlockDeviceMappings - The information to include in the launch template.
 The block device mapping.     Supplying both a snapshot ID and an encryption value as arguments for block-device mappi...(description truncated)
   * @param {array} launchTemplateDataNetworkInterfaces - The information to include in the launch template.
One or more network interfaces. If you specify a network interface, you must specify any security groups as part of the...(description truncated)
   * @param {string} launchTemplateDataImageId - The information to include in the launch template.
The ID of the AMI, which you can get by using  DescribeImages .
   * @param {string} launchTemplateDataInstanceType - The information to include in the launch template.
The instance type. For more information, see  Instance Types  in the  Amazon Elastic Compute Cloud User Guide .
   * @param {string} launchTemplateDataKeyName - The information to include in the launch template.
 The name of the key pair. You can create a key pair using  CreateKeyPair  or  ImportKeyPair .     If you do not specif...(description truncated)
   * @param {string} launchTemplateDataMonitoring - The information to include in the launch template.
The monitoring for the instance.
   * @param {string} launchTemplateDataPlacement - The information to include in the launch template.
The placement for the instance.
   * @param {string} launchTemplateDataRamDiskId - The information to include in the launch template.
 The ID of the RAM disk.     We recommend that you use PV-GRUB instead of kernels and RAM disks. For more information, ...(description truncated)
   * @param {string} launchTemplateDataDisableApiTermination - The information to include in the launch template.
If set to  true , you can't terminate the instance using the Amazon EC2 console, CLI, or API. To change this attribute ...(description truncated)
   * @param {string} launchTemplateDataInstanceInitiatedShutdownBehavior - The information to include in the launch template.
 Indicates whether an instance stops or terminates when you initiate shutdown from the instance (using the operating sy...(description truncated)
   * @param {string} launchTemplateDataUserData - The information to include in the launch template.
The Base64-encoded user data to make available to the instance. For more information, see  Running Commands on Your Lin...(description truncated)
   * @param {array} launchTemplateDataTagSpecifications - The information to include in the launch template.
The tags to apply to the resources during launch. You can only tag instances and volumes on launch. The specified tags ...(description truncated)
   * @param {array} launchTemplateDataElasticGpuSpecifications - The information to include in the launch template.
An elastic GPU to associate with the instance.
   * @param {array} launchTemplateDataElasticInferenceAccelerators - The information to include in the launch template.
 The elastic inference accelerator for the instance.
   * @param {array} launchTemplateDataSecurityGroupIds - The information to include in the launch template.
One or more security group IDs. You can create a security group using  CreateSecurityGroup . You cannot specify both a ...(description truncated)
   * @param {array} launchTemplateDataSecurityGroups - The information to include in the launch template.
[EC2-Classic, default VPC] One or more security group names. For a nondefault VPC, you must use security group IDs inst...(description truncated)
   * @param {string} launchTemplateDataInstanceMarketOptions - The information to include in the launch template.
The market (purchasing) option for the instances.
   * @param {string} launchTemplateDataCreditSpecification - The information to include in the launch template.
The credit option for CPU usage of the instance. Valid for T2 or T3 instances only.
   * @param {string} launchTemplateDataCpuOptions - The information to include in the launch template.
The CPU options for the instance. For more information, see  Optimizing CPU Options  in the  Amazon Elastic Compute Clo...(description truncated)
   * @param {string} launchTemplateDataCapacityReservationSpecification - The information to include in the launch template.
The Capacity Reservation targeting option. If you do not specify this parameter, the instance's Capacity Reservation pr...(description truncated)
   * @param {array} launchTemplateDataLicenseSpecifications - The information to include in the launch template.
The license configurations.
   * @param {string} launchTemplateDataHibernationOptions - The information to include in the launch template.
Indicates whether an instance is enabled for hibernation. This parameter is valid only if the instance meets the  hiber...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createLaunchTemplateVersion(dryRun, clientToken, launchTemplateId, launchTemplateName, sourceVersion, versionDescription, launchTemplateDataKernelId, launchTemplateDataEbsOptimized, launchTemplateDataIamInstanceProfile, launchTemplateDataBlockDeviceMappings, launchTemplateDataNetworkInterfaces, launchTemplateDataImageId, launchTemplateDataInstanceType, launchTemplateDataKeyName, launchTemplateDataMonitoring, launchTemplateDataPlacement, launchTemplateDataRamDiskId, launchTemplateDataDisableApiTermination, launchTemplateDataInstanceInitiatedShutdownBehavior, launchTemplateDataUserData, launchTemplateDataTagSpecifications, launchTemplateDataElasticGpuSpecifications, launchTemplateDataElasticInferenceAccelerators, launchTemplateDataSecurityGroupIds, launchTemplateDataSecurityGroups, launchTemplateDataInstanceMarketOptions, launchTemplateDataCreditSpecification, launchTemplateDataCpuOptions, launchTemplateDataCapacityReservationSpecification, launchTemplateDataLicenseSpecifications, launchTemplateDataHibernationOptions, callback) {
    const meth = 'adapter-createLaunchTemplateVersion';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateLaunchTemplateVersion';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientToken: clientToken,
      LaunchTemplateId: launchTemplateId,
      LaunchTemplateName: launchTemplateName,
      SourceVersion: sourceVersion,
      VersionDescription: versionDescription,
      LaunchTemplateDataKernelId: launchTemplateDataKernelId,
      LaunchTemplateDataEbsOptimized: launchTemplateDataEbsOptimized,
      LaunchTemplateDataIamInstanceProfile: launchTemplateDataIamInstanceProfile,
      LaunchTemplateDataImageId: launchTemplateDataImageId,
      LaunchTemplateDataInstanceType: launchTemplateDataInstanceType,
      LaunchTemplateDataKeyName: launchTemplateDataKeyName,
      LaunchTemplateDataMonitoring: launchTemplateDataMonitoring,
      LaunchTemplateDataPlacement: launchTemplateDataPlacement,
      LaunchTemplateDataRamDiskId: launchTemplateDataRamDiskId,
      LaunchTemplateDataDisableApiTermination: launchTemplateDataDisableApiTermination,
      LaunchTemplateDataInstanceInitiatedShutdownBehavior: launchTemplateDataInstanceInitiatedShutdownBehavior,
      LaunchTemplateDataUserData: launchTemplateDataUserData,
      LaunchTemplateDataInstanceMarketOptions: launchTemplateDataInstanceMarketOptions,
      LaunchTemplateDataCreditSpecification: launchTemplateDataCreditSpecification,
      LaunchTemplateDataCpuOptions: launchTemplateDataCpuOptions,
      LaunchTemplateDataCapacityReservationSpecification: launchTemplateDataCapacityReservationSpecification,
      LaunchTemplateDataHibernationOptions: launchTemplateDataHibernationOptions,
      Action: action,
      Version: version
    };
    if (launchTemplateDataBlockDeviceMappings && launchTemplateDataBlockDeviceMappings.length > 0) {
      processInArray(launchTemplateDataBlockDeviceMappings, queryParamsAvailable, 'LaunchTemplateDataBlockDeviceMappings');
    }
    if (launchTemplateDataNetworkInterfaces && launchTemplateDataNetworkInterfaces.length > 0) {
      processInArray(launchTemplateDataNetworkInterfaces, queryParamsAvailable, 'LaunchTemplateDataNetworkInterfaces');
    }
    if (launchTemplateDataTagSpecifications && launchTemplateDataTagSpecifications.length > 0) {
      processInArray(launchTemplateDataTagSpecifications, queryParamsAvailable, 'LaunchTemplateDataTagSpecifications');
    }
    if (launchTemplateDataElasticGpuSpecifications && launchTemplateDataElasticGpuSpecifications.length > 0) {
      processInArray(launchTemplateDataElasticGpuSpecifications, queryParamsAvailable, 'LaunchTemplateDataElasticGpuSpecifications');
    }
    if (launchTemplateDataElasticInferenceAccelerators && launchTemplateDataElasticInferenceAccelerators.length > 0) {
      processInArray(launchTemplateDataElasticInferenceAccelerators, queryParamsAvailable, 'LaunchTemplateDataElasticInferenceAccelerators');
    }
    if (launchTemplateDataSecurityGroupIds && launchTemplateDataSecurityGroupIds.length > 0) {
      processInArray(launchTemplateDataSecurityGroupIds, queryParamsAvailable, 'LaunchTemplateDataSecurityGroupIds');
    }
    if (launchTemplateDataSecurityGroups && launchTemplateDataSecurityGroups.length > 0) {
      processInArray(launchTemplateDataSecurityGroups, queryParamsAvailable, 'LaunchTemplateDataSecurityGroups');
    }
    if (launchTemplateDataLicenseSpecifications && launchTemplateDataLicenseSpecifications.length > 0) {
      processInArray(launchTemplateDataLicenseSpecifications, queryParamsAvailable, 'LaunchTemplateDataLicenseSpecifications');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('LaunchTemplate', 'createLaunchTemplateVersion', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createLaunchTemplateVersion'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple new version for a launch template.
   *
   * @function createLaunchTemplateVersions
   * @param {array} LaunchTemplateVersionsArray - An array of objects(LaunchTemplateVersions parameters). dryRun, clientToken, launchTemplateId, launchTemplateName, sourceVersion, versionDescription, launchTemplateDataKernelId, launchTemplateDataEbsOptimized, launchTemplateDataIamInstanceProfile, launchTemplateDataBlockDeviceMappings, launchTemplateDataNetworkInterfaces, launchTemplateDataImageId, launchTemplateDataInstanceType, launchTemplateDataKeyName, launchTemplateDataMonitoring, launchTemplateDataPlacement, launchTemplateDataRamDiskId, launchTemplateDataDisableApiTermination, launchTemplateDataInstanceInitiatedShutdownBehavior, launchTemplateDataUserData, launchTemplateDataTagSpecifications, launchTemplateDataElasticGpuSpecifications, launchTemplateDataElasticInferenceAccelerators, launchTemplateDataSecurityGroupIds, launchTemplateDataSecurityGroups, launchTemplateDataInstanceMarketOptions, launchTemplateDataCreditSpecification, launchTemplateDataCpuOptions, launchTemplateDataCapacityReservationSpecification, launchTemplateDataLicenseSpecifications, launchTemplateDataHibernationOptions.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createLaunchTemplateVersions(LaunchTemplateVersionsArray, callback) {
    const meth = 'adapter-createLaunchTemplateVersions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (LaunchTemplateVersionsArray === undefined || LaunchTemplateVersionsArray === null || LaunchTemplateVersionsArray === '' || LaunchTemplateVersionsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['LaunchTemplateVersionsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      LaunchTemplateVersionsArray.forEach((item) => {
        const { dryRun, clientToken, launchTemplateId, launchTemplateName, sourceVersion, versionDescription, launchTemplateDataKernelId, launchTemplateDataEbsOptimized, launchTemplateDataIamInstanceProfile, launchTemplateDataBlockDeviceMappings, launchTemplateDataNetworkInterfaces, launchTemplateDataImageId, launchTemplateDataInstanceType, launchTemplateDataKeyName, launchTemplateDataMonitoring, launchTemplateDataPlacement, launchTemplateDataRamDiskId, launchTemplateDataDisableApiTermination, launchTemplateDataInstanceInitiatedShutdownBehavior, launchTemplateDataUserData, launchTemplateDataTagSpecifications, launchTemplateDataElasticGpuSpecifications, launchTemplateDataElasticInferenceAccelerators, launchTemplateDataSecurityGroupIds, launchTemplateDataSecurityGroups, launchTemplateDataInstanceMarketOptions, launchTemplateDataCreditSpecification, launchTemplateDataCpuOptions, launchTemplateDataCapacityReservationSpecification, launchTemplateDataLicenseSpecifications, launchTemplateDataHibernationOptions } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createLaunchTemplateVersion(dryRun, clientToken, launchTemplateId, launchTemplateName, sourceVersion, versionDescription, launchTemplateDataKernelId, launchTemplateDataEbsOptimized, launchTemplateDataIamInstanceProfile, launchTemplateDataBlockDeviceMappings, launchTemplateDataNetworkInterfaces, launchTemplateDataImageId, launchTemplateDataInstanceType, launchTemplateDataKeyName, launchTemplateDataMonitoring, launchTemplateDataPlacement, launchTemplateDataRamDiskId, launchTemplateDataDisableApiTermination, launchTemplateDataInstanceInitiatedShutdownBehavior, launchTemplateDataUserData, launchTemplateDataTagSpecifications, launchTemplateDataElasticGpuSpecifications, launchTemplateDataElasticInferenceAccelerators, launchTemplateDataSecurityGroupIds, launchTemplateDataSecurityGroups, launchTemplateDataInstanceMarketOptions, launchTemplateDataCreditSpecification, launchTemplateDataCpuOptions, launchTemplateDataCapacityReservationSpecification, launchTemplateDataLicenseSpecifications, launchTemplateDataHibernationOptions, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a NAT gateway in the specified public subnet. This action creates a network interface in the specified subnet with a private IP address from the IP address range of the subnet. Internet-bound traffic from a private subnet can be routed to the NAT gateway, therefore enabling instances in the private subnet to connect to the internet. For more information, see  NAT Gateways  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function createNatGateway
   * @param {string} allocationId - The allocation ID of an Elastic IP address to associate with the NAT gateway. If the Elastic IP address is associated with another resource, you must first disassociate i...(description truncated)
   * @param {string} clientToken - Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see  How to Ensure Idempotency .   Constraint: Maximum...(description truncated)
   * @param {string} subnetId - The subnet in which to create the NAT gateway.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createNatGateway(allocationId, clientToken, subnetId, callback) {
    const meth = 'adapter-createNatGateway';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateNatGateway';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (allocationId === undefined || allocationId === null || allocationId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['allocationId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (subnetId === undefined || subnetId === null || subnetId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['subnetId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      AllocationId: allocationId,
      ClientToken: clientToken,
      SubnetId: subnetId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Gateway', 'createNatGateway', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createNatGateway'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple NAT gateways in the specified public subnet.
   *
   * @function createNatGateways
   * @param {array} NatGatewaysArray - An array of objects(NatGateway parameters).allocationId, clientToken, subnetId
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createNatGateways(NatGatewaysArray, callback) {
    const meth = 'adapter-createNatGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (NatGatewaysArray === undefined || NatGatewaysArray === null || NatGatewaysArray === '' || NatGatewaysArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['NatGatewaysArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      NatGatewaysArray.forEach((item) => {
        const { allocationId, clientToken, subnetId } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createNatGateway(allocationId, clientToken, subnetId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a network ACL in a VPC. Network ACLs provide an optional layer of security (in addition to security groups) for the instances in your VPC.   For more information, see  Network ACLs  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function createNetworkAcl
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} vpcId - The ID of the VPC.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createNetworkAcl(dryRun, vpcId, callback) {
    const meth = 'adapter-createNetworkAcl';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateNetworkAcl';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkAcl', 'createNetworkAcl', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createNetworkAcl'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple network ACL in a VPC.
   *
   * @function createNetworkAcls
   * @param {array} NetworkAclsArray - An array of objects(NetworkAcl parameters).dryRun, vpcId
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createNetworkAcls(NetworkAclsArray, callback) {
    const meth = 'adapter-createNetworkAcls';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (NetworkAclsArray === undefined || NetworkAclsArray === null || NetworkAclsArray === '' || NetworkAclsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['NetworkAclsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      NetworkAclsArray.forEach((item) => {
        const { dryRun, vpcId } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createNetworkAcl(dryRun, vpcId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates an entry (a rule) in a network ACL with the specified rule number. Each network ACL has a set of numbered ingress rules and a separate set of numbered egress rules. When determining whether a packet should be allowed in or out of a subnet associated with the ACL, we process the entries in the ACL according to the rule numbers, in ascending order. Each network ACL has a set of ingress rules and a separate set of egress rules.   We recommend that you leave room between the rule numbers (f...(description truncated)
   *
   * @function createNetworkAclEntry
   * @param {string} cidrBlock - The IPv4 network range to allow or deny, in CIDR notation (for example  172.16.0.0/24 ).
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {boolean} egress - Indicates whether this is an egress rule (rule is applied to traffic leaving the subnet).
   * @param {string} icmpCode - Describes the ICMP type and code.
The ICMP code. A value of -1 means all codes for the specified ICMP type.
   * @param {string} icmpType - Describes the ICMP type and code.
The ICMP type. A value of -1 means all types.
   * @param {string} ipv6CidrBlock - The IPv6 network range to allow or deny, in CIDR notation (for example  2001:db8:1234:1a00::/64 ).
   * @param {string} networkAclId - The ID of the network ACL.
   * @param {string} portRangeFrom - Describes a range of ports.
The first port in the range.
   * @param {string} portRangeTo - Describes a range of ports.
The last port in the range.
   * @param {string} protocol - The protocol number. A value of "-1" means all protocols. If you specify "-1" or a protocol number other than "6" (TCP), "17" (UDP), or "1" (ICMP), traffic on all ports i...(description truncated)
   * @param {string} ruleAction - Indicates whether to allow or deny the traffic that matches the rule.
   * @param {number} ruleNumber - The rule number for the entry (for example, 100). ACL entries are processed in ascending order by rule number.   Constraints: Positive integer from 1 to 32766. The range...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createNetworkAclEntry(cidrBlock, dryRun, egress, icmpCode, icmpType, ipv6CidrBlock, networkAclId, portRangeFrom, portRangeTo, protocol, ruleAction = 'allow', ruleNumber, callback) {
    const meth = 'adapter-createNetworkAclEntry';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateNetworkAclEntry';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (egress === undefined || egress === null || egress === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['egress'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (networkAclId === undefined || networkAclId === null || networkAclId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['networkAclId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (protocol === undefined || protocol === null || protocol === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['protocol'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (ruleAction === undefined || ruleAction === null || ruleAction === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['ruleAction'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (ruleNumber === undefined || ruleNumber === null || ruleNumber === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['ruleNumber'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      CidrBlock: cidrBlock,
      Egress: egress,
      IcmpCode: icmpCode,
      IcmpType: icmpType,
      Ipv6CidrBlock: ipv6CidrBlock,
      NetworkAclId: networkAclId,
      PortRangeFrom: portRangeFrom,
      PortRangeTo: portRangeTo,
      Protocol: protocol,
      RuleAction: ruleAction,
      RuleNumber: ruleNumber,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkAcl', 'createNetworkAclEntry', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createNetworkAclEntry'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple entries (a rule) in a network ACL with the specified rule number.
   *
   * @function createNetworkAclEntries
   * @param {array} NetworkAclEntriesArray - An array of objects(NetworkAclEntries parameters).cidrBlock, dryRun, egress, icmpCode, icmpType, ipv6CidrBlock, networkAclId, portRangeFrom, portRangeTo, protocol, ruleAction, ruleNumber
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createNetworkAclEntries(NetworkAclEntriesArray, callback) {
    const meth = 'adapter-createNetworkAclEntries';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (NetworkAclEntriesArray === undefined || NetworkAclEntriesArray === null || NetworkAclEntriesArray === '' || NetworkAclEntriesArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['NetworkAclEntriesArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      NetworkAclEntriesArray.forEach((item) => {
        const { cidrBlock, dryRun, egress, icmpCode, icmpType, ipv6CidrBlock, networkAclId, portRangeFrom, portRangeTo, protocol, ruleAction, ruleNumber } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createNetworkAclEntry(cidrBlock, dryRun, egress, icmpCode, icmpType, ipv6CidrBlock, networkAclId, portRangeFrom, portRangeTo, protocol, ruleAction, ruleNumber, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a network interface in the specified subnet.   For more information about network interfaces, see  Elastic Network Interfaces  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function createNetworkInterface
   * @param {string} description - A description for the network interface.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} securityGroupId - The IDs of one or more security groups.
   * @param {number} ipv6AddressCount - The number of IPv6 addresses to assign to a network interface. Amazon EC2 automatically selects the IPv6 addresses from the subnet range. You can't use this option if spe...(description truncated)
   * @param {array} ipv6Addresses - One or more specific IPv6 addresses from the IPv6 CIDR block range of your subnet. You can't use this option if you're specifying a number of IPv6 addresses.
   * @param {string} privateIpAddress - The primary private IPv4 address of the network interface. If you don't specify an IPv4 address, Amazon EC2 selects one for you from the subnet's IPv4 CIDR range. If you ...(description truncated)
   * @param {array} privateIpAddresses - One or more private IPv4 addresses.
   * @param {number} secondaryPrivateIpAddressCount - The number of secondary private IPv4 addresses to assign to a network interface. When you specify a number of secondary IPv4 addresses, Amazon EC2 selects these IP addre...(description truncated)
   * @param {string} interfaceType - Indicates whether the network interface is an Elastic Fabric Adapter (EFA). Only specify this parameter to create an EFA. For more information, see  Elastic Fabric Adapt...(description truncated)
   * @param {string} subnetId - The ID of the subnet to associate with the network interface.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createNetworkInterface(description, dryRun, securityGroupId, ipv6AddressCount, ipv6Addresses, privateIpAddress, privateIpAddresses, secondaryPrivateIpAddressCount, interfaceType = 'efa', subnetId, callback) {
    const meth = 'adapter-createNetworkInterface';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateNetworkInterface';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (subnetId === undefined || subnetId === null || subnetId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['subnetId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Description: description,
      Ipv6AddressCount: ipv6AddressCount,
      PrivateIpAddress: privateIpAddress,
      SecondaryPrivateIpAddressCount: secondaryPrivateIpAddressCount,
      InterfaceType: interfaceType,
      SubnetId: subnetId,
      Action: action,
      Version: version
    };
    if (securityGroupId && securityGroupId.length > 0) {
      processInArray(securityGroupId, queryParamsAvailable, 'SecurityGroupId');
    }
    if (ipv6Addresses && ipv6Addresses.length > 0) {
      processInArray(ipv6Addresses, queryParamsAvailable, 'Ipv6Addresses');
    }
    if (privateIpAddresses && privateIpAddresses.length > 0) {
      processInArray(privateIpAddresses, queryParamsAvailable, 'PrivateIpAddresses');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkInterface', 'createNetworkInterface', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createNetworkInterface'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple network interfaces in the specified subnet.
   *
   * @function createNetworkInterfaces
   * @param {array} NetworkInterfacesArray - An array of objects(NetworkInterface parameters). description, dryRun, securityGroupId, ipv6AddressCount, ipv6Addresses, privateIpAddress, privateIpAddresses, secondaryPrivateIpAddressCount, interfaceType, subnetId
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createNetworkInterfaces(NetworkInterfacesArray, callback) {
    const meth = 'adapter-createNetworkInterfaces';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (NetworkInterfacesArray === undefined || NetworkInterfacesArray === null || NetworkInterfacesArray === '' || NetworkInterfacesArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['NetworkInterfacesArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      NetworkInterfacesArray.forEach((item) => {
        const { description, dryRun, securityGroupId, ipv6AddressCount, ipv6Addresses, privateIpAddress, privateIpAddresses, secondaryPrivateIpAddressCount, interfaceType, subnetId } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createNetworkInterface(description, dryRun, securityGroupId, ipv6AddressCount, ipv6Addresses, privateIpAddress, privateIpAddresses, secondaryPrivateIpAddressCount, interfaceType, subnetId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Grants an AWS-authorized account permission to attach the specified network interface to an instance in their account.   You can grant permission to a single AWS account only, and only one account at a time.
   *
   * @function createNetworkInterfacePermission
   * @param {string} networkInterfaceId - The ID of the network interface.
   * @param {string} awsAccountId - The AWS account ID.
   * @param {string} awsService - The AWS service. Currently not supported.
   * @param {string} permission - The type of permission to grant.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createNetworkInterfacePermission(networkInterfaceId, awsAccountId, awsService, permission = 'INSTANCE-ATTACH', dryRun, callback) {
    const meth = 'adapter-createNetworkInterfacePermission';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateNetworkInterfacePermission';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (networkInterfaceId === undefined || networkInterfaceId === null || networkInterfaceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['networkInterfaceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (permission === undefined || permission === null || permission === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['permission'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NetworkInterfaceId: networkInterfaceId,
      AwsAccountId: awsAccountId,
      AwsService: awsService,
      Permission: permission,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkInterface', 'createNetworkInterfacePermission', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createNetworkInterfacePermission'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Grants multiple AWS-authorized account permissions to attach the specified network interface to an instance in their account.
   *
   * @function createNetworkInterfacePermissions
   * @param {array} InterfacePermissionsArray - An array of objects(InterfacePermission parameters). networkInterfaceId, awsAccountId, awsService, permission, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createNetworkInterfacePermissions(InterfacePermissionsArray, callback) {
    const meth = 'adapter-createNetworkInterfacePermissions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (InterfacePermissionsArray === undefined || InterfacePermissionsArray === null || InterfacePermissionsArray === '' || InterfacePermissionsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['InterfacePermissionsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      InterfacePermissionsArray.forEach((item) => {
        const { networkInterfaceId, awsAccountId, awsService, permission, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createNetworkInterfacePermission(networkInterfaceId, awsAccountId, awsService, permission, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a placement group in which to launch instances. The strategy of the placement group determines how the instances are organized within the group.    A  cluster  placement group is a logical grouping of instances within a single Availability Zone that benefit from low network latency, high network throughput. A  spread  placement group places instances on distinct hardware. A  partition  placement group places groups of instances in different partitions, where instances in one partition d...(description truncated)
   *
   * @function createPlacementGroup
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} groupName - A name for the placement group. Must be unique within the scope of your account for the Region.   Constraints: Up to 255 ASCII characters
   * @param {string} strategy - The placement strategy.
   * @param {number} partitionCount - The number of partitions. Valid only when  Strategy  is set to  partition .
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createPlacementGroup(dryRun, groupName, strategy = 'cluster', partitionCount, callback) {
    const meth = 'adapter-createPlacementGroup';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreatePlacementGroup';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      GroupName: groupName,
      Strategy: strategy,
      PartitionCount: partitionCount,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionCreatePlacementGroup', 'getCreatePlacementGroup', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createPlacementGroup'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple placement groups in which to launch instances.
   *
   * @function createPlacementGroups
   * @param {array} PlacementGroupsArray - An array of objects(PlacementGroup parameters). dryRun, groupName, strategy, partitionCount
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createPlacementGroups(PlacementGroupsArray, callback) {
    const meth = 'adapter-createPlacementGroups';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (PlacementGroupsArray === undefined || PlacementGroupsArray === null || PlacementGroupsArray === '' || PlacementGroupsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['PlacementGroupsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      PlacementGroupsArray.forEach((item) => {
        const { dryRun, groupName, strategy, partitionCount } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createPlacementGroup(dryRun, groupName, strategy, partitionCount, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a listing for Amazon EC2 Standard Reserved Instances to be sold in the Reserved Instance Marketplace. You can submit one Standard Reserved Instance listing at a time. To get a list of your Standard Reserved Instances, you can use the  DescribeReservedInstances  operation.     Only Standard Reserved Instances can be sold in the Reserved Instance Marketplace. Convertible Reserved Instances cannot be sold.     The Reserved Instance Marketplace matches sellers who want to resell Standard Re...(description truncated)
   *
   * @function createReservedInstancesListing
   * @param {string} clientToken - Unique, case-sensitive identifier you provide to ensure idempotency of your listings. This helps avoid duplicate listings. For more information, see  Ensuring Idempotency...(description truncated)
   * @param {number} instanceCount - The number of instances that are a part of a Reserved Instance account to be listed in the Reserved Instance Marketplace. This number should be less than or equal to the ...(description truncated)
   * @param {array} priceSchedules - A list specifying the price of the Standard Reserved Instance for each month remaining in the Reserved Instance term.
   * @param {string} reservedInstancesId - The ID of the active Standard Reserved Instance.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createReservedInstancesListing(clientToken, instanceCount, priceSchedules, reservedInstancesId, callback) {
    const meth = 'adapter-createReservedInstancesListing';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateReservedInstancesListing';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientToken === undefined || clientToken === null || clientToken === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientToken'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (instanceCount === undefined || instanceCount === null || instanceCount === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceCount'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (priceSchedules === undefined || priceSchedules === null || priceSchedules === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['priceSchedules'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (reservedInstancesId === undefined || reservedInstancesId === null || reservedInstancesId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['reservedInstancesId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      ClientToken: clientToken,
      InstanceCount: instanceCount,
      ReservedInstancesId: reservedInstancesId,
      Action: action,
      Version: version
    };
    if (priceSchedules && priceSchedules.length > 0) {
      processInArray(priceSchedules, queryParamsAvailable, 'PriceSchedules');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'createReservedInstancesListing', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createReservedInstancesListing'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple listing for Amazon EC2 Standard Reserved Instances to be sold in the Reserved Instance Marketplace.
   *
   * @function createReservedInstancesListings
   * @param {array} ReservedInstancesListingArray - An array of objects(ReservedInstancesListing parameters). clientToken, instanceCount, priceSchedules, reservedInstancesId
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createReservedInstancesListings(ReservedInstancesListingArray, callback) {
    const meth = 'adapter-createReservedInstancesListings';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (ReservedInstancesListingArray === undefined || ReservedInstancesListingArray === null || ReservedInstancesListingArray === '' || ReservedInstancesListingArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['ReservedInstancesListingArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      ReservedInstancesListingArray.forEach((item) => {
        const { clientToken, instanceCount, priceSchedules, reservedInstancesId } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createReservedInstancesListing(clientToken, instanceCount, priceSchedules, reservedInstancesId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a route in a route table within a VPC.   You must specify one of the following targets: internet gateway or virtual private gateway, NAT instance, NAT gateway, VPC peering connection, network interface, or egress-only internet gateway.   When determining how to route traffic, we use the route with the most specific match. For example, traffic is destined for the IPv4 address  192.0.2.3 , and the route table includes the following two IPv4 routes:         192.0.2.0/24  (goes to some targ...(description truncated)
   *
   * @function createRoute
   * @param {string} destinationCidrBlock - The IPv4 CIDR address block used for the destination match. Routing decisions are based on the most specific match.
   * @param {string} destinationIpv6CidrBlock - The IPv6 CIDR block used for the destination match. Routing decisions are based on the most specific match.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} egressOnlyInternetGatewayId - [IPv6 traffic only] The ID of an egress-only internet gateway.
   * @param {string} gatewayId - The ID of an internet gateway or virtual private gateway attached to your VPC.
   * @param {string} instanceId - The ID of a NAT instance in your VPC. The operation fails if you specify an instance ID unless exactly one network interface is attached.
   * @param {string} natGatewayId - [IPv4 traffic only] The ID of a NAT gateway.
   * @param {string} transitGatewayId - The ID of a transit gateway.
   * @param {string} networkInterfaceId - The ID of a network interface.
   * @param {string} routeTableId - The ID of the route table for the route.
   * @param {string} vpcPeeringConnectionId - The ID of a VPC peering connection.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createRoute(destinationCidrBlock, destinationIpv6CidrBlock, dryRun, egressOnlyInternetGatewayId, gatewayId, instanceId, natGatewayId, transitGatewayId, networkInterfaceId, routeTableId, vpcPeeringConnectionId, callback) {
    const meth = 'adapter-createRoute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateRoute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (routeTableId === undefined || routeTableId === null || routeTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['routeTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      DestinationCidrBlock: destinationCidrBlock,
      DestinationIpv6CidrBlock: destinationIpv6CidrBlock,
      EgressOnlyInternetGatewayId: egressOnlyInternetGatewayId,
      GatewayId: gatewayId,
      InstanceId: instanceId,
      NatGatewayId: natGatewayId,
      TransitGatewayId: transitGatewayId,
      NetworkInterfaceId: networkInterfaceId,
      RouteTableId: routeTableId,
      VpcPeeringConnectionId: vpcPeeringConnectionId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Route', 'createRoute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createRoute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple routes in a route table within a VPC. You must specify one of the following targets: internet gateway or virtual private gateway, NAT instance, NAT gateway, VPC peering connection, network interface, or egress-only internet gateway.   When determining how to route traffic, we use the route with the most specific match. For example, traffic is destined for the IPv4 address  192.0.2.3 , and the route table includes the following two IPv4 routes:         192.0.2.0/24  (goes to some targ...(description truncated)
   *
   * @function createRoutes
   * @param {array} routeArray - An array of objects(route parameters). destinationCidrBlock, destinationIpv6CidrBlock, dryRun, egressOnlyInternetGatewayId, gatewayId, instanceId, natGatewayId, transitGatewayId, networkInterfaceId, routeTableId, vpcPeeringConnectionId.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createRoutes(routeArray, callback) {
    const meth = 'adapter-createRoutes';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (routeArray === undefined || routeArray === null || routeArray === '' || routeArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['routeArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each route individually.
      routeArray.forEach((route) => {
        const { destinationCidrBlock, destinationIpv6CidrBlock, dryRun, egressOnlyInternetGatewayId, gatewayId, instanceId, natGatewayId, transitGatewayId, networkInterfaceId, routeTableId, vpcPeeringConnectionId } = route;
        returnArray.push(
          new Promise((resolve) => {
            this.createRoute(destinationCidrBlock, destinationIpv6CidrBlock, dryRun, egressOnlyInternetGatewayId, gatewayId, instanceId, natGatewayId, transitGatewayId, networkInterfaceId, routeTableId, vpcPeeringConnectionId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a route table for the specified VPC. After you create a route table, you can add routes and associate the table with a subnet.   For more information, see  Route Tables  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function createRouteTable
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} vpcId - The ID of the VPC.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createRouteTable(dryRun, vpcId, callback) {
    const meth = 'adapter-createRouteTable';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateRouteTable';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Route', 'createRouteTable', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createRouteTable'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple route tables for the specified VPC. After you create a route table, you can add routes and associate the table with a subnet.   For more information, see  Route Tables  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function createRouteTables
   * @param {array} routeTableArray - An array of objects(routeTable parameters). dryRun, vpcId.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createRouteTables(routeTableArray, callback) {
    const meth = 'adapter-createRouteTables';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (routeTableArray === undefined || routeTableArray === null || routeTableArray === '' || routeTableArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['routeTableArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each route table individually.
      routeTableArray.forEach((routeTable) => {
        const { dryRun, vpcId } = routeTable;
        returnArray.push(
          new Promise((resolve) => {
            this.createRouteTable(dryRun, vpcId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a security group.   A security group acts as a virtual firewall for your instance to control inbound and outbound traffic. For more information, see  Amazon EC2 Security Groups  in the  Amazon Elastic Compute Cloud User Guide  and  Security Groups for Your VPC  in the  Amazon Virtual Private Cloud User Guide .   When you create a security group, you specify a friendly name of your choice. You can have a security group for use in EC2-Classic with the same name as a security group for use...(description truncated)
   *
   * @function createSecurityGroup
   * @param {string} groupDescription - A description for the security group. This is informational only.   Constraints: Up to 255 characters in length   Constraints for EC2-Classic: ASCII characters   Constra...(description truncated)
   * @param {string} groupName - The name of the security group.   Constraints: Up to 255 characters in length. Cannot start with  sg- .   Constraints for EC2-Classic: ASCII characters   Constraints for...(description truncated)
   * @param {string} vpcId - [EC2-VPC] The ID of the VPC. Required for EC2-VPC.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createSecurityGroup(groupDescription, groupName, vpcId, dryRun, callback) {
    const meth = 'adapter-createSecurityGroup';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateSecurityGroup';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (groupDescription === undefined || groupDescription === null || groupDescription === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['groupDescription'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (groupName === undefined || groupName === null || groupName === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['groupName'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      GroupDescription: groupDescription,
      GroupName: groupName,
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('SecurityGroup', 'createSecurityGroup', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createSecurityGroup'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple security groups.   A security group acts as a virtual firewall for your instance to control inbound and outbound traffic. For more information, see  Amazon EC2 Security Groups  in the  Amazon Elastic Compute Cloud User Guide  and  Security Groups for Your VPC  in the  Amazon Virtual Private Cloud User Guide .   When you create a security group, you specify a friendly name of your choice. You can have a security group for use in EC2-Classic with the same name as a security group for use...(description truncated)
   *
   * @function createSecurityGroups
   * @param {array} securityGroupArray - An array of objects(SecurityGroup parameters). groupDescription, groupName, vpcId, dryRun.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createSecurityGroups(securityGroupArray, callback) {
    const meth = 'adapter-createSecurityGroups';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (securityGroupArray === undefined || securityGroupArray === null || securityGroupArray === '' || securityGroupArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['securityGroupArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each securityGroup individually.
      securityGroupArray.forEach((securityGroup) => {
        const { groupDescription, groupName, vpcId, dryRun } = securityGroup;
        returnArray.push(
          new Promise((resolve) => {
            this.createSecurityGroup(groupDescription, groupName, vpcId, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a snapshot of an EBS volume and stores it in Amazon S3. You can use snapshots for backups, to make copies of EBS volumes, and to save data before shutting down an instance.   When a snapshot is created, any AWS Marketplace product codes that are associated with the source volume are propagated to the snapshot.   You can take a snapshot of an attached volume that is in use. However, snapshots only capture data that has been written to your EBS volume at the time the snapshot command is i...(description truncated)
   *
   * @function createSnapshot
   * @param {string} description - A description for the snapshot.
   * @param {string} volumeId - The ID of the EBS volume.
   * @param {array} tagSpecification - The tags to apply to the snapshot during creation.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createSnapshot(description, volumeId, tagSpecification, dryRun, callback) {
    const meth = 'adapter-createSnapshot';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateSnapshot';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (volumeId === undefined || volumeId === null || volumeId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['volumeId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Description: description,
      VolumeId: volumeId,
      Action: action,
      Version: version
    };
    if (tagSpecification && tagSpecification.length > 0) {
      processInArray(tagSpecification, queryParamsAvailable, 'TagSpecification');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Snapshot', 'createSnapshot', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createSnapshot'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple snapshots of an EBS volume and stores it in Amazon S3.
   *
   * @function createSnapshots
   * @param {array} SnapshotsArray - An array of objects(Snapshot parameters). description, volumeId, tagSpecification, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createSnapshots(SnapshotsArray, callback) {
    const meth = 'adapter-createSnapshots';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (SnapshotsArray === undefined || SnapshotsArray === null || SnapshotsArray === '' || SnapshotsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['SnapshotsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      SnapshotsArray.forEach((item) => {
        const { description, volumeId, tagSpecification, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createSnapshot(description, volumeId, tagSpecification, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a data feed for Spot Instances, enabling you to view Spot Instance usage logs. You can create one data feed per AWS account. For more information, see  Spot Instance Data Feed  in the  Amazon EC2 User Guide for Linux Instances .
   *
   * @function createSpotDatafeedSubscription
   * @param {string} bucket - The Amazon S3 bucket in which to store the Spot Instance data feed.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} prefix - A prefix for the data feed file names.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createSpotDatafeedSubscription(bucket, dryRun, prefix, callback) {
    const meth = 'adapter-createSpotDatafeedSubscription';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateSpotDatafeedSubscription';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (bucket === undefined || bucket === null || bucket === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['bucket'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Bucket: bucket,
      Prefix: prefix,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionCreateSpotDatafeedSubscription', 'getCreateSpotDatafeedSubscription', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createSpotDatafeedSubscription'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple data feeds for Spot Instances, enabling you to view Spot Instance usage logs.
   *
   * @function createSpotDatafeedSubscriptions
   * @param {array} SpotDatafeedSubscriptionsArray - An array of objects(SpotDatafeedSubscription parameters). bucket, dryRun, prefix
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createSpotDatafeedSubscriptions(SpotDatafeedSubscriptionsArray, callback) {
    const meth = 'adapter-createSpotDatafeedSubscriptions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (SpotDatafeedSubscriptionsArray === undefined || SpotDatafeedSubscriptionsArray === null || SpotDatafeedSubscriptionsArray === '' || SpotDatafeedSubscriptionsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['SpotDatafeedSubscriptionsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      SpotDatafeedSubscriptionsArray.forEach((item) => {
        const { bucket, dryRun, prefix } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createSpotDatafeedSubscription(bucket, dryRun, prefix, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a subnet in an existing VPC.   When you create each subnet, you provide the VPC ID and IPv4 CIDR block for the subnet. After you create a subnet, you can't change its CIDR block. The size of the subnet's IPv4 CIDR block can be the same as a VPC's IPv4 CIDR block, or a subset of a VPC's IPv4 CIDR block. If you create more than one subnet in a VPC, the subnets' CIDR blocks must not overlap. The smallest IPv4 subnet (and VPC) you can create uses a /28 netmask (16 IPv4 addresses), and the l...(description truncated)
   *
   * @function createSubnet
   * @param {string} availabilityZone - The Availability Zone for the subnet.   Default: AWS selects one for you. If you create more than one subnet in your VPC, we may not necessarily select a different zone ...(description truncated)
   * @param {string} availabilityZoneId - The AZ ID of the subnet.
   * @param {string} cidrBlock - The IPv4 network range for the subnet, in CIDR notation. For example,  10.0.0.0/24 .
   * @param {string} ipv6CidrBlock - The IPv6 network range for the subnet, in CIDR notation. The subnet size must use a /64 prefix length.
   * @param {string} vpcId - The ID of the VPC.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createSubnet(availabilityZone, availabilityZoneId, cidrBlock, ipv6CidrBlock, vpcId, dryRun, callback) {
    const meth = 'adapter-createSubnet';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateSubnet';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (cidrBlock === undefined || cidrBlock === null || cidrBlock === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['cidrBlock'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AvailabilityZone: availabilityZone,
      AvailabilityZoneId: availabilityZoneId,
      CidrBlock: cidrBlock,
      Ipv6CidrBlock: ipv6CidrBlock,
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Subnet', 'createSubnet', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createSubnet'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates subnets in an existing VPC. When you create each subnet, you provide the VPC ID and IPv4 CIDR block for the subnet. After you create a subnet, you can't change its CIDR block. The size of the subnet's IPv4 CIDR block can be the same as a VPC's IPv4 CIDR block, or a subset of a VPC's IPv4 CIDR block. If you create more than one subnet in a VPC, the subnets' CIDR blocks must not overlap. The smallest IPv4 subnet (and VPC) you can create uses a /28 netmask (16 IPv4 addresses), and the l...(description truncated)
   *
   * @function createSubnets
   * @param {array} subnetArray - An array of objects(subnet parameters). availabilityZone, availabilityZoneId, cidrBlock, ipv6CidrBlock, vpcId, dryRun.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createSubnets(subnetArray, callback) {
    const meth = 'adapter-createSubnets';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (subnetArray === undefined || subnetArray === null || subnetArray === '' || subnetArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['subnetArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each subnet individually.
      subnetArray.forEach((subnet) => {
        const { availabilityZone, availabilityZoneId, cidrBlock, ipv6CidrBlock, vpcId, dryRun } = subnet;
        returnArray.push(
          new Promise((resolve) => {
            this.createSubnet(availabilityZone, availabilityZoneId, cidrBlock, ipv6CidrBlock, vpcId, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Adds or overwrites the specified tags for the specified Amazon EC2 resource or resources. Each resource can have a maximum of 50 tags. Each tag consists of a key and optional value. Tag keys must be unique per resource.   For more information about tags, see  Tagging Your Resources  in the  Amazon Elastic Compute Cloud User Guide . For more information about creating IAM policies that control users' access to resources based on tags, see  Supported Resource-Level Permissions for Amazon EC2 API ...(description truncated)
   *
   * @function createTags
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} resourceId - The IDs of the resources, separated by spaces.   Constraints: Up to 1000 resource IDs. We recommend breaking up this request into smaller batches.
   * @param {array} tag - The tags. The  value  parameter is required, but if you don't want the tag to have a value, specify the parameter with no value, and we set the value to an empty string.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createTags(dryRun, resourceId, tag, callback) {
    const meth = 'adapter-createTags';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateTags';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (resourceId === undefined || resourceId === null || resourceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['resourceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (tag === undefined || tag === null || tag === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['tag'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (resourceId && resourceId.length > 0) {
      processInArray(resourceId, queryParamsAvailable, 'ResourceId');
    }
    if (tag && tag.length > 0) {
      processInArray(tag, queryParamsAvailable, 'Tag');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionCreateTags', 'getCreateTags', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createTags'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Adds or overwrites multiple specified tags for the specified Amazon EC2 resource or resources.
   *
   * @function createMultipleTags
   * @param {array} MultipleTagsArray - An array of objects(Tags parameters). dryRun, resourceId, tag
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createMultipleTags(MultipleTagsArray, callback) {
    const meth = 'adapter-createMultipleTags';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (MultipleTagsArray === undefined || MultipleTagsArray === null || MultipleTagsArray === '' || MultipleTagsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['MultipleTagsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      MultipleTagsArray.forEach((item) => {
        const { dryRun, resourceId, tag } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createTags(dryRun, resourceId, tag, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a transit gateway.   You can use a transit gateway to interconnect your virtual private clouds (VPC) and on-premises networks. After the transit gateway enters the  available  state, you can attach your VPCs and VPN connections to the transit gateway.   To attach your VPCs, use  CreateTransitGatewayVpcAttachment .   To attach a VPN connection, use  CreateCustomerGateway  to create a customer gateway and specify the ID of the customer gateway and the ID of the transit gateway in a call t...(description truncated)
   *
   * @function createTransitGateway
   * @param {string} description - A description of the transit gateway.
   * @param {string} optionsAmazonSideAsn - Describes the options for a transit gateway.
A private Autonomous System Number (ASN) for the Amazon side of a BGP session. The range is 64512 to 65534 for 16-bit ASNs an...(description truncated)
   * @param {string} optionsAutoAcceptSharedAttachments - Describes the options for a transit gateway.
Enable or disable automatic acceptance of attachment requests. The default is  disable .
   * @param {string} optionsDefaultRouteTableAssociation - Describes the options for a transit gateway.
Enable or disable automatic association with the default association route table. The default is  enable .
   * @param {string} optionsDefaultRouteTablePropagation - Describes the options for a transit gateway.
Enable or disable automatic propagation of routes to the default propagation route table. The default is  enable .
   * @param {string} optionsVpnEcmpSupport - Describes the options for a transit gateway.
Enable or disable Equal Cost Multipath Protocol support.
   * @param {string} optionsDnsSupport - Describes the options for a transit gateway.
Enable or disable DNS support.
   * @param {array} tagSpecification - The tags to apply to the transit gateway.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createTransitGateway(description, optionsAmazonSideAsn, optionsAutoAcceptSharedAttachments, optionsDefaultRouteTableAssociation, optionsDefaultRouteTablePropagation, optionsVpnEcmpSupport, optionsDnsSupport, tagSpecification, dryRun, callback) {
    const meth = 'adapter-createTransitGateway';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateTransitGateway';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Description: description,
      OptionsAmazonSideAsn: optionsAmazonSideAsn,
      OptionsAutoAcceptSharedAttachments: optionsAutoAcceptSharedAttachments,
      OptionsDefaultRouteTableAssociation: optionsDefaultRouteTableAssociation,
      OptionsDefaultRouteTablePropagation: optionsDefaultRouteTablePropagation,
      OptionsVpnEcmpSupport: optionsVpnEcmpSupport,
      OptionsDnsSupport: optionsDnsSupport,
      Action: action,
      Version: version
    };
    if (tagSpecification && tagSpecification.length > 0) {
      processInArray(tagSpecification, queryParamsAvailable, 'TagSpecification');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'createTransitGateway', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createTransitGateway'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple transit gateways.
   *
   * @function createTransitGateways
   * @param {array} TransitGatewaysArray - An array of objects(TransitGateway parameters). description, optionsAmazonSideAsn, optionsAutoAcceptSharedAttachments, optionsDefaultRouteTableAssociation, optionsDefaultRouteTablePropagation, optionsVpnEcmpSupport, optionsDnsSupport, tagSpecification, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createTransitGateways(TransitGatewaysArray, callback) {
    const meth = 'adapter-createTransitGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (TransitGatewaysArray === undefined || TransitGatewaysArray === null || TransitGatewaysArray === '' || TransitGatewaysArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['TransitGatewaysArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      TransitGatewaysArray.forEach((item) => {
        const { description, optionsAmazonSideAsn, optionsAutoAcceptSharedAttachments, optionsDefaultRouteTableAssociation, optionsDefaultRouteTablePropagation, optionsVpnEcmpSupport, optionsDnsSupport, tagSpecification, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createTransitGateway(description, optionsAmazonSideAsn, optionsAutoAcceptSharedAttachments, optionsDefaultRouteTableAssociation, optionsDefaultRouteTablePropagation, optionsVpnEcmpSupport, optionsDnsSupport, tagSpecification, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a static route for the specified transit gateway route table.
   *
   * @function createTransitGatewayRoute
   * @param {string} destinationCidrBlock - The CIDR range used for destination matches. Routing decisions are based on the most specific match.
   * @param {string} transitGatewayRouteTableId - The ID of the transit gateway route table.
   * @param {string} transitGatewayAttachmentId - The ID of the attachment.
   * @param {boolean} blackhole - Indicates whether to drop traffic that matches this route.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createTransitGatewayRoute(destinationCidrBlock, transitGatewayRouteTableId, transitGatewayAttachmentId, blackhole, dryRun, callback) {
    const meth = 'adapter-createTransitGatewayRoute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateTransitGatewayRoute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (destinationCidrBlock === undefined || destinationCidrBlock === null || destinationCidrBlock === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['destinationCidrBlock'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (transitGatewayRouteTableId === undefined || transitGatewayRouteTableId === null || transitGatewayRouteTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayRouteTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      DestinationCidrBlock: destinationCidrBlock,
      TransitGatewayRouteTableId: transitGatewayRouteTableId,
      TransitGatewayAttachmentId: transitGatewayAttachmentId,
      Blackhole: blackhole,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'createTransitGatewayRoute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createTransitGatewayRoute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple static routes for the specified transit gateway route table.
   *
   * @function createTransitGatewayRoutes
   * @param {array} TransitGatewayRoutesArray - An array of objects(TransitGatewayRoute parameters). destinationCidrBlock, transitGatewayRouteTableId, transitGatewayAttachmentId, blackhole, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createTransitGatewayRoutes(TransitGatewayRoutesArray, callback) {
    const meth = 'adapter-createTransitGatewayRoutes';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (TransitGatewayRoutesArray === undefined || TransitGatewayRoutesArray === null || TransitGatewayRoutesArray === '' || TransitGatewayRoutesArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['TransitGatewayRoutesArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      TransitGatewayRoutesArray.forEach((item) => {
        const { destinationCidrBlock, transitGatewayRouteTableId, transitGatewayAttachmentId, blackhole, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createTransitGatewayRoute(destinationCidrBlock, transitGatewayRouteTableId, transitGatewayAttachmentId, blackhole, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a route table for the specified transit gateway.
   *
   * @function createTransitGatewayRouteTable
   * @param {string} transitGatewayId - The ID of the transit gateway.
   * @param {array} tagSpecifications - The tags to apply to the transit gateway route table.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createTransitGatewayRouteTable(transitGatewayId, tagSpecifications, dryRun, callback) {
    const meth = 'adapter-createTransitGatewayRouteTable';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateTransitGatewayRouteTable';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayId === undefined || transitGatewayId === null || transitGatewayId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      TransitGatewayId: transitGatewayId,
      Action: action,
      Version: version
    };
    if (tagSpecifications && tagSpecifications.length > 0) {
      processInArray(tagSpecifications, queryParamsAvailable, 'TagSpecifications');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'createTransitGatewayRouteTable', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createTransitGatewayRouteTable'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple route tables for the specified transit gateway.
   *
   * @function createTransitGatewayRouteTables
   * @param {array} TransitGatewayRouteTablesArray - An array of objects(TransitGatewayRouteTable parameters). transitGatewayId, tagSpecifications, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createTransitGatewayRouteTables(TransitGatewayRouteTablesArray, callback) {
    const meth = 'adapter-createTransitGatewayRouteTables';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (TransitGatewayRouteTablesArray === undefined || TransitGatewayRouteTablesArray === null || TransitGatewayRouteTablesArray === '' || TransitGatewayRouteTablesArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['TransitGatewayRouteTablesArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      TransitGatewayRouteTablesArray.forEach((item) => {
        const { transitGatewayId, tagSpecifications, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createTransitGatewayRouteTable(transitGatewayId, tagSpecifications, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Attaches the specified VPC to the specified transit gateway.   If you attach a VPC with a CIDR range that overlaps the CIDR range of a VPC that is already attached, the new VPC CIDR range is not propagated to the default propagation route table.   To send VPC traffic to an attached transit gateway, add a route to the VPC route table using  CreateRoute .
   *
   * @function createTransitGatewayVpcAttachment
   * @param {string} transitGatewayId - The ID of the transit gateway.
   * @param {string} vpcId - The ID of the VPC.
   * @param {array} subnetIds - The IDs of one or more subnets. You can specify only one subnet per Availability Zone. You must specify at least one subnet, but we recommend that you specify two subnets...(description truncated)
   * @param {string} optionsDnsSupport - Describes the options for a VPC attachment.
Enable or disable DNS support. The default is  enable .
   * @param {string} optionsIpv6Support - Describes the options for a VPC attachment.
Enable or disable IPv6 support. The default is  enable .
   * @param {array} tagSpecifications - The tags to apply to the VPC attachment.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createTransitGatewayVpcAttachment(transitGatewayId, vpcId, subnetIds, optionsDnsSupport, optionsIpv6Support, tagSpecifications, dryRun, callback) {
    const meth = 'adapter-createTransitGatewayVpcAttachment';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateTransitGatewayVpcAttachment';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayId === undefined || transitGatewayId === null || transitGatewayId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (subnetIds === undefined || subnetIds === null || subnetIds === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['subnetIds'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      TransitGatewayId: transitGatewayId,
      VpcId: vpcId,
      OptionsDnsSupport: optionsDnsSupport,
      OptionsIpv6Support: optionsIpv6Support,
      Action: action,
      Version: version
    };
    if (subnetIds && subnetIds.length > 0) {
      processInArray(subnetIds, queryParamsAvailable, 'SubnetIds');
    }
    if (tagSpecifications && tagSpecifications.length > 0) {
      processInArray(tagSpecifications, queryParamsAvailable, 'TagSpecifications');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'createTransitGatewayVpcAttachment', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createTransitGatewayVpcAttachment'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Attaches the specified VPCs to the specified transit gateway.
   *
   * @function createTransitGatewayVpcAttachments
   * @param {array} TransitGatewayVpcAttachmentsArray - An array of objects(TransitGatewayVpcAttachment parameters). transitGatewayId, vpcId, subnetIds, optionsDnsSupport, optionsIpv6Support, tagSpecifications, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createTransitGatewayVpcAttachments(TransitGatewayVpcAttachmentsArray, callback) {
    const meth = 'adapter-createTransitGatewayVpcAttachments';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (TransitGatewayVpcAttachmentsArray === undefined || TransitGatewayVpcAttachmentsArray === null || TransitGatewayVpcAttachmentsArray === '' || TransitGatewayVpcAttachmentsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['TransitGatewayVpcAttachmentsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      TransitGatewayVpcAttachmentsArray.forEach((item) => {
        const { transitGatewayId, vpcId, subnetIds, optionsDnsSupport, optionsIpv6Support, tagSpecifications, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createTransitGatewayVpcAttachment(transitGatewayId, vpcId, subnetIds, optionsDnsSupport, optionsIpv6Support, tagSpecifications, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates an EBS volume that can be attached to an instance in the same Availability Zone. The volume is created in the regional endpoint that you send the HTTP request to. For more information see  Regions and Endpoints .   You can create a new empty volume or restore a volume from an EBS snapshot. Any AWS Marketplace product codes from the snapshot are propagated to the volume.   You can create encrypted volumes with the  Encrypted  parameter. Encrypted volumes may only be attached to instances...(description truncated)
   *
   * @function createVolume
   * @param {string} availabilityZone - The Availability Zone in which to create the volume. Use  DescribeAvailabilityZones  to list the Availability Zones that are currently available to you.
   * @param {boolean} encrypted - Specifies whether the volume should be encrypted. Encrypted Amazon EBS volumes may only be attached to instances that support Amazon EBS encryption. Volumes that are crea...(description truncated)
   * @param {number} iops - The number of I/O operations per second (IOPS) to provision for the volume, with a maximum ratio of 50 IOPS/GiB. Range is 100 to 64,000 IOPS for volumes in most Regions....(description truncated)
   * @param {string} kmsKeyId - An identifier for the AWS Key Management Service (AWS KMS) customer master key (CMK) to use when creating the encrypted volume. This parameter is only required if you wa...(description truncated)
   * @param {number} size - The size of the volume, in GiBs.   Constraints: 1-16,384 for  gp2 , 4-16,384 for  io1 , 500-16,384 for  st1 , 500-16,384 for  sc1 , and 1-1,024 for  standard . If you sp...(description truncated)
   * @param {string} snapshotId - The snapshot from which to create the volume.     At least one of Size or SnapshotId are required.
   * @param {string} volumeType - The volume type. This can be  gp2  for General Purpose SSD,  io1  for Provisioned IOPS SSD,  st1  for Throughput Optimized HDD,  sc1  for Cold HDD, or  standard  for Mag...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} tagSpecification - The tags to apply to the volume during creation.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createVolume(availabilityZone, encrypted, iops, kmsKeyId, size, snapshotId, volumeType = 'standard', dryRun, tagSpecification, callback) {
    const meth = 'adapter-createVolume';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateVolume';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (availabilityZone === undefined || availabilityZone === null || availabilityZone === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['availabilityZone'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AvailabilityZone: availabilityZone,
      Encrypted: encrypted,
      Iops: iops,
      KmsKeyId: kmsKeyId,
      Size: size,
      SnapshotId: snapshotId,
      VolumeType: volumeType,
      Action: action,
      Version: version
    };
    if (tagSpecification && tagSpecification.length > 0) {
      processInArray(tagSpecification, queryParamsAvailable, 'TagSpecification');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Volume', 'createVolume', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createVolume'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple EBS volumes that can be attached to an instance in the same Availability Zone.
   *
   * @function createVolumes
   * @param {array} createVolumesArray - An array of objects(Volume parameters). availabilityZone, encrypted, iops, kmsKeyId, size, snapshotId, volumeType, dryRun, tagSpecification
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createVolumes(TransitGatewayVpcAttachmentssArray, callback) {
    const meth = 'adapter-createVolumes';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (TransitGatewayVpcAttachmentssArray === undefined || TransitGatewayVpcAttachmentssArray === null || TransitGatewayVpcAttachmentssArray === '' || TransitGatewayVpcAttachmentssArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['TransitGatewayVpcAttachmentssArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      TransitGatewayVpcAttachmentssArray.forEach((item) => {
        const { availabilityZone, encrypted, iops, kmsKeyId, size, snapshotId, volumeType, dryRun, tagSpecification } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createVolume(availabilityZone, encrypted, iops, kmsKeyId, size, snapshotId, volumeType, dryRun, tagSpecification, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a VPC with the specified IPv4 CIDR block. The smallest VPC you can create uses a /28 netmask (16 IPv4 addresses), and the largest uses a /16 netmask (65,536 IPv4 addresses). For more information about how large to make your VPC, see  Your VPC and Subnets  in the  Amazon Virtual Private Cloud User Guide .   You can optionally request an Amazon-provided IPv6 CIDR block for the VPC. The IPv6 CIDR block uses a /56 prefix length, and is allocated from Amazon's pool of IPv6 addresses. You can...(description truncated)
   *
   * @function createVpc
   * @param {string} cidrBlock - The IPv4 network range for the VPC, in CIDR notation. For example,  10.0.0.0/16 .
   * @param {boolean} amazonProvidedIpv6CidrBlock - Requests an Amazon-provided IPv6 CIDR block with a /56 prefix length for the VPC. You cannot specify the range of IP addresses, or the size of the CIDR block.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} instanceTenancy - The tenancy options for instances launched into the VPC. For  default , instances are launched with shared tenancy by default. You can launch instances with any tenancy ...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createVpc(cidrBlock, amazonProvidedIpv6CidrBlock, dryRun, instanceTenancy = 'default', callback) {
    const meth = 'adapter-createVpc';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateVpc';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (cidrBlock === undefined || cidrBlock === null || cidrBlock === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['cidrBlock'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      CidrBlock: cidrBlock,
      AmazonProvidedIpv6CidrBlock: amazonProvidedIpv6CidrBlock,
      InstanceTenancy: instanceTenancy,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'createVpc', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createVpc'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates VPCs with the specified IPv4 CIDR block. The smallest VPC you can create uses a /28 netmask (16 IPv4 addresses), and the largest uses a /16 netmask (65,536 IPv4 addresses). For more information about how large to make your VPC, see  Your VPC and Subnets  in the  Amazon Virtual Private Cloud User Guide .   You can optionally request an Amazon-provided IPv6 CIDR block for the VPC. The IPv6 CIDR block uses a /56 prefix length, and is allocated from Amazon's pool of IPv6 addresses. You can...(description truncated)
   *
   * @function createVpcs
   * @param {array} vpcArray - An array of objects(vpc parameters). cidrBlock, amazonProvidedIpv6CidrBlock, dryRun, instanceTenancy.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createVpcs(vpcArray, callback) {
    const meth = 'adapter-createVpcs';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcArray === undefined || vpcArray === null || vpcArray === '' || vpcArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each vpc individually.
      vpcArray.forEach((vpc) => {
        const { cidrBlock, amazonProvidedIpv6CidrBlock, dryRun, instanceTenancy } = vpc;
        returnArray.push(
          new Promise((resolve) => {
            this.createVpc(cidrBlock, amazonProvidedIpv6CidrBlock, dryRun, instanceTenancy, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a VPC endpoint for a specified service. An endpoint enables you to create a private connection between your VPC and the service. The service may be provided by AWS, an AWS Marketplace partner, or another AWS account. For more information, see  VPC Endpoints  in the  Amazon Virtual Private Cloud User Guide .   A  gateway  endpoint serves as a target for a route in your route table for traffic destined for the AWS service. You can specify an endpoint policy to attach to the endpoint that ...(description truncated)
   *
   * @function createVpcEndpoint
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} vpcEndpointType - The type of endpoint.   Default: Gateway
   * @param {string} vpcId - The ID of the VPC in which the endpoint will be used.
   * @param {string} serviceName - The service name. To get a list of available services, use the  DescribeVpcEndpointServices  request, or get the name from the service provider.
   * @param {string} policyDocument - A policy to attach to the endpoint that controls access to the service. The policy must be in valid JSON format. If this parameter is not specified, we attach a default p...(description truncated)
   * @param {array} routeTableId - (Gateway endpoint) One or more route table IDs.
   * @param {array} subnetId - (Interface endpoint) The ID of one or more subnets in which to create an endpoint network interface.
   * @param {array} securityGroupId - (Interface endpoint) The ID of one or more security groups to associate with the endpoint network interface.
   * @param {string} clientToken - Unique, case-sensitive identifier you provide to ensure the idempotency of the request. For more information, see  How to Ensure Idempotency .
   * @param {boolean} privateDnsEnabled - (Interface endpoint) Indicate whether to associate a private hosted zone with the specified VPC. The private hosted zone contains a record set for the default public DNS...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createVpcEndpoint(dryRun, vpcEndpointType = 'Interface', vpcId, serviceName, policyDocument, routeTableId, subnetId, securityGroupId, clientToken, privateDnsEnabled, callback) {
    const meth = 'adapter-createVpcEndpoint';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateVpcEndpoint';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (serviceName === undefined || serviceName === null || serviceName === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['serviceName'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VpcEndpointType: vpcEndpointType,
      VpcId: vpcId,
      ServiceName: serviceName,
      PolicyDocument: policyDocument,
      ClientToken: clientToken,
      PrivateDnsEnabled: privateDnsEnabled,
      Action: action,
      Version: version
    };
    if (routeTableId && routeTableId.length > 0) {
      processInArray(routeTableId, queryParamsAvailable, 'RouteTableId');
    }
    if (subnetId && subnetId.length > 0) {
      processInArray(subnetId, queryParamsAvailable, 'SubnetId');
    }
    if (securityGroupId && securityGroupId.length > 0) {
      processInArray(securityGroupId, queryParamsAvailable, 'SecurityGroupId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'createVpcEndpoint', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createVpcEndpoint'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple VPC endpoints for a specified service.
   *
   * @function createVpcEndpoints
   * @param {array} createVpcEndpointsArray - An array of objects(VpcEndpoint parameters). dryRun, vpcEndpointType, vpcId, serviceName, policyDocument, routeTableId, subnetId, securityGroupId, clientToken, privateDnsEnabled
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createVpcEndpoints(createVpcEndpointsArray, callback) {
    const meth = 'adapter-createVpcEndpoints';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (createVpcEndpointsArray === undefined || createVpcEndpointsArray === null || createVpcEndpointsArray === '' || createVpcEndpointsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['createVpcEndpointsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      createVpcEndpointsArray.forEach((item) => {
        const { dryRun, vpcEndpointType, vpcId, serviceName, policyDocument, routeTableId, subnetId, securityGroupId, clientToken, privateDnsEnabled } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createVpcEndpoint(dryRun, vpcEndpointType, vpcId, serviceName, policyDocument, routeTableId, subnetId, securityGroupId, clientToken, privateDnsEnabled, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a connection notification for a specified VPC endpoint or VPC endpoint service. A connection notification notifies you of specific endpoint events. You must create an SNS topic to receive notifications. For more information, see  Create a Topic  in the  Amazon Simple Notification Service Developer Guide .   You can create a connection notification for interface endpoints only.
   *
   * @function createVpcEndpointConnectionNotification
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} serviceId - The ID of the endpoint service.
   * @param {string} vpcEndpointId - The ID of the endpoint.
   * @param {string} connectionNotificationArn - The ARN of the SNS topic for the notifications.
   * @param {array} connectionEvents - One or more endpoint events for which to receive notifications. Valid values are  Accept ,  Connect ,  Delete , and  Reject .
   * @param {string} clientToken - Unique, case-sensitive identifier you provide to ensure the idempotency of the request. For more information, see  How to Ensure Idempotency .
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createVpcEndpointConnectionNotification(dryRun, serviceId, vpcEndpointId, connectionNotificationArn, connectionEvents, clientToken, callback) {
    const meth = 'adapter-createVpcEndpointConnectionNotification';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateVpcEndpointConnectionNotification';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (connectionNotificationArn === undefined || connectionNotificationArn === null || connectionNotificationArn === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['connectionNotificationArn'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (connectionEvents === undefined || connectionEvents === null || connectionEvents === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['connectionEvents'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ServiceId: serviceId,
      VpcEndpointId: vpcEndpointId,
      ConnectionNotificationArn: connectionNotificationArn,
      ClientToken: clientToken,
      Action: action,
      Version: version
    };
    if (connectionEvents && connectionEvents.length > 0) {
      processInArray(connectionEvents, queryParamsAvailable, 'ConnectionEvents');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'createVpcEndpointConnectionNotification', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createVpcEndpointConnectionNotification'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple connection notifications for a specified VPC endpoint or VPC endpoint service.
   *
   * @function createVpcEndpointConnectionNotifications
   * @param {array} createVpcEndpointConnectionNotificationsArray - An array of objects(VpcEndpointConnectionNotification parameters). dryRun, serviceId, vpcEndpointId, connectionNotificationArn, connectionEvents, clientToken
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createVpcEndpointConnectionNotifications(createVpcEndpointConnectionNotificationsArray, callback) {
    const meth = 'adapter-createVpcEndpointConnectionNotifications';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (createVpcEndpointConnectionNotificationsArray === undefined || createVpcEndpointConnectionNotificationsArray === null || createVpcEndpointConnectionNotificationsArray === '' || createVpcEndpointConnectionNotificationsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['createVpcEndpointConnectionNotificationsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      createVpcEndpointConnectionNotificationsArray.forEach((item) => {
        const { dryRun, serviceId, vpcEndpointId, connectionNotificationArn, connectionEvents, clientToken } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createVpcEndpointConnectionNotification(dryRun, serviceId, vpcEndpointId, connectionNotificationArn, connectionEvents, clientToken, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a VPC endpoint service configuration to which service consumers (AWS accounts, IAM users, and IAM roles) can connect. Service consumers can create an interface VPC endpoint to connect to your service.   To create an endpoint service configuration, you must first create a Network Load Balancer for your service. For more information, see  VPC Endpoint Services  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function createVpcEndpointServiceConfiguration
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {boolean} acceptanceRequired - Indicate whether requests from service consumers to create an endpoint to your service must be accepted. To accept a request, use  AcceptVpcEndpointConnections .
   * @param {array} networkLoadBalancerArn - The Amazon Resource Names (ARNs) of one or more Network Load Balancers for your service.
   * @param {string} clientToken - Unique, case-sensitive identifier you provide to ensure the idempotency of the request. For more information, see  How to Ensure Idempotency .
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createVpcEndpointServiceConfiguration(dryRun, acceptanceRequired, networkLoadBalancerArn, clientToken, callback) {
    const meth = 'adapter-createVpcEndpointServiceConfiguration';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateVpcEndpointServiceConfiguration';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (networkLoadBalancerArn === undefined || networkLoadBalancerArn === null || networkLoadBalancerArn === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['networkLoadBalancerArn'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AcceptanceRequired: acceptanceRequired,
      ClientToken: clientToken,
      Action: action,
      Version: version
    };
    if (networkLoadBalancerArn && networkLoadBalancerArn.length > 0) {
      processInArray(networkLoadBalancerArn, queryParamsAvailable, 'NetworkLoadBalancerArn');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'createVpcEndpointServiceConfiguration', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createVpcEndpointServiceConfiguration'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple VPC endpoint service configurations to which service consumers (AWS accounts, IAM users, and IAM roles) can connect.
   *
   * @function createVpcEndpointServiceConfigurations
   * @param {array} createVpcEndpointServiceConfigurationsArray - An array of objects(VpcEndpointServiceConfiguration parameters). dryRun, acceptanceRequired, networkLoadBalancerArn, clientToken
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createVpcEndpointServiceConfigurations(createVpcEndpointServiceConfigurationsArray, callback) {
    const meth = 'adapter-createVpcEndpointServiceConfigurations';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (createVpcEndpointServiceConfigurationsArray === undefined || createVpcEndpointServiceConfigurationsArray === null || createVpcEndpointServiceConfigurationsArray === '' || createVpcEndpointServiceConfigurationsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['createVpcEndpointServiceConfigurationsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      createVpcEndpointServiceConfigurationsArray.forEach((item) => {
        const { dryRun, acceptanceRequired, networkLoadBalancerArn, clientToken } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createVpcEndpointServiceConfiguration(dryRun, acceptanceRequired, networkLoadBalancerArn, clientToken, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Requests a VPC peering connection between two VPCs: a requester VPC that you own and an accepter VPC with which to create the connection. The accepter VPC can belong to another AWS account and can be in a different Region to the requester VPC. The requester VPC and accepter VPC cannot have overlapping CIDR blocks.     Limitations and rules apply to a VPC peering connection. For more information, see the  limitations  section in the  VPC Peering Guide .     The owner of the accepter VPC must acc...(description truncated)
   *
   * @function createVpcPeeringConnection
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} peerOwnerId - The AWS account ID of the owner of the accepter VPC.   Default: Your AWS account ID
   * @param {string} peerVpcId - The ID of the VPC with which you are creating the VPC peering connection. You must specify this parameter in the request.
   * @param {string} vpcId - The ID of the requester VPC. You must specify this parameter in the request.
   * @param {string} peerRegion - The Region code for the accepter VPC, if the accepter VPC is located in a Region other than the Region in which you make the request.   Default: The Region in which you ...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createVpcPeeringConnection(dryRun, peerOwnerId, peerVpcId, vpcId, peerRegion, callback) {
    const meth = 'adapter-createVpcPeeringConnection';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateVpcPeeringConnection';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      PeerOwnerId: peerOwnerId,
      PeerVpcId: peerVpcId,
      VpcId: vpcId,
      PeerRegion: peerRegion,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'createVpcPeeringConnection', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createVpcPeeringConnection'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Requests multiple VPC peering connections between two VPCs: a requester VPC that you own and an accepter VPC with which to create the connection.
   *
   * @function createVpcPeeringConnections
   * @param {array} createVpcPeeringConnectionsArray - An array of objects(VpcPeeringConnection parameters). dryRun, peerOwnerId, peerVpcId, vpcId, peerRegion
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createVpcPeeringConnections(createVpcPeeringConnectionsArray, callback) {
    const meth = 'adapter-createVpcPeeringConnections';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (createVpcPeeringConnectionsArray === undefined || createVpcPeeringConnectionsArray === null || createVpcPeeringConnectionsArray === '' || createVpcPeeringConnectionsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['createVpcPeeringConnectionsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      createVpcPeeringConnectionsArray.forEach((item) => {
        const { dryRun, peerOwnerId, peerVpcId, vpcId, peerRegion } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createVpcPeeringConnection(dryRun, peerOwnerId, peerVpcId, vpcId, peerRegion, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a VPN connection between an existing virtual private gateway and a VPN customer gateway. The only supported connection type is  ipsec.1 .   The response includes information that you need to give to your network administrator to configure your customer gateway.     We strongly recommend that you use HTTPS when calling this operation because the response contains sensitive cryptographic information for configuring your customer gateway.     If you decide to shut down your VPN connection ...(description truncated)
   *
   * @function createVpnConnection
   * @param {string} customerGatewayId - The ID of the customer gateway.
   * @param {string} type - The type of VPN connection ( ipsec.1 ).
   * @param {string} vpnGatewayId - The ID of the virtual private gateway. If you specify a virtual private gateway, you cannot specify a transit gateway.
   * @param {string} transitGatewayId - The ID of the transit gateway. If you specify a transit gateway, you cannot specify a virtual private gateway.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {boolean} optionsStaticRoutesOnly - Describes VPN connection options.
 Indicate whether the VPN connection uses static routes only. If you are creating a VPN connection for a device that does not support BG...(description truncated)
   * @param {array} optionsTunnelOptions - Describes VPN connection options.
The tunnel options for the VPN connection.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createVpnConnection(customerGatewayId, type, vpnGatewayId, transitGatewayId, dryRun, optionsStaticRoutesOnly, optionsTunnelOptions, callback) {
    const meth = 'adapter-createVpnConnection';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateVpnConnection';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (customerGatewayId === undefined || customerGatewayId === null || customerGatewayId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['customerGatewayId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (type === undefined || type === null || type === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['type'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      CustomerGatewayId: customerGatewayId,
      Type: type,
      VpnGatewayId: vpnGatewayId,
      TransitGatewayId: transitGatewayId,
      Action: action,
      Version: version
    };
    if (optionsStaticRoutesOnly !== undefined && optionsStaticRoutesOnly !== null) {
      queryParamsAvailable['Options.StaticRoutesOnly'] = optionsStaticRoutesOnly;
    }
    if (optionsTunnelOptions && optionsTunnelOptions.length > 0) {
      processInArray(optionsTunnelOptions, queryParamsAvailable, 'Options.TunnelOptions');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'createVpnConnection', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createVpnConnection'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple VPN connections between an existing virtual private gateway and a VPN customer gateway. The only supported connection type is  ipsec.1 .
   *
   * @function createVpnConnections
   * @param {array} createVpnConnectionsArray - An array of objects(VpnConnection parameters). customerGatewayId, type, vpnGatewayId, transitGatewayId, dryRun, optionsStaticRoutesOnly, optionsTunnelOptions
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createVpnConnections(createVpnConnectionsArray, callback) {
    const meth = 'adapter-createVpnConnections';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (createVpnConnectionsArray === undefined || createVpnConnectionsArray === null || createVpnConnectionsArray === '' || createVpnConnectionsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['createVpnConnectionsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      createVpnConnectionsArray.forEach((item) => {
        const { customerGatewayId, type, vpnGatewayId, transitGatewayId, dryRun, optionsStaticRoutesOnly, optionsTunnelOptions } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createVpnConnection(customerGatewayId, type, vpnGatewayId, transitGatewayId, dryRun, optionsStaticRoutesOnly, optionsTunnelOptions, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a static route associated with a VPN connection between an existing virtual private gateway and a VPN customer gateway. The static route allows traffic to be routed from the virtual private gateway to the VPN customer gateway.   For more information, see  AWS Site-to-Site VPN  in the  AWS Site-to-Site VPN User Guide .
   *
   * @function createVpnConnectionRoute
   * @param {string} destinationCidrBlock - The CIDR block associated with the local subnet of the customer network.
   * @param {string} vpnConnectionId - The ID of the VPN connection.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createVpnConnectionRoute(destinationCidrBlock, vpnConnectionId, callback) {
    const meth = 'adapter-createVpnConnectionRoute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateVpnConnectionRoute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (destinationCidrBlock === undefined || destinationCidrBlock === null || destinationCidrBlock === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['destinationCidrBlock'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (vpnConnectionId === undefined || vpnConnectionId === null || vpnConnectionId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpnConnectionId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DestinationCidrBlock: destinationCidrBlock,
      VpnConnectionId: vpnConnectionId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'createVpnConnectionRoute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createVpnConnectionRoute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple static routes associated with a VPN connection between an existing virtual private gateway and a VPN customer gateway. The static route allows traffic to be routed from the virtual private gateway to the VPN customer gateway.
   *
   * @function createVpnConnectionRoutes
   * @param {array} createVpnConnectionRoutesArray - An array of objects(VpnConnectionRoute parameters). destinationCidrBlock, vpnConnectionId
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createVpnConnectionRoutes(createVpnConnectionRoutesArray, callback) {
    const meth = 'adapter-createVpnConnectionRoutes';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (createVpnConnectionRoutesArray === undefined || createVpnConnectionRoutesArray === null || createVpnConnectionRoutesArray === '' || createVpnConnectionRoutesArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['createVpnConnectionRoutesArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      createVpnConnectionRoutesArray.forEach((item) => {
        const { destinationCidrBlock, vpnConnectionId } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createVpnConnectionRoute(destinationCidrBlock, vpnConnectionId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a virtual private gateway. A virtual private gateway is the endpoint on the VPC side of your VPN connection. You can create a virtual private gateway before creating the VPC itself.   For more information, see  AWS Site-to-Site VPN  in the  AWS Site-to-Site VPN User Guide .
   *
   * @function createVpnGateway
   * @param {string} availabilityZone - The Availability Zone for the virtual private gateway.
   * @param {string} type - The type of VPN connection this virtual private gateway supports.
   * @param {number} amazonSideAsn - A private Autonomous System Number (ASN) for the Amazon side of a BGP session. If you're using a 16-bit ASN, it must be in the 64512 to 65534 range. If you're using a 32...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  createVpnGateway(availabilityZone, type = 'ipsec.1', amazonSideAsn, dryRun, callback) {
    const meth = 'adapter-createVpnGateway';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'CreateVpnGateway';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (type === undefined || type === null || type === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['type'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AvailabilityZone: availabilityZone,
      Type: type,
      AmazonSideAsn: amazonSideAsn,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'createVpnGateway', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['createVpnGateway'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates multiple virtual private gateways.
   *
   * @function createVpnGateways
   * @param {array} createVpnGatewaysArray - An array of objects(VpnGateway parameters). availabilityZone, type, amazonSideAsn, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  createVpnGateways(createVpnGatewaysArray, callback) {
    const meth = 'adapter-createVpnGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (createVpnGatewaysArray === undefined || createVpnGatewaysArray === null || createVpnGatewaysArray === '' || createVpnGatewaysArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['createVpnGatewaysArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Create each action individually.
      createVpnGatewaysArray.forEach((item) => {
        const { availabilityZone, type, amazonSideAsn, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.createVpnGateway(availabilityZone, type, amazonSideAsn, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified Client VPN endpoint. You must disassociate all target networks before you can delete a Client VPN endpoint.
   *
   * @function deleteClientVpnEndpoint
   * @param {string} clientVpnEndpointId - The ID of the Client VPN to be deleted.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteClientVpnEndpoint(clientVpnEndpointId, dryRun, callback) {
    const meth = 'adapter-deleteClientVpnEndpoint';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteClientVpnEndpoint';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointId === undefined || clientVpnEndpointId === null || clientVpnEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientVpnEndpointId: clientVpnEndpointId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'deleteClientVpnEndpoint', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteClientVpnEndpoint'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes a route from a Client VPN endpoint. You can only delete routes that you manually added using the  CreateClientVpnRoute  action. You cannot delete routes that were automatically added when associating a subnet. To remove routes that have been automatically added, disassociate the target subnet from the Client VPN endpoint.
   *
   * @function deleteClientVpnRoute
   * @param {string} clientVpnEndpointId - The ID of the Client VPN endpoint from which the route is to be deleted.
   * @param {string} targetVpcSubnetId - The ID of the target subnet used by the route.
   * @param {string} destinationCidrBlock - The IPv4 address range, in CIDR notation, of the route to be deleted.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteClientVpnRoute(clientVpnEndpointId, targetVpcSubnetId, destinationCidrBlock, dryRun, callback) {
    const meth = 'adapter-deleteClientVpnRoute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteClientVpnRoute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointId === undefined || clientVpnEndpointId === null || clientVpnEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (destinationCidrBlock === undefined || destinationCidrBlock === null || destinationCidrBlock === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['destinationCidrBlock'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientVpnEndpointId: clientVpnEndpointId,
      TargetVpcSubnetId: targetVpcSubnetId,
      DestinationCidrBlock: destinationCidrBlock,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'deleteClientVpnRoute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteClientVpnRoute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified customer gateway. You must delete the VPN connection before you can delete the customer gateway.
   *
   * @function deleteCustomerGateway
   * @param {string} customerGatewayId - The ID of the customer gateway.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteCustomerGateway(customerGatewayId, dryRun, callback) {
    const meth = 'adapter-deleteCustomerGateway';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteCustomerGateway';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (customerGatewayId === undefined || customerGatewayId === null || customerGatewayId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['customerGatewayId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      CustomerGatewayId: customerGatewayId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Gateway', 'deleteCustomerGateway', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteCustomerGateway'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified set of DHCP options. You must disassociate the set of DHCP options before you can delete it. You can disassociate the set of DHCP options by associating either a new set of options or the default set of options with the VPC.
   *
   * @function deleteDhcpOptions
   * @param {string} dhcpOptionsId - The ID of the DHCP options set.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteDhcpOptions(dhcpOptionsId, dryRun, callback) {
    const meth = 'adapter-deleteDhcpOptions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteDhcpOptions';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (dhcpOptionsId === undefined || dhcpOptionsId === null || dhcpOptionsId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['dhcpOptionsId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      DhcpOptionsId: dhcpOptionsId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('DHCP', 'deleteDhcpOptions', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteDhcpOptions'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes an egress-only internet gateway.
   *
   * @function deleteEgressOnlyInternetGateway
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} egressOnlyInternetGatewayId - The ID of the egress-only internet gateway.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteEgressOnlyInternetGateway(dryRun, egressOnlyInternetGatewayId, callback) {
    const meth = 'adapter-deleteEgressOnlyInternetGateway';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteEgressOnlyInternetGateway';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (egressOnlyInternetGatewayId === undefined || egressOnlyInternetGatewayId === null || egressOnlyInternetGatewayId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['egressOnlyInternetGatewayId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      EgressOnlyInternetGatewayId: egressOnlyInternetGatewayId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Gateway', 'deleteEgressOnlyInternetGateway', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteEgressOnlyInternetGateway'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified EC2 Fleet.   After you delete an EC2 Fleet, it launches no new instances. You must specify whether an EC2 Fleet should also terminate its instances. If you terminate the instances, the EC2 Fleet enters the  deleted_terminating  state. Otherwise, the EC2 Fleet enters the  deleted_running  state, and the instances continue to run until they are interrupted or you terminate them manually.
   *
   * @function deleteFleets
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} fleetId - The IDs of the EC2 Fleets.
   * @param {boolean} terminateInstances - Indicates whether to terminate instances for an EC2 Fleet if it is deleted successfully.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteFleets(dryRun, fleetId, terminateInstances, callback) {
    const meth = 'adapter-deleteFleets';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteFleets';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (fleetId === undefined || fleetId === null || fleetId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['fleetId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (terminateInstances === undefined || terminateInstances === null || terminateInstances === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['terminateInstances'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      TerminateInstances: terminateInstances,
      Action: action,
      Version: version
    };
    if (fleetId && fleetId.length > 0) {
      processInArray(fleetId, queryParamsAvailable, 'FleetId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Fleet', 'deleteFleets', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteFleets'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes one or more flow logs.
   *
   * @function deleteFlowLogs
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} flowLogId - One or more flow log IDs.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteFlowLogs(dryRun, flowLogId, callback) {
    const meth = 'adapter-deleteFlowLogs';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteFlowLogs';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (flowLogId === undefined || flowLogId === null || flowLogId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['flowLogId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (flowLogId && flowLogId.length > 0) {
      processInArray(flowLogId, queryParamsAvailable, 'FlowLogId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionDeleteFlowLogs', 'getDeleteFlowLogs', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteFlowLogs'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified Amazon FPGA Image (AFI).
   *
   * @function deleteFpgaImage
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} fpgaImageId - The ID of the AFI.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteFpgaImage(dryRun, fpgaImageId, callback) {
    const meth = 'adapter-deleteFpgaImage';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteFpgaImage';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (fpgaImageId === undefined || fpgaImageId === null || fpgaImageId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['fpgaImageId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      FpgaImageId: fpgaImageId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Image', 'deleteFpgaImage', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteFpgaImage'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified internet gateway. You must detach the internet gateway from the VPC before you can delete it.
   *
   * @function deleteInternetGateway
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} internetGatewayId - The ID of the internet gateway.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteInternetGateway(dryRun, internetGatewayId, callback) {
    const meth = 'adapter-deleteInternetGateway';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteInternetGateway';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (internetGatewayId === undefined || internetGatewayId === null || internetGatewayId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['internetGatewayId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      InternetGatewayId: internetGatewayId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Gateway', 'deleteInternetGateway', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteInternetGateway'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple internet gateways. You must detach the internet gateway from the VPC before you can delete it.
   *
   * @function deleteInternetGateways
   * @param {array} internetGatewayArray - An array of objects. dryRun, internetGatewayId.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteInternetGateways(internetGatewayArray, callback) {
    const meth = 'adapter-deleteInternetGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (internetGatewayArray === undefined || internetGatewayArray === null || internetGatewayArray === '' || internetGatewayArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['internetGatewayArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // delete each internet gateway individually.
      internetGatewayArray.forEach((internetGateway) => {
        const { dryRun, internetGatewayId } = internetGateway;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteInternetGateway(dryRun, internetGatewayId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified key pair, by removing the public key from Amazon EC2.
   *
   * @function deleteKeyPair
   * @param {string} keyName - The name of the key pair.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteKeyPair(keyName, dryRun, callback) {
    const meth = 'adapter-deleteKeyPair';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteKeyPair';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (keyName === undefined || keyName === null || keyName === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['keyName'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      KeyName: keyName,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('KeyPair', 'deleteKeyPair', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteKeyPair'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes a launch template. Deleting a launch template deletes all of its versions.
   *
   * @function deleteLaunchTemplate
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} launchTemplateId - The ID of the launch template. You must specify either the launch template ID or launch template name in the request.
   * @param {string} launchTemplateName - The name of the launch template. You must specify either the launch template ID or launch template name in the request.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteLaunchTemplate(dryRun, launchTemplateId, launchTemplateName, callback) {
    const meth = 'adapter-deleteLaunchTemplate';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteLaunchTemplate';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      LaunchTemplateId: launchTemplateId,
      LaunchTemplateName: launchTemplateName,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('LaunchTemplate', 'deleteLaunchTemplate', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteLaunchTemplate'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes one or more versions of a launch template. You cannot delete the default version of a launch template; you must first assign a different version as the default. If the default version is the only version for the launch template, you must delete the entire launch template using  DeleteLaunchTemplate .
   *
   * @function deleteLaunchTemplateVersions
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} launchTemplateId - The ID of the launch template. You must specify either the launch template ID or launch template name in the request.
   * @param {string} launchTemplateName - The name of the launch template. You must specify either the launch template ID or launch template name in the request.
   * @param {array} launchTemplateVersion - The version numbers of one or more launch template versions to delete.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteLaunchTemplateVersions(dryRun, launchTemplateId, launchTemplateName, launchTemplateVersion, callback) {
    const meth = 'adapter-deleteLaunchTemplateVersions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteLaunchTemplateVersions';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (launchTemplateVersion === undefined || launchTemplateVersion === null || launchTemplateVersion === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['launchTemplateVersion'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      LaunchTemplateId: launchTemplateId,
      LaunchTemplateName: launchTemplateName,
      Action: action,
      Version: version
    };
    if (launchTemplateVersion && launchTemplateVersion.length > 0) {
      processInArray(launchTemplateVersion, queryParamsAvailable, 'LaunchTemplateVersion');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('LaunchTemplate', 'deleteLaunchTemplateVersions', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteLaunchTemplateVersions'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified NAT gateway. Deleting a NAT gateway disassociates its Elastic IP address, but does not release the address from your account. Deleting a NAT gateway does not delete any NAT gateway routes in your route tables.
   *
   * @function deleteNatGateway
   * @param {string} natGatewayId - The ID of the NAT gateway.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteNatGateway(natGatewayId, callback) {
    const meth = 'adapter-deleteNatGateway';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteNatGateway';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (natGatewayId === undefined || natGatewayId === null || natGatewayId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['natGatewayId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      NatGatewayId: natGatewayId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Gateway', 'deleteNatGateway', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteNatGateway'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified network ACL. You can't delete the ACL if it's associated with any subnets. You can't delete the default network ACL.
   *
   * @function deleteNetworkAcl
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} networkAclId - The ID of the network ACL.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteNetworkAcl(dryRun, networkAclId, callback) {
    const meth = 'adapter-deleteNetworkAcl';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteNetworkAcl';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (networkAclId === undefined || networkAclId === null || networkAclId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['networkAclId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NetworkAclId: networkAclId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkAcl', 'deleteNetworkAcl', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteNetworkAcl'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified ingress or egress entry (rule) from the specified network ACL.
   *
   * @function deleteNetworkAclEntry
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {boolean} egress - Indicates whether the rule is an egress rule.
   * @param {string} networkAclId - The ID of the network ACL.
   * @param {number} ruleNumber - The rule number of the entry to delete.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteNetworkAclEntry(dryRun, egress, networkAclId, ruleNumber, callback) {
    const meth = 'adapter-deleteNetworkAclEntry';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteNetworkAclEntry';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (egress === undefined || egress === null || egress === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['egress'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (networkAclId === undefined || networkAclId === null || networkAclId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['networkAclId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (ruleNumber === undefined || ruleNumber === null || ruleNumber === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['ruleNumber'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Egress: egress,
      NetworkAclId: networkAclId,
      RuleNumber: ruleNumber,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkAcl', 'deleteNetworkAclEntry', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteNetworkAclEntry'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified network interface. You must detach the network interface before you can delete it.
   *
   * @function deleteNetworkInterface
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} networkInterfaceId - The ID of the network interface.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteNetworkInterface(dryRun, networkInterfaceId, callback) {
    const meth = 'adapter-deleteNetworkInterface';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteNetworkInterface';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (networkInterfaceId === undefined || networkInterfaceId === null || networkInterfaceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['networkInterfaceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NetworkInterfaceId: networkInterfaceId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkInterface', 'deleteNetworkInterface', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteNetworkInterface'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes a permission for a network interface. By default, you cannot delete the permission if the account for which you're removing the permission has attached the network interface to an instance. However, you can force delete the permission, regardless of any attachment.
   *
   * @function deleteNetworkInterfacePermission
   * @param {string} networkInterfacePermissionId - The ID of the network interface permission.
   * @param {boolean} force - Specify  true  to remove the permission even if the network interface is attached to an instance.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteNetworkInterfacePermission(networkInterfacePermissionId, force, dryRun, callback) {
    const meth = 'adapter-deleteNetworkInterfacePermission';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteNetworkInterfacePermission';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (networkInterfacePermissionId === undefined || networkInterfacePermissionId === null || networkInterfacePermissionId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['networkInterfacePermissionId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NetworkInterfacePermissionId: networkInterfacePermissionId,
      Force: force,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkInterface', 'deleteNetworkInterfacePermission', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteNetworkInterfacePermission'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified placement group. You must terminate all instances in the placement group before you can delete the placement group. For more information, see  Placement Groups  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function deletePlacementGroup
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} groupName - The name of the placement group.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deletePlacementGroup(dryRun, groupName, callback) {
    const meth = 'adapter-deletePlacementGroup';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeletePlacementGroup';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (groupName === undefined || groupName === null || groupName === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['groupName'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      GroupName: groupName,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionDeletePlacementGroup', 'getDeletePlacementGroup', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deletePlacementGroup'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified route from the specified route table.
   *
   * @function deleteRoute
   * @param {string} destinationCidrBlock - The IPv4 CIDR range for the route. The value you specify must match the CIDR for the route exactly.
   * @param {string} destinationIpv6CidrBlock - The IPv6 CIDR range for the route. The value you specify must match the CIDR for the route exactly.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} routeTableId - The ID of the route table.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteRoute(destinationCidrBlock, destinationIpv6CidrBlock, dryRun, routeTableId, callback) {
    const meth = 'adapter-deleteRoute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteRoute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (routeTableId === undefined || routeTableId === null || routeTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['routeTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      DestinationCidrBlock: destinationCidrBlock,
      DestinationIpv6CidrBlock: destinationIpv6CidrBlock,
      RouteTableId: routeTableId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Route', 'deleteRoute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteRoute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple routes from the specified route table.
   *
   * @function deleteRoutes
   * @param {array} routeArray - An array of objects. destinationCidrBlock, destinationIpv6CidrBlock, dryRun, routeTableId.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteRoutes(routeArray, callback) {
    const meth = 'adapter-deleteRoutes';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (routeArray === undefined || routeArray === null || routeArray === '' || routeArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['routeArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // delete each route individually.
      routeArray.forEach((route) => {
        const { destinationCidrBlock, destinationIpv6CidrBlock, dryRun, routeTableId } = route;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteRoute(destinationCidrBlock, destinationIpv6CidrBlock, dryRun, routeTableId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified route table. You must disassociate the route table from any subnets before you can delete it. You can't delete the main route table.
   *
   * @function deleteRouteTable
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} routeTableId - The ID of the route table.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteRouteTable(dryRun, routeTableId, callback) {
    const meth = 'adapter-deleteRouteTable';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteRouteTable';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (routeTableId === undefined || routeTableId === null || routeTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['routeTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      RouteTableId: routeTableId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Route', 'deleteRouteTable', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteRouteTable'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple route tables. You must disassociate the route table from any subnets before you can delete it. You can't delete the main route table.
   *
   * @function deleteRouteTables
   * @param {array} routeTableArray - An array of objects. dryRun, routeTableId.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteRouteTables(routeTableArray, callback) {
    const meth = 'adapter-deleteRouteTables';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (routeTableArray === undefined || routeTableArray === null || routeTableArray === '' || routeTableArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['routeTableArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // delete each route table individually.
      routeTableArray.forEach((routeTable) => {
        const { dryRun, routeTableId } = routeTable;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteRouteTable(dryRun, routeTableId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes a security group. If you attempt to delete a security group that is associated with an instance, or is referenced by another security group, the operation fails with  InvalidGroup.InUse  in EC2-Classic or  DependencyViolation  in EC2-VPC.
   *
   * @function deleteSecurityGroup
   * @param {string} groupId - The ID of the security group. Required for a nondefault VPC.
   * @param {string} groupName - [EC2-Classic, default VPC] The name of the security group. You can specify either the security group name or the security group ID.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteSecurityGroup(groupId, groupName, dryRun, callback) {
    const meth = 'adapter-deleteSecurityGroup';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteSecurityGroup';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      GroupId: groupId,
      GroupName: groupName,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('SecurityGroup', 'deleteSecurityGroup', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteSecurityGroup'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple security groups. If you attempt to delete a security group that is associated with an instance, or is referenced by another security group, the operation fails with  InvalidGroup.InUse  in EC2-Classic or  DependencyViolation  in EC2-VPC.
   *
   * @function deleteSecurityGroups
   * @param {array} securityGroupArray - An array of objects. groupId, groupName, dryRun.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteSecurityGroups(securityGroupArray, callback) {
    const meth = 'adapter-deleteSecurityGroups';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (securityGroupArray === undefined || securityGroupArray === null || securityGroupArray === '' || securityGroupArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['securityGroupArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // delete each security group individually.
      securityGroupArray.forEach((securityGroup) => {
        const { groupId, groupName, dryRun } = securityGroup;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteSecurityGroup(groupId, groupName, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified snapshot.   When you make periodic snapshots of a volume, the snapshots are incremental, and only the blocks on the device that have changed since your last snapshot are saved in the new snapshot. When you delete a snapshot, only the data not needed for any other snapshot is removed. So regardless of which prior snapshots have been deleted, all active snapshots will have access to all the information needed to restore the volume.   You cannot delete a snapshot of the root ...(description truncated)
   *
   * @function deleteSnapshot
   * @param {string} snapshotId - The ID of the EBS snapshot.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteSnapshot(snapshotId, dryRun, callback) {
    const meth = 'adapter-deleteSnapshot';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteSnapshot';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (snapshotId === undefined || snapshotId === null || snapshotId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['snapshotId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      SnapshotId: snapshotId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Snapshot', 'deleteSnapshot', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteSnapshot'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the data feed for Spot Instances.
   *
   * @function deleteSpotDatafeedSubscription
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteSpotDatafeedSubscription(dryRun, callback) {
    const meth = 'adapter-deleteSpotDatafeedSubscription';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteSpotDatafeedSubscription';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionDeleteSpotDatafeedSubscription', 'getDeleteSpotDatafeedSubscription', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteSpotDatafeedSubscription'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified subnet. You must terminate all running instances in the subnet before you can delete the subnet.
   *
   * @function deleteSubnet
   * @param {string} subnetId - The ID of the subnet.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteSubnet(subnetId, dryRun, callback) {
    const meth = 'adapter-deleteSubnet';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteSubnet';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (subnetId === undefined || subnetId === null || subnetId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['subnetId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      SubnetId: subnetId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Subnet', 'deleteSubnet', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteSubnet'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple subnets. You must terminate all running instances in the subnet before you can delete the subnet.
   *
   * @function deleteSubnets
   * @param {array} subnetArray - An array of objects. subnetId, dryRun.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteSubnets(subnetArray, callback) {
    const meth = 'adapter-deleteSubnets';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (subnetArray === undefined || subnetArray === null || subnetArray === '' || subnetArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['subnetArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // delete each subnet individually.
      subnetArray.forEach((subnet) => {
        const { subnetId, dryRun } = subnet;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteSubnet(subnetId, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified set of tags from the specified set of resources.   To list the current tags, use  DescribeTags . For more information about tags, see  Tagging Your Resources  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function deleteTags
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} resourceId - The IDs of the resources, separated by spaces.   Constraints: Up to 1000 resource IDs. We recommend breaking up this request into smaller batches.
   * @param {array} tag - The tags to delete. Specify a tag key and an optional tag value to delete specific tags. If you specify a tag key without a tag value, we delete any tag with this key re...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteTags(dryRun, resourceId, tag, callback) {
    const meth = 'adapter-deleteTags';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteTags';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (resourceId === undefined || resourceId === null || resourceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['resourceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (resourceId && resourceId.length > 0) {
      processInArray(resourceId, queryParamsAvailable, 'ResourceId');
    }
    if (tag && tag.length > 0) {
      processInArray(tag, queryParamsAvailable, 'Tag');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionDeleteTags', 'getDeleteTags', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteTags'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified transit gateway.
   *
   * @function deleteTransitGateway
   * @param {string} transitGatewayId - The ID of the transit gateway.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteTransitGateway(transitGatewayId, dryRun, callback) {
    const meth = 'adapter-deleteTransitGateway';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteTransitGateway';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayId === undefined || transitGatewayId === null || transitGatewayId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      TransitGatewayId: transitGatewayId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'deleteTransitGateway', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteTransitGateway'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified route from the specified transit gateway route table.
   *
   * @function deleteTransitGatewayRoute
   * @param {string} transitGatewayRouteTableId - The ID of the transit gateway route table.
   * @param {string} destinationCidrBlock - The CIDR range for the route. This must match the CIDR for the route exactly.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteTransitGatewayRoute(transitGatewayRouteTableId, destinationCidrBlock, dryRun, callback) {
    const meth = 'adapter-deleteTransitGatewayRoute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteTransitGatewayRoute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayRouteTableId === undefined || transitGatewayRouteTableId === null || transitGatewayRouteTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayRouteTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (destinationCidrBlock === undefined || destinationCidrBlock === null || destinationCidrBlock === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['destinationCidrBlock'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      TransitGatewayRouteTableId: transitGatewayRouteTableId,
      DestinationCidrBlock: destinationCidrBlock,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'deleteTransitGatewayRoute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteTransitGatewayRoute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified transit gateway route table. You must disassociate the route table from any transit gateway route tables before you can delete it.
   *
   * @function deleteTransitGatewayRouteTable
   * @param {string} transitGatewayRouteTableId - The ID of the transit gateway route table.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteTransitGatewayRouteTable(transitGatewayRouteTableId, dryRun, callback) {
    const meth = 'adapter-deleteTransitGatewayRouteTable';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteTransitGatewayRouteTable';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayRouteTableId === undefined || transitGatewayRouteTableId === null || transitGatewayRouteTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayRouteTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      TransitGatewayRouteTableId: transitGatewayRouteTableId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'deleteTransitGatewayRouteTable', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteTransitGatewayRouteTable'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified VPC attachment.
   *
   * @function deleteTransitGatewayVpcAttachment
   * @param {string} transitGatewayAttachmentId - The ID of the attachment.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteTransitGatewayVpcAttachment(transitGatewayAttachmentId, dryRun, callback) {
    const meth = 'adapter-deleteTransitGatewayVpcAttachment';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteTransitGatewayVpcAttachment';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayAttachmentId === undefined || transitGatewayAttachmentId === null || transitGatewayAttachmentId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayAttachmentId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      TransitGatewayAttachmentId: transitGatewayAttachmentId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'deleteTransitGatewayVpcAttachment', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteTransitGatewayVpcAttachment'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified EBS volume. The volume must be in the  available  state (not attached to an instance).   The volume can remain in the  deleting  state for several minutes.   For more information, see  Deleting an Amazon EBS Volume  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function deleteVolume
   * @param {string} volumeId - The ID of the volume.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteVolume(volumeId, dryRun, callback) {
    const meth = 'adapter-deleteVolume';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteVolume';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (volumeId === undefined || volumeId === null || volumeId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['volumeId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VolumeId: volumeId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Volume', 'deleteVolume', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteVolume'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified VPC. You must detach or delete all gateways and resources that are associated with the VPC before you can delete it. For example, you must terminate all instances running in the VPC, delete all security groups associated with the VPC (except the default one), delete all route tables associated with the VPC (except the default one), and so on.
   *
   * @function deleteVpc
   * @param {string} vpcId - The ID of the VPC.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteVpc(vpcId, dryRun, callback) {
    const meth = 'adapter-deleteVpc';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteVpc';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'deleteVpc', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteVpc'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple VPCs. You must detach or delete all gateways and resources that are associated with the VPC before you can delete it. For example, you must terminate all instances running in the VPC, delete all security groups associated with the VPC (except the default one), delete all route tables associated with the VPC (except the default one), and so on.
   *
   * @function deleteVpcs
   * @param {array} vpcArray - An array of objects. vpcId, dryRun.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteVpcs(vpcArray, callback) {
    const meth = 'adapter-deleteVpcs';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcArray === undefined || vpcArray === null || vpcArray === '' || vpcArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // delete each vpc individually.
      vpcArray.forEach((vpc) => {
        const { vpcId, dryRun } = vpc;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteVpc(vpcId, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes one or more VPC endpoint connection notifications.
   *
   * @function deleteVpcEndpointConnectionNotifications
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} connectionNotificationId - One or more notification IDs.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteVpcEndpointConnectionNotifications(dryRun, connectionNotificationId, callback) {
    const meth = 'adapter-deleteVpcEndpointConnectionNotifications';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteVpcEndpointConnectionNotifications';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (connectionNotificationId === undefined || connectionNotificationId === null || connectionNotificationId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['connectionNotificationId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (connectionNotificationId && connectionNotificationId.length > 0) {
      processInArray(connectionNotificationId, queryParamsAvailable, 'ConnectionNotificationId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'deleteVpcEndpointConnectionNotifications', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteVpcEndpointConnectionNotifications'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes one or more VPC endpoint service configurations in your account. Before you delete the endpoint service configuration, you must reject any  Available  or  PendingAcceptance  interface endpoint connections that are attached to the service.
   *
   * @function deleteVpcEndpointServiceConfigurations
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} serviceId - The IDs of one or more services.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteVpcEndpointServiceConfigurations(dryRun, serviceId, callback) {
    const meth = 'adapter-deleteVpcEndpointServiceConfigurations';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteVpcEndpointServiceConfigurations';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (serviceId === undefined || serviceId === null || serviceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['serviceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (serviceId && serviceId.length > 0) {
      processInArray(serviceId, queryParamsAvailable, 'ServiceId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'deleteVpcEndpointServiceConfigurations', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteVpcEndpointServiceConfigurations'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes one or more specified VPC endpoints. Deleting a gateway endpoint also deletes the endpoint routes in the route tables that were associated with the endpoint. Deleting an interface endpoint deletes the endpoint network interfaces.
   *
   * @function deleteVpcEndpoints
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} vpcEndpointId - One or more VPC endpoint IDs.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteVpcEndpoints(dryRun, vpcEndpointId, callback) {
    const meth = 'adapter-deleteVpcEndpoints';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteVpcEndpoints';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcEndpointId === undefined || vpcEndpointId === null || vpcEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (vpcEndpointId && vpcEndpointId.length > 0) {
      processInArray(vpcEndpointId, queryParamsAvailable, 'VpcEndpointId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'deleteVpcEndpoints', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteVpcEndpoints'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes a VPC peering connection. Either the owner of the requester VPC or the owner of the accepter VPC can delete the VPC peering connection if it's in the  active  state. The owner of the requester VPC can delete a VPC peering connection in the  pending-acceptance  state. You cannot delete a VPC peering connection that's in the  failed  state.
   *
   * @function deleteVpcPeeringConnection
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} vpcPeeringConnectionId - The ID of the VPC peering connection.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteVpcPeeringConnection(dryRun, vpcPeeringConnectionId, callback) {
    const meth = 'adapter-deleteVpcPeeringConnection';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteVpcPeeringConnection';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcPeeringConnectionId === undefined || vpcPeeringConnectionId === null || vpcPeeringConnectionId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcPeeringConnectionId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VpcPeeringConnectionId: vpcPeeringConnectionId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'deleteVpcPeeringConnection', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteVpcPeeringConnection'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified VPN connection.   If you're deleting the VPC and its associated components, we recommend that you detach the virtual private gateway from the VPC and delete the VPC before deleting the VPN connection. If you believe that the tunnel credentials for your VPN connection have been compromised, you can delete the VPN connection and create a new one that has new keys, without needing to delete the VPC or virtual private gateway. If you create a new VPN connection, you must recon...(description truncated)
   *
   * @function deleteVpnConnection
   * @param {string} vpnConnectionId - The ID of the VPN connection.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteVpnConnection(vpnConnectionId, dryRun, callback) {
    const meth = 'adapter-deleteVpnConnection';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteVpnConnection';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpnConnectionId === undefined || vpnConnectionId === null || vpnConnectionId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpnConnectionId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VpnConnectionId: vpnConnectionId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'deleteVpnConnection', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteVpnConnection'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified static route associated with a VPN connection between an existing virtual private gateway and a VPN customer gateway. The static route allows traffic to be routed from the virtual private gateway to the VPN customer gateway.
   *
   * @function deleteVpnConnectionRoute
   * @param {string} destinationCidrBlock - The CIDR block associated with the local subnet of the customer network.
   * @param {string} vpnConnectionId - The ID of the VPN connection.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteVpnConnectionRoute(destinationCidrBlock, vpnConnectionId, callback) {
    const meth = 'adapter-deleteVpnConnectionRoute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteVpnConnectionRoute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (destinationCidrBlock === undefined || destinationCidrBlock === null || destinationCidrBlock === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['destinationCidrBlock'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (vpnConnectionId === undefined || vpnConnectionId === null || vpnConnectionId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpnConnectionId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DestinationCidrBlock: destinationCidrBlock,
      VpnConnectionId: vpnConnectionId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'deleteVpnConnectionRoute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteVpnConnectionRoute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes the specified virtual private gateway. We recommend that before you delete a virtual private gateway, you detach it from the VPC and delete the VPN connection. Note that you don't need to delete the virtual private gateway if you plan to delete and recreate the VPN connection between your VPC and your network.
   *
   * @function deleteVpnGateway
   * @param {string} vpnGatewayId - The ID of the virtual private gateway.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deleteVpnGateway(vpnGatewayId, dryRun, callback) {
    const meth = 'adapter-deleteVpnGateway';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeleteVpnGateway';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpnGatewayId === undefined || vpnGatewayId === null || vpnGatewayId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpnGatewayId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VpnGatewayId: vpnGatewayId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'deleteVpnGateway', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deleteVpnGateway'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Delete multiple Client Vpn Endpoints.
   *
   * @function deleteClientVpnEndpoints
   * @param {array} ClientVpnEndpointsArray - An array of objects(ClientVpnEndpoint parameters). clientVpnEndpointId, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteClientVpnEndpoints(ClientVpnEndpointsArray, callback) {
    const meth = 'adapter-deleteClientVpnEndpoints';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (ClientVpnEndpointsArray === undefined || ClientVpnEndpointsArray === null || ClientVpnEndpointsArray === '' || ClientVpnEndpointsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['ClientVpnEndpointsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      ClientVpnEndpointsArray.forEach((item) => {
        const { clientVpnEndpointId, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteClientVpnEndpoint(clientVpnEndpointId, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Delete multiple routes from a Client VPN endpoint.
   *
   * @function deleteClientVpnRoutes
   * @param {array} ClientVpnRoutesArray - An array of objects(ClientVpnRoute parameters). clientVpnEndpointId, targetVpcSubnetId, destinationCidrBlock, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteClientVpnRoutes(ClientVpnRoutesArray, callback) {
    const meth = 'adapter-deleteClientVpnRoutes';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (ClientVpnRoutesArray === undefined || ClientVpnRoutesArray === null || ClientVpnRoutesArray === '' || ClientVpnRoutesArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['ClientVpnRoutesArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      ClientVpnRoutesArray.forEach((item) => {
        const { clientVpnEndpointId, targetVpcSubnetId, destinationCidrBlock, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteClientVpnRoute(clientVpnEndpointId, targetVpcSubnetId, destinationCidrBlock, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Customer Gateways.
   *
   * @function deleteCustomerGateways
   * @param {array} CustomerGatewaysArray - An array of objects(CustomerGateway parameters). customerGatewayId, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteCustomerGateways(CustomerGatewaysArray, callback) {
    const meth = 'adapter-deleteCustomerGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (CustomerGatewaysArray === undefined || CustomerGatewaysArray === null || CustomerGatewaysArray === '' || CustomerGatewaysArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['CustomerGatewaysArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      CustomerGatewaysArray.forEach((item) => {
        const { customerGatewayId, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteCustomerGateway(customerGatewayId, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Egress Only Internet Gateways.
   *
   * @function deleteEgressOnlyInternetGateways
   * @param {array} EgressOnlyInternetGatewaysArray - An array of objects(EgressOnlyInternetGateway parameters). dryRun, egressOnlyInternetGatewayId
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteEgressOnlyInternetGateways(EgressOnlyInternetGatewaysArray, callback) {
    const meth = 'adapter-deleteEgressOnlyInternetGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (EgressOnlyInternetGatewaysArray === undefined || EgressOnlyInternetGatewaysArray === null || EgressOnlyInternetGatewaysArray === '' || EgressOnlyInternetGatewaysArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['EgressOnlyInternetGatewaysArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      EgressOnlyInternetGatewaysArray.forEach((item) => {
        const { dryRun, egressOnlyInternetGatewayId } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteEgressOnlyInternetGateway(dryRun, egressOnlyInternetGatewayId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Multiple Fleets.
   *
   * @function deleteMultipleFleets
   * @param {array} MultipleFleetsArray - An array of objects(MultipleFleet parameters). dryRun, fleetId, terminateInstances
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteMultipleFleets(MultipleFleetsArray, callback) {
    const meth = 'adapter-deleteMultipleFleets';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (MultipleFleetsArray === undefined || MultipleFleetsArray === null || MultipleFleetsArray === '' || MultipleFleetsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['MultipleFleetsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      MultipleFleetsArray.forEach((item) => {
        const { dryRun, fleetId, terminateInstances } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteFleets(dryRun, fleetId, terminateInstances, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Dhcp Options.
   *
   * @function deleteMultipleDhcpOptions
   * @param {array} MultipleDhcpOptionsArray - An array of objects(MultipleDhcpOption parameters). dhcpOptionsId, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteMultipleDhcpOptions(MultipleDhcpOptionsArray, callback) {
    const meth = 'adapter-deleteMultipleDhcpOptions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (MultipleDhcpOptionsArray === undefined || MultipleDhcpOptionsArray === null || MultipleDhcpOptionsArray === '' || MultipleDhcpOptionsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['MultipleDhcpOptionsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      MultipleDhcpOptionsArray.forEach((item) => {
        const { dhcpOptionsId, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteDhcpOptions(dhcpOptionsId, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Fpga Images.
   *
   * @function deleteFpgaImages
   * @param {array} FpgaImagesArray - An array of objects(FpgaImage parameters). dryRun, fpgaImageId
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteFpgaImages(FpgaImagesArray, callback) {
    const meth = 'adapter-deleteFpgaImages';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (FpgaImagesArray === undefined || FpgaImagesArray === null || FpgaImagesArray === '' || FpgaImagesArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['FpgaImagesArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      FpgaImagesArray.forEach((item) => {
        const { dryRun, fpgaImageId } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteFpgaImage(dryRun, fpgaImageId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Multiple Flow Logs.
   *
   * @function deleteMultipleFlowLogs
   * @param {array} MultipleFlowLogsArray - An array of objects(MultipleFlowLog parameters). dryRun, flowLogId
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteMultipleFlowLogs(MultipleFlowLogsArray, callback) {
    const meth = 'adapter-deleteMultipleFlowLogs';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (MultipleFlowLogsArray === undefined || MultipleFlowLogsArray === null || MultipleFlowLogsArray === '' || MultipleFlowLogsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['MultipleFlowLogsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      MultipleFlowLogsArray.forEach((item) => {
        const { dryRun, flowLogId } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteFlowLogs(dryRun, flowLogId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Key Pairs.
   *
   * @function deleteKeyPairs
   * @param {array} KeyPairsArray - An array of objects(KeyPair parameters). keyName, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteKeyPairs(KeyPairsArray, callback) {
    const meth = 'adapter-deleteKeyPairs';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (KeyPairsArray === undefined || KeyPairsArray === null || KeyPairsArray === '' || KeyPairsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['KeyPairsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      KeyPairsArray.forEach((item) => {
        const { keyName, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteKeyPair(keyName, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple  Launch Template Versions.
   *
   * @function deleteMultipleLaunchTemplateVersions
   * @param {array} MultipleLaunchTemplateVersionsArray - An array of objects(MultipleLaunchTemplateVersion parameters). dryRun, launchTemplateId, launchTemplateName, launchTemplateVersion
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteMultipleLaunchTemplateVersions(MultipleLaunchTemplateVersionsArray, callback) {
    const meth = 'adapter-deleteMultipleLaunchTemplateVersions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (MultipleLaunchTemplateVersionsArray === undefined || MultipleLaunchTemplateVersionsArray === null || MultipleLaunchTemplateVersionsArray === '' || MultipleLaunchTemplateVersionsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['MultipleLaunchTemplateVersionsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      MultipleLaunchTemplateVersionsArray.forEach((item) => {
        const { dryRun, launchTemplateId, launchTemplateName, launchTemplateVersion } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteLaunchTemplateVersions(dryRun, launchTemplateId, launchTemplateName, launchTemplateVersion, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Network Acl Entries.
   *
   * @function deleteNetworkAclEntries
   * @param {array} NetworkAclEntriesArray - An array of objects(NetworkAclEntrie parameters). dryRun, egress, networkAclId, ruleNumber
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteNetworkAclEntries(NetworkAclEntriesArray, callback) {
    const meth = 'adapter-deleteNetworkAclEntries';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (NetworkAclEntriesArray === undefined || NetworkAclEntriesArray === null || NetworkAclEntriesArray === '' || NetworkAclEntriesArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['NetworkAclEntriesArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      NetworkAclEntriesArray.forEach((item) => {
        const { dryRun, egress, networkAclId, ruleNumber } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteNetworkAclEntry(dryRun, egress, networkAclId, ruleNumber, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Launch Templates.
   *
   * @function deleteLaunchTemplates
   * @param {array} LaunchTemplatesArray - An array of objects(LaunchTemplate parameters). dryRun, launchTemplateId, launchTemplateName
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteLaunchTemplates(LaunchTemplatesArray, callback) {
    const meth = 'adapter-deleteLaunchTemplates';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (LaunchTemplatesArray === undefined || LaunchTemplatesArray === null || LaunchTemplatesArray === '' || LaunchTemplatesArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['LaunchTemplatesArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      LaunchTemplatesArray.forEach((item) => {
        const { dryRun, launchTemplateId, launchTemplateName } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteLaunchTemplate(dryRun, launchTemplateId, launchTemplateName, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Network Interface Permissions.
   *
   * @function deleteNetworkInterfacePermissions
   * @param {array} NetworkInterfacePermissionsArray - An array of objects(NetworkInterfacePermission parameters). networkInterfacePermissionId, force, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteNetworkInterfacePermissions(NetworkInterfacePermissionsArray, callback) {
    const meth = 'adapter-deleteNetworkInterfacePermissions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (NetworkInterfacePermissionsArray === undefined || NetworkInterfacePermissionsArray === null || NetworkInterfacePermissionsArray === '' || NetworkInterfacePermissionsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['NetworkInterfacePermissionsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      NetworkInterfacePermissionsArray.forEach((item) => {
        const { networkInterfacePermissionId, force, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteNetworkInterfacePermission(networkInterfacePermissionId, force, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Placement Groups.
   *
   * @function deletePlacementGroups
   * @param {array} PlacementGroupsArray - An array of objects(PlacementGroup parameters). dryRun, groupName
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deletePlacementGroups(PlacementGroupsArray, callback) {
    const meth = 'adapter-deletePlacementGroups';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (PlacementGroupsArray === undefined || PlacementGroupsArray === null || PlacementGroupsArray === '' || PlacementGroupsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['PlacementGroupsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      PlacementGroupsArray.forEach((item) => {
        const { dryRun, groupName } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deletePlacementGroup(dryRun, groupName, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Snapshots.
   *
   * @function deleteSnapshots
   * @param {array} SnapshotsArray - An array of objects(Snapshot parameters). snapshotId, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteSnapshots(SnapshotsArray, callback) {
    const meth = 'adapter-deleteSnapshots';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (SnapshotsArray === undefined || SnapshotsArray === null || SnapshotsArray === '' || SnapshotsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['SnapshotsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      SnapshotsArray.forEach((item) => {
        const { snapshotId, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteSnapshot(snapshotId, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Spot Datafeed Subscriptions.
   *
   * @function deleteSpotDatafeedSubscriptions
   * @param {array} SpotDatafeedSubscriptionsArray - An array of objects(SpotDatafeedSubscription parameters). dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteSpotDatafeedSubscriptions(SpotDatafeedSubscriptionsArray, callback) {
    const meth = 'adapter-deleteSpotDatafeedSubscriptions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (SpotDatafeedSubscriptionsArray === undefined || SpotDatafeedSubscriptionsArray === null || SpotDatafeedSubscriptionsArray === '' || SpotDatafeedSubscriptionsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['SpotDatafeedSubscriptionsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      SpotDatafeedSubscriptionsArray.forEach((item) => {
        const { dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteSpotDatafeedSubscription(dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Multiple Tags.
   *
   * @function deleteMultipleTags
   * @param {array} MultipleTagsArray - An array of objects(Tags parameters). dryRun, resourceId, tag
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteMultipleTags(MultipleTagsArray, callback) {
    const meth = 'adapter-deleteMultipleTags';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (MultipleTagsArray === undefined || MultipleTagsArray === null || MultipleTagsArray === '' || MultipleTagsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['MultipleTagsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      MultipleTagsArray.forEach((item) => {
        const { dryRun, resourceId, tag } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteTags(dryRun, resourceId, tag, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Transit Gateways.
   *
   * @function deleteTransitGateways
   * @param {array} TransitGatewaysArray - An array of objects(TransitGateway parameters). transitGatewayId, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteTransitGateways(TransitGatewaysArray, callback) {
    const meth = 'adapter-deleteTransitGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (TransitGatewaysArray === undefined || TransitGatewaysArray === null || TransitGatewaysArray === '' || TransitGatewaysArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['TransitGatewaysArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      TransitGatewaysArray.forEach((item) => {
        const { transitGatewayId, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteTransitGateway(transitGatewayId, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Transit Gateway Routes.
   *
   * @function deleteTransitGatewayRoutes
   * @param {array} TransitGatewayRoutesArray - An array of objects(TransitGatewayRoute parameters). transitGatewayRouteTableId, destinationCidrBlock, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteTransitGatewayRoutes(TransitGatewayRoutesArray, callback) {
    const meth = 'adapter-deleteTransitGatewayRoutes';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (TransitGatewayRoutesArray === undefined || TransitGatewayRoutesArray === null || TransitGatewayRoutesArray === '' || TransitGatewayRoutesArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['TransitGatewayRoutesArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      TransitGatewayRoutesArray.forEach((item) => {
        const { transitGatewayRouteTableId, destinationCidrBlock, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteTransitGatewayRoute(transitGatewayRouteTableId, destinationCidrBlock, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Transit Gateway Route Tables.
   *
   * @function deleteTransitGatewayRouteTables
   * @param {array} TransitGatewayRouteTablesArray - An array of objects(TransitGatewayRouteTable parameters). transitGatewayRouteTableId, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteTransitGatewayRouteTables(TransitGatewayRouteTablesArray, callback) {
    const meth = 'adapter-deleteTransitGatewayRouteTables';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (TransitGatewayRouteTablesArray === undefined || TransitGatewayRouteTablesArray === null || TransitGatewayRouteTablesArray === '' || TransitGatewayRouteTablesArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['TransitGatewayRouteTablesArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      TransitGatewayRouteTablesArray.forEach((item) => {
        const { transitGatewayRouteTableId, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteTransitGatewayRouteTable(transitGatewayRouteTableId, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Transit Gateway Vpc Attachments.
   *
   * @function deleteTransitGatewayVpcAttachments
   * @param {array} TransitGatewayVpcAttachmentsArray - An array of objects(TransitGatewayVpcAttachment parameters). transitGatewayAttachmentId, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteTransitGatewayVpcAttachments(TransitGatewayVpcAttachmentsArray, callback) {
    const meth = 'adapter-deleteTransitGatewayVpcAttachments';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (TransitGatewayVpcAttachmentsArray === undefined || TransitGatewayVpcAttachmentsArray === null || TransitGatewayVpcAttachmentsArray === '' || TransitGatewayVpcAttachmentsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['TransitGatewayVpcAttachmentsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      TransitGatewayVpcAttachmentsArray.forEach((item) => {
        const { transitGatewayAttachmentId, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteTransitGatewayVpcAttachment(transitGatewayAttachmentId, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Volumes.
   *
   * @function deleteVolumes
   * @param {array} VolumesArray - An array of objects(Volume parameters). volumeId, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteVolumes(VolumesArray, callback) {
    const meth = 'adapter-deleteVolumes';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (VolumesArray === undefined || VolumesArray === null || VolumesArray === '' || VolumesArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['VolumesArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      VolumesArray.forEach((item) => {
        const { volumeId, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteVolume(volumeId, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Multiple Vpc Endpoint Connection Notifications.
   *
   * @function deleteMultipleVpcEndpointConnectionNotifications
   * @param {array} MultipleVpcEndpointConnectionNotificationsArray - An array of objects(VpcEndpointConnectionNotification parameters). dryRun, connectionNotificationId
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteMultipleVpcEndpointConnectionNotifications(MultipleVpcEndpointConnectionNotificationsArray, callback) {
    const meth = 'adapter-deleteMultipleVpcEndpointConnectionNotifications';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (MultipleVpcEndpointConnectionNotificationsArray === undefined || MultipleVpcEndpointConnectionNotificationsArray === null || MultipleVpcEndpointConnectionNotificationsArray === '' || MultipleVpcEndpointConnectionNotificationsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['MultipleVpcEndpointConnectionNotificationsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      MultipleVpcEndpointConnectionNotificationsArray.forEach((item) => {
        const { dryRun, connectionNotificationId } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteVpcEndpointConnectionNotifications(dryRun, connectionNotificationId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Network Interfaces.
   *
   * @function deleteNetworkInterfaces
   * @param {array} NetworkInterfacesArray - An array of objects(NetworkInterface parameters). dryRun, networkInterfaceId
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteNetworkInterfaces(NetworkInterfacesArray, callback) {
    const meth = 'adapter-deleteNetworkInterfaces';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (NetworkInterfacesArray === undefined || NetworkInterfacesArray === null || NetworkInterfacesArray === '' || NetworkInterfacesArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['NetworkInterfacesArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      NetworkInterfacesArray.forEach((item) => {
        const { dryRun, networkInterfaceId } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteNetworkInterface(dryRun, networkInterfaceId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Network Acls.
   *
   * @function deleteNetworkAcls
   * @param {array} NetworkAclsArray - An array of objects(NetworkAcl parameters). dryRun, networkAclId
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteNetworkAcls(NetworkAclsArray, callback) {
    const meth = 'adapter-deleteNetworkAcls';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (NetworkAclsArray === undefined || NetworkAclsArray === null || NetworkAclsArray === '' || NetworkAclsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['NetworkAclsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      NetworkAclsArray.forEach((item) => {
        const { dryRun, networkAclId } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteNetworkAcl(dryRun, networkAclId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Nat Gateways.
   *
   * @function deleteNatGateways
   * @param {array} NatGatewaysArray - An array of objects(NatGateway parameters). natGatewayId
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteNatGateways(NatGatewaysArray, callback) {
    const meth = 'adapter-deleteNatGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (NatGatewaysArray === undefined || NatGatewaysArray === null || NatGatewaysArray === '' || NatGatewaysArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['NatGatewaysArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      NatGatewaysArray.forEach((item) => {
        const { natGatewayId } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteNatGateway(natGatewayId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Multiple Vpc Endpoint Service Configurations.
   *
   * @function deleteMultipleVpcEndpointServiceConfigurations
   * @param {array} MultipleVpcEndpointServiceConfigurationsArray - An array of objects(MultipleVpcEndpointServiceConfiguration parameters). dryRun, serviceId
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteMultipleVpcEndpointServiceConfigurations(MultipleVpcEndpointServiceConfigurationsArray, callback) {
    const meth = 'adapter-deleteMultipleVpcEndpointServiceConfigurations';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (MultipleVpcEndpointServiceConfigurationsArray === undefined || MultipleVpcEndpointServiceConfigurationsArray === null || MultipleVpcEndpointServiceConfigurationsArray === '' || MultipleVpcEndpointServiceConfigurationsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['MultipleVpcEndpointServiceConfigurationsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      MultipleVpcEndpointServiceConfigurationsArray.forEach((item) => {
        const { dryRun, serviceId } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteVpcEndpointServiceConfigurations(dryRun, serviceId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Multiple Vpc Endpoints.
   *
   * @function deleteMultipleVpcEndpoints
   * @param {array} MultipleVpcEndpointsArray - An array of objects(MultipleVpcEndpoint parameters). dryRun, vpcEndpointId
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteMultipleVpcEndpoints(MultipleVpcEndpointsArray, callback) {
    const meth = 'adapter-deleteMultipleVpcEndpoints';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (MultipleVpcEndpointsArray === undefined || MultipleVpcEndpointsArray === null || MultipleVpcEndpointsArray === '' || MultipleVpcEndpointsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['MultipleVpcEndpointsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      MultipleVpcEndpointsArray.forEach((item) => {
        const { dryRun, vpcEndpointId } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteVpcEndpoints(dryRun, vpcEndpointId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Vpn Connections.
   *
   * @function deleteVpnConnections
   * @param {array} VpnConnectionsArray - An array of objects(VpnConnection parameters). vpnConnectionId, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteVpnConnections(VpnConnectionsArray, callback) {
    const meth = 'adapter-deleteVpnConnections';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (VpnConnectionsArray === undefined || VpnConnectionsArray === null || VpnConnectionsArray === '' || VpnConnectionsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['VpnConnectionsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      VpnConnectionsArray.forEach((item) => {
        const { vpnConnectionId, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteVpnConnection(vpnConnectionId, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Vpn Connection Routes.
   *
   * @function deleteVpnConnectionRoutes
   * @param {array} VpnConnectionRoutesArray - An array of objects(VpnConnectionRoute parameters). destinationCidrBlock, vpnConnectionId
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteVpnConnectionRoutes(VpnConnectionRoutesArray, callback) {
    const meth = 'adapter-deleteVpnConnectionRoutes';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (VpnConnectionRoutesArray === undefined || VpnConnectionRoutesArray === null || VpnConnectionRoutesArray === '' || VpnConnectionRoutesArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['VpnConnectionRoutesArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      VpnConnectionRoutesArray.forEach((item) => {
        const { destinationCidrBlock, vpnConnectionId } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteVpnConnectionRoute(destinationCidrBlock, vpnConnectionId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Vpn Gateways.
   *
   * @function deleteVpnGateways
   * @param {array} VpnGatewaysArray - An array of objects(VpnGateway parameters). vpnGatewayId, dryRun
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteVpnGateways(VpnGatewaysArray, callback) {
    const meth = 'adapter-deleteVpnGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (VpnGatewaysArray === undefined || VpnGatewaysArray === null || VpnGatewaysArray === '' || VpnGatewaysArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['VpnGatewaysArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      VpnGatewaysArray.forEach((item) => {
        const { vpnGatewayId, dryRun } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteVpnGateway(vpnGatewayId, dryRun, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deletes multiple Vpc Peering Connections.
   *
   * @function deleteVpcPeeringConnections
   * @param {array} VpcPeeringConnectionsArray - An array of objects(VpcPeeringConnection parameters). dryRun, vpcPeeringConnectionId
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */

  deleteVpcPeeringConnections(VpcPeeringConnectionsArray, callback) {
    const meth = 'adapter-deleteVpcPeeringConnections';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (VpcPeeringConnectionsArray === undefined || VpcPeeringConnectionsArray === null || VpcPeeringConnectionsArray === '' || VpcPeeringConnectionsArray.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['VpcPeeringConnectionsArray'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const returnArray = [];
    try {
      // Delete each action individually.
      VpcPeeringConnectionsArray.forEach((item) => {
        const { dryRun, vpcPeeringConnectionId } = item;
        returnArray.push(
          new Promise((resolve) => {
            this.deleteVpcPeeringConnection(dryRun, vpcPeeringConnectionId, (irReturnData, irReturnError) => {
              // if we received an error or their is no response on the results
              // return an error
              if (irReturnError) {
                resolve(irReturnError);
              }
              // return the data
              resolve(irReturnData);
            });
          })
        );
      });
      // return the responses
      return Promise.all(returnArray).then((result) => callback(result, null));
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Releases the specified address range that you provisioned for use with your AWS resources through bring your own IP addresses (BYOIP) and deletes the corresponding address pool.   Before you can release an address range, you must stop advertising it using  WithdrawByoipCidr  and you must not have any IP addresses allocated from its address range.
   *
   * @function deprovisionByoipCidr
   * @param {string} cidr - The public IPv4 address range, in CIDR notation. The prefix must be the same prefix that you specified when you provisioned the address range.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deprovisionByoipCidr(cidr, dryRun, callback) {
    const meth = 'adapter-deprovisionByoipCidr';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeprovisionByoipCidr';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (cidr === undefined || cidr === null || cidr === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['cidr'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Cidr: cidr,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'deprovisionByoipCidr', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deprovisionByoipCidr'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Deregisters the specified AMI. After you deregister an AMI, it can't be used to launch new instances; however, it doesn't affect any instances that you've already launched from the AMI. You'll continue to incur usage costs for those instances until you terminate them.   When you deregister an Amazon EBS-backed AMI, it doesn't affect the snapshot that was created for the root volume of the instance during the AMI creation process. When you deregister an instance store-backed AMI, it doesn't affe...(description truncated)
   *
   * @function deregisterImage
   * @param {string} imageId - The ID of the AMI.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  deregisterImage(imageId, dryRun, callback) {
    const meth = 'adapter-deregisterImage';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DeregisterImage';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (imageId === undefined || imageId === null || imageId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['imageId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ImageId: imageId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Image', 'deregisterImage', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['deregisterImage'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes attributes of your AWS account. The following are the supported account attributes:         supported-platforms : Indicates whether your account can launch instances into EC2-Classic and EC2-VPC, or only into EC2-VPC.         default-vpc : The ID of the default VPC for your account, or  none .         max-instances : The maximum number of On-Demand Instances that you can run.         vpc-max-security-groups-per-interface : The maximum number of security groups that you can assign to a...(description truncated)
   *
   * @function describeAccountAttributes
   * @param {array} attributeName - The account attribute names.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeAccountAttributes(attributeName, dryRun, callback) {
    const meth = 'adapter-describeAccountAttributes';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeAccountAttributes';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (attributeName && attributeName.length > 0) {
      processInArray(attributeName, queryParamsAvailable, 'AttributeName');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionDescribeAccountAttributes', 'getDescribeAccountAttributes', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeAccountAttributes'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified Elastic IP addresses or all of your Elastic IP addresses.   An Elastic IP address is for use in either the EC2-Classic platform or in a VPC. For more information, see  Elastic IP Addresses  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function describeAddresses
   * @param {array} filter - One or more filters. Filter names and values are case-sensitive.         allocation-id  - [EC2-VPC] The allocation ID for the address.         association-id  - [EC2-VPC...(description truncated)
   * @param {array} publicIp - One or more Elastic IP addresses.   Default: Describes all your Elastic IP addresses.
   * @param {array} allocationId - [EC2-VPC] Information about the allocation IDs.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeAddresses(filter, publicIp, allocationId, dryRun, callback) {
    const meth = 'adapter-describeAddresses';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeAddresses';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (publicIp && publicIp.length > 0) {
      processInArray(publicIp, queryParamsAvailable, 'PublicIp');
    }
    if (allocationId && allocationId.length > 0) {
      processInArray(allocationId, queryParamsAvailable, 'AllocationId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'describeAddresses', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeAddresses'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the longer ID format settings for all resource types in a specific Region. This request is useful for performing a quick audit to determine whether a specific Region is fully opted in for longer IDs (17-character IDs).   This request only returns information about resource types that support longer IDs.   The following resource types support longer IDs:  bundle  |  conversion-task  |  customer-gateway  |  dhcp-options  |  elastic-ip-allocation  |  elastic-ip-association  |  export-tas...(description truncated)
   *
   * @function describeAggregateIdFormat
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeAggregateIdFormat(dryRun, callback) {
    const meth = 'adapter-describeAggregateIdFormat';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeAggregateIdFormat';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Id', 'describeAggregateIdFormat', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeAggregateIdFormat'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the Availability Zones that are available to you. The results include zones only for the Region you're currently using. If there is an event impacting an Availability Zone, you can use this request to view the state and any provided message for that Availability Zone.   For more information, see  Regions and Availability Zones  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function describeAvailabilityZones
   * @param {array} filter - The filters.         message  - Information about the Availability Zone.         region-name  - The name of the Region for the Availability Zone (for example,  us-east-1...(description truncated)
   * @param {array} zoneName - The names of the Availability Zones.
   * @param {array} zoneId - The IDs of the Availability Zones.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeAvailabilityZones(filter, zoneName, zoneId, dryRun, callback) {
    const meth = 'adapter-describeAvailabilityZones';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeAvailabilityZones';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (zoneName && zoneName.length > 0) {
      processInArray(zoneName, queryParamsAvailable, 'ZoneName');
    }
    if (zoneId && zoneId.length > 0) {
      processInArray(zoneId, queryParamsAvailable, 'ZoneId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionDescribeAvailabilityZones', 'getDescribeAvailabilityZones', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeAvailabilityZones'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified bundle tasks or all of your bundle tasks.     Completed bundle tasks are listed for only a limited time. If your bundle task is no longer in the list, you can still register an AMI from it. Just use  RegisterImage  with the Amazon S3 bucket name and image manifest name you provided to the bundle task.
   *
   * @function describeBundleTasks
   * @param {array} bundleId - The bundle task IDs.   Default: Describes all your bundle tasks.
   * @param {array} filter - The filters.         bundle-id  - The ID of the bundle task.         error-code  - If the task failed, the error code returned.         error-message  - If the task fail...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeBundleTasks(bundleId, filter, dryRun, callback) {
    const meth = 'adapter-describeBundleTasks';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeBundleTasks';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (bundleId && bundleId.length > 0) {
      processInArray(bundleId, queryParamsAvailable, 'BundleId');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionDescribeBundleTasks', 'getDescribeBundleTasks', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeBundleTasks'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the IP address ranges that were specified in calls to  ProvisionByoipCidr .   To describe the address pools that were created when you provisioned the address ranges, use  DescribePublicIpv4Pools .
   *
   * @function describeByoipCidrs
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {string} nextToken - The token for the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeByoipCidrs(dryRun, maxResults, nextToken, callback) {
    const meth = 'adapter-describeByoipCidrs';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeByoipCidrs';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (maxResults === undefined || maxResults === null || maxResults === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['maxResults'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'describeByoipCidrs', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeByoipCidrs'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more of your Capacity Reservations. The results describe only the Capacity Reservations in the AWS Region that you're currently using.
   *
   * @function describeCapacityReservations
   * @param {array} capacityReservationId - The ID of the Capacity Reservation.
   * @param {string} nextToken - The token to retrieve the next page of results.
   * @param {number} maxResults - The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the returned nextToken value.
   * @param {array} filter - One or more filters.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeCapacityReservations(capacityReservationId, nextToken, maxResults, filter, dryRun, callback) {
    const meth = 'adapter-describeCapacityReservations';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeCapacityReservations';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (capacityReservationId && capacityReservationId.length > 0) {
      processInArray(capacityReservationId, queryParamsAvailable, 'CapacityReservationId');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Capacity', 'describeCapacityReservations', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeCapacityReservations'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more of your linked EC2-Classic instances. This request only returns information about EC2-Classic instances linked to a VPC through ClassicLink. You cannot use this request to return information about other instances.
   *
   * @function describeClassicLinkInstances
   * @param {array} filter - One or more filters.         group-id  - The ID of a VPC security group that's associated with the instance.         instance-id  - The ID of the instance.         tag :...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} instanceId - One or more instance IDs. Must be instances linked to a VPC through ClassicLink.
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.   Constraint: If th...(description truncated)
   * @param {string} nextToken - The token for the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeClassicLinkInstances(filter, dryRun, instanceId, maxResults, nextToken, callback) {
    const meth = 'adapter-describeClassicLinkInstances';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeClassicLinkInstances';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (instanceId && instanceId.length > 0) {
      processInArray(instanceId, queryParamsAvailable, 'InstanceId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'describeClassicLinkInstances', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeClassicLinkInstances'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the authorization rules for a specified Client VPN endpoint.
   *
   * @function describeClientVpnAuthorizationRules
   * @param {string} clientVpnEndpointId - The ID of the Client VPN endpoint.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} nextToken - The token to retrieve the next page of results.
   * @param {array} filter - One or more filters. Filter names and values are case-sensitive.
   * @param {number} maxResults - The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the nextToken value.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeClientVpnAuthorizationRules(clientVpnEndpointId, dryRun, nextToken, filter, maxResults, callback) {
    const meth = 'adapter-describeClientVpnAuthorizationRules';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeClientVpnAuthorizationRules';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointId === undefined || clientVpnEndpointId === null || clientVpnEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      ClientVpnEndpointId: clientVpnEndpointId,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'describeClientVpnAuthorizationRules', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeClientVpnAuthorizationRules'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes active client connections and connections that have been terminated within the last 60 minutes for the specified Client VPN endpoint.
   *
   * @function describeClientVpnConnections
   * @param {string} clientVpnEndpointId - The ID of the Client VPN endpoint.
   * @param {array} filter - One or more filters. Filter names and values are case-sensitive.
   * @param {string} nextToken - The token to retrieve the next page of results.
   * @param {number} maxResults - The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the nextToken value.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeClientVpnConnections(clientVpnEndpointId, filter, nextToken, maxResults, dryRun, callback) {
    const meth = 'adapter-describeClientVpnConnections';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeClientVpnConnections';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointId === undefined || clientVpnEndpointId === null || clientVpnEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      ClientVpnEndpointId: clientVpnEndpointId,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'describeClientVpnConnections', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeClientVpnConnections'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more Client VPN endpoints in the account.
   *
   * @function describeClientVpnEndpoints
   * @param {array} clientVpnEndpointId - The ID of the Client VPN endpoint.
   * @param {number} maxResults - The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the nextToken value.
   * @param {string} nextToken - The token to retrieve the next page of results.
   * @param {array} filter - One or more filters. Filter names and values are case-sensitive.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeClientVpnEndpoints(clientVpnEndpointId, maxResults, nextToken, filter, dryRun, callback) {
    const meth = 'adapter-describeClientVpnEndpoints';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeClientVpnEndpoints';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (clientVpnEndpointId && clientVpnEndpointId.length > 0) {
      processInArray(clientVpnEndpointId, queryParamsAvailable, 'ClientVpnEndpointId');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'describeClientVpnEndpoints', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeClientVpnEndpoints'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the routes for the specified Client VPN endpoint.
   *
   * @function describeClientVpnRoutes
   * @param {string} clientVpnEndpointId - The ID of the Client VPN endpoint.
   * @param {array} filter - One or more filters. Filter names and values are case-sensitive.
   * @param {number} maxResults - The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the nextToken value.
   * @param {string} nextToken - The token to retrieve the next page of results.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeClientVpnRoutes(clientVpnEndpointId, filter, maxResults, nextToken, dryRun, callback) {
    const meth = 'adapter-describeClientVpnRoutes';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeClientVpnRoutes';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointId === undefined || clientVpnEndpointId === null || clientVpnEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      ClientVpnEndpointId: clientVpnEndpointId,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'describeClientVpnRoutes', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeClientVpnRoutes'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the target networks associated with the specified Client VPN endpoint.
   *
   * @function describeClientVpnTargetNetworks
   * @param {string} clientVpnEndpointId - The ID of the Client VPN endpoint.
   * @param {array} associationIds - The IDs of the target network associations.
   * @param {number} maxResults - The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the nextToken value.
   * @param {string} nextToken - The token to retrieve the next page of results.
   * @param {array} filter - One or more filters. Filter names and values are case-sensitive.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeClientVpnTargetNetworks(clientVpnEndpointId, associationIds, maxResults, nextToken, filter, dryRun, callback) {
    const meth = 'adapter-describeClientVpnTargetNetworks';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeClientVpnTargetNetworks';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointId === undefined || clientVpnEndpointId === null || clientVpnEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      ClientVpnEndpointId: clientVpnEndpointId,
      Action: action,
      Version: version
    };
    if (associationIds && associationIds.length > 0) {
      processInArray(associationIds, queryParamsAvailable, 'AssociationIds');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'describeClientVpnTargetNetworks', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeClientVpnTargetNetworks'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified conversion tasks or all your conversion tasks. For more information, see the  VM Import/Export User Guide .   For information about the import manifest referenced by this API action, see  VM Import Manifest .
   *
   * @function describeConversionTasks
   * @param {array} conversionTaskId - The conversion task IDs.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeConversionTasks(conversionTaskId, dryRun, callback) {
    const meth = 'adapter-describeConversionTasks';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeConversionTasks';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (conversionTaskId && conversionTaskId.length > 0) {
      processInArray(conversionTaskId, queryParamsAvailable, 'ConversionTaskId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionDescribeConversionTasks', 'getDescribeConversionTasks', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeConversionTasks'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more of your VPN customer gateways.   For more information, see  AWS Site-to-Site VPN  in the  AWS Site-to-Site VPN User Guide .
   *
   * @function describeCustomerGateways
   * @param {array} customerGatewayId - One or more customer gateway IDs.   Default: Describes all your customer gateways.
   * @param {array} filter - One or more filters.         bgp-asn  - The customer gateway's Border Gateway Protocol (BGP) Autonomous System Number (ASN).         customer-gateway-id  - The ID of the...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeCustomerGateways(customerGatewayId, filter, dryRun, callback) {
    const meth = 'adapter-describeCustomerGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeCustomerGateways';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (customerGatewayId && customerGatewayId.length > 0) {
      processInArray(customerGatewayId, queryParamsAvailable, 'CustomerGatewayId');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Gateway', 'describeCustomerGateways', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeCustomerGateways'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more of your DHCP options sets.   For more information, see  DHCP Options Sets  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function describeDhcpOptions
   * @param {array} dhcpOptionsId - The IDs of one or more DHCP options sets.   Default: Describes all your DHCP options sets.
   * @param {array} filter - One or more filters.         dhcp-options-id  - The ID of a DHCP options set.         key  - The key for one of the options (for example,  domain-name ).         value  ...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} nextToken - The token for the next page of results.
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeDhcpOptions(dhcpOptionsId, filter, dryRun, nextToken, maxResults, callback) {
    const meth = 'adapter-describeDhcpOptions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeDhcpOptions';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (dhcpOptionsId && dhcpOptionsId.length > 0) {
      processInArray(dhcpOptionsId, queryParamsAvailable, 'DhcpOptionsId');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('DHCP', 'describeDhcpOptions', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeDhcpOptions'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more of your egress-only internet gateways.
   *
   * @function describeEgressOnlyInternetGateways
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} egressOnlyInternetGatewayId - One or more egress-only internet gateway IDs.
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {string} nextToken - The token for the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeEgressOnlyInternetGateways(dryRun, egressOnlyInternetGatewayId, maxResults, nextToken, callback) {
    const meth = 'adapter-describeEgressOnlyInternetGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeEgressOnlyInternetGateways';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (egressOnlyInternetGatewayId && egressOnlyInternetGatewayId.length > 0) {
      processInArray(egressOnlyInternetGatewayId, queryParamsAvailable, 'EgressOnlyInternetGatewayId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Gateway', 'describeEgressOnlyInternetGateways', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeEgressOnlyInternetGateways'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the Elastic Graphics accelerator associated with your instances. For more information about Elastic Graphics, see  Amazon Elastic Graphics .
   *
   * @function describeElasticGpus
   * @param {array} elasticGpuId - The Elastic Graphics accelerator IDs.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} filter - The filters.         availability-zone  - The Availability Zone in which the Elastic Graphics accelerator resides.         elastic-gpu-health  - The status of the Elasti...(description truncated)
   * @param {number} maxResults - The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned  NextToken  value. This value can be betw...(description truncated)
   * @param {string} nextToken - The token to request the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeElasticGpus(elasticGpuId, dryRun, filter, maxResults, nextToken, callback) {
    const meth = 'adapter-describeElasticGpus';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeElasticGpus';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (elasticGpuId && elasticGpuId.length > 0) {
      processInArray(elasticGpuId, queryParamsAvailable, 'ElasticGpuId');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionDescribeElasticGpus', 'getDescribeElasticGpus', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeElasticGpus'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified export tasks or all your export tasks.
   *
   * @function describeExportTasks
   * @param {array} exportTaskId - The export task IDs.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeExportTasks(exportTaskId, callback) {
    const meth = 'adapter-describeExportTasks';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeExportTasks';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      Action: action,
      Version: version
    };
    if (exportTaskId && exportTaskId.length > 0) {
      processInArray(exportTaskId, queryParamsAvailable, 'ExportTaskId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionDescribeExportTasks', 'getDescribeExportTasks', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeExportTasks'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the events for the specified EC2 Fleet during the specified time.
   *
   * @function describeFleetHistory
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} eventType - The type of events to describe. By default, all events are described.
   * @param {number} maxResults - The maximum number of results to return in a single call. Specify a value between 1 and 1000. The default value is 1000. To retrieve the remaining results, make another c...(description truncated)
   * @param {string} nextToken - The token for the next set of results.
   * @param {string} fleetId - The ID of the EC2 Fleet.
   * @param {string} startTime - The start date and time for the events, in UTC format (for example,  YYYY - MM - DD T HH : MM : SS Z).
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeFleetHistory(dryRun, eventType = 'instance-change', maxResults, nextToken, fleetId, startTime, callback) {
    const meth = 'adapter-describeFleetHistory';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeFleetHistory';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (fleetId === undefined || fleetId === null || fleetId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['fleetId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (startTime === undefined || startTime === null || startTime === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['startTime'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      EventType: eventType,
      FleetId: fleetId,
      StartTime: startTime,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Fleet', 'describeFleetHistory', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeFleetHistory'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the running instances for the specified EC2 Fleet.
   *
   * @function describeFleetInstances
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {number} maxResults - The maximum number of results to return in a single call. Specify a value between 1 and 1000. The default value is 1000. To retrieve the remaining results, make another c...(description truncated)
   * @param {string} nextToken - The token for the next set of results.
   * @param {string} fleetId - The ID of the EC2 Fleet.
   * @param {array} filter - The filters.         instance-type  - The instance type.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeFleetInstances(dryRun, maxResults, nextToken, fleetId, filter, callback) {
    const meth = 'adapter-describeFleetInstances';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeFleetInstances';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (fleetId === undefined || fleetId === null || fleetId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['fleetId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      FleetId: fleetId,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'describeFleetInstances', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeFleetInstances'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified EC2 Fleets or all your EC2 Fleets.
   *
   * @function describeFleets
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {number} maxResults - The maximum number of results to return in a single call. Specify a value between 1 and 1000. The default value is 1000. To retrieve the remaining results, make another c...(description truncated)
   * @param {string} nextToken - The token for the next set of results.
   * @param {array} fleetId - The ID of the EC2 Fleets.
   * @param {array} filter - The filters.         activity-status  - The progress of the EC2 Fleet (  error  |  pending-fulfillment  |  pending-termination  |  fulfilled ).         excess-capacity-t...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeFleets(dryRun, maxResults, nextToken, fleetId, filter, callback) {
    const meth = 'adapter-describeFleets';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeFleets';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (fleetId && fleetId.length > 0) {
      processInArray(fleetId, queryParamsAvailable, 'FleetId');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Fleet', 'describeFleets', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeFleets'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more flow logs. To view the information in your flow logs (the log streams for the network interfaces), you must use the CloudWatch Logs console or the CloudWatch Logs API.
   *
   * @function describeFlowLogs
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} filter - One or more filters.         deliver-log-status  - The status of the logs delivery ( SUCCESS  |  FAILED ).         log-destination-type  - The type of destination to whi...(description truncated)
   * @param {array} flowLogId - One or more flow log IDs.
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {string} nextToken - The token for the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeFlowLogs(dryRun, filter, flowLogId, maxResults, nextToken, callback) {
    const meth = 'adapter-describeFlowLogs';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeFlowLogs';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (flowLogId && flowLogId.length > 0) {
      processInArray(flowLogId, queryParamsAvailable, 'FlowLogId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionDescribeFlowLogs', 'getDescribeFlowLogs', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeFlowLogs'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified attribute of the specified Amazon FPGA Image (AFI).
   *
   * @function describeFpgaImageAttribute
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} fpgaImageId - The ID of the AFI.
   * @param {string} attribute - The AFI attribute.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeFpgaImageAttribute(dryRun, fpgaImageId, attribute = 'description', callback) {
    const meth = 'adapter-describeFpgaImageAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeFpgaImageAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (fpgaImageId === undefined || fpgaImageId === null || fpgaImageId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['fpgaImageId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (attribute === undefined || attribute === null || attribute === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['attribute'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      FpgaImageId: fpgaImageId,
      Attribute: attribute,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Image', 'describeFpgaImageAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeFpgaImageAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the Amazon FPGA Images (AFIs) available to you. These include public AFIs, private AFIs that you own, and AFIs owned by other AWS accounts for which you have load permissions.
   *
   * @function describeFpgaImages
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} fpgaImageId - The AFI IDs.
   * @param {array} owner - Filters the AFI by owner. Specify an AWS account ID,  self  (owner is the sender of the request), or an AWS owner alias (valid values are  amazon  |  aws-marketplace ).
   * @param {array} filter - The filters.         create-time  - The creation time of the AFI.         fpga-image-id  - The FPGA image identifier (AFI ID).         fpga-image-global-id  - The global...(description truncated)
   * @param {string} nextToken - The token to retrieve the next page of results.
   * @param {number} maxResults - The maximum number of results to return in a single call.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeFpgaImages(dryRun, fpgaImageId, owner, filter, nextToken, maxResults, callback) {
    const meth = 'adapter-describeFpgaImages';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeFpgaImages';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (fpgaImageId && fpgaImageId.length > 0) {
      processInArray(fpgaImageId, queryParamsAvailable, 'FpgaImageId');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (owner && owner.length > 0) {
      processInArray(owner, queryParamsAvailable, 'Owner');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Image', 'describeFpgaImages', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeFpgaImages'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the Dedicated Host reservations that are available to purchase.   The results describe all the Dedicated Host reservation offerings, including offerings that may not match the instance family and Region of your Dedicated Hosts. When purchasing an offering, ensure that the instance family and Region of the offering matches that of the Dedicated Hosts with which it is to be associated. For more information about supported instance types, see  Dedicated Hosts Overview  in the  Amazon Ela...(description truncated)
   *
   * @function describeHostReservationOfferings
   * @param {array} filter - The filters.         instance-family  - The instance family of the offering (for example,  m4 ).         payment-option  - The payment option ( NoUpfront  |  PartialUpfr...(description truncated)
   * @param {number} maxDuration - This is the maximum duration of the reservation to purchase, specified in seconds. Reservations are available in one-year and three-year terms. The number of seconds spec...(description truncated)
   * @param {number} maxResults - The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the returned  nextToken  value...(description truncated)
   * @param {number} minDuration - This is the minimum duration of the reservation you'd like to purchase, specified in seconds. Reservations are available in one-year and three-year terms. The number of s...(description truncated)
   * @param {string} nextToken - The token to use to retrieve the next page of results.
   * @param {string} offeringId - The ID of the reservation offering.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeHostReservationOfferings(filter, maxDuration, maxResults, minDuration, nextToken, offeringId, callback) {
    const meth = 'adapter-describeHostReservationOfferings';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeHostReservationOfferings';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      NextToken: nextToken,
      MaxResults: maxResults,
      MaxDuration: maxDuration,
      MinDuration: minDuration,
      OfferingId: offeringId,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Host', 'describeHostReservationOfferings', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeHostReservationOfferings'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes reservations that are associated with Dedicated Hosts in your account.
   *
   * @function describeHostReservations
   * @param {array} filter - The filters.         instance-family  - The instance family (for example,  m4 ).         payment-option  - The payment option ( NoUpfront  |  PartialUpfront  |  AllUpfro...(description truncated)
   * @param {array} hostReservationIdSet - The host reservation IDs.
   * @param {number} maxResults - The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the returned  nextToken  value...(description truncated)
   * @param {string} nextToken - The token to use to retrieve the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeHostReservations(filter, hostReservationIdSet, maxResults, nextToken, callback) {
    const meth = 'adapter-describeHostReservations';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeHostReservations';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (hostReservationIdSet && hostReservationIdSet.length > 0) {
      processInArray(hostReservationIdSet, queryParamsAvailable, 'HostReservationIdSet');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Host', 'describeHostReservations', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeHostReservations'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified Dedicated Hosts or all your Dedicated Hosts.   The results describe only the Dedicated Hosts in the Region you're currently using. All listed instances consume capacity on your Dedicated Host. Dedicated Hosts that have recently been released are listed with the state  released .
   *
   * @function describeHosts
   * @param {array} filter - The filters.         auto-placement  - Whether auto-placement is enabled or disabled ( on  |  off ).         availability-zone  - The Availability Zone of the host.     ...(description truncated)
   * @param {array} hostId - The IDs of the Dedicated Hosts. The IDs are used for targeted instance launches.
   * @param {number} maxResults - The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the returned  nextToken  value...(description truncated)
   * @param {string} nextToken - The token to retrieve the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeHosts(filter, hostId, maxResults, nextToken, callback) {
    const meth = 'adapter-describeHosts';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeHosts';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (hostId && hostId.length > 0) {
      processInArray(hostId, queryParamsAvailable, 'HostId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Host', 'describeHosts', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeHosts'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes your IAM instance profile associations.
   *
   * @function describeIamInstanceProfileAssociations
   * @param {array} associationId - The IAM instance profile associations.
   * @param {array} filter - The filters.         instance-id  - The ID of the instance.         state  - The state of the association ( associating  |  associated  |  disassociating  |  disassociat...(description truncated)
   * @param {number} maxResults - The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned  NextToken  value.
   * @param {string} nextToken - The token to request the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeIamInstanceProfileAssociations(associationId, filter, maxResults, nextToken, callback) {
    const meth = 'adapter-describeIamInstanceProfileAssociations';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeIamInstanceProfileAssociations';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (associationId && associationId.length > 0) {
      processInArray(associationId, queryParamsAvailable, 'AssociationId');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'describeIamInstanceProfileAssociations', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeIamInstanceProfileAssociations'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the ID format settings for your resources on a per-Region basis, for example, to view which resource types are enabled for longer IDs. This request only returns information about resource types whose ID formats can be modified; it does not return information about other resource types.   The following resource types support longer IDs:  bundle  |  conversion-task  |  customer-gateway  |  dhcp-options  |  elastic-ip-allocation  |  elastic-ip-association  |  export-task  |  flow-log  | ...(description truncated)
   *
   * @function describeIdFormat
   * @param {string} resource - The type of resource:  bundle  |  conversion-task  |  customer-gateway  |  dhcp-options  |  elastic-ip-allocation  |  elastic-ip-association  |  export-task  |  flow-log ...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeIdFormat(resource, callback) {
    const meth = 'adapter-describeIdFormat';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeIdFormat';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      Resource: resource,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Id', 'describeIdFormat', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeIdFormat'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the ID format settings for resources for the specified IAM user, IAM role, or root user. For example, you can view the resource types that are enabled for longer IDs. This request only returns information about resource types whose ID formats can be modified; it does not return information about other resource types. For more information, see  Resource IDs  in the  Amazon Elastic Compute Cloud User Guide .    The following resource types support longer IDs:  bundle  |  conversion-task...(description truncated)
   *
   * @function describeIdentityIdFormat
   * @param {string} principalArn - The ARN of the principal, which can be an IAM role, IAM user, or the root user.
   * @param {string} resource - The type of resource:  bundle  |  conversion-task  |  customer-gateway  |  dhcp-options  |  elastic-ip-allocation  |  elastic-ip-association  |  export-task  |  flow-log ...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeIdentityIdFormat(principalArn, resource, callback) {
    const meth = 'adapter-describeIdentityIdFormat';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeIdentityIdFormat';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (principalArn === undefined || principalArn === null || principalArn === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['principalArn'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      PrincipalArn: principalArn,
      Resource: resource,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Id', 'describeIdentityIdFormat', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeIdentityIdFormat'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified attribute of the specified AMI. You can specify only one attribute at a time.
   *
   * @function describeImageAttribute
   * @param {string} attribute - The AMI attribute.     Note : Depending on your account privileges, the  blockDeviceMapping  attribute may return a  Client.AuthFailure  error. If this happens, use  Des...(description truncated)
   * @param {string} imageId - The ID of the AMI.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeImageAttribute(attribute = 'description', imageId, dryRun, callback) {
    const meth = 'adapter-describeImageAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeImageAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (attribute === undefined || attribute === null || attribute === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['attribute'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (imageId === undefined || imageId === null || imageId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['imageId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Attribute: attribute,
      ImageId: imageId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Image', 'describeImageAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeImageAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified images (AMIs, AKIs, and ARIs) available to you or all of the images available to you.   The images available to you include public images, private images that you own, and private images owned by other AWS accounts for which you have explicit launch permissions.   Recently deregistered images appear in the returned results for a short interval and then return empty results. After all instances that reference a deregistered AMI are terminated, specifying the ID of the ima...(description truncated)
   *
   * @function describeImages
   * @param {array} executableBy - Scopes the images by users with explicit launch permissions. Specify an AWS account ID,  self  (the sender of the request), or  all  (public AMIs).
   * @param {array} filter - The filters.         architecture  - The image architecture ( i386  |  x86_64 ).         block-device-mapping.delete-on-termination  - A Boolean value that indicates whe...(description truncated)
   * @param {array} imageId - The image IDs.   Default: Describes all images available to you.
   * @param {array} owner - Filters the images by the owner. Specify an AWS account ID,  self  (owner is the sender of the request), or an AWS owner alias (valid values are  amazon  |  aws-marketpla...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeImages(executableBy, filter, imageId, owner, dryRun, callback) {
    const meth = 'adapter-describeImages';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeImages';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (executableBy && executableBy.length > 0) {
      processInArray(executableBy, queryParamsAvailable, 'ExecutableBy');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (imageId && imageId.length > 0) {
      processInArray(imageId, queryParamsAvailable, 'ImageId');
    }
    if (owner && owner.length > 0) {
      processInArray(owner, queryParamsAvailable, 'Owner');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Image', 'describeImages', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeImages'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Displays details about an import virtual machine or import snapshot tasks that are already created.
   *
   * @function describeImportImageTasks
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} filters - Filter tasks using the  task-state  filter and one of the following values: active, completed, deleting, deleted.
   * @param {array} importTaskId - A list of import image task IDs.
   * @param {number} maxResults - The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned  NextToken  value.
   * @param {string} nextToken - A token that indicates the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeImportImageTasks(dryRun, filters, importTaskId, maxResults, nextToken, callback) {
    const meth = 'adapter-describeImportImageTasks';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeImportImageTasks';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (importTaskId && importTaskId.length > 0) {
      processInArray(importTaskId, queryParamsAvailable, 'ImportTaskId');
    }
    if (filters && filters.length > 0) {
      processInArray(filters, queryParamsAvailable, 'Filters');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Image', 'describeImportImageTasks', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeImportImageTasks'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes your import snapshot tasks.
   *
   * @function describeImportSnapshotTasks
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} filters - The filters.
   * @param {array} importTaskId - A list of import snapshot task IDs.
   * @param {number} maxResults - The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned  NextToken  value.
   * @param {string} nextToken - A token that indicates the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeImportSnapshotTasks(dryRun, filters, importTaskId, maxResults, nextToken, callback) {
    const meth = 'adapter-describeImportSnapshotTasks';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeImportSnapshotTasks';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filters && filters.length > 0) {
      processInArray(filters, queryParamsAvailable, 'Filters');
    }
    if (importTaskId && importTaskId.length > 0) {
      processInArray(importTaskId, queryParamsAvailable, 'ImportTaskId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Snapshot', 'describeImportSnapshotTasks', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeImportSnapshotTasks'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified attribute of the specified instance. You can specify only one attribute at a time. Valid attribute values are:  instanceType  |  kernel  |  ramdisk  |  userData  |  disableApiTermination  |  instanceInitiatedShutdownBehavior  |  rootDeviceName  |  blockDeviceMapping  |  productCodes  |  sourceDestCheck  |  groupSet  |  ebsOptimized  |  sriovNetSupport
   *
   * @function describeInstanceAttribute
   * @param {string} attribute - The instance attribute.   Note: The  enaSupport  attribute is not supported at this time.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} instanceId - The ID of the instance.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeInstanceAttribute(attribute = 'instanceType', dryRun, instanceId, callback) {
    const meth = 'adapter-describeInstanceAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeInstanceAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (attribute === undefined || attribute === null || attribute === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['attribute'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Attribute: attribute,
      InstanceId: instanceId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'describeInstanceAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeInstanceAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the credit option for CPU usage of the specified T2 or T3 instances. The credit options are  standard  and  unlimited .   If you do not specify an instance ID, Amazon EC2 returns T2 and T3 instances with the  unlimited  credit option, as well as instances that were previously configured as T2 or T3 with the  unlimited  credit option. For example, if you resize a T2 instance, while it is configured as  unlimited , to an M4 instance, Amazon EC2 returns the M4 instance.   If you specify ...(description truncated)
   *
   * @function describeInstanceCreditSpecifications
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} filter - The filters.         instance-id  - The ID of the instance.
   * @param {array} instanceId - The instance IDs.   Default: Describes all your instances.   Constraints: Maximum 1000 explicitly specified instance IDs.
   * @param {number} maxResults - The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned  NextToken  value. This value can be betw...(description truncated)
   * @param {string} nextToken - The token to retrieve the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeInstanceCreditSpecifications(dryRun, filter, instanceId, maxResults, nextToken, callback) {
    const meth = 'adapter-describeInstanceCreditSpecifications';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeInstanceCreditSpecifications';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (instanceId && instanceId.length > 0) {
      processInArray(instanceId, queryParamsAvailable, 'InstanceId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'describeInstanceCreditSpecifications', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeInstanceCreditSpecifications'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the status of the specified instances or all of your instances. By default, only running instances are described, unless you specifically indicate to return the status of all instances.   Instance status includes the following components:         Status checks  - Amazon EC2 performs status checks on running EC2 instances to identify hardware and software issues. For more information, see  Status Checks for Your Instances  and  Troubleshooting Instances with Failed Status Checks  in th...(description truncated)
   *
   * @function describeInstanceStatus
   * @param {array} filter - The filters.         availability-zone  - The Availability Zone of the instance.         event.code  - The code for the scheduled event ( instance-reboot  |  system-rebo...(description truncated)
   * @param {array} instanceId - The instance IDs.   Default: Describes all your instances.   Constraints: Maximum 100 explicitly specified instance IDs.
   * @param {number} maxResults - The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned  NextToken  value. This value can be betw...(description truncated)
   * @param {string} nextToken - The token to retrieve the next page of results.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {boolean} includeAllInstances - When  true , includes the health status for all instances. When  false , includes the health status for running instances only.   Default:  false
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeInstanceStatus(filter, instanceId, maxResults, nextToken, dryRun, includeAllInstances, callback) {
    const meth = 'adapter-describeInstanceStatus';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeInstanceStatus';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      IncludeAllInstances: includeAllInstances,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (instanceId && instanceId.length > 0) {
      processInArray(instanceId, queryParamsAvailable, 'InstanceId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'describeInstanceStatus', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeInstanceStatus'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified instances or all of your instances.   If you specify one or more instance IDs, Amazon EC2 returns information for those instances. If you do not specify instance IDs, Amazon EC2 returns information for all relevant instances. If you specify an instance ID that is not valid, an error is returned. If you specify an instance that you do not own, it is not included in the returned results.   Recently terminated instances might appear in the returned results. This interval is...(description truncated)
   *
   * @function describeInstances
   * @param {array} filter - The filters.         affinity  - The affinity setting for an instance running on a Dedicated Host ( default  |  host ).         architecture  - The instance architecture...(description truncated)
   * @param {array} instanceId - The instance IDs.   Default: Describes all your instances.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {number} maxResults - The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned  NextToken  value. This value can be betw...(description truncated)
   * @param {string} nextToken - The token to request the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeInstances(filter, instanceId, dryRun, maxResults, nextToken, callback) {
    const meth = 'adapter-describeInstances';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeInstances';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (instanceId && instanceId.length > 0) {
      processInArray(instanceId, queryParamsAvailable, 'InstanceId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'describeInstances', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeInstances'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more of your internet gateways.
   *
   * @function describeInternetGateways
   * @param {array} filter - One or more filters.         attachment.state  - The current state of the attachment between the gateway and the VPC ( available ). Present only if a VPC is attached.   ...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} internetGatewayId - One or more internet gateway IDs.   Default: Describes all your internet gateways.
   * @param {string} nextToken - The token for the next page of results.
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeInternetGateways(filter, dryRun, internetGatewayId, nextToken, maxResults, callback) {
    const meth = 'adapter-describeInternetGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeInternetGateways';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (internetGatewayId && internetGatewayId.length > 0) {
      processInArray(internetGatewayId, queryParamsAvailable, 'InternetGatewayId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Gateway', 'describeInternetGateways', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeInternetGateways'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified key pairs or all of your key pairs.   For more information about key pairs, see  Key Pairs  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function describeKeyPairs
   * @param {array} filter - The filters.         fingerprint  - The fingerprint of the key pair.         key-name  - The name of the key pair.
   * @param {array} keyName - The key pair names.   Default: Describes all your key pairs.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeKeyPairs(filter, keyName, dryRun, callback) {
    const meth = 'adapter-describeKeyPairs';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeKeyPairs';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (keyName && keyName.length > 0) {
      processInArray(keyName, queryParamsAvailable, 'KeyName');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('KeyPair', 'describeKeyPairs', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeKeyPairs'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more versions of a specified launch template. You can describe all versions, individual versions, or a range of versions.
   *
   * @function describeLaunchTemplateVersions
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} launchTemplateId - The ID of the launch template. You must specify either the launch template ID or launch template name in the request.
   * @param {string} launchTemplateName - The name of the launch template. You must specify either the launch template ID or launch template name in the request.
   * @param {array} launchTemplateVersion - One or more versions of the launch template.
   * @param {string} minVersion - The version number after which to describe launch template versions.
   * @param {string} maxVersion - The version number up to which to describe launch template versions.
   * @param {string} nextToken - The token to request the next page of results.
   * @param {number} maxResults - The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned  NextToken  value. This value can be betw...(description truncated)
   * @param {array} filter - One or more filters.         create-time  - The time the launch template version was created.         ebs-optimized  - A boolean that indicates whether the instance is o...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeLaunchTemplateVersions(dryRun, launchTemplateId, launchTemplateName, launchTemplateVersion, minVersion, maxVersion, nextToken, maxResults, filter, callback) {
    const meth = 'adapter-describeLaunchTemplateVersions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeLaunchTemplateVersions';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      LaunchTemplateId: launchTemplateId,
      LaunchTemplateName: launchTemplateName,
      MinVersion: minVersion,
      MaxVersion: maxVersion,
      Action: action,
      Version: version
    };
    if (launchTemplateVersion && launchTemplateVersion.length > 0) {
      processInArray(launchTemplateVersion, queryParamsAvailable, 'LaunchTemplateVersion');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('LaunchTemplate', 'describeLaunchTemplateVersions', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeLaunchTemplateVersions'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more launch templates.
   *
   * @function describeLaunchTemplates
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} launchTemplateId - One or more launch template IDs.
   * @param {array} launchTemplateName - One or more launch template names.
   * @param {array} filter - One or more filters.         create-time  - The time the launch template was created.         launch-template-name  - The name of the launch template.         tag :&lt;k...(description truncated)
   * @param {string} nextToken - The token to request the next page of results.
   * @param {number} maxResults - The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned  NextToken  value. This value can be betw...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeLaunchTemplates(dryRun, launchTemplateId, launchTemplateName, filter, nextToken, maxResults, callback) {
    const meth = 'adapter-describeLaunchTemplates';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeLaunchTemplates';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (launchTemplateId && launchTemplateId.length > 0) {
      processInArray(launchTemplateId, queryParamsAvailable, 'LaunchTemplateId');
    }
    if (launchTemplateName && launchTemplateName.length > 0) {
      processInArray(launchTemplateName, queryParamsAvailable, 'LaunchTemplateName');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('LaunchTemplate', 'describeLaunchTemplates', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeLaunchTemplates'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes your Elastic IP addresses that are being moved to the EC2-VPC platform, or that are being restored to the EC2-Classic platform. This request does not return information about any other Elastic IP addresses in your account.
   *
   * @function describeMovingAddresses
   * @param {array} filter - One or more filters.         moving-status  - The status of the Elastic IP address ( MovingToVpc  |  RestoringToClassic ).
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {number} maxResults - The maximum number of results to return for the request in a single page. The remaining results of the initial request can be seen by sending another request with the re...(description truncated)
   * @param {string} nextToken - The token for the next page of results.
   * @param {array} publicIp - One or more Elastic IP addresses.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeMovingAddresses(filter, dryRun, maxResults, nextToken, publicIp, callback) {
    const meth = 'adapter-describeMovingAddresses';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeMovingAddresses';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (publicIp && publicIp.length > 0) {
      processInArray(publicIp, queryParamsAvailable, 'PublicIp');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'describeMovingAddresses', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeMovingAddresses'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more of your NAT gateways.
   *
   * @function describeNatGateways
   * @param {array} filter - One or more filters.         nat-gateway-id  - The ID of the NAT gateway.         state  - The state of the NAT gateway ( pending  |  failed  |  available  |  deleting  ...(description truncated)
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {array} natGatewayId - One or more NAT gateway IDs.
   * @param {string} nextToken - The token for the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeNatGateways(filter, maxResults, natGatewayId, nextToken, callback) {
    const meth = 'adapter-describeNatGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeNatGateways';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (natGatewayId && natGatewayId.length > 0) {
      processInArray(natGatewayId, queryParamsAvailable, 'NatGatewayId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Gateway', 'describeNatGateways', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeNatGateways'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more of your network ACLs.   For more information, see  Network ACLs  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function describeNetworkAcls
   * @param {array} filter - One or more filters.         association.association-id  - The ID of an association ID for the ACL.         association.network-acl-id  - The ID of the network ACL invol...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} networkAclId - One or more network ACL IDs.   Default: Describes all your network ACLs.
   * @param {string} nextToken - The token for the next page of results.
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeNetworkAcls(filter, dryRun, networkAclId, nextToken, maxResults, callback) {
    const meth = 'adapter-describeNetworkAcls';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeNetworkAcls';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (networkAclId && networkAclId.length > 0) {
      processInArray(networkAclId, queryParamsAvailable, 'NetworkAclId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkAcl', 'describeNetworkAcls', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeNetworkAcls'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes a network interface attribute. You can specify only one attribute at a time.
   *
   * @function describeNetworkInterfaceAttribute
   * @param {string} attribute - The attribute of the network interface. This parameter is required.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} networkInterfaceId - The ID of the network interface.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeNetworkInterfaceAttribute(attribute = 'description', dryRun, networkInterfaceId, callback) {
    const meth = 'adapter-describeNetworkInterfaceAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeNetworkInterfaceAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (networkInterfaceId === undefined || networkInterfaceId === null || networkInterfaceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['networkInterfaceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Attribute: attribute,
      NetworkInterfaceId: networkInterfaceId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkInterface', 'describeNetworkInterfaceAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeNetworkInterfaceAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the permissions for your network interfaces.
   *
   * @function describeNetworkInterfacePermissions
   * @param {array} networkInterfacePermissionId - One or more network interface permission IDs.
   * @param {array} filter - One or more filters.         network-interface-permission.network-interface-permission-id  - The ID of the permission.         network-interface-permission.network-inter...(description truncated)
   * @param {string} nextToken - The token to request the next page of results.
   * @param {number} maxResults - The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned  NextToken  value. If this parameter is n...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeNetworkInterfacePermissions(networkInterfacePermissionId, filter, nextToken, maxResults, callback) {
    const meth = 'adapter-describeNetworkInterfacePermissions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeNetworkInterfacePermissions';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (networkInterfacePermissionId && networkInterfacePermissionId.length > 0) {
      processInArray(networkInterfacePermissionId, queryParamsAvailable, 'NetworkInterfacePermissionId');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkInterface', 'describeNetworkInterfacePermissions', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeNetworkInterfacePermissions'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more of your network interfaces.
   *
   * @function describeNetworkInterfaces
   * @param {array} filter - One or more filters.         addresses.private-ip-address  - The private IPv4 addresses associated with the network interface.         addresses.primary  - Whether the p...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} networkInterfaceId - One or more network interface IDs.   Default: Describes all your network interfaces.
   * @param {string} nextToken - The token to retrieve the next page of results.
   * @param {number} maxResults - The maximum number of items to return for this request. The request returns a token that you can specify in a subsequent call to get the next set of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeNetworkInterfaces(filter, dryRun, networkInterfaceId, nextToken, maxResults, callback) {
    const meth = 'adapter-describeNetworkInterfaces';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeNetworkInterfaces';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (networkInterfaceId && networkInterfaceId.length > 0) {
      processInArray(networkInterfaceId, queryParamsAvailable, 'NetworkInterfaceId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkInterface', 'describeNetworkInterfaces', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeNetworkInterfaces'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified placement groups or all of your placement groups. For more information, see  Placement Groups  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function describePlacementGroups
   * @param {array} filter - The filters.         group-name  - The name of the placement group.         state  - The state of the placement group ( pending  |  available  |  deleting  |  deleted )....(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} groupName - The names of the placement groups.   Default: Describes all your placement groups, or only those otherwise specified.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describePlacementGroups(filter, dryRun, groupName, callback) {
    const meth = 'adapter-describePlacementGroups';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribePlacementGroups';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (groupName && groupName.length > 0) {
      processInArray(groupName, queryParamsAvailable, 'GroupName');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionDescribePlacementGroups', 'getDescribePlacementGroups', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describePlacementGroups'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes available AWS services in a prefix list format, which includes the prefix list name and prefix list ID of the service and the IP address range for the service. A prefix list ID is required for creating an outbound security group rule that allows traffic from a VPC to access an AWS service through a gateway VPC endpoint. Currently, the services that support this action are Amazon S3 and Amazon DynamoDB.
   *
   * @function describePrefixLists
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} filter - One or more filters.         prefix-list-id : The ID of a prefix list.         prefix-list-name : The name of a prefix list.
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {string} nextToken - The token for the next page of results.
   * @param {array} prefixListId - One or more prefix list IDs.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describePrefixLists(dryRun, filter, maxResults, nextToken, prefixListId, callback) {
    const meth = 'adapter-describePrefixLists';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribePrefixLists';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (prefixListId && prefixListId.length > 0) {
      processInArray(prefixListId, queryParamsAvailable, 'PrefixListId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionDescribePrefixLists', 'getDescribePrefixLists', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describePrefixLists'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the ID format settings for the root user and all IAM roles and IAM users that have explicitly specified a longer ID (17-character ID) preference.    By default, all IAM roles and IAM users default to the same ID settings as the root user, unless they explicitly override the settings. This request is useful for identifying those IAM users and IAM roles that have overridden the default ID settings.   The following resource types support longer IDs:  bundle  |  conversion-task  |  custom...(description truncated)
   *
   * @function describePrincipalIdFormat
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} resource - The type of resource:  bundle  |  conversion-task  |  customer-gateway  |  dhcp-options  |  elastic-ip-allocation  |  elastic-ip-association  |  export-task  |  flow-log ...(description truncated)
   * @param {number} maxResults - The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned NextToken value.
   * @param {string} nextToken - The token to request the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describePrincipalIdFormat(dryRun, resource, maxResults, nextToken, callback) {
    const meth = 'adapter-describePrincipalIdFormat';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribePrincipalIdFormat';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (resource && resource.length > 0) {
      processInArray(resource, queryParamsAvailable, 'Resource');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Id', 'describePrincipalIdFormat', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describePrincipalIdFormat'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified IPv4 address pools.
   *
   * @function describePublicIpv4Pools
   * @param {array} poolId - The IDs of the address pools.
   * @param {string} nextToken - The token for the next page of results.
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describePublicIpv4Pools(poolId, nextToken, maxResults, callback) {
    const meth = 'adapter-describePublicIpv4Pools';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribePublicIpv4Pools';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (poolId && poolId.length > 0) {
      processInArray(poolId, queryParamsAvailable, 'PoolId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'describePublicIpv4Pools', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describePublicIpv4Pools'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the Regions that are currently available to you. The API returns a list of all the Regions, including Regions that are disabled for your account. For information about enabling Regions for your account, see  Enabling and Disabling Regions  in the  AWS Billing and Cost Management User Guide .   For a list of the Regions supported by Amazon EC2, see   Regions and Endpoints .
   *
   * @function describeRegions
   * @param {array} filter - The filters.         endpoint  - The endpoint of the Region (for example,  ec2.us-east-1.amazonaws.com ).         region-name  - The name of the Region (for example,  us...(description truncated)
   * @param {array} regionName - The names of the Regions.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeRegions(filter, regionName, dryRun, callback) {
    const meth = 'adapter-describeRegions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeRegions';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (regionName && regionName.length > 0) {
      processInArray(regionName, queryParamsAvailable, 'RegionName');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionDescribeRegions', 'getDescribeRegions', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeRegions'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more of the Reserved Instances that you purchased.   For more information about Reserved Instances, see  Reserved Instances  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function describeReservedInstances
   * @param {array} filter - One or more filters.         availability-zone  - The Availability Zone where the Reserved Instance can be used.         duration  - The duration of the Reserved Instanc...(description truncated)
   * @param {string} offeringClass - Describes whether the Reserved Instance is Standard or Convertible.
   * @param {array} reservedInstancesId - One or more Reserved Instance IDs.   Default: Describes all your Reserved Instances, or only those otherwise specified.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} offeringType - The Reserved Instance offering type. If you are using tools that predate the 2011-11-01 API version, you only have access to the  Medium Utilization  Reserved Instance of...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeReservedInstances(filter, offeringClass = 'standard', reservedInstancesId, dryRun, offeringType = 'Heavy Utilization', callback) {
    const meth = 'adapter-describeReservedInstances';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeReservedInstances';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      OfferingClass: offeringClass,
      OfferingType: offeringType,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (reservedInstancesId && reservedInstancesId.length > 0) {
      processInArray(reservedInstancesId, queryParamsAvailable, 'ReservedInstancesId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'describeReservedInstances', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeReservedInstances'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes your account's Reserved Instance listings in the Reserved Instance Marketplace.   The Reserved Instance Marketplace matches sellers who want to resell Reserved Instance capacity that they no longer need with buyers who want to purchase additional capacity. Reserved Instances bought and sold through the Reserved Instance Marketplace work like any other Reserved Instances.   As a seller, you choose to list some or all of your Reserved Instances, and you specify the upfront price to rece...(description truncated)
   *
   * @function describeReservedInstancesListings
   * @param {array} filter - One or more filters.         reserved-instances-id  - The ID of the Reserved Instances.         reserved-instances-listing-id  - The ID of the Reserved Instances listing...(description truncated)
   * @param {string} reservedInstancesId - One or more Reserved Instance IDs.
   * @param {string} reservedInstancesListingId - One or more Reserved Instance listing IDs.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeReservedInstancesListings(filter, reservedInstancesId, reservedInstancesListingId, callback) {
    const meth = 'adapter-describeReservedInstancesListings';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeReservedInstancesListings';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      ReservedInstancesId: reservedInstancesId,
      ReservedInstancesListingId: reservedInstancesListingId,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'describeReservedInstancesListings', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeReservedInstancesListings'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the modifications made to your Reserved Instances. If no parameter is specified, information about all your Reserved Instances modification requests is returned. If a modification ID is specified, only information about the specific modification is returned.   For more information, see  Modifying Reserved Instances  in the Amazon Elastic Compute Cloud User Guide.
   *
   * @function describeReservedInstancesModifications
   * @param {array} filter - One or more filters.         client-token  - The idempotency token for the modification request.         create-date  - The time when the modification request was create...(description truncated)
   * @param {array} reservedInstancesModificationId - IDs for the submitted modification request.
   * @param {string} nextToken - The token to retrieve the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeReservedInstancesModifications(filter, reservedInstancesModificationId, nextToken, callback) {
    const meth = 'adapter-describeReservedInstancesModifications';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeReservedInstancesModifications';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      NextToken: nextToken,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (reservedInstancesModificationId && reservedInstancesModificationId.length > 0) {
      processInArray(reservedInstancesModificationId, queryParamsAvailable, 'ReservedInstancesModificationId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'describeReservedInstancesModifications', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeReservedInstancesModifications'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes Reserved Instance offerings that are available for purchase. With Reserved Instances, you purchase the right to launch instances for a period of time. During that time period, you do not receive insufficient capacity errors, and you pay a lower usage rate than the rate charged for On-Demand instances for the actual time used.   If you have listed your own Reserved Instances for sale in the Reserved Instance Marketplace, they will be excluded from these results. This is to ensure that ...(description truncated)
   *
   * @function describeReservedInstancesOfferings
   * @param {string} availabilityZone - The Availability Zone in which the Reserved Instance can be used.
   * @param {array} filter - One or more filters.         availability-zone  - The Availability Zone where the Reserved Instance can be used.         duration  - The duration of the Reserved Instanc...(description truncated)
   * @param {boolean} includeMarketplace - Include Reserved Instance Marketplace offerings in the response.
   * @param {string} instanceType - The instance type that the reservation will cover (for example,  m1.small ). For more information, see  Instance Types  in the  Amazon Elastic Compute Cloud User Guide .
   * @param {number} maxDuration - The maximum duration (in seconds) to filter when searching for offerings.   Default: 94608000 (3 years)
   * @param {number} maxInstanceCount - The maximum number of instances to filter when searching for offerings.   Default: 20
   * @param {number} minDuration - The minimum duration (in seconds) to filter when searching for offerings.   Default: 2592000 (1 month)
   * @param {string} offeringClass - The offering class of the Reserved Instance. Can be  standard  or  convertible .
   * @param {string} productDescription - The Reserved Instance product platform description. Instances that include  (Amazon VPC)  in the description are for use with Amazon VPC.
   * @param {array} reservedInstancesOfferingId - One or more Reserved Instances offering IDs.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} instanceTenancy - The tenancy of the instances covered by the reservation. A Reserved Instance with a tenancy of  dedicated  is applied to instances that run in a VPC on single-tenant har...(description truncated)
   * @param {number} maxResults - The maximum number of results to return for the request in a single page. The remaining results of the initial request can be seen by sending another request with the re...(description truncated)
   * @param {string} nextToken - The token to retrieve the next page of results.
   * @param {string} offeringType - The Reserved Instance offering type. If you are using tools that predate the 2011-11-01 API version, you only have access to the  Medium Utilization  Reserved Instance of...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeReservedInstancesOfferings(availabilityZone, filter, includeMarketplace, instanceType = 't1.micro', maxDuration, maxInstanceCount, minDuration, offeringClass = 'standard', productDescription = 'Linux/UNIX', reservedInstancesOfferingId, dryRun, instanceTenancy = 'default', maxResults, nextToken, offeringType = 'Heavy Utilization', callback) {
    const meth = 'adapter-describeReservedInstancesOfferings';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeReservedInstancesOfferings';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      AvailabilityZone: availabilityZone,
      IncludeMarketplace: includeMarketplace,
      InstanceType: instanceType,
      MaxDuration: maxDuration,
      MaxInstanceCount: maxInstanceCount,
      MinDuration: minDuration,
      OfferingClass: offeringClass,
      ProductDescription: productDescription,
      InstanceTenancy: instanceTenancy,
      OfferingType: offeringType,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (reservedInstancesOfferingId && reservedInstancesOfferingId.length > 0) {
      processInArray(reservedInstancesOfferingId, queryParamsAvailable, 'ReservedInstancesOfferingId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'describeReservedInstancesOfferings', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeReservedInstancesOfferings'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more of your route tables.   Each subnet in your VPC must be associated with a route table. If a subnet is not explicitly associated with any route table, it is implicitly associated with the main route table. This command does not return the subnet ID for implicit associations.   For more information, see  Route Tables  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function describeRouteTables
   * @param {array} filter - One or more filters.         association.route-table-association-id  - The ID of an association ID for the route table.         association.route-table-id  - The ID of t...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} routeTableId - One or more route table IDs.   Default: Describes all your route tables.
   * @param {string} nextToken - The token for the next page of results.
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeRouteTables(filter, dryRun, routeTableId, nextToken, maxResults, callback) {
    const meth = 'adapter-describeRouteTables';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeRouteTables';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (routeTableId && routeTableId.length > 0) {
      processInArray(routeTableId, queryParamsAvailable, 'RouteTableId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Route', 'describeRouteTables', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeRouteTables'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Finds available schedules that meet the specified criteria.   You can search for an available schedule no more than 3 months in advance. You must meet the minimum required duration of 1,200 hours per year. For example, the minimum daily schedule is 4 hours, the minimum weekly schedule is 24 hours, and the minimum monthly schedule is 100 hours.   After you find a schedule that meets your needs, call  PurchaseScheduledInstances  to purchase Scheduled Instances with that schedule.
   *
   * @function describeScheduledInstanceAvailability
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} filter - The filters.         availability-zone  - The Availability Zone (for example,  us-west-2a ).         instance-type  - The instance type (for example,  c4.large ).       ...(description truncated)
   * @param {string} firstSlotStartTimeRangeEarliestTime - Describes the time period for a Scheduled Instance to start its first schedule. The time period must span less than one day.
The earliest date and time, in UTC, for the S...(description truncated)
   * @param {string} firstSlotStartTimeRangeLatestTime - Describes the time period for a Scheduled Instance to start its first schedule. The time period must span less than one day.
The latest date and time, in UTC, for the Sch...(description truncated)
   * @param {number} maxResults - The maximum number of results to return in a single call. This value can be between 5 and 300. The default value is 300. To retrieve the remaining results, make another c...(description truncated)
   * @param {number} maxSlotDurationInHours - The maximum available duration, in hours. This value must be greater than  MinSlotDurationInHours  and less than 1,720.
   * @param {number} minSlotDurationInHours - The minimum available duration, in hours. The minimum required duration is 1,200 hours per year. For example, the minimum daily schedule is 4 hours, the minimum weekly sc...(description truncated)
   * @param {string} nextToken - The token for the next set of results.
   * @param {string} recurrenceFrequency - Describes the recurring schedule for a Scheduled Instance.
The frequency ( Daily ,  Weekly , or  Monthly ).
   * @param {string} recurrenceInterval - Describes the recurring schedule for a Scheduled Instance.
The interval quantity. The interval unit depends on the value of  Frequency . For example, every 2 weeks or eve...(description truncated)
   * @param {array} recurrenceOccurrenceDays - Describes the recurring schedule for a Scheduled Instance.
The days. For a monthly schedule, this is one or more days of the month (1-31). For a weekly schedule, this is ...(description truncated)
   * @param {string} recurrenceOccurrenceRelativeToEnd - Describes the recurring schedule for a Scheduled Instance.
Indicates whether the occurrence is relative to the end of the specified week or month. You can't specify this ...(description truncated)
   * @param {string} recurrenceOccurrenceUnit - Describes the recurring schedule for a Scheduled Instance.
The unit for  OccurrenceDays  ( DayOfWeek  or  DayOfMonth ). This value is required for a monthly schedule. You...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeScheduledInstanceAvailability(dryRun, filter, firstSlotStartTimeRangeEarliestTime, firstSlotStartTimeRangeLatestTime, maxResults, maxSlotDurationInHours, minSlotDurationInHours, nextToken, recurrenceFrequency, recurrenceInterval, recurrenceOccurrenceDays, recurrenceOccurrenceRelativeToEnd, recurrenceOccurrenceUnit, callback) {
    const meth = 'adapter-describeScheduledInstanceAvailability';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeScheduledInstanceAvailability';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      FirstSlotStartTimeRangeEarliestTime: firstSlotStartTimeRangeEarliestTime,
      FirstSlotStartTimeRangeLatestTime: firstSlotStartTimeRangeLatestTime,
      MaxSlotDurationInHours: maxSlotDurationInHours,
      MinSlotDurationInHours: minSlotDurationInHours,
      RecurrenceFrequency: recurrenceFrequency,
      RecurrenceInterval: recurrenceInterval,
      RecurrenceOccurrenceRelativeToEnd: recurrenceOccurrenceRelativeToEnd,
      RecurrenceOccurrenceUnit: recurrenceOccurrenceUnit,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (recurrenceOccurrenceDays && recurrenceOccurrenceDays.length > 0) {
      processInArray(recurrenceOccurrenceDays, queryParamsAvailable, 'RecurrenceOccurrenceDays');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'describeScheduledInstanceAvailability', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeScheduledInstanceAvailability'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified Scheduled Instances or all your Scheduled Instances.
   *
   * @function describeScheduledInstances
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} filter - The filters.         availability-zone  - The Availability Zone (for example,  us-west-2a ).         instance-type  - The instance type (for example,  c4.large ).       ...(description truncated)
   * @param {number} maxResults - The maximum number of results to return in a single call. This value can be between 5 and 300. The default value is 100. To retrieve the remaining results, make another c...(description truncated)
   * @param {string} nextToken - The token for the next set of results.
   * @param {array} scheduledInstanceId - The Scheduled Instance IDs.
   * @param {string} slotStartTimeRangeEarliestTime - Describes the time period for a Scheduled Instance to start its first schedule.
The earliest date and time, in UTC, for the Scheduled Instance to start.
   * @param {string} slotStartTimeRangeLatestTime - Describes the time period for a Scheduled Instance to start its first schedule.
The latest date and time, in UTC, for the Scheduled Instance to start.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeScheduledInstances(dryRun, filter, maxResults, nextToken, scheduledInstanceId, slotStartTimeRangeEarliestTime, slotStartTimeRangeLatestTime, callback) {
    const meth = 'adapter-describeScheduledInstances';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeScheduledInstances';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      SlotStartTimeRangeEarliestTime: slotStartTimeRangeEarliestTime,
      SlotStartTimeRangeLatestTime: slotStartTimeRangeLatestTime,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (scheduledInstanceId && scheduledInstanceId.length > 0) {
      processInArray(scheduledInstanceId, queryParamsAvailable, 'ScheduledInstanceId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'describeScheduledInstances', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeScheduledInstances'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary [VPC only] Describes the VPCs on the other side of a VPC peering connection that are referencing the security groups you've specified in this request.
   *
   * @function describeSecurityGroupReferences
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} groupId - The IDs of the security groups in your account.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeSecurityGroupReferences(dryRun, groupId, callback) {
    const meth = 'adapter-describeSecurityGroupReferences';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeSecurityGroupReferences';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (groupId === undefined || groupId === null || groupId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['groupId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (groupId && groupId.length > 0) {
      processInArray(groupId, queryParamsAvailable, 'GroupId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('SecurityGroup', 'describeSecurityGroupReferences', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeSecurityGroupReferences'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified security groups or all of your security groups.   A security group is for use with instances either in the EC2-Classic platform or in a specific VPC. For more information, see  Amazon EC2 Security Groups  in the  Amazon Elastic Compute Cloud User Guide  and  Security Groups for Your VPC  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function describeSecurityGroups
   * @param {array} filter - The filters. If using multiple filters for rules, the results include security groups for which any combination of rules - not necessarily a single rule - match all filt...(description truncated)
   * @param {array} groupId - The IDs of the security groups. Required for security groups in a nondefault VPC.   Default: Describes all your security groups.
   * @param {array} groupName - [EC2-Classic and default VPC only] The names of the security groups. You can specify either the security group name or the security group ID. For security groups in a no...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} nextToken - The token to request the next page of results.
   * @param {number} maxResults - The maximum number of results to return in a single call. To retrieve the remaining results, make another request with the returned  NextToken  value. This value can be b...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeSecurityGroups(filter, groupId, groupName, dryRun, nextToken, maxResults, callback) {
    const meth = 'adapter-describeSecurityGroups';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeSecurityGroups';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (groupId && groupId.length > 0) {
      processInArray(groupId, queryParamsAvailable, 'GroupId');
    }
    if (groupName && groupName.length > 0) {
      processInArray(groupName, queryParamsAvailable, 'GroupName');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('SecurityGroup', 'describeSecurityGroups', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeSecurityGroups'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified attribute of the specified snapshot. You can specify only one attribute at a time.   For more information about EBS snapshots, see  Amazon EBS Snapshots  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function describeSnapshotAttribute
   * @param {string} attribute - The snapshot attribute you would like to view.
   * @param {string} snapshotId - The ID of the EBS snapshot.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeSnapshotAttribute(attribute = 'productCodes', snapshotId, dryRun, callback) {
    const meth = 'adapter-describeSnapshotAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeSnapshotAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (attribute === undefined || attribute === null || attribute === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['attribute'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (snapshotId === undefined || snapshotId === null || snapshotId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['snapshotId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Attribute: attribute,
      SnapshotId: snapshotId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Snapshot', 'describeSnapshotAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeSnapshotAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified EBS snapshots available to you or all of the EBS snapshots available to you.   The snapshots available to you include public snapshots, private snapshots that you own, and private snapshots owned by other AWS accounts for which you have explicit create volume permissions.   The create volume permissions fall into the following categories:         public : The owner of the snapshot granted create volume permissions for the snapshot to the  all  group. All AWS accounts hav...(description truncated)
   *
   * @function describeSnapshots
   * @param {array} filter - The filters.         description  - A description of the snapshot.         encrypted  - Indicates whether the snapshot is encrypted ( true  |  false )         owner-alia...(description truncated)
   * @param {number} maxResults - The maximum number of snapshot results returned by  DescribeSnapshots  in paginated output. When this parameter is used,  DescribeSnapshots  only returns  MaxResults  res...(description truncated)
   * @param {string} nextToken - The  NextToken  value returned from a previous paginated  DescribeSnapshots  request where  MaxResults  was used and the results exceeded the value of that parameter. Pag...(description truncated)
   * @param {array} owner - Describes the snapshots owned by these owners.
   * @param {array} restorableBy - The IDs of the AWS accounts that can create volumes from the snapshot.
   * @param {array} snapshotId - The snapshot IDs.   Default: Describes the snapshots for which you have create volume permissions.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeSnapshots(filter, maxResults, nextToken, owner, restorableBy, snapshotId, dryRun, callback) {
    const meth = 'adapter-describeSnapshots';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeSnapshots';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (owner && owner.length > 0) {
      processInArray(owner, queryParamsAvailable, 'Owner');
    }
    if (restorableBy && restorableBy.length > 0) {
      processInArray(restorableBy, queryParamsAvailable, 'RestorableBy');
    }
    if (snapshotId && snapshotId.length > 0) {
      processInArray(snapshotId, queryParamsAvailable, 'SnapshotId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Snapshot', 'describeSnapshots', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeSnapshots'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the data feed for Spot Instances. For more information, see  Spot Instance Data Feed  in the  Amazon EC2 User Guide for Linux Instances .
   *
   * @function describeSpotDatafeedSubscription
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeSpotDatafeedSubscription(dryRun, callback) {
    const meth = 'adapter-describeSpotDatafeedSubscription';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeSpotDatafeedSubscription';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionDescribeSpotDatafeedSubscription', 'getDescribeSpotDatafeedSubscription', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeSpotDatafeedSubscription'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the running instances for the specified Spot Fleet.
   *
   * @function describeSpotFleetInstances
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {number} maxResults - The maximum number of results to return in a single call. Specify a value between 1 and 1000. The default value is 1000. To retrieve the remaining results, make another c...(description truncated)
   * @param {string} nextToken - The token for the next set of results.
   * @param {string} spotFleetRequestId - The ID of the Spot Fleet request.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeSpotFleetInstances(dryRun, maxResults, nextToken, spotFleetRequestId, callback) {
    const meth = 'adapter-describeSpotFleetInstances';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeSpotFleetInstances';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (spotFleetRequestId === undefined || spotFleetRequestId === null || spotFleetRequestId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['spotFleetRequestId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      SpotFleetRequestId: spotFleetRequestId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'describeSpotFleetInstances', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeSpotFleetInstances'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the events for the specified Spot Fleet request during the specified time.   Spot Fleet events are delayed by up to 30 seconds before they can be described. This ensures that you can query by the last evaluated time and not miss a recorded event. Spot Fleet events are available for 48 hours.
   *
   * @function describeSpotFleetRequestHistory
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} eventType - The type of events to describe. By default, all events are described.
   * @param {number} maxResults - The maximum number of results to return in a single call. Specify a value between 1 and 1000. The default value is 1000. To retrieve the remaining results, make another c...(description truncated)
   * @param {string} nextToken - The token for the next set of results.
   * @param {string} spotFleetRequestId - The ID of the Spot Fleet request.
   * @param {string} startTime - The starting date and time for the events, in UTC format (for example,  YYYY - MM - DD T HH : MM : SS Z).
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeSpotFleetRequestHistory(dryRun, eventType = 'instanceChange', maxResults, nextToken, spotFleetRequestId, startTime, callback) {
    const meth = 'adapter-describeSpotFleetRequestHistory';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeSpotFleetRequestHistory';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (spotFleetRequestId === undefined || spotFleetRequestId === null || spotFleetRequestId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['spotFleetRequestId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (startTime === undefined || startTime === null || startTime === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['startTime'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      EventType: eventType,
      SpotFleetRequestId: spotFleetRequestId,
      StartTime: startTime,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Fleet', 'describeSpotFleetRequestHistory', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeSpotFleetRequestHistory'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes your Spot Fleet requests.   Spot Fleet requests are deleted 48 hours after they are canceled and their instances are terminated.
   *
   * @function describeSpotFleetRequests
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {number} maxResults - The maximum number of results to return in a single call. Specify a value between 1 and 1000. The default value is 1000. To retrieve the remaining results, make another c...(description truncated)
   * @param {string} nextToken - The token for the next set of results.
   * @param {array} spotFleetRequestId - The IDs of the Spot Fleet requests.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeSpotFleetRequests(dryRun, maxResults, nextToken, spotFleetRequestId, callback) {
    const meth = 'adapter-describeSpotFleetRequests';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeSpotFleetRequests';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (spotFleetRequestId && spotFleetRequestId.length > 0) {
      processInArray(spotFleetRequestId, queryParamsAvailable, 'SpotFleetRequestId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Fleet', 'describeSpotFleetRequests', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeSpotFleetRequests'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified Spot Instance requests.   You can use  DescribeSpotInstanceRequests  to find a running Spot Instance by examining the response. If the status of the Spot Instance is  fulfilled , the instance ID appears in the response and contains the identifier of the instance. Alternatively, you can use  DescribeInstances  with a filter to look for instances where the instance lifecycle is  spot .   We recommend that you set  MaxResults  to a value between 5 and 1000 to limit the numb...(description truncated)
   *
   * @function describeSpotInstanceRequests
   * @param {array} filter - One or more filters.         availability-zone-group  - The Availability Zone group.         create-time  - The time stamp when the Spot Instance request was created.   ...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} spotInstanceRequestId - One or more Spot Instance request IDs.
   * @param {string} nextToken - The token to request the next set of results. This value is  null  when there are no more results to return.
   * @param {number} maxResults - The maximum number of results to return in a single call. Specify a value between 5 and 1000. To retrieve the remaining results, make another call with the returned  Next...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeSpotInstanceRequests(filter, dryRun, spotInstanceRequestId, nextToken, maxResults, callback) {
    const meth = 'adapter-describeSpotInstanceRequests';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeSpotInstanceRequests';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (spotInstanceRequestId && spotInstanceRequestId.length > 0) {
      processInArray(spotInstanceRequestId, queryParamsAvailable, 'SpotInstanceRequestId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'describeSpotInstanceRequests', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeSpotInstanceRequests'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the Spot price history. For more information, see  Spot Instance Pricing History  in the  Amazon EC2 User Guide for Linux Instances .   When you specify a start and end time, this operation returns the prices of the instance types within the time range that you specified and the time when the price changed. The price is valid within the time period that you specified; the response merely indicates the last time that the price changed.
   *
   * @function describeSpotPriceHistory
   * @param {array} filter - One or more filters.         availability-zone  - The Availability Zone for which prices should be returned.         instance-type  - The type of instance (for example, ...(description truncated)
   * @param {string} availabilityZone - Filters the results by the specified Availability Zone.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} endTime - The date and time, up to the current date, from which to stop retrieving the price history data, in UTC format (for example,  YYYY - MM - DD T HH : MM : SS Z).
   * @param {array} instanceType - Filters the results by the specified instance types.
   * @param {number} maxResults - The maximum number of results to return in a single call. Specify a value between 1 and 1000. The default value is 1000. To retrieve the remaining results, make another c...(description truncated)
   * @param {string} nextToken - The token for the next set of results.
   * @param {array} productDescription - Filters the results by the specified basic product descriptions.
   * @param {string} startTime - The date and time, up to the past 90 days, from which to start retrieving the price history data, in UTC format (for example,  YYYY - MM - DD T HH : MM : SS Z).
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeSpotPriceHistory(filter, availabilityZone, dryRun, endTime, instanceType, maxResults, nextToken, productDescription, startTime, callback) {
    const meth = 'adapter-describeSpotPriceHistory';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeSpotPriceHistory';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      AvailabilityZone: availabilityZone,
      EndTime: endTime,
      StartTime: startTime,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (instanceType && instanceType.length > 0) {
      processInArray(instanceType, queryParamsAvailable, 'InstanceType');
    }
    if (productDescription && productDescription.length > 0) {
      processInArray(productDescription, queryParamsAvailable, 'ProductDescription');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionDescribeSpotPriceHistory', 'getDescribeSpotPriceHistory', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeSpotPriceHistory'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary [VPC only] Describes the stale security group rules for security groups in a specified VPC. Rules are stale when they reference a deleted security group in a peer VPC, or a security group in a peer VPC for which the VPC peering connection has been deleted.
   *
   * @function describeStaleSecurityGroups
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {number} maxResults - The maximum number of items to return for this request. The request returns a token that you can specify in a subsequent call to get the next set of results.
   * @param {string} nextToken - The token for the next set of items to return. (You received this token from a prior call.)
   * @param {string} vpcId - The ID of the VPC.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeStaleSecurityGroups(dryRun, maxResults, nextToken, vpcId, callback) {
    const meth = 'adapter-describeStaleSecurityGroups';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeStaleSecurityGroups';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('SecurityGroup', 'describeStaleSecurityGroups', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeStaleSecurityGroups'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more of your subnets.   For more information, see  Your VPC and Subnets  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function describeSubnets
   * @param {array} filter - One or more filters.         availability-zone  - The Availability Zone for the subnet. You can also use  availabilityZone  as the filter name.         availability-zone...(description truncated)
   * @param {array} subnetId - One or more subnet IDs.   Default: Describes all your subnets.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} nextToken - The token for the next page of results.
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeSubnets(filter, subnetId, dryRun, nextToken, maxResults, callback) {
    const meth = 'adapter-describeSubnets';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeSubnets';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (subnetId && subnetId.length > 0) {
      processInArray(subnetId, queryParamsAvailable, 'SubnetId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Subnet', 'describeSubnets', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeSubnets'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified tags for your EC2 resources.   For more information about tags, see  Tagging Your Resources  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function describeTags
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} filter - The filters.         key  - The tag key.         resource-id  - The ID of the resource.         resource-type  - The resource type ( customer-gateway  |  dedicated-host ...(description truncated)
   * @param {number} maxResults - The maximum number of results to return in a single call. This value can be between 5 and 1000. To retrieve the remaining results, make another call with the returned  Ne...(description truncated)
   * @param {string} nextToken - The token to retrieve the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeTags(dryRun, filter, maxResults, nextToken, callback) {
    const meth = 'adapter-describeTags';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeTags';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionDescribeTags', 'getDescribeTags', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeTags'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more attachments between resources and transit gateways. By default, all attachments are described. Alternatively, you can filter the results by attachment ID, attachment state, resource ID, or resource owner.
   *
   * @function describeTransitGatewayAttachments
   * @param {array} transitGatewayAttachmentIds - The IDs of the attachments.
   * @param {array} filter - One or more filters. The possible values are:         association.state  - The state of the association ( associating  |  associated  |  disassociating ).         associ...(description truncated)
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {string} nextToken - The token for the next page of results.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeTransitGatewayAttachments(transitGatewayAttachmentIds, filter, maxResults, nextToken, dryRun, callback) {
    const meth = 'adapter-describeTransitGatewayAttachments';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeTransitGatewayAttachments';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (transitGatewayAttachmentIds && transitGatewayAttachmentIds.length > 0) {
      processInArray(transitGatewayAttachmentIds, queryParamsAvailable, 'TransitGatewayAttachmentIds');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'describeTransitGatewayAttachments', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeTransitGatewayAttachments'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more transit gateway route tables. By default, all transit gateway route tables are described. Alternatively, you can filter the results.
   *
   * @function describeTransitGatewayRouteTables
   * @param {array} transitGatewayRouteTableIds - The IDs of the transit gateway route tables.
   * @param {array} filter - One or more filters. The possible values are:         default-association-route-table  - Indicates whether this is the default association route table for the transit ga...(description truncated)
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {string} nextToken - The token for the next page of results.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeTransitGatewayRouteTables(transitGatewayRouteTableIds, filter, maxResults, nextToken, dryRun, callback) {
    const meth = 'adapter-describeTransitGatewayRouteTables';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeTransitGatewayRouteTables';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (transitGatewayRouteTableIds && transitGatewayRouteTableIds.length > 0) {
      processInArray(transitGatewayRouteTableIds, queryParamsAvailable, 'TransitGatewayRouteTableIds');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'describeTransitGatewayRouteTables', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeTransitGatewayRouteTables'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more VPC attachments. By default, all VPC attachments are described. Alternatively, you can filter the results.
   *
   * @function describeTransitGatewayVpcAttachments
   * @param {array} transitGatewayAttachmentIds - The IDs of the attachments.
   * @param {array} filter - One or more filters. The possible values are:         state  - The state of the attachment ( available  |  deleted  |  deleting  |  failed  |  modifying  |  pendingAccep...(description truncated)
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {string} nextToken - The token for the next page of results.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeTransitGatewayVpcAttachments(transitGatewayAttachmentIds, filter, maxResults, nextToken, dryRun, callback) {
    const meth = 'adapter-describeTransitGatewayVpcAttachments';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeTransitGatewayVpcAttachments';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (transitGatewayAttachmentIds && transitGatewayAttachmentIds.length > 0) {
      processInArray(transitGatewayAttachmentIds, queryParamsAvailable, 'TransitGatewayAttachmentIds');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'describeTransitGatewayVpcAttachments', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeTransitGatewayVpcAttachments'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more transit gateways. By default, all transit gateways are described. Alternatively, you can filter the results.
   *
   * @function describeTransitGateways
   * @param {array} transitGatewayIds - The IDs of the transit gateways.
   * @param {array} filter - One or more filters. The possible values are:         options.propagation-default-route-table-id  - The ID of the default propagation route table.         options.amazon...(description truncated)
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {string} nextToken - The token for the next page of results.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeTransitGateways(transitGatewayIds, filter, maxResults, nextToken, dryRun, callback) {
    const meth = 'adapter-describeTransitGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeTransitGateways';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (transitGatewayIds && transitGatewayIds.length > 0) {
      processInArray(transitGatewayIds, queryParamsAvailable, 'TransitGatewayIds');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'describeTransitGateways', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeTransitGateways'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified attribute of the specified volume. You can specify only one attribute at a time.   For more information about EBS volumes, see  Amazon EBS Volumes  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function describeVolumeAttribute
   * @param {string} attribute - The attribute of the volume. This parameter is required.
   * @param {string} volumeId - The ID of the volume.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVolumeAttribute(attribute = 'autoEnableIO', volumeId, dryRun, callback) {
    const meth = 'adapter-describeVolumeAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeVolumeAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (attribute === undefined || attribute === null || attribute === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['attribute'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (volumeId === undefined || volumeId === null || volumeId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['volumeId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Attribute: attribute,
      VolumeId: volumeId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Volume', 'describeVolumeAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeVolumeAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the status of the specified volumes. Volume status provides the result of the checks performed on your volumes to determine events that can impair the performance of your volumes. The performance of a volume can be affected if an issue occurs on the volume's underlying host. If the volume's underlying host experiences a power outage or system issue, after the system is restored, there could be data inconsistencies on the volume. Volume events notify you if this occurs. Volume actions ...(description truncated)
   *
   * @function describeVolumeStatus
   * @param {array} filter - The filters.         action.code  - The action code for the event (for example,  enable-volume-io ).         action.description  - A description of the action.         a...(description truncated)
   * @param {number} maxResults - The maximum number of volume results returned by  DescribeVolumeStatus  in paginated output. When this parameter is used, the request only returns  MaxResults  results in...(description truncated)
   * @param {string} nextToken - The  NextToken  value to include in a future  DescribeVolumeStatus  request. When the results of the request exceed  MaxResults , this value can be used to retrieve the n...(description truncated)
   * @param {array} volumeId - The IDs of the volumes.   Default: Describes all your volumes.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVolumeStatus(filter, maxResults, nextToken, volumeId, dryRun, callback) {
    const meth = 'adapter-describeVolumeStatus';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeVolumeStatus';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (volumeId && volumeId.length > 0) {
      processInArray(volumeId, queryParamsAvailable, 'VolumeId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Volume', 'describeVolumeStatus', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeVolumeStatus'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified EBS volumes or all of your EBS volumes.   If you are describing a long list of volumes, you can paginate the output to make the list more manageable. The  MaxResults  parameter sets the maximum number of results returned in a single page. If the list of results exceeds your  MaxResults  value, then that number of results is returned along with a  NextToken  value that can be passed to a subsequent  DescribeVolumes  request to retrieve the remaining results.   For more in...(description truncated)
   *
   * @function describeVolumes
   * @param {array} filter - The filters.         attachment.attach-time  - The time stamp when the attachment initiated.         attachment.delete-on-termination  - Whether the volume is deleted on...(description truncated)
   * @param {array} volumeId - The volume IDs.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {number} maxResults - The maximum number of volume results returned by  DescribeVolumes  in paginated output. When this parameter is used,  DescribeVolumes  only returns  MaxResults  results i...(description truncated)
   * @param {string} nextToken - The  NextToken  value returned from a previous paginated  DescribeVolumes  request where  MaxResults  was used and the results exceeded the value of that parameter. Pagin...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVolumes(filter, volumeId, dryRun, maxResults, nextToken, callback) {
    const meth = 'adapter-describeVolumes';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeVolumes';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (volumeId && volumeId.length > 0) {
      processInArray(volumeId, queryParamsAvailable, 'VolumeId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Volume', 'describeVolumes', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeVolumes'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Reports the current modification status of EBS volumes.   Current-generation EBS volumes support modification of attributes including type, size, and (for  io1  volumes) IOPS provisioning while either attached to or detached from an instance. Following an action from the API or the console to modify a volume, the status of the modification may be  modifying ,  optimizing ,  completed , or  failed . If a volume has never been modified, then certain elements of the returned  VolumeModification  o...(description truncated)
   *
   * @function describeVolumesModifications
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} volumeId - The IDs of the volumes for which in-progress modifications will be described.
   * @param {array} filter - The filters. Supported filters:  volume-id ,  modification-state ,  target-size ,  target-iops ,  target-volume-type ,  original-size ,  original-iops ,  original-volume-...(description truncated)
   * @param {string} nextToken - The  nextToken  value returned by a previous paginated request.
   * @param {number} maxResults - The maximum number of results (up to a limit of 500) to be returned in a paginated request.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVolumesModifications(dryRun, volumeId, filter, nextToken, maxResults, callback) {
    const meth = 'adapter-describeVolumesModifications';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeVolumesModifications';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (volumeId && volumeId.length > 0) {
      processInArray(volumeId, queryParamsAvailable, 'VolumeId');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Volume', 'describeVolumesModifications', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeVolumesModifications'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the specified attribute of the specified VPC. You can specify only one attribute at a time.
   *
   * @function describeVpcAttribute
   * @param {string} attribute - The VPC attribute.
   * @param {string} vpcId - The ID of the VPC.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVpcAttribute(attribute = 'enableDnsSupport', vpcId, dryRun, callback) {
    const meth = 'adapter-describeVpcAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeVpcAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (attribute === undefined || attribute === null || attribute === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['attribute'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Attribute: attribute,
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'describeVpcAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeVpcAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the ClassicLink status of one or more VPCs.
   *
   * @function describeVpcClassicLink
   * @param {array} filter - One or more filters.         is-classic-link-enabled  - Whether the VPC is enabled for ClassicLink ( true  |  false ).         tag :&lt;key&gt; - The key/value combinati...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} vpcId - One or more VPCs for which you want to describe the ClassicLink status.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVpcClassicLink(filter, dryRun, vpcId, callback) {
    const meth = 'adapter-describeVpcClassicLink';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeVpcClassicLink';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (vpcId && vpcId.length > 0) {
      processInArray(vpcId, queryParamsAvailable, 'VpcId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'describeVpcClassicLink', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeVpcClassicLink'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the ClassicLink DNS support status of one or more VPCs. If enabled, the DNS hostname of a linked EC2-Classic instance resolves to its private IP address when addressed from an instance in the VPC to which it's linked. Similarly, the DNS hostname of an instance in a VPC resolves to its private IP address when addressed from a linked EC2-Classic instance. For more information, see  ClassicLink  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function describeVpcClassicLinkDnsSupport
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {string} nextToken - The token for the next page of results.
   * @param {array} vpcIds - One or more VPC IDs.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVpcClassicLinkDnsSupport(maxResults, nextToken, vpcIds, callback) {
    const meth = 'adapter-describeVpcClassicLinkDnsSupport';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeVpcClassicLinkDnsSupport';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (vpcIds && vpcIds.length > 0) {
      processInArray(vpcIds, queryParamsAvailable, 'VpcIds');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'describeVpcClassicLinkDnsSupport', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeVpcClassicLinkDnsSupport'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the connection notifications for VPC endpoints and VPC endpoint services.
   *
   * @function describeVpcEndpointConnectionNotifications
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} connectionNotificationId - The ID of the notification.
   * @param {array} filter - One or more filters.         connection-notification-arn  - The ARN of SNS topic for the notification.         connection-notification-id  - The ID of the notification. ...(description truncated)
   * @param {number} maxResults - The maximum number of results to return in a single call. To retrieve the remaining results, make another request with the returned  NextToken  value.
   * @param {string} nextToken - The token to request the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVpcEndpointConnectionNotifications(dryRun, connectionNotificationId, filter, maxResults, nextToken, callback) {
    const meth = 'adapter-describeVpcEndpointConnectionNotifications';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeVpcEndpointConnectionNotifications';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      ConnectionNotificationId: connectionNotificationId,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'describeVpcEndpointConnectionNotifications', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeVpcEndpointConnectionNotifications'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the VPC endpoint connections to your VPC endpoint services, including any endpoints that are pending your acceptance.
   *
   * @function describeVpcEndpointConnections
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} filter - One or more filters.         service-id  - The ID of the service.         vpc-endpoint-owner  - The AWS account number of the owner of the endpoint.         vpc-endpoint...(description truncated)
   * @param {number} maxResults - The maximum number of results to return for the request in a single page. The remaining results of the initial request can be seen by sending another request with the ret...(description truncated)
   * @param {string} nextToken - The token to retrieve the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVpcEndpointConnections(dryRun, filter, maxResults, nextToken, callback) {
    const meth = 'adapter-describeVpcEndpointConnections';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeVpcEndpointConnections';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'describeVpcEndpointConnections', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeVpcEndpointConnections'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the VPC endpoint service configurations in your account (your services).
   *
   * @function describeVpcEndpointServiceConfigurations
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} serviceId - The IDs of one or more services.
   * @param {array} filter - One or more filters.         service-name  - The name of the service.         service-id  - The ID of the service.         service-state  - The state of the service ( Pe...(description truncated)
   * @param {number} maxResults - The maximum number of results to return for the request in a single page. The remaining results of the initial request can be seen by sending another request with the ret...(description truncated)
   * @param {string} nextToken - The token to retrieve the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVpcEndpointServiceConfigurations(dryRun, serviceId, filter, maxResults, nextToken, callback) {
    const meth = 'adapter-describeVpcEndpointServiceConfigurations';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeVpcEndpointServiceConfigurations';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (serviceId && serviceId.length > 0) {
      processInArray(serviceId, queryParamsAvailable, 'ServiceId');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'describeVpcEndpointServiceConfigurations', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeVpcEndpointServiceConfigurations'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes the principals (service consumers) that are permitted to discover your VPC endpoint service.
   *
   * @function describeVpcEndpointServicePermissions
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} serviceId - The ID of the service.
   * @param {array} filter - One or more filters.         principal  - The ARN of the principal.         principal-type  - The principal type ( All  |  Service  |  OrganizationUnit  |  Account  |  U...(description truncated)
   * @param {number} maxResults - The maximum number of results to return for the request in a single page. The remaining results of the initial request can be seen by sending another request with the ret...(description truncated)
   * @param {string} nextToken - The token to retrieve the next page of results.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVpcEndpointServicePermissions(dryRun, serviceId, filter, maxResults, nextToken, callback) {
    const meth = 'adapter-describeVpcEndpointServicePermissions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeVpcEndpointServicePermissions';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (serviceId === undefined || serviceId === null || serviceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['serviceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      ServiceId: serviceId,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'describeVpcEndpointServicePermissions', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeVpcEndpointServicePermissions'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes available services to which you can create a VPC endpoint.
   *
   * @function describeVpcEndpointServices
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} serviceName - One or more service names.
   * @param {array} filter - One or more filters.         service-name : The name of the service.         tag :&lt;key&gt; - The key/value combination of a tag assigned to the resource. Use the tag ...(description truncated)
   * @param {number} maxResults - The maximum number of items to return for this request. The request returns a token that you can specify in a subsequent call to get the next set of results.   Constrain...(description truncated)
   * @param {string} nextToken - The token for the next set of items to return. (You received this token from a prior call.)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVpcEndpointServices(dryRun, serviceName, filter, maxResults, nextToken, callback) {
    const meth = 'adapter-describeVpcEndpointServices';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeVpcEndpointServices';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (serviceName && serviceName.length > 0) {
      processInArray(serviceName, queryParamsAvailable, 'ServiceName');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'describeVpcEndpointServices', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeVpcEndpointServices'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more of your VPC endpoints.
   *
   * @function describeVpcEndpoints
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} vpcEndpointId - One or more endpoint IDs.
   * @param {array} filter - One or more filters.         service-name : The name of the service.         vpc-id : The ID of the VPC in which the endpoint resides.         vpc-endpoint-id : The ID o...(description truncated)
   * @param {number} maxResults - The maximum number of items to return for this request. The request returns a token that you can specify in a subsequent call to get the next set of results.   Constrain...(description truncated)
   * @param {string} nextToken - The token for the next set of items to return. (You received this token from a prior call.)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVpcEndpoints(dryRun, vpcEndpointId, filter, maxResults, nextToken, callback) {
    const meth = 'adapter-describeVpcEndpoints';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeVpcEndpoints';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (vpcEndpointId && vpcEndpointId.length > 0) {
      processInArray(vpcEndpointId, queryParamsAvailable, 'VpcEndpointId');
    }
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'describeVpcEndpoints', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeVpcEndpoints'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more of your VPC peering connections.
   *
   * @function describeVpcPeeringConnections
   * @param {array} filter - One or more filters.         accepter-vpc-info.cidr-block  - The IPv4 CIDR block of the accepter VPC.         accepter-vpc-info.owner-id  - The AWS account ID of the own...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} vpcPeeringConnectionId - One or more VPC peering connection IDs.   Default: Describes all your VPC peering connections.
   * @param {string} nextToken - The token for the next page of results.
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVpcPeeringConnections(filter, dryRun, vpcPeeringConnectionId, nextToken, maxResults, callback) {
    const meth = 'adapter-describeVpcPeeringConnections';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeVpcPeeringConnections';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (vpcPeeringConnectionId && vpcPeeringConnectionId.length > 0) {
      processInArray(vpcPeeringConnectionId, queryParamsAvailable, 'VpcPeeringConnectionId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'describeVpcPeeringConnections', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeVpcPeeringConnections'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more of your VPCs.
   *
   * @function describeVpcs
   * @param {array} filter - One or more filters.         cidr  - The primary IPv4 CIDR block of the VPC. The CIDR block you specify must exactly match the VPC's CIDR block for information to be ret...(description truncated)
   * @param {array} vpcId - One or more VPC IDs.   Default: Describes all your VPCs.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} nextToken - The token for the next page of results.
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVpcs(filter, vpcId, dryRun, nextToken, maxResults, callback) {
    const meth = 'adapter-describeVpcs';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeVpcs';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (vpcId && vpcId.length > 0) {
      processInArray(vpcId, queryParamsAvailable, 'VpcId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'describeVpcs', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeVpcs'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes all the VPC details associated with a vpc-id.
   *
   * @function describeVpcDetails
   * @param {string} vpcId - a vpc id.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVpcDetails(vpcId, callback) {
    const meth = 'adapter-describeVpcDetails';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcId === undefined || vpcId === null || vpcId === '' || vpcId.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    const vpcPromises = [];
    let filter = [];
    try {
      // Perform component calls here.
      vpcPromises.push(
        new Promise((resolve) => {
          // set filter for describeInternetGateways
          filter = [{
            Name: 'attachment.vpc-id',
            Value: vpcId
          }];
          this.describeInternetGateways(filter, null, null, null, null, (InternetGatewaysResult, InternetGatewaysError) => {
            // return an error
            if (InternetGatewaysError) {
              const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Unable to get VPC internet gateways details', ['describeVpcDetails'], null, null, null);
              log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
              resolve({ action: 'describeInternetGateways', icode: 400, response: InternetGatewaysError });
            }
            // return the data
            resolve({ action: 'describeInternetGateways', icode: InternetGatewaysResult.icode, response: InternetGatewaysResult.response.DescribeInternetGatewaysResponse.internetGatewaySet, metrics: InternetGatewaysResult.metrics });
          });
        })
      );

      vpcPromises.push(
        new Promise((resolve) => {
          // set filter for describeSubnets
          filter = [{
            Name: 'vpc-id',
            Value: vpcId
          }];
          this.describeSubnets(filter, null, null, null, null, (SubnetsResult, SubnetsError) => {
            // return an error
            if (SubnetsError) {
              const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Unable to get VPC subnets details', ['describeVpcDetails'], null, null, null);
              log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
              resolve({ action: 'describeSubnets', icode: 400, response: SubnetsError });
            }
            // return the data
            resolve({ action: 'describeSubnets', icode: SubnetsResult.icode, response: SubnetsResult.response.DescribeSubnetsResponse.subnetSet, metrics: SubnetsResult.metrics });
          });
        })
      );

      vpcPromises.push(
        new Promise((resolve) => {
          // set filter for describeRouteTables
          filter = [{
            Name: 'vpc-id',
            Value: vpcId
          }];
          this.describeRouteTables(filter, null, null, null, null, (RouteTablesResult, RouteTablesError) => {
            // return an error
            if (RouteTablesError) {
              const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Unable to get VPC route table details', ['describeVpcDetails'], null, null, null);
              log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
              resolve({ action: 'describeRouteTables', icode: 400, response: RouteTablesError });
            }
            // return the data
            resolve({ action: 'describeRouteTables', icode: RouteTablesResult.icode, response: RouteTablesResult.response.DescribeRouteTablesResponse.routeTableSet, metrics: RouteTablesResult.metrics });
          });
        })
      );

      vpcPromises.push(
        new Promise((resolve) => {
          // set filter for describeNetworkAcls
          filter = [{
            Name: 'vpc-id',
            Value: vpcId
          }];
          this.describeNetworkAcls(filter, null, null, null, null, (NetworkAclsResult, NetworkAclsError) => {
            // return an error
            if (NetworkAclsError) {
              const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Unable to get VPC networkAclSet details', ['describeVpcDetails'], null, null, null);
              log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
              resolve({ action: 'describeNetworkAcls', icode: 400, response: NetworkAclsError });
            }
            // return the data
            resolve({ action: 'describeNetworkAcls', icode: NetworkAclsResult.icode, response: NetworkAclsResult.response.DescribeNetworkAclsResponse.networkAclSet, metrics: NetworkAclsResult.metrics });
          });
        })
      );

      vpcPromises.push(
        new Promise((resolve) => {
          // set filter for describeVpcPeeringConnections
          filter = [{
            Name: 'requester-vpc-info.vpc-id',
            Value: vpcId
          }];
          this.describeVpcPeeringConnections(filter, null, null, null, null, (VpcPeeringConnectionsResult, VpcPeeringConnectionsError) => {
            // return an error
            if (VpcPeeringConnectionsError) {
              const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Unable to get VPC vpcPeeringConnectionSet details', ['describeVpcDetails'], null, null, null);
              log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
              resolve({ action: 'describeVpcPeeringConnections', icode: 400, response: VpcPeeringConnectionsError });
            }
            // return the data
            resolve({ action: 'describeVpcPeeringConnections', icode: VpcPeeringConnectionsResult.icode, response: VpcPeeringConnectionsResult.response.DescribeVpcPeeringConnectionsResponse.vpcPeeringConnectionSet, metrics: VpcPeeringConnectionsResult.metrics });
          });
        })
      );

      vpcPromises.push(
        new Promise((resolve) => {
          // set filter for describeVpcEndpoints
          filter = [{
            Name: 'vpc-id',
            Value: vpcId
          }];
          this.describeVpcEndpoints(null, null, filter, null, null, (VpcEndpointsResult, VpcEndpointsError) => {
            // return an error
            if (VpcEndpointsError) {
              const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Unable to get VPC vpcEndpointSet details', ['describeVpcDetails'], null, null, null);
              log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
              resolve({ action: 'describeVpcEndpoints', icode: 400, response: VpcEndpointsError });
            }
            // return the data
            resolve({ action: 'describeVpcEndpoints', icode: VpcEndpointsResult.icode, response: VpcEndpointsResult.response.DescribeVpcEndpointsResponse.vpcEndpointSet, metrics: VpcEndpointsResult.metrics });
          });
        })
      );

      vpcPromises.push(
        new Promise((resolve) => {
          // set filter for describeNatGateways
          filter = [{
            Name: 'vpc-id',
            Value: vpcId
          }];
          this.describeNatGateways(filter, null, null, null, (NatGatewaysResult, NatGatewaysError) => {
            // return an error
            if (NatGatewaysError) {
              const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Unable to get VPC natGatewaySet details', ['describeVpcDetails'], null, null, null);
              log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
              resolve({ action: 'describeNatGateways', icode: 400, response: NatGatewaysError });
            }
            // return the data
            resolve({ action: 'describeNatGateways', icode: NatGatewaysResult.icode, response: NatGatewaysResult.response.DescribeNatGatewaysResponse.natGatewaySet, metrics: NatGatewaysResult.metrics });
          });
        })
      );

      vpcPromises.push(
        new Promise((resolve) => {
          // set filter for describeSecurityGroups
          filter = [{
            Name: 'vpc-id',
            Value: vpcId
          }];
          this.describeSecurityGroups(filter, null, null, null, null, null, (SecurityGroupsResult, SecurityGroupsError) => {
            // return an error
            if (SecurityGroupsError) {
              const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Unable to get VPC securityGroupInfo details', ['describeVpcDetails'], null, null, null);
              log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
              resolve({ action: 'describeSecurityGroups', icode: 400, response: SecurityGroupsError });
            }
            // return the data
            resolve({ action: 'describeSecurityGroups', icode: SecurityGroupsResult.icode, response: SecurityGroupsResult.response.DescribeSecurityGroupsResponse.securityGroupInfo, metrics: SecurityGroupsResult.metrics });
          });
        })
      );

      vpcPromises.push(
        new Promise((resolve) => {
          // set filter for describeInstances
          filter = [{
            Name: 'vpc-id',
            Value: vpcId
          }];
          this.describeInstances(filter, null, null, null, null, (InstancesResult, InstancesError) => {
            // return an error
            if (InstancesError) {
              const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Unable to get VPC reservationSet details', ['describeVpcDetails'], null, null, null);
              log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
              resolve({ action: 'describeInstances', icode: 400, response: InstancesError });
            }
            // return the data
            resolve({ action: 'describeInstances', icode: InstancesResult.icode, response: InstancesResult.response.DescribeInstancesResponse.reservationSet, metrics: InstancesResult.metrics });
          });
        })
      );

      vpcPromises.push(
        new Promise((resolve) => {
          // set filter for describeVpnConnections
          filter = [{
            Name: 'vpc-id',
            Value: vpcId
          }];
          this.describeVpnConnections(filter, null, null, (VpnConnectionsResult, VpnConnectionsError) => {
            // return an error
            if (VpnConnectionsError) {
              const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Unable to get VPC vpnConnectionSet details', ['describeVpcDetails'], null, null, null);
              log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
              resolve({ action: 'describeVpnConnections', icode: 400, response: VpnConnectionsError });
            }
            // return the data
            resolve({ action: 'describeVpnConnections', icode: VpnConnectionsResult.icode, response: VpnConnectionsResult.response.DescribeVpnConnectionsResponse.vpnConnectionSet, metrics: VpnConnectionsResult.metrics });
          });
        })
      );

      vpcPromises.push(
        new Promise((resolve) => {
          // set filter for describeVpnGateways
          filter = [{
            Name: 'attachment.vpc-id',
            Value: vpcId
          }];
          this.describeVpnGateways(filter, null, null, (VpnGatewaysResult, VpnGatewaysError) => {
            // return an error
            if (VpnGatewaysError) {
              const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Unable to get VPC vpnGatewaySet details', ['describeVpcDetails'], null, null, null);
              log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
              resolve({ action: 'describeVpnGateways', icode: 400, response: VpnGatewaysError });
            }
            // return the data
            resolve({ action: 'describeVpnGateways', icode: VpnGatewaysResult.icode, response: VpnGatewaysResult.response.DescribeVpnGatewaysResponse.vpnGatewaySet, metrics: VpnGatewaysResult.metrics });
          });
        })
      );

      vpcPromises.push(
        new Promise((resolve) => {
          // set filter for describeNetworkInterfaces
          filter = [{
            Name: 'vpc-id',
            Value: vpcId
          }];
          this.describeNetworkInterfaces(filter, null, null, null, null, (NetworkInterfacesResult, NetworkInterfacesError) => {
            // return an error
            if (NetworkInterfacesError) {
              const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Unable to get VPC networkInterfaceSet details', ['describeVpcDetails'], null, null, null);
              log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
              resolve({ action: 'describeNetworkInterfaces', icode: 400, response: NetworkInterfacesError });
            }
            // return the data
            resolve({ action: 'describeNetworkInterfaces', icode: NetworkInterfacesResult.icode, response: NetworkInterfacesResult.response.DescribeNetworkInterfacesResponse.networkInterfaceSet, metrics: NetworkInterfacesResult.metrics });
          });
        })
      );

      vpcPromises.push(
        new Promise((resolve) => {
          // set filter for describeNetworkInterfaces
          filter = [{
            Name: 'vpc-id',
            Value: vpcId
          }];
          this.describeVpcs(filter, null, null, null, null, (VpcResult, VpcError) => {
            // return an error
            if (VpcError) {
              const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Unable to get VPC', ['describeVpcDetails'], null, null, null);
              log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
              resolve({ action: 'describeVpcs', icode: 400, response: VpcError });
            }
            // return the data
            resolve({ action: 'describeVpcs', icode: VpcResult.icode, response: VpcResult.response.DescribeVpcsResponse.vpcSet, metrics: VpcResult.metrics });
          });
        })
      );

      // return an array of repsonses
      return Promise.all(vpcPromises).then((results) => {
        const icodes = [];
        const mergedResponses = {};
        let totalMetrics = {
          capabilityTime: '0ms'
        };
        results.sort((a, b) => {
          const actionA = a.action.toLowerCase();
          const actionB = b.action.toLowerCase();
          if (actionA < actionB) {
            return -1;
          }
          if (actionA > actionB) {
            return 1;
          }
          return 0;
        });
        results.forEach((result) => {
          const { action, icode, response, metrics } = result;
          const temp = {};
          temp[action] = icode;
          icodes.push(temp);
          mergedResponses[action] = response;

          if (metrics && metrics.capabilityTime) {
            const cap = Number(totalMetrics.capabilityTime.substring(0, totalMetrics.capabilityTime.length - 2));
            const met = Number(metrics.capabilityTime.substring(0, metrics.capabilityTime.length - 2));
            if (met > cap) {
              totalMetrics = metrics;
            }
          }
        });
        const newResult = {
          icode: icodes,
          response: mergedResponses,
          metrics: totalMetrics
        };
        callback(newResult);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Get VPC that match the vpcId
   *
   * @function getDevice
   * @param {String} vpcId - the vpc to find (required)
   *
   * @param {getCallback} callback - a callback function to return the result
   *                                 (vpc) or the error
   */
  getDevice(vpcId, callback) {
    const meth = 'adapter-getDevice';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcId === undefined || vpcId === null || vpcId === '' || vpcId.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    try {
      // make call to get the VPCs
      return this.describeVpcs(null, [vpcId], null, null, null, (result, error) => {
        // if we received an error or their is no response on the results return an error
        if (error) {
          return callback(null, error);
        }
        if (!result.response || !result.response.DescribeVpcsResponse || !result.response.DescribeVpcsResponse.vpcSet || !result.response.DescribeVpcsResponse.vpcSet.item) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['getDevice'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        // return the response
        const thisVPC = result.response.DescribeVpcsResponse.vpcSet.item;
        thisVPC.name = thisVPC.vpcId;
        thisVPC.ostype = 'aws-vpc';
        thisVPC.ipaddress = thisVPC.cidrBlock;
        return callback(thisVPC);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Get VPCs that match the filter
   *
   * @function getDevicesFiltered
   * @param {Object} options - the vpc data to use to filter the vpc (optional)
   *
   * @param {getCallback} callback - a callback function to return the result
   *                                 (vpcs) or the error
   */
  getDevicesFiltered(options, callback) {
    const meth = 'adapter-getDevicesFiltered';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    // verify the required fields have been provided
    if (options === undefined || options === null || options === '' || options.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['options'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    // TODO - get pagination working
    // const nextToken = options.start;
    // const maxResults = options.limit;

    // TODO - get contains working for partial filters
    // set up the filter for AWS
    let filter = [];
    if (options && options.filter && options.filter.name) {
      // when this hack is removed, remove the lint ignore above
      let temp = options.filter.name;
      if (Array.isArray(options.filter.name)) {
        // eslint-disable-next-line prefer-destructuring
        temp = options.filter.name[0];
      }
      filter = [{
        Name: 'vpc-id',
        Value: temp
      }];
    }

    // TODO - get sort and order working
    /*
    if (options && options.sort) {
      reqObj.uriOptions.sort = JSON.stringify(options.sort);
    }
    if (options && options.order) {
      reqObj.uriOptions.order = options.order;
    }
    */
    try {
      // make call to get the VPCs
      return this.describeVpcs(filter, null, null, null, null, (result, error) => {
        // if we received an error or their is no response on the results return an error
        if (error) {
          return callback(null, error);
        }
        if (!result.response || !result.response.DescribeVpcsResponse || !result.response.DescribeVpcsResponse.vpcSet) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['getDevicesFiltered'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        // return the response
        if (result.response.DescribeVpcsResponse.vpcSet.item && Array.isArray(result.response.DescribeVpcsResponse.vpcSet.item)) {
          const myVPCs = [];

          for (let v = 0; v < result.response.DescribeVpcsResponse.vpcSet.item.length; v += 1) {
            const thisVPC = result.response.DescribeVpcsResponse.vpcSet.item[v];
            thisVPC.name = thisVPC.vpcId;
            thisVPC.ostype = 'aws-vpc';
            thisVPC.ipaddress = thisVPC.cidrBlock;
            myVPCs.push(thisVPC);
          }
          log.debug(`${origin}: Found #${myVPCs.length} vpcs.`);
          return callback({ total: myVPCs.length, list: myVPCs });
        }
        if (result.response.DescribeVpcsResponse.vpcSet.item) {
          const thisVPC = result.response.DescribeVpcsResponse.vpcSet.item;
          log.debug(`${origin}: Found #1 vpc.`);
          thisVPC.name = thisVPC.vpcId;
          thisVPC.ostype = 'aws-vpc';
          thisVPC.ipaddress = thisVPC.cidrBlock;
          return callback({ total: 1, list: [thisVPC] });
        }

        // if we have a filter - need to see if partials are found
        if (options && options.filter && options.filter.name) {
          // make call to get the VPCs again with no filter -- need to check contains not done in AWS
          return this.describeVpcs(null, null, null, null, null, (secres, secerr) => {
            // if we received an error or their is no response on the results return an error
            if (secerr) {
              return callback(null, secerr);
            }
            if (!secres.response || !secres.response.DescribeVpcsResponse || !secres.response.DescribeVpcsResponse.vpcSet) {
              const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['getDevicesFiltered'], null, null, null);
              log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
              return callback(null, errorObj);
            }

            // return the response
            if (secres.response.DescribeVpcsResponse.vpcSet.item && Array.isArray(secres.response.DescribeVpcsResponse.vpcSet.item)) {
              const myVPCs = [];

              for (let v = 0; v < secres.response.DescribeVpcsResponse.vpcSet.item.length; v += 1) {
                const thisVPC = secres.response.DescribeVpcsResponse.vpcSet.item[v];
                if (thisVPC.vpcId.contains(options.filter.name)) {
                  thisVPC.name = thisVPC.vpcId;
                  thisVPC.ostype = 'aws-vpc';
                  thisVPC.ipaddress = thisVPC.cidrBlock;
                  myVPCs.push(thisVPC);
                }
              }
              log.debug(`${origin}: Found #${myVPCs.length} vpcs.`);
              return callback({ total: myVPCs.length, list: myVPCs });
            }
            if (secres.response.DescribeVpcsResponse.vpcSet.item) {
              const thisVPC = secres.response.DescribeVpcsResponse.vpcSet.item;
              if (thisVPC.vpcId.contains(options.filter.name)) {
                log.debug(`${origin}: Found #1 vpc.`);
                thisVPC.name = thisVPC.vpcId;
                thisVPC.ostype = 'aws-vpc';
                thisVPC.ipaddress = thisVPC.cidrBlock;
                return callback({ total: 1, list: [thisVPC] });
              }
            }

            log.debug(`${origin}: Found #0 vpc.`);
            return callback({ total: 0, list: [] });
          });
        }
        log.debug(`${origin}: Found #0 vpc.`);
        return callback({ total: 0, list: [] });
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Gets the status for the provided vpc
   *
   * @function connectDevice
   * @param {String} vpcId - the id of the vpc. (required)
   *
   * @param {configCallback} callback - callback function to return the result
   *                                    (vpc connectDevice) or the error
   */
  connectDevice(vpcId, callback) {
    const meth = 'adapter-connectDevice';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    // verify the required fields have been provided
    if (vpcId === undefined || vpcId === null || vpcId === '' || vpcId.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    try {
      return this.describeVpcs(null, [vpcId], null, null, null, (result, error) => {
        // if we received an error or their is no response on the results return an error
        if (error) {
          return callback(null, error);
        }
        if (!result.response || !result.response.DescribeVpcsResponse || !result.response.DescribeVpcsResponse.vpcSet) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['connectDevice'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        // return the response
        if (result.response.DescribeVpcsResponse.vpcSet.item && Array.isArray(result.response.DescribeVpcsResponse.vpcSet.item)
            && result.response.DescribeVpcsResponse.vpcSet.item[0].state === 'available') {
          return callback(true);
        }
        if (result.response.DescribeVpcsResponse.vpcSet.item && result.response.DescribeVpcsResponse.vpcSet.item.state === 'available') {
          return callback(true);
        }
        return callback(false);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Gets the status for the provided vpc
   *
   * @function isAlive
   * @param {String} vpcId - the id of the vpc. (required)
   *
   * @param {configCallback} callback - callback function to return the result
   *                                    (vpc isAlive) or the error
   */
  isAlive(vpcId, callback) {
    const meth = 'adapter-isAlive';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    // verify the required fields have been provided
    if (vpcId === undefined || vpcId === null || vpcId === '' || vpcId.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    try {
      return this.describeVpcs(null, [vpcId], null, null, null, (result, error) => {
        // if we received an error or their is no response on the results return an error
        if (error) {
          return callback(null, error);
        }
        if (!result.response || !result.response.DescribeVpcsResponse || !result.response.DescribeVpcsResponse.vpcSet) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['isAlive'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        // return the response
        if (result.response.DescribeVpcsResponse.vpcSet.item && Array.isArray(result.response.DescribeVpcsResponse.vpcSet.item)
            && result.response.DescribeVpcsResponse.vpcSet.item[0].state === 'available') {
          return callback(true);
        }
        if (result.response.DescribeVpcsResponse.vpcSet.item && result.response.DescribeVpcsResponse.vpcSet.item.state === 'available') {
          return callback(true);
        }
        return callback(false);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Gets a config for the provided vpc
   *
   * @function getConfig
   * @param {String} vpcId - the id of the vpc. (required)
   * @param {String} format - the desired format of the config. (optional)
   *
   * @param {configCallback} callback - callback function to return the result
   *                                    (vpc config) or the error
   */
  getConfig(vpcId, format, callback) {
    const meth = 'adapter-getConfig';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);

    // verify the required fields have been provided
    if (vpcId === undefined || vpcId === null || vpcId === '' || vpcId.length === 0) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    try {
      return this.describeVpcDetails(vpcId, (result, error) => {
        // if we received an error or their is no response on the results return an error
        if (error) {
          return callback(null, error);
        }

        /*
        if (!Object.hasOwnProperty.call(result, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['getConfig'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }
        */

        /*
        // DO WE NEED THIS -- EG IF ANY PEICES FAIL????
        if (Array.isArray(result.icode)) {
          for (let r = 0; r < result.icode.length; r += 1) {
            // set the possible fail reason from Automation Gateway
            if (result.icode[r].status !== 'SUCCESS' && !failReason) {
              failReason = returnObj.response[r].results.msg;
            }
          }
        }
        */
        // return the result
        const newResponse = {
          response: JSON.stringify(result.response, null, 2)
        };
        return callback(newResponse);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more of your VPN connections.   For more information, see  AWS Site-to-Site VPN  in the  AWS Site-to-Site VPN User Guide .
   *
   * @function describeVpnConnections
   * @param {array} filter - One or more filters.         customer-gateway-configuration  - The configuration information for the customer gateway.         customer-gateway-id  - The ID of a custome...(description truncated)
   * @param {array} vpnConnectionId - One or more VPN connection IDs.   Default: Describes your VPN connections.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVpnConnections(filter, vpnConnectionId, dryRun, callback) {
    const meth = 'adapter-describeVpnConnections';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeVpnConnections';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (vpnConnectionId && vpnConnectionId.length > 0) {
      processInArray(vpnConnectionId, queryParamsAvailable, 'VpnConnectionId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'describeVpnConnections', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeVpnConnections'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Describes one or more of your virtual private gateways.   For more information, see  AWS Site-to-Site VPN  in the  AWS Site-to-Site VPN User Guide .
   *
   * @function describeVpnGateways
   * @param {array} filter - One or more filters.         amazon-side-asn  - The Autonomous System Number (ASN) for the Amazon side of the gateway.         attachment.state  - The current state of t...(description truncated)
   * @param {array} vpnGatewayId - One or more virtual private gateway IDs.   Default: Describes all your virtual private gateways.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  describeVpnGateways(filter, vpnGatewayId, dryRun, callback) {
    const meth = 'adapter-describeVpnGateways';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DescribeVpnGateways';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }
    if (vpnGatewayId && vpnGatewayId.length > 0) {
      processInArray(vpnGatewayId, queryParamsAvailable, 'VpnGatewayId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'describeVpnGateways', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['describeVpnGateways'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Unlinks (detaches) a linked EC2-Classic instance from a VPC. After the instance has been unlinked, the VPC security groups are no longer associated with it. An instance is automatically unlinked from a VPC when it's stopped.
   *
   * @function detachClassicLinkVpc
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} instanceId - The ID of the instance to unlink from the VPC.
   * @param {string} vpcId - The ID of the VPC to which the instance is linked.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  detachClassicLinkVpc(dryRun, instanceId, vpcId, callback) {
    const meth = 'adapter-detachClassicLinkVpc';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DetachClassicLinkVpc';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      InstanceId: instanceId,
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'detachClassicLinkVpc', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['detachClassicLinkVpc'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Detaches an internet gateway from a VPC, disabling connectivity between the internet and the VPC. The VPC must not contain any running instances with Elastic IP addresses or public IPv4 addresses.
   *
   * @function detachInternetGateway
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} internetGatewayId - The ID of the internet gateway.
   * @param {string} vpcId - The ID of the VPC.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  detachInternetGateway(dryRun, internetGatewayId, vpcId, callback) {
    const meth = 'adapter-detachInternetGateway';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DetachInternetGateway';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (internetGatewayId === undefined || internetGatewayId === null || internetGatewayId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['internetGatewayId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      InternetGatewayId: internetGatewayId,
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Gateway', 'detachInternetGateway', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['detachInternetGateway'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Detaches a network interface from an instance.
   *
   * @function detachNetworkInterface
   * @param {string} attachmentId - The ID of the attachment.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {boolean} force - Specifies whether to force a detachment.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  detachNetworkInterface(attachmentId, dryRun, force, callback) {
    const meth = 'adapter-detachNetworkInterface';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DetachNetworkInterface';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (attachmentId === undefined || attachmentId === null || attachmentId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['attachmentId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AttachmentId: attachmentId,
      Force: force,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkInterface', 'detachNetworkInterface', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['detachNetworkInterface'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Detaches an EBS volume from an instance. Make sure to unmount any file systems on the device within your operating system before detaching the volume. Failure to do so can result in the volume becoming stuck in the  busy  state while detaching. If this happens, detachment can be delayed indefinitely until you unmount the volume, force detachment, reboot the instance, or all three. If an EBS volume is the root device of an instance, it can't be detached while the instance is running. To detach t...(description truncated)
   *
   * @function detachVolume
   * @param {string} device - The device name.
   * @param {boolean} force - Forces detachment if the previous detachment attempt did not occur cleanly (for example, logging into an instance, unmounting the volume, and detaching normally). This op...(description truncated)
   * @param {string} instanceId - The ID of the instance.
   * @param {string} volumeId - The ID of the volume.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  detachVolume(device, force, instanceId, volumeId, dryRun, callback) {
    const meth = 'adapter-detachVolume';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DetachVolume';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (volumeId === undefined || volumeId === null || volumeId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['volumeId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Device: device,
      Force: force,
      InstanceId: instanceId,
      VolumeId: volumeId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Volume', 'detachVolume', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['detachVolume'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Detaches a virtual private gateway from a VPC. You do this if you're planning to turn off the VPC and not use it anymore. You can confirm a virtual private gateway has been completely detached from a VPC by describing the virtual private gateway (any attachments to the virtual private gateway are also described).   You must wait for the attachment's state to switch to  detached  before you can delete the VPC or attach a different VPC to the virtual private gateway.
   *
   * @function detachVpnGateway
   * @param {string} vpcId - The ID of the VPC.
   * @param {string} vpnGatewayId - The ID of the virtual private gateway.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  detachVpnGateway(vpcId, vpnGatewayId, dryRun, callback) {
    const meth = 'adapter-detachVpnGateway';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DetachVpnGateway';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (vpnGatewayId === undefined || vpnGatewayId === null || vpnGatewayId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpnGatewayId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VpcId: vpcId,
      VpnGatewayId: vpnGatewayId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'detachVpnGateway', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['detachVpnGateway'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Disables the specified resource attachment from propagating routes to the specified propagation route table.
   *
   * @function disableTransitGatewayRouteTablePropagation
   * @param {string} transitGatewayRouteTableId - The ID of the propagation route table.
   * @param {string} transitGatewayAttachmentId - The ID of the attachment.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  disableTransitGatewayRouteTablePropagation(transitGatewayRouteTableId, transitGatewayAttachmentId, dryRun, callback) {
    const meth = 'adapter-disableTransitGatewayRouteTablePropagation';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DisableTransitGatewayRouteTablePropagation';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayRouteTableId === undefined || transitGatewayRouteTableId === null || transitGatewayRouteTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayRouteTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (transitGatewayAttachmentId === undefined || transitGatewayAttachmentId === null || transitGatewayAttachmentId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayAttachmentId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      TransitGatewayRouteTableId: transitGatewayRouteTableId,
      TransitGatewayAttachmentId: transitGatewayAttachmentId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'disableTransitGatewayRouteTablePropagation', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['disableTransitGatewayRouteTablePropagation'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Disables a virtual private gateway (VGW) from propagating routes to a specified route table of a VPC.
   *
   * @function disableVgwRoutePropagation
   * @param {string} gatewayId - The ID of the virtual private gateway.
   * @param {string} routeTableId - The ID of the route table.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  disableVgwRoutePropagation(gatewayId, routeTableId, callback) {
    const meth = 'adapter-disableVgwRoutePropagation';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DisableVgwRoutePropagation';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (gatewayId === undefined || gatewayId === null || gatewayId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['gatewayId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (routeTableId === undefined || routeTableId === null || routeTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['routeTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      GatewayId: gatewayId,
      RouteTableId: routeTableId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Route', 'disableVgwRoutePropagation', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['disableVgwRoutePropagation'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Disables ClassicLink for a VPC. You cannot disable ClassicLink for a VPC that has EC2-Classic instances linked to it.
   *
   * @function disableVpcClassicLink
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} vpcId - The ID of the VPC.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  disableVpcClassicLink(dryRun, vpcId, callback) {
    const meth = 'adapter-disableVpcClassicLink';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DisableVpcClassicLink';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'disableVpcClassicLink', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['disableVpcClassicLink'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Disables ClassicLink DNS support for a VPC. If disabled, DNS hostnames resolve to public IP addresses when addressed between a linked EC2-Classic instance and instances in the VPC to which it's linked. For more information, see  ClassicLink  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function disableVpcClassicLinkDnsSupport
   * @param {string} vpcId - The ID of the VPC.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  disableVpcClassicLinkDnsSupport(vpcId, callback) {
    const meth = 'adapter-disableVpcClassicLinkDnsSupport';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DisableVpcClassicLinkDnsSupport';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'disableVpcClassicLinkDnsSupport', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['disableVpcClassicLinkDnsSupport'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Disassociates an Elastic IP address from the instance or network interface it's associated with.   An Elastic IP address is for use in either the EC2-Classic platform or in a VPC. For more information, see  Elastic IP Addresses  in the  Amazon Elastic Compute Cloud User Guide .   This is an idempotent operation. If you perform the operation more than once, Amazon EC2 doesn't return an error.
   *
   * @function disassociateAddress
   * @param {string} associationId - [EC2-VPC] The association ID. Required for EC2-VPC.
   * @param {string} publicIp - [EC2-Classic] The Elastic IP address. Required for EC2-Classic.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  disassociateAddress(associationId, publicIp, dryRun, callback) {
    const meth = 'adapter-disassociateAddress';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DisassociateAddress';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AssociationId: associationId,
      PublicIp: publicIp,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'disassociateAddress', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['disassociateAddress'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Disassociates a target network from the specified Client VPN endpoint. When you disassociate the last target network from a Client VPN, the following happens:       The route that was automatically added for the VPC is deleted       All active client connections are terminated       New client connections are disallowed       The Client VPN endpoint's status changes to  pending-associate
   *
   * @function disassociateClientVpnTargetNetwork
   * @param {string} clientVpnEndpointId - The ID of the Client VPN endpoint from which to disassociate the target network.
   * @param {string} associationId - The ID of the target network association.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  disassociateClientVpnTargetNetwork(clientVpnEndpointId, associationId, dryRun, callback) {
    const meth = 'adapter-disassociateClientVpnTargetNetwork';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DisassociateClientVpnTargetNetwork';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointId === undefined || clientVpnEndpointId === null || clientVpnEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (associationId === undefined || associationId === null || associationId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['associationId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientVpnEndpointId: clientVpnEndpointId,
      AssociationId: associationId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'disassociateClientVpnTargetNetwork', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['disassociateClientVpnTargetNetwork'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Disassociates an IAM instance profile from a running or stopped instance.   Use  DescribeIamInstanceProfileAssociations  to get the association ID.
   *
   * @function disassociateIamInstanceProfile
   * @param {string} associationId - The ID of the IAM instance profile association.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  disassociateIamInstanceProfile(associationId, callback) {
    const meth = 'adapter-disassociateIamInstanceProfile';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DisassociateIamInstanceProfile';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (associationId === undefined || associationId === null || associationId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['associationId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      AssociationId: associationId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'disassociateIamInstanceProfile', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['disassociateIamInstanceProfile'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Disassociates a subnet from a route table.   After you perform this action, the subnet no longer uses the routes in the route table. Instead, it uses the routes in the VPC's main route table. For more information about route tables, see  Route Tables  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function disassociateRouteTable
   * @param {string} associationId - The association ID representing the current association between the route table and subnet.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  disassociateRouteTable(associationId, dryRun, callback) {
    const meth = 'adapter-disassociateRouteTable';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DisassociateRouteTable';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (associationId === undefined || associationId === null || associationId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['associationId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AssociationId: associationId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Route', 'disassociateRouteTable', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['disassociateRouteTable'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Disassociates a CIDR block from a subnet. Currently, you can disassociate an IPv6 CIDR block only. You must detach or delete all gateways and resources that are associated with the CIDR block before you can disassociate it.
   *
   * @function disassociateSubnetCidrBlock
   * @param {string} associationId - The association ID for the CIDR block.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  disassociateSubnetCidrBlock(associationId, callback) {
    const meth = 'adapter-disassociateSubnetCidrBlock';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DisassociateSubnetCidrBlock';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (associationId === undefined || associationId === null || associationId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['associationId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      AssociationId: associationId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Subnet', 'disassociateSubnetCidrBlock', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['disassociateSubnetCidrBlock'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Disassociates a resource attachment from a transit gateway route table.
   *
   * @function disassociateTransitGatewayRouteTable
   * @param {string} transitGatewayRouteTableId - The ID of the transit gateway route table.
   * @param {string} transitGatewayAttachmentId - The ID of the attachment.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  disassociateTransitGatewayRouteTable(transitGatewayRouteTableId, transitGatewayAttachmentId, dryRun, callback) {
    const meth = 'adapter-disassociateTransitGatewayRouteTable';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DisassociateTransitGatewayRouteTable';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayRouteTableId === undefined || transitGatewayRouteTableId === null || transitGatewayRouteTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayRouteTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (transitGatewayAttachmentId === undefined || transitGatewayAttachmentId === null || transitGatewayAttachmentId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayAttachmentId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      TransitGatewayRouteTableId: transitGatewayRouteTableId,
      TransitGatewayAttachmentId: transitGatewayAttachmentId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'disassociateTransitGatewayRouteTable', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['disassociateTransitGatewayRouteTable'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Disassociates a CIDR block from a VPC. To disassociate the CIDR block, you must specify its association ID. You can get the association ID by using  DescribeVpcs . You must detach or delete all gateways and resources that are associated with the CIDR block before you can disassociate it.    You cannot disassociate the CIDR block with which you originally created the VPC (the primary CIDR block).
   *
   * @function disassociateVpcCidrBlock
   * @param {string} associationId - The association ID for the CIDR block.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  disassociateVpcCidrBlock(associationId, callback) {
    const meth = 'adapter-disassociateVpcCidrBlock';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'DisassociateVpcCidrBlock';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (associationId === undefined || associationId === null || associationId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['associationId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      AssociationId: associationId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'disassociateVpcCidrBlock', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['disassociateVpcCidrBlock'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Enables the specified attachment to propagate routes to the specified propagation route table.
   *
   * @function enableTransitGatewayRouteTablePropagation
   * @param {string} transitGatewayRouteTableId - The ID of the propagation route table.
   * @param {string} transitGatewayAttachmentId - The ID of the attachment.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  enableTransitGatewayRouteTablePropagation(transitGatewayRouteTableId, transitGatewayAttachmentId, dryRun, callback) {
    const meth = 'adapter-enableTransitGatewayRouteTablePropagation';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'EnableTransitGatewayRouteTablePropagation';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayRouteTableId === undefined || transitGatewayRouteTableId === null || transitGatewayRouteTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayRouteTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (transitGatewayAttachmentId === undefined || transitGatewayAttachmentId === null || transitGatewayAttachmentId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayAttachmentId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      TransitGatewayRouteTableId: transitGatewayRouteTableId,
      TransitGatewayAttachmentId: transitGatewayAttachmentId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'enableTransitGatewayRouteTablePropagation', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['enableTransitGatewayRouteTablePropagation'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Enables a virtual private gateway (VGW) to propagate routes to the specified route table of a VPC.
   *
   * @function enableVgwRoutePropagation
   * @param {string} gatewayId - The ID of the virtual private gateway that is attached to a VPC. The virtual private gateway must be attached to the same VPC that the routing tables are associated with....(description truncated)
   * @param {string} routeTableId - The ID of the route table. The routing table must be associated with the same VPC that the virtual private gateway is attached to.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  enableVgwRoutePropagation(gatewayId, routeTableId, callback) {
    const meth = 'adapter-enableVgwRoutePropagation';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'EnableVgwRoutePropagation';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (gatewayId === undefined || gatewayId === null || gatewayId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['gatewayId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (routeTableId === undefined || routeTableId === null || routeTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['routeTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      GatewayId: gatewayId,
      RouteTableId: routeTableId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Route', 'enableVgwRoutePropagation', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['enableVgwRoutePropagation'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Enables I/O operations for a volume that had I/O operations disabled because the data on the volume was potentially inconsistent.
   *
   * @function enableVolumeIO
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} volumeId - The ID of the volume.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  enableVolumeIO(dryRun, volumeId, callback) {
    const meth = 'adapter-enableVolumeIO';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'EnableVolumeIO';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (volumeId === undefined || volumeId === null || volumeId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['volumeId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VolumeId: volumeId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Volume', 'enableVolumeIO', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['enableVolumeIO'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Enables a VPC for ClassicLink. You can then link EC2-Classic instances to your ClassicLink-enabled VPC to allow communication over private IP addresses. You cannot enable your VPC for ClassicLink if any of your VPC route tables have existing routes for address ranges within the  10.0.0.0/8  IP address range, excluding local routes for VPCs in the  10.0.0.0/16  and  10.1.0.0/16  IP address ranges. For more information, see  ClassicLink  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function enableVpcClassicLink
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} vpcId - The ID of the VPC.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  enableVpcClassicLink(dryRun, vpcId, callback) {
    const meth = 'adapter-enableVpcClassicLink';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'EnableVpcClassicLink';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'enableVpcClassicLink', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['enableVpcClassicLink'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Enables a VPC to support DNS hostname resolution for ClassicLink. If enabled, the DNS hostname of a linked EC2-Classic instance resolves to its private IP address when addressed from an instance in the VPC to which it's linked. Similarly, the DNS hostname of an instance in a VPC resolves to its private IP address when addressed from a linked EC2-Classic instance. For more information, see  ClassicLink  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function enableVpcClassicLinkDnsSupport
   * @param {string} vpcId - The ID of the VPC.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  enableVpcClassicLinkDnsSupport(vpcId, callback) {
    const meth = 'adapter-enableVpcClassicLinkDnsSupport';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'EnableVpcClassicLinkDnsSupport';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'enableVpcClassicLinkDnsSupport', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['enableVpcClassicLinkDnsSupport'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Downloads the client certificate revocation list for the specified Client VPN endpoint.
   *
   * @function exportClientVpnClientCertificateRevocationList
   * @param {string} clientVpnEndpointId - The ID of the Client VPN endpoint.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  exportClientVpnClientCertificateRevocationList(clientVpnEndpointId, dryRun, callback) {
    const meth = 'adapter-exportClientVpnClientCertificateRevocationList';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ExportClientVpnClientCertificateRevocationList';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointId === undefined || clientVpnEndpointId === null || clientVpnEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientVpnEndpointId: clientVpnEndpointId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'exportClientVpnClientCertificateRevocationList', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['exportClientVpnClientCertificateRevocationList'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Downloads the contents of the Client VPN endpoint configuration file for the specified Client VPN endpoint. The Client VPN endpoint configuration file includes the Client VPN endpoint and certificate information clients need to establish a connection with the Client VPN endpoint.
   *
   * @function exportClientVpnClientConfiguration
   * @param {string} clientVpnEndpointId - The ID of the Client VPN endpoint.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  exportClientVpnClientConfiguration(clientVpnEndpointId, dryRun, callback) {
    const meth = 'adapter-exportClientVpnClientConfiguration';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ExportClientVpnClientConfiguration';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointId === undefined || clientVpnEndpointId === null || clientVpnEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientVpnEndpointId: clientVpnEndpointId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'exportClientVpnClientConfiguration', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['exportClientVpnClientConfiguration'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Exports routes from the specified transit gateway route table to the specified S3 bucket. By default, all routes are exported. Alternatively, you can filter by CIDR range.
   *
   * @function exportTransitGatewayRoutes
   * @param {string} transitGatewayRouteTableId - The ID of the route table.
   * @param {array} filter - One or more filters. The possible values are:         attachment.transit-gateway-attachment-id - The id of the transit gateway attachment.         attachment.resource-id...(description truncated)
   * @param {string} s3Bucket - The name of the S3 bucket.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  exportTransitGatewayRoutes(transitGatewayRouteTableId, filter, s3Bucket, dryRun, callback) {
    const meth = 'adapter-exportTransitGatewayRoutes';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ExportTransitGatewayRoutes';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayRouteTableId === undefined || transitGatewayRouteTableId === null || transitGatewayRouteTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayRouteTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (s3Bucket === undefined || s3Bucket === null || s3Bucket === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['s3Bucket'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      TransitGatewayRouteTableId: transitGatewayRouteTableId,
      S3Bucket: s3Bucket,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'exportTransitGatewayRoutes', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['exportTransitGatewayRoutes'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Gets the console output for the specified instance. For Linux instances, the instance console output displays the exact console output that would normally be displayed on a physical monitor attached to a computer. For Windows instances, the instance console output includes the last three system event log errors.   By default, the console output returns buffered information that was posted shortly after an instance transition state (start, stop, reboot, or terminate). This information is availab...(description truncated)
   *
   * @function getConsoleOutput
   * @param {string} instanceId - The ID of the instance.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {boolean} latest - When enabled, retrieves the latest console output for the instance.   Default: disabled ( false )
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  getConsoleOutput(instanceId, dryRun, latest, callback) {
    const meth = 'adapter-getConsoleOutput';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'GetConsoleOutput';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      InstanceId: instanceId,
      Latest: latest,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionGetConsoleOutput', 'getGetConsoleOutput', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['getConsoleOutput'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Retrieve a JPG-format screenshot of a running instance to help with troubleshooting.   The returned content is Base64-encoded.
   *
   * @function getConsoleScreenshot
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} instanceId - The ID of the instance.
   * @param {boolean} wakeUp - When set to  true , acts as keystroke input and wakes up an instance that's in standby or "sleep" mode.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  getConsoleScreenshot(dryRun, instanceId, wakeUp, callback) {
    const meth = 'adapter-getConsoleScreenshot';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'GetConsoleScreenshot';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      InstanceId: instanceId,
      WakeUp: wakeUp,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionGetConsoleScreenshot', 'getGetConsoleScreenshot', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['getConsoleScreenshot'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Preview a reservation purchase with configurations that match those of your Dedicated Host. You must have active Dedicated Hosts in your account before you purchase a reservation.   This is a preview of the  PurchaseHostReservation  action and does not result in the offering being purchased.
   *
   * @function getHostReservationPurchasePreview
   * @param {array} hostIdSet - The IDs of the Dedicated Hosts with which the reservation is associated.
   * @param {string} offeringId - The offering ID of the reservation.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  getHostReservationPurchasePreview(hostIdSet, offeringId, callback) {
    const meth = 'adapter-getHostReservationPurchasePreview';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'GetHostReservationPurchasePreview';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (hostIdSet === undefined || hostIdSet === null || hostIdSet === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['hostIdSet'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (offeringId === undefined || offeringId === null || offeringId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['offeringId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      OfferingId: offeringId,
      Action: action,
      Version: version
    };
    if (hostIdSet && hostIdSet.length > 0) {
      processInArray(hostIdSet, queryParamsAvailable, 'HostIdSet');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('ActionGetHostReservationPurchasePreview', 'getGetHostReservationPurchasePreview', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['getHostReservationPurchasePreview'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Retrieves the configuration data of the specified instance. You can use this data to create a launch template.
   *
   * @function getLaunchTemplateData
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} instanceId - The ID of the instance.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  getLaunchTemplateData(dryRun, instanceId, callback) {
    const meth = 'adapter-getLaunchTemplateData';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'GetLaunchTemplateData';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      InstanceId: instanceId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('LaunchTemplate', 'getLaunchTemplateData', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['getLaunchTemplateData'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Retrieves the encrypted administrator password for a running Windows instance.   The Windows password is generated at boot by the  EC2Config  service or  EC2Launch  scripts (Windows Server 2016 and later). This usually only happens the first time an instance is launched. For more information, see  EC2Config  and  EC2Launch  in the Amazon Elastic Compute Cloud User Guide.   For the  EC2Config  service, the password is not generated for rebundled AMIs unless  Ec2SetPassword  is enabled before bun...(description truncated)
   *
   * @function getPasswordData
   * @param {string} instanceId - The ID of the Windows instance.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  getPasswordData(instanceId, dryRun, callback) {
    const meth = 'adapter-getPasswordData';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'GetPasswordData';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      InstanceId: instanceId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Miscellaneous', 'getPasswordData', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['getPasswordData'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Returns a quote and exchange information for exchanging one or more specified Convertible Reserved Instances for a new Convertible Reserved Instance. If the exchange cannot be performed, the reason is returned in the response. Use  AcceptReservedInstancesExchangeQuote  to perform the exchange.
   *
   * @function getReservedInstancesExchangeQuote
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} reservedInstanceId - The IDs of the Convertible Reserved Instances to exchange.
   * @param {array} targetConfiguration - The configuration of the target Convertible Reserved Instance to exchange for your current Convertible Reserved Instances.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  getReservedInstancesExchangeQuote(dryRun, reservedInstanceId, targetConfiguration, callback) {
    const meth = 'adapter-getReservedInstancesExchangeQuote';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'GetReservedInstancesExchangeQuote';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (reservedInstanceId === undefined || reservedInstanceId === null || reservedInstanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['reservedInstanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (reservedInstanceId && reservedInstanceId.length > 0) {
      processInArray(reservedInstanceId, queryParamsAvailable, 'ReservedInstanceId');
    }
    if (targetConfiguration && targetConfiguration.length > 0) {
      processInArray(targetConfiguration, queryParamsAvailable, 'TargetConfiguration');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'getReservedInstancesExchangeQuote', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['getReservedInstancesExchangeQuote'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Lists the route tables to which the specified resource attachment propagates routes.
   *
   * @function getTransitGatewayAttachmentPropagations
   * @param {string} transitGatewayAttachmentId - The ID of the attachment.
   * @param {array} filter - One or more filters. The possible values are:         transit-gateway-route-table-id  - The ID of the transit gateway route table.
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {string} nextToken - The token for the next page of results.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  getTransitGatewayAttachmentPropagations(transitGatewayAttachmentId, filter, maxResults, nextToken, dryRun, callback) {
    const meth = 'adapter-getTransitGatewayAttachmentPropagations';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'GetTransitGatewayAttachmentPropagations';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayAttachmentId === undefined || transitGatewayAttachmentId === null || transitGatewayAttachmentId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayAttachmentId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      TransitGatewayAttachmentId: transitGatewayAttachmentId,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'getTransitGatewayAttachmentPropagations', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['getTransitGatewayAttachmentPropagations'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Gets information about the associations for the specified transit gateway route table.
   *
   * @function getTransitGatewayRouteTableAssociations
   * @param {string} transitGatewayRouteTableId - The ID of the transit gateway route table.
   * @param {array} filter - One or more filters. The possible values are:         resource-id  - The ID of the resource.         resource-type  - The resource type ( vpc  |  vpn ).         transit-...(description truncated)
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {string} nextToken - The token for the next page of results.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  getTransitGatewayRouteTableAssociations(transitGatewayRouteTableId, filter, maxResults, nextToken, dryRun, callback) {
    const meth = 'adapter-getTransitGatewayRouteTableAssociations';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'GetTransitGatewayRouteTableAssociations';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayRouteTableId === undefined || transitGatewayRouteTableId === null || transitGatewayRouteTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayRouteTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      TransitGatewayRouteTableId: transitGatewayRouteTableId,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'getTransitGatewayRouteTableAssociations', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['getTransitGatewayRouteTableAssociations'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Gets information about the route table propagations for the specified transit gateway route table.
   *
   * @function getTransitGatewayRouteTablePropagations
   * @param {string} transitGatewayRouteTableId - The ID of the transit gateway route table.
   * @param {array} filter - One or more filters. The possible values are:         resource-id  - The ID of the resource.         resource-type  - The resource type ( vpc  |  vpn ).         transit-...(description truncated)
   * @param {number} maxResults - The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned  nextToken  value.
   * @param {string} nextToken - The token for the next page of results.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  getTransitGatewayRouteTablePropagations(transitGatewayRouteTableId, filter, maxResults, nextToken, dryRun, callback) {
    const meth = 'adapter-getTransitGatewayRouteTablePropagations';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'GetTransitGatewayRouteTablePropagations';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayRouteTableId === undefined || transitGatewayRouteTableId === null || transitGatewayRouteTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayRouteTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NextToken: nextToken,
      MaxResults: maxResults,
      TransitGatewayRouteTableId: transitGatewayRouteTableId,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'getTransitGatewayRouteTablePropagations', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['getTransitGatewayRouteTablePropagations'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Uploads a client certificate revocation list to the specified Client VPN endpoint. Uploading a client certificate revocation list overwrites the existing client certificate revocation list.   Uploading a client certificate revocation list resets existing client connections.
   *
   * @function importClientVpnClientCertificateRevocationList
   * @param {string} clientVpnEndpointId - The ID of the Client VPN endpoint to which the client certificate revocation list applies.
   * @param {string} certificateRevocationList - The client certificate revocation list file. For more information, see  Generate a Client Certificate Revocation List  in the  AWS Client VPN Administrator Guide .
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  importClientVpnClientCertificateRevocationList(clientVpnEndpointId, certificateRevocationList, dryRun, callback) {
    const meth = 'adapter-importClientVpnClientCertificateRevocationList';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ImportClientVpnClientCertificateRevocationList';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointId === undefined || clientVpnEndpointId === null || clientVpnEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (certificateRevocationList === undefined || certificateRevocationList === null || certificateRevocationList === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['certificateRevocationList'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientVpnEndpointId: clientVpnEndpointId,
      CertificateRevocationList: certificateRevocationList,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'importClientVpnClientCertificateRevocationList', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['importClientVpnClientCertificateRevocationList'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Import single or multi-volume disk images or EBS snapshots into an Amazon Machine Image (AMI). For more information, see  Importing a VM as an Image Using VM Import/Export  in the  VM Import/Export User Guide .
   *
   * @function importImage
   * @param {string} architecture - The architecture of the virtual machine.   Valid values:  i386  |  x86_64
   * @param {string} clientDataComment - Describes the client-specific data.
A user-defined comment about the disk upload.
   * @param {string} clientDataUploadEnd - Describes the client-specific data.
The time that the disk upload ends.
   * @param {string} clientDataUploadSize - Describes the client-specific data.
The size of the uploaded disk image, in GiB.
   * @param {string} clientDataUploadStart - Describes the client-specific data.
The time that the disk upload starts.
   * @param {string} clientToken - The token to enable idempotency for VM import requests.
   * @param {string} description - A description string for the import image task.
   * @param {array} diskContainer - Information about the disk containers.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {boolean} encrypted - Specifies whether the destination AMI of the imported image should be encrypted. The default CMK for EBS is used unless you specify a non-default AWS Key Management Servi...(description truncated)
   * @param {string} hypervisor - The target hypervisor platform.   Valid values:  xen
   * @param {string} kmsKeyId - An identifier for the AWS Key Management Service (AWS KMS) customer master key (CMK) to use when creating the encrypted AMI. This parameter is only required if you want ...(description truncated)
   * @param {string} licenseType - The license type to be used for the Amazon Machine Image (AMI) after importing.     Note:  You may only use BYOL if you have existing licenses with rights to use these l...(description truncated)
   * @param {string} platform - The operating system of the virtual machine.   Valid values:  Windows  |  Linux
   * @param {string} roleName - The name of the role to use when not using the default role, 'vmimport'.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  importImage(architecture, clientDataComment, clientDataUploadEnd, clientDataUploadSize, clientDataUploadStart, clientToken, description, diskContainer, dryRun, encrypted, hypervisor, kmsKeyId, licenseType, platform, roleName, callback) {
    const meth = 'adapter-importImage';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ImportImage';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Architecture: architecture,
      ClientDataComment: clientDataComment,
      ClientDataUploadEnd: clientDataUploadEnd,
      ClientDataUploadSize: clientDataUploadSize,
      ClientDataUploadStart: clientDataUploadStart,
      ClientToken: clientToken,
      Description: description,
      Encrypted: encrypted,
      Hypervisor: hypervisor,
      KmsKeyId: kmsKeyId,
      LicenseType: licenseType,
      Platform: platform,
      RoleName: roleName,
      Action: action,
      Version: version
    };
    if (diskContainer && diskContainer.length > 0) {
      processInArray(diskContainer, queryParamsAvailable, 'DiskContainer');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Image', 'importImage', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['importImage'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates an import instance task using metadata from the specified disk image.  ImportInstance  only supports single-volume VMs. To import multi-volume VMs, use  ImportImage . For more information, see  Importing a Virtual Machine Using the Amazon EC2 CLI .   For information about the import manifest referenced by this API action, see  VM Import Manifest .
   *
   * @function importInstance
   * @param {string} description - A description for the instance being imported.
   * @param {array} diskImage - The disk image.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} launchSpecificationAdditionalInfo - Describes the launch specification for VM import.
Reserved.
   * @param {string} launchSpecificationArchitecture - Describes the launch specification for VM import.
The architecture of the instance.
   * @param {array} launchSpecificationGroupIds - Describes the launch specification for VM import.
The security group IDs.
   * @param {array} launchSpecificationGroupNames - Describes the launch specification for VM import.
The security group names.
   * @param {string} launchSpecificationInstanceInitiatedShutdownBehavior - Describes the launch specification for VM import.
Indicates whether an instance stops or terminates when you initiate shutdown from the instance (using the operating syst...(description truncated)
   * @param {string} launchSpecificationInstanceType - Describes the launch specification for VM import.
The instance type. For more information about the instance types that you can import, see  Instance Types  in the VM Imp...(description truncated)
   * @param {string} launchSpecificationMonitoring - Describes the launch specification for VM import.
Indicates whether monitoring is enabled.
   * @param {string} launchSpecificationPlacement - Describes the launch specification for VM import.
The placement information for the instance.
   * @param {string} launchSpecificationPrivateIpAddress - Describes the launch specification for VM import.
[EC2-VPC] An available IP address from the IP address range of the subnet.
   * @param {string} launchSpecificationSubnetId - Describes the launch specification for VM import.
[EC2-VPC] The ID of the subnet in which to launch the instance.
   * @param {string} launchSpecificationUserData - Describes the launch specification for VM import.
The Base64-encoded user data to make available to the instance.
   * @param {string} platform - The instance operating system.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  importInstance(description, diskImage, dryRun, launchSpecificationAdditionalInfo, launchSpecificationArchitecture, launchSpecificationGroupIds, launchSpecificationGroupNames, launchSpecificationInstanceInitiatedShutdownBehavior, launchSpecificationInstanceType, launchSpecificationMonitoring, launchSpecificationPlacement, launchSpecificationPrivateIpAddress, launchSpecificationSubnetId, launchSpecificationUserData, platform = 'Windows', callback) {
    const meth = 'adapter-importInstance';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ImportInstance';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (platform === undefined || platform === null || platform === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['platform'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Description: description,
      LaunchSpecificationAdditionalInfo: launchSpecificationAdditionalInfo,
      LaunchSpecificationArchitecture: launchSpecificationArchitecture,
      LaunchSpecificationInstanceInitiatedShutdownBehavior: launchSpecificationInstanceInitiatedShutdownBehavior,
      LaunchSpecificationInstanceType: launchSpecificationInstanceType,
      LaunchSpecificationMonitoring: launchSpecificationMonitoring,
      LaunchSpecificationPlacement: launchSpecificationPlacement,
      LaunchSpecificationPrivateIpAddress: launchSpecificationPrivateIpAddress,
      LaunchSpecificationSubnetId: launchSpecificationSubnetId,
      LaunchSpecificationUserData: launchSpecificationUserData,
      Platform: platform,
      Action: action,
      Version: version
    };
    if (diskImage && diskImage.length > 0) {
      processInArray(diskImage, queryParamsAvailable, 'DiskImage');
    }
    if (launchSpecificationGroupIds && launchSpecificationGroupIds.length > 0) {
      processInArray(launchSpecificationGroupIds, queryParamsAvailable, 'LaunchSpecificationGroupIds');
    }
    if (launchSpecificationGroupNames && launchSpecificationGroupNames.length > 0) {
      processInArray(launchSpecificationGroupNames, queryParamsAvailable, 'LaunchSpecificationGroupNames');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'importInstance', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['importInstance'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Imports the public key from an RSA key pair that you created with a third-party tool. Compare this with  CreateKeyPair , in which AWS creates the key pair and gives the keys to you (AWS keeps a copy of the public key). With ImportKeyPair, you create the key pair and give AWS just the public key. The private key is never transferred between you and AWS.   For more information about key pairs, see  Key Pairs  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function importKeyPair
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} keyName - A unique name for the key pair.
   * @param {string} publicKeyMaterial - The public key. For API calls, the text must be base64-encoded. For command line tools, base64 encoding is performed for you.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  importKeyPair(dryRun, keyName, publicKeyMaterial, callback) {
    const meth = 'adapter-importKeyPair';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ImportKeyPair';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (keyName === undefined || keyName === null || keyName === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['keyName'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (publicKeyMaterial === undefined || publicKeyMaterial === null || publicKeyMaterial === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['publicKeyMaterial'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      KeyName: keyName,
      PublicKeyMaterial: publicKeyMaterial,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('KeyPair', 'importKeyPair', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['importKeyPair'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Imports a disk into an EBS snapshot.
   *
   * @function importSnapshot
   * @param {string} clientDataComment - Describes the client-specific data.
A user-defined comment about the disk upload.
   * @param {string} clientDataUploadEnd - Describes the client-specific data.
The time that the disk upload ends.
   * @param {string} clientDataUploadSize - Describes the client-specific data.
The size of the uploaded disk image, in GiB.
   * @param {string} clientDataUploadStart - Describes the client-specific data.
The time that the disk upload starts.
   * @param {string} clientToken - Token to enable idempotency for VM import requests.
   * @param {string} description - The description string for the import snapshot task.
   * @param {string} diskContainerDescription - The disk container object for the import snapshot request.
The description of the disk image being imported.
   * @param {string} diskContainerFormat - The disk container object for the import snapshot request.
 The format of the disk image being imported.   Valid values:  VHD  |  VMDK
   * @param {string} diskContainerUrl - The disk container object for the import snapshot request.
The URL to the Amazon S3-based disk image being imported. It can either be a https URL (https://..) or an Amazo...(description truncated)
   * @param {string} diskContainerUserBucket - The disk container object for the import snapshot request.
The S3 bucket for the disk image.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {boolean} encrypted - Specifies whether the destination snapshot of the imported image should be encrypted. The default CMK for EBS is used unless you specify a non-default AWS Key Management ...(description truncated)
   * @param {string} kmsKeyId - An identifier for the AWS Key Management Service (AWS KMS) customer master key (CMK) to use when creating the encrypted snapshot. This parameter is only required if you ...(description truncated)
   * @param {string} roleName - The name of the role to use when not using the default role, 'vmimport'.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  importSnapshot(clientDataComment, clientDataUploadEnd, clientDataUploadSize, clientDataUploadStart, clientToken, description, diskContainerDescription, diskContainerFormat, diskContainerUrl, diskContainerUserBucket, dryRun, encrypted, kmsKeyId, roleName, callback) {
    const meth = 'adapter-importSnapshot';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ImportSnapshot';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientDataComment: clientDataComment,
      ClientDataUploadEnd: clientDataUploadEnd,
      ClientDataUploadSize: clientDataUploadSize,
      ClientDataUploadStart: clientDataUploadStart,
      ClientToken: clientToken,
      Description: description,
      DiskContainerDescription: diskContainerDescription,
      DiskContainerFormat: diskContainerFormat,
      DiskContainerUrl: diskContainerUrl,
      DiskContainerUserBucket: diskContainerUserBucket,
      Encrypted: encrypted,
      KmsKeyId: kmsKeyId,
      RoleName: roleName,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Snapshot', 'importSnapshot', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['importSnapshot'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates an import volume task using metadata from the specified disk image.For more information, see  Importing Disks to Amazon EBS .   For information about the import manifest referenced by this API action, see  VM Import Manifest .
   *
   * @function importVolume
   * @param {string} availabilityZone - The Availability Zone for the resulting EBS volume.
   * @param {string} description - A description of the volume.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} imageBytes - Describes a disk image.
The size of the disk image, in GiB.
   * @param {string} imageFormat - Describes a disk image.
The disk image format.
   * @param {string} imageImportManifestUrl - Describes a disk image.
 A presigned URL for the import manifest stored in Amazon S3 and presented here as an Amazon S3 presigned URL. For information about creating a pr...(description truncated)
   * @param {string} volumeSize - Describes an EBS volume.
The size of the volume, in GiB.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  importVolume(availabilityZone, description, dryRun, imageBytes, imageFormat, imageImportManifestUrl, volumeSize, callback) {
    const meth = 'adapter-importVolume';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ImportVolume';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (availabilityZone === undefined || availabilityZone === null || availabilityZone === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['availabilityZone'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AvailabilityZone: availabilityZone,
      Description: description,
      ImageBytes: imageBytes,
      ImageFormat: imageFormat,
      ImageImportManifestUrl: imageImportManifestUrl,
      VolumeSize: volumeSize,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Volume', 'importVolume', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['importVolume'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies a Capacity Reservation's capacity and the conditions under which it is to be released. You cannot change a Capacity Reservation's instance type, EBS optimization, instance store settings, platform, Availability Zone, or instance eligibility. If you need to modify any of these attributes, we recommend that you cancel the Capacity Reservation, and then create a new one with the required attributes.
   *
   * @function modifyCapacityReservation
   * @param {string} capacityReservationId - The ID of the Capacity Reservation.
   * @param {number} instanceCount - The number of instances for which to reserve capacity.
   * @param {string} endDate - The date and time at which the Capacity Reservation expires. When a Capacity Reservation expires, the reserved capacity is released and you can no longer launch instance...(description truncated)
   * @param {string} endDateType - Indicates the way in which the Capacity Reservation ends. A Capacity Reservation can have one of the following end types:         unlimited  - The Capacity Reservation r...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyCapacityReservation(capacityReservationId, instanceCount, endDate, endDateType = 'unlimited', dryRun, callback) {
    const meth = 'adapter-modifyCapacityReservation';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyCapacityReservation';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (capacityReservationId === undefined || capacityReservationId === null || capacityReservationId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['capacityReservationId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      CapacityReservationId: capacityReservationId,
      InstanceCount: instanceCount,
      EndDate: endDate,
      EndDateType: endDateType,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Capacity', 'modifyCapacityReservation', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyCapacityReservation'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the specified Client VPN endpoint. You can only modify an endpoint's server certificate information, client connection logging information, DNS server, and description. Modifying the DNS server resets existing client connections.
   *
   * @function modifyClientVpnEndpoint
   * @param {string} clientVpnEndpointId - The ID of the Client VPN endpoint to modify.
   * @param {string} serverCertificateArn - The ARN of the server certificate to be used. The server certificate must be provisioned in AWS Certificate Manager (ACM).
   * @param {string} connectionLogOptionsEnabled - Describes the client connection logging options for the Client VPN endpoint.
Indicates whether connection logging is enabled.
   * @param {string} connectionLogOptionsCloudwatchLogGroup - Describes the client connection logging options for the Client VPN endpoint.
The name of the CloudWatch Logs log group.
   * @param {string} connectionLogOptionsCloudwatchLogStream - Describes the client connection logging options for the Client VPN endpoint.
The name of the CloudWatch Logs log stream to which the connection data is published.
   * @param {array} dnsServersCustomDnsServers - Information about the DNS server to be used.
The IPv4 address range, in CIDR notation, of the DNS servers to be used. You can specify up to two DNS servers. Ensure that t...(description truncated)
   * @param {string} dnsServersEnabled - Information about the DNS server to be used.
Indicates whether DNS servers should be used. Specify  False  to delete the existing DNS servers.
   * @param {string} description - A brief description of the Client VPN endpoint.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyClientVpnEndpoint(clientVpnEndpointId, serverCertificateArn, connectionLogOptionsEnabled, connectionLogOptionsCloudwatchLogGroup, connectionLogOptionsCloudwatchLogStream, dnsServersCustomDnsServers, dnsServersEnabled, description, dryRun, callback) {
    const meth = 'adapter-modifyClientVpnEndpoint';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyClientVpnEndpoint';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointId === undefined || clientVpnEndpointId === null || clientVpnEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientVpnEndpointId: clientVpnEndpointId,
      ServerCertificateArn: serverCertificateArn,
      ConnectionLogOptionsEnabled: connectionLogOptionsEnabled,
      ConnectionLogOptionsCloudwatchLogGroup: connectionLogOptionsCloudwatchLogGroup,
      ConnectionLogOptionsCloudwatchLogStream: connectionLogOptionsCloudwatchLogStream,
      DnsServersEnabled: dnsServersEnabled,
      Description: description,
      Action: action,
      Version: version
    };
    if (dnsServersCustomDnsServers && dnsServersCustomDnsServers.length > 0) {
      processInArray(dnsServersCustomDnsServers, queryParamsAvailable, 'DnsServersCustomDnsServers');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'modifyClientVpnEndpoint', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyClientVpnEndpoint'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the specified EC2 Fleet.   While the EC2 Fleet is being modified, it is in the  modifying  state.
   *
   * @function modifyFleet
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} excessCapacityTerminationPolicy - Indicates whether running instances should be terminated if the total target capacity of the EC2 Fleet is decreased below the current size of the EC2 Fleet.
   * @param {string} fleetId - The ID of the EC2 Fleet.
   * @param {string} targetCapacitySpecificationTotalTargetCapacity - The number of units to request. You can choose to set the target capacity in terms of instances or a performance characteristic that is important to your application work...(description truncated)
   * @param {string} targetCapacitySpecificationOnDemandTargetCapacity - The number of units to request. You can choose to set the target capacity in terms of instances or a performance characteristic that is important to your application work...(description truncated)
   * @param {string} targetCapacitySpecificationSpotTargetCapacity - The number of units to request. You can choose to set the target capacity in terms of instances or a performance characteristic that is important to your application work...(description truncated)
   * @param {string} targetCapacitySpecificationDefaultTargetCapacityType - The number of units to request. You can choose to set the target capacity in terms of instances or a performance characteristic that is important to your application work...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyFleet(dryRun, excessCapacityTerminationPolicy = 'no-termination', fleetId, targetCapacitySpecificationTotalTargetCapacity, targetCapacitySpecificationOnDemandTargetCapacity, targetCapacitySpecificationSpotTargetCapacity, targetCapacitySpecificationDefaultTargetCapacityType, callback) {
    const meth = 'adapter-modifyFleet';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyFleet';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (fleetId === undefined || fleetId === null || fleetId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['fleetId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ExcessCapacityTerminationPolicy: excessCapacityTerminationPolicy,
      FleetId: fleetId,
      TargetCapacitySpecificationTotalTargetCapacity: targetCapacitySpecificationTotalTargetCapacity,
      TargetCapacitySpecificationOnDemandTargetCapacity: targetCapacitySpecificationOnDemandTargetCapacity,
      TargetCapacitySpecificationSpotTargetCapacity: targetCapacitySpecificationSpotTargetCapacity,
      TargetCapacitySpecificationDefaultTargetCapacityType: targetCapacitySpecificationDefaultTargetCapacityType,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Fleet', 'modifyFleet', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyFleet'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the specified attribute of the specified Amazon FPGA Image (AFI).
   *
   * @function modifyFpgaImageAttribute
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} fpgaImageId - The ID of the AFI.
   * @param {string} attribute - The name of the attribute.
   * @param {string} operationType - The operation type.
   * @param {array} userId - The AWS account IDs. This parameter is valid only when modifying the  loadPermission  attribute.
   * @param {array} userGroup - The user groups. This parameter is valid only when modifying the  loadPermission  attribute.
   * @param {array} productCode - The product codes. After you add a product code to an AFI, it can't be removed. This parameter is valid only when modifying the  productCodes  attribute.
   * @param {array} loadPermissionAdd - Describes modifications to the load permissions of an Amazon FPGA image (AFI).
The load permissions to add.
   * @param {array} loadPermissionRemove - Describes modifications to the load permissions of an Amazon FPGA image (AFI).
The load permissions to remove.
   * @param {string} description - A description for the AFI.
   * @param {string} name - A name for the AFI.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyFpgaImageAttribute(dryRun, fpgaImageId, attribute = 'description', operationType = 'add', userId, userGroup, productCode, loadPermissionAdd, loadPermissionRemove, description, name, callback) {
    const meth = 'adapter-modifyFpgaImageAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyFpgaImageAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (fpgaImageId === undefined || fpgaImageId === null || fpgaImageId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['fpgaImageId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      FpgaImageId: fpgaImageId,
      Attribute: attribute,
      OperationType: operationType,
      Description: description,
      Name: name,
      Action: action,
      Version: version
    };
    if (userId && userId.length > 0) {
      processInArray(userId, queryParamsAvailable, 'UserId');
    }
    if (userGroup && userGroup.length > 0) {
      processInArray(userGroup, queryParamsAvailable, 'UserGroup');
    }
    if (productCode && productCode.length > 0) {
      processInArray(productCode, queryParamsAvailable, 'ProductCode');
    }
    if (loadPermissionAdd && loadPermissionAdd.length > 0) {
      processInArray(loadPermissionAdd, queryParamsAvailable, 'LoadPermissionAdd');
    }
    if (loadPermissionRemove && loadPermissionRemove.length > 0) {
      processInArray(loadPermissionRemove, queryParamsAvailable, 'LoadPermissionRemove');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Image', 'modifyFpgaImageAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyFpgaImageAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modify the auto-placement setting of a Dedicated Host. When auto-placement is enabled, any instances that you launch with a tenancy of  host  but without a specific host ID are placed onto any available Dedicated Host in your account that has auto-placement enabled. When auto-placement is disabled, you need to provide a host ID to have the instance launch onto a specific host. If no host ID is provided, the instance is launched onto a suitable host with auto-placement enabled.
   *
   * @function modifyHosts
   * @param {string} autoPlacement - Specify whether to enable or disable auto-placement.
   * @param {array} hostId - The IDs of the Dedicated Hosts to modify.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyHosts(autoPlacement = 'on', hostId, callback) {
    const meth = 'adapter-modifyHosts';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyHosts';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (autoPlacement === undefined || autoPlacement === null || autoPlacement === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['autoPlacement'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (hostId === undefined || hostId === null || hostId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['hostId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      AutoPlacement: autoPlacement,
      Action: action,
      Version: version
    };
    if (hostId && hostId.length > 0) {
      processInArray(hostId, queryParamsAvailable, 'HostId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Host', 'modifyHosts', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyHosts'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the ID format for the specified resource on a per-Region basis. You can specify that resources should receive longer IDs (17-character IDs) when they are created.   This request can only be used to modify longer ID settings for resource types that are within the opt-in period. Resources currently in their opt-in period include:  bundle  |  conversion-task  |  customer-gateway  |  dhcp-options  |  elastic-ip-allocation  |  elastic-ip-association  |  export-task  |  flow-log  |  image  |...(description truncated)
   *
   * @function modifyIdFormat
   * @param {string} resource - The type of resource:  bundle  |  conversion-task  |  customer-gateway  |  dhcp-options  |  elastic-ip-allocation  |  elastic-ip-association  |  export-task  |  flow-log...(description truncated)
   * @param {boolean} useLongIds - Indicate whether the resource should use longer IDs (17-character IDs).
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyIdFormat(resource, useLongIds, callback) {
    const meth = 'adapter-modifyIdFormat';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyIdFormat';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (resource === undefined || resource === null || resource === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['resource'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (useLongIds === undefined || useLongIds === null || useLongIds === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['useLongIds'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      Resource: resource,
      UseLongIds: useLongIds,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Id', 'modifyIdFormat', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyIdFormat'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the ID format of a resource for a specified IAM user, IAM role, or the root user for an account; or all IAM users, IAM roles, and the root user for an account. You can specify that resources should receive longer IDs (17-character IDs) when they are created.    This request can only be used to modify longer ID settings for resource types that are within the opt-in period. Resources currently in their opt-in period include:  bundle  |  conversion-task  |  customer-gateway  |  dhcp-optio...(description truncated)
   *
   * @function modifyIdentityIdFormat
   * @param {string} principalArn - The ARN of the principal, which can be an IAM user, IAM role, or the root user. Specify  all  to modify the ID format for all IAM users, IAM roles, and the root user of t...(description truncated)
   * @param {string} resource - The type of resource:  bundle  |  conversion-task  |  customer-gateway  |  dhcp-options  |  elastic-ip-allocation  |  elastic-ip-association  |  export-task  |  flow-log...(description truncated)
   * @param {boolean} useLongIds - Indicates whether the resource should use longer IDs (17-character IDs)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyIdentityIdFormat(principalArn, resource, useLongIds, callback) {
    const meth = 'adapter-modifyIdentityIdFormat';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyIdentityIdFormat';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (principalArn === undefined || principalArn === null || principalArn === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['principalArn'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (resource === undefined || resource === null || resource === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['resource'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (useLongIds === undefined || useLongIds === null || useLongIds === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['useLongIds'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      PrincipalArn: principalArn,
      Resource: resource,
      UseLongIds: useLongIds,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Id', 'modifyIdentityIdFormat', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyIdentityIdFormat'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the specified attribute of the specified AMI. You can specify only one attribute at a time. You can use the  Attribute  parameter to specify the attribute or one of the following parameters:  Description ,  LaunchPermission , or  ProductCode .   AWS Marketplace product codes cannot be modified. Images with an AWS Marketplace product code cannot be made public.   To enable the SriovNetSupport enhanced networking attribute of an image, enable SriovNetSupport on an instance and create an ...(description truncated)
   *
   * @function modifyImageAttribute
   * @param {string} attribute - The name of the attribute to modify. The valid values are  description ,  launchPermission , and  productCodes .
   * @param {string} descriptionValue - Describes a value for a resource attribute that is a String.
The attribute value. The value is case-sensitive.
   * @param {string} imageId - The ID of the AMI.
   * @param {array} launchPermissionAdd - Describes a launch permission modification.
The AWS account ID to add to the list of launch permissions for the AMI.
   * @param {array} launchPermissionRemove - Describes a launch permission modification.
The AWS account ID to remove from the list of launch permissions for the AMI.
   * @param {string} operationType - The operation type. This parameter can be used only when the  Attribute  parameter is  launchPermission .
   * @param {array} productCode - The DevPay product codes. After you add a product code to an AMI, it can't be removed.
   * @param {array} userGroup - The user groups. This parameter can be used only when the  Attribute  parameter is  launchPermission .
   * @param {array} userId - The AWS account IDs. This parameter can be used only when the  Attribute  parameter is  launchPermission .
   * @param {string} value - The value of the attribute being modified. This parameter can be used only when the  Attribute  parameter is  description  or  productCodes .
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyImageAttribute(attribute, descriptionValue, imageId, launchPermissionAdd, launchPermissionRemove, operationType = 'add', productCode, userGroup, userId, value, dryRun, callback) {
    const meth = 'adapter-modifyImageAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyImageAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (imageId === undefined || imageId === null || imageId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['imageId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Attribute: attribute,
      DescriptionValue: descriptionValue,
      ImageId: imageId,
      OperationType: operationType,
      Value: value,
      Action: action,
      Version: version
    };
    if (launchPermissionAdd && launchPermissionAdd.length > 0) {
      processInArray(launchPermissionAdd, queryParamsAvailable, 'LaunchPermissionAdd');
    }
    if (launchPermissionRemove && launchPermissionRemove.length > 0) {
      processInArray(launchPermissionRemove, queryParamsAvailable, 'LaunchPermissionRemove');
    }
    if (productCode && productCode.length > 0) {
      processInArray(productCode, queryParamsAvailable, 'ProductCode');
    }
    if (userGroup && userGroup.length > 0) {
      processInArray(userGroup, queryParamsAvailable, 'UserGroup');
    }
    if (userId && userId.length > 0) {
      processInArray(userId, queryParamsAvailable, 'UserId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Image', 'modifyImageAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyImageAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the specified attribute of the specified instance. You can specify only one attribute at a time.     Note:  Using this action to change the security groups associated with an elastic network interface (ENI) attached to an instance in a VPC can result in an error if the instance has more than one ENI. To change the security groups associated with an ENI attached to an instance that has multiple ENIs, we recommend that you use the  ModifyNetworkInterfaceAttribute  action.   To modify som...(description truncated)
   *
   * @function modifyInstanceAttribute
   * @param {string} sourceDestCheckValue - Describes a value for a resource attribute that is a Boolean value.
The attribute value. The valid values are  true  or  false .
   * @param {string} attribute - The name of the attribute.
   * @param {array} blockDeviceMapping - Modifies the  DeleteOnTermination  attribute for volumes that are currently attached. The volume must be owned by the caller. If no value is specified for  DeleteOnTermi...(description truncated)
   * @param {string} disableApiTerminationValue - Describes a value for a resource attribute that is a Boolean value.
The attribute value. The valid values are  true  or  false .
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} ebsOptimizedValue - Describes a value for a resource attribute that is a Boolean value.
The attribute value. The valid values are  true  or  false .
   * @param {string} enaSupportValue - Describes a value for a resource attribute that is a Boolean value.
The attribute value. The valid values are  true  or  false .
   * @param {array} groupId - [EC2-VPC] Changes the security groups of the instance. You must specify at least one security group, even if it's just the default security group for the VPC. You must sp...(description truncated)
   * @param {string} instanceId - The ID of the instance.
   * @param {string} instanceInitiatedShutdownBehaviorValue - Describes a value for a resource attribute that is a String.
The attribute value. The value is case-sensitive.
   * @param {string} instanceTypeValue - Describes a value for a resource attribute that is a String.
The attribute value. The value is case-sensitive.
   * @param {string} kernelValue - Describes a value for a resource attribute that is a String.
The attribute value. The value is case-sensitive.
   * @param {string} ramdiskValue - Describes a value for a resource attribute that is a String.
The attribute value. The value is case-sensitive.
   * @param {string} sriovNetSupportValue - Describes a value for a resource attribute that is a String.
The attribute value. The value is case-sensitive.
   * @param {string} userDataValue - Changes the instance's user data to the specified value. If you are using an AWS SDK or command line tool, base64-encoding is performed for you, and you can load the text...(description truncated)
   * @param {string} value - A new value for the attribute. Use only with the  kernel ,  ramdisk ,  userData ,  disableApiTermination , or  instanceInitiatedShutdownBehavior  attribute.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyInstanceAttribute(sourceDestCheckValue, attribute = 'instanceType', blockDeviceMapping, disableApiTerminationValue, dryRun, ebsOptimizedValue, enaSupportValue, groupId, instanceId, instanceInitiatedShutdownBehaviorValue, instanceTypeValue, kernelValue, ramdiskValue, sriovNetSupportValue, userDataValue, value, callback) {
    const meth = 'adapter-modifyInstanceAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyInstanceAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      SourceDestCheckValue: sourceDestCheckValue,
      Attribute: attribute,
      DisableApiTerminationValue: disableApiTerminationValue,
      EbsOptimizedValue: ebsOptimizedValue,
      EnaSupportValue: enaSupportValue,
      InstanceId: instanceId,
      InstanceInitiatedShutdownBehaviorValue: instanceInitiatedShutdownBehaviorValue,
      InstanceTypeValue: instanceTypeValue,
      KernelValue: kernelValue,
      RamdiskValue: ramdiskValue,
      SriovNetSupportValue: sriovNetSupportValue,
      UserDataValue: userDataValue,
      Value: value,
      Action: action,
      Version: version
    };
    if (blockDeviceMapping && blockDeviceMapping.length > 0) {
      processInArray(blockDeviceMapping, queryParamsAvailable, 'BlockDeviceMapping');
    }
    if (groupId && groupId.length > 0) {
      processInArray(groupId, queryParamsAvailable, 'GroupId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'modifyInstanceAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyInstanceAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the Capacity Reservation settings for a stopped instance. Use this action to configure an instance to target a specific Capacity Reservation, run in any  open  Capacity Reservation with matching attributes, or run On-Demand Instance capacity.
   *
   * @function modifyInstanceCapacityReservationAttributes
   * @param {string} instanceId - The ID of the instance to be modified.
   * @param {string} capacityReservationSpecificationCapacityReservationPreference - Describes an instance's Capacity Reservation targeting option. You can specify only one parameter at a time. If you specify  CapacityReservationPreference  and  Capacity...(description truncated)
   * @param {string} capacityReservationSpecificationCapacityReservationTarget - Describes an instance's Capacity Reservation targeting option. You can specify only one parameter at a time. If you specify  CapacityReservationPreference  and  Capacity...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyInstanceCapacityReservationAttributes(instanceId, capacityReservationSpecificationCapacityReservationPreference, capacityReservationSpecificationCapacityReservationTarget, dryRun, callback) {
    const meth = 'adapter-modifyInstanceCapacityReservationAttributes';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyInstanceCapacityReservationAttributes';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      InstanceId: instanceId,
      CapacityReservationSpecificationCapacityReservationPreference: capacityReservationSpecificationCapacityReservationPreference,
      CapacityReservationSpecificationCapacityReservationTarget: capacityReservationSpecificationCapacityReservationTarget,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'modifyInstanceCapacityReservationAttributes', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyInstanceCapacityReservationAttributes'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the credit option for CPU usage on a running or stopped T2 or T3 instance. The credit options are  standard  and  unlimited .   For more information, see  Burstable Performance Instances  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function modifyInstanceCreditSpecification
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} clientToken - A unique, case-sensitive token that you provide to ensure idempotency of your modification request. For more information, see  Ensuring Idempotency .
   * @param {array} instanceCreditSpecification - Information about the credit option for CPU usage.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyInstanceCreditSpecification(dryRun, clientToken, instanceCreditSpecification, callback) {
    const meth = 'adapter-modifyInstanceCreditSpecification';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyInstanceCreditSpecification';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceCreditSpecification === undefined || instanceCreditSpecification === null || instanceCreditSpecification === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceCreditSpecification'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientToken: clientToken,
      Action: action,
      Version: version
    };
    if (instanceCreditSpecification && instanceCreditSpecification.length > 0) {
      processInArray(instanceCreditSpecification, queryParamsAvailable, 'InstanceCreditSpecification');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'modifyInstanceCreditSpecification', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyInstanceCreditSpecification'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the start time for a scheduled Amazon EC2 instance event.
   *
   * @function modifyInstanceEventStartTime
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} instanceId - The ID of the instance with the scheduled event.
   * @param {string} instanceEventId - The ID of the event whose date and time you are modifying.
   * @param {string} notBefore - The new date and time when the event will take place.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyInstanceEventStartTime(dryRun, instanceId, instanceEventId, notBefore, callback) {
    const meth = 'adapter-modifyInstanceEventStartTime';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyInstanceEventStartTime';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (instanceEventId === undefined || instanceEventId === null || instanceEventId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceEventId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (notBefore === undefined || notBefore === null || notBefore === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['notBefore'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      InstanceId: instanceId,
      InstanceEventId: instanceEventId,
      NotBefore: notBefore,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'modifyInstanceEventStartTime', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyInstanceEventStartTime'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the placement attributes for a specified instance. You can do the following:       Modify the affinity between an instance and a  Dedicated Host . When affinity is set to  host  and the instance is not associated with a specific Dedicated Host, the next time the instance is launched, it is automatically associated with the host on which it lands. If the instance is restarted or rebooted, this relationship persists.       Change the Dedicated Host with which an instance is associated.  ...(description truncated)
   *
   * @function modifyInstancePlacement
   * @param {string} affinity - The affinity setting for the instance.
   * @param {string} groupName - The name of the placement group in which to place the instance. For spread placement groups, the instance must have a tenancy of  default . For cluster and partition pla...(description truncated)
   * @param {string} hostId - The ID of the Dedicated Host with which to associate the instance.
   * @param {string} instanceId - The ID of the instance that you are modifying.
   * @param {string} tenancy - The tenancy for the instance.
   * @param {number} partitionNumber - Reserved for future use.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyInstancePlacement(affinity = 'default', groupName, hostId, instanceId, tenancy = 'dedicated', partitionNumber, callback) {
    const meth = 'adapter-modifyInstancePlacement';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyInstancePlacement';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      Affinity: affinity,
      GroupName: groupName,
      HostId: hostId,
      InstanceId: instanceId,
      Tenancy: tenancy,
      PartitionNumber: partitionNumber,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'modifyInstancePlacement', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyInstancePlacement'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies a launch template. You can specify which version of the launch template to set as the default version. When launching an instance, the default version applies when a launch template version is not specified.
   *
   * @function modifyLaunchTemplate
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} clientToken - Unique, case-sensitive identifier you provide to ensure the idempotency of the request. For more information, see  Ensuring Idempotency .   Constraint: Maximum 128 ASCII...(description truncated)
   * @param {string} launchTemplateId - The ID of the launch template. You must specify either the launch template ID or launch template name in the request.
   * @param {string} launchTemplateName - The name of the launch template. You must specify either the launch template ID or launch template name in the request.
   * @param {string} setDefaultVersion - The version number of the launch template to set as the default version.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyLaunchTemplate(dryRun, clientToken, launchTemplateId, launchTemplateName, setDefaultVersion, callback) {
    const meth = 'adapter-modifyLaunchTemplate';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyLaunchTemplate';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientToken: clientToken,
      LaunchTemplateId: launchTemplateId,
      LaunchTemplateName: launchTemplateName,
      SetDefaultVersion: setDefaultVersion,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('LaunchTemplate', 'modifyLaunchTemplate', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyLaunchTemplate'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the specified network interface attribute. You can specify only one attribute at a time. You can use this action to attach and detach security groups from an existing EC2 instance.
   *
   * @function modifyNetworkInterfaceAttribute
   * @param {string} attachmentAttachmentId - Describes an attachment change.
The ID of the network interface attachment.
   * @param {string} attachmentDeleteOnTermination - Describes an attachment change.
Indicates whether the network interface is deleted when the instance is terminated.
   * @param {string} descriptionValue - Describes a value for a resource attribute that is a String.
The attribute value. The value is case-sensitive.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} securityGroupId - Changes the security groups for the network interface. The new set of groups you specify replaces the current set. You must specify at least one group, even if it's just ...(description truncated)
   * @param {string} networkInterfaceId - The ID of the network interface.
   * @param {string} sourceDestCheckValue - Describes a value for a resource attribute that is a Boolean value.
The attribute value. The valid values are  true  or  false .
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyNetworkInterfaceAttribute(attachmentAttachmentId, attachmentDeleteOnTermination, descriptionValue, dryRun, securityGroupId, networkInterfaceId, sourceDestCheckValue, callback) {
    const meth = 'adapter-modifyNetworkInterfaceAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyNetworkInterfaceAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (networkInterfaceId === undefined || networkInterfaceId === null || networkInterfaceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['networkInterfaceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AttachmentAttachmentId: attachmentAttachmentId,
      AttachmentDeleteOnTermination: attachmentDeleteOnTermination,
      DescriptionValue: descriptionValue,
      NetworkInterfaceId: networkInterfaceId,
      SourceDestCheckValue: sourceDestCheckValue,
      Action: action,
      Version: version
    };
    if (securityGroupId && securityGroupId.length > 0) {
      processInArray(securityGroupId, queryParamsAvailable, 'SecurityGroupId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkInterface', 'modifyNetworkInterfaceAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyNetworkInterfaceAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the Availability Zone, instance count, instance type, or network platform (EC2-Classic or EC2-VPC) of your Reserved Instances. The Reserved Instances to be modified must be identical, except for Availability Zone, network platform, and instance type.   For more information, see  Modifying Reserved Instances  in the Amazon Elastic Compute Cloud User Guide.
   *
   * @function modifyReservedInstances
   * @param {array} reservedInstancesId - The IDs of the Reserved Instances to modify.
   * @param {string} clientToken - A unique, case-sensitive token you provide to ensure idempotency of your modification request. For more information, see  Ensuring Idempotency .
   * @param {array} reservedInstancesConfigurationSetItemType - The configuration settings for the Reserved Instances to modify.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyReservedInstances(reservedInstancesId, clientToken, reservedInstancesConfigurationSetItemType, callback) {
    const meth = 'adapter-modifyReservedInstances';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyReservedInstances';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (reservedInstancesId === undefined || reservedInstancesId === null || reservedInstancesId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['reservedInstancesId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (reservedInstancesConfigurationSetItemType === undefined || reservedInstancesConfigurationSetItemType === null || reservedInstancesConfigurationSetItemType === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['reservedInstancesConfigurationSetItemType'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      ClientToken: clientToken,
      Action: action,
      Version: version
    };
    if (reservedInstancesId && reservedInstancesId.length > 0) {
      processInArray(reservedInstancesId, queryParamsAvailable, 'ReservedInstancesId');
    }
    if (reservedInstancesConfigurationSetItemType && reservedInstancesConfigurationSetItemType.length > 0) {
      processInArray(reservedInstancesConfigurationSetItemType, queryParamsAvailable, 'ReservedInstancesConfigurationSetItemType');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'modifyReservedInstances', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyReservedInstances'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Adds or removes permission settings for the specified snapshot. You may add or remove specified AWS account IDs from a snapshot's list of create volume permissions, but you cannot do both in a single API call. If you need to both add and remove account IDs for a snapshot, you must use multiple API calls.   Encrypted snapshots and snapshots with AWS Marketplace product codes cannot be made public. Snapshots encrypted with your default CMK cannot be shared with other accounts.   For more informat...(description truncated)
   *
   * @function modifySnapshotAttribute
   * @param {string} attribute - The snapshot attribute to modify. Only volume creation permissions can be modified.
   * @param {array} createVolumePermissionAdd - Describes modifications to the list of create volume permissions for a volume.
Adds the specified AWS account ID or group to the list.
   * @param {array} createVolumePermissionRemove - Describes modifications to the list of create volume permissions for a volume.
Removes the specified AWS account ID or group from the list.
   * @param {array} userGroup - The group to modify for the snapshot.
   * @param {string} operationType - The type of operation to perform to the attribute.
   * @param {string} snapshotId - The ID of the snapshot.
   * @param {array} userId - The account ID to modify for the snapshot.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifySnapshotAttribute(attribute = 'productCodes', createVolumePermissionAdd, createVolumePermissionRemove, userGroup, operationType = 'add', snapshotId, userId, dryRun, callback) {
    const meth = 'adapter-modifySnapshotAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifySnapshotAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (snapshotId === undefined || snapshotId === null || snapshotId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['snapshotId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Attribute: attribute,
      OperationType: operationType,
      SnapshotId: snapshotId,
      Action: action,
      Version: version
    };
    if (createVolumePermissionAdd && createVolumePermissionAdd.length > 0) {
      processInArray(createVolumePermissionAdd, queryParamsAvailable, 'CreateVolumePermissionAdd');
    }
    if (createVolumePermissionRemove && createVolumePermissionRemove.length > 0) {
      processInArray(createVolumePermissionRemove, queryParamsAvailable, 'CreateVolumePermissionRemove');
    }
    if (userGroup && userGroup.length > 0) {
      processInArray(userGroup, queryParamsAvailable, 'UserGroup');
    }
    if (userId && userId.length > 0) {
      processInArray(userId, queryParamsAvailable, 'UserId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Snapshot', 'modifySnapshotAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifySnapshotAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the specified Spot Fleet request.   While the Spot Fleet request is being modified, it is in the  modifying  state.   To scale up your Spot Fleet, increase its target capacity. The Spot Fleet launches the additional Spot Instances according to the allocation strategy for the Spot Fleet request. If the allocation strategy is  lowestPrice , the Spot Fleet launches instances using the Spot pool with the lowest price. If the allocation strategy is  diversified , the Spot Fleet distributes ...(description truncated)
   *
   * @function modifySpotFleetRequest
   * @param {string} excessCapacityTerminationPolicy - Indicates whether running Spot Instances should be terminated if the target capacity of the Spot Fleet request is decreased below the current size of the Spot Fleet.
   * @param {string} spotFleetRequestId - The ID of the Spot Fleet request.
   * @param {number} targetCapacity - The size of the fleet.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifySpotFleetRequest(excessCapacityTerminationPolicy = 'noTermination', spotFleetRequestId, targetCapacity, callback) {
    const meth = 'adapter-modifySpotFleetRequest';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifySpotFleetRequest';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (spotFleetRequestId === undefined || spotFleetRequestId === null || spotFleetRequestId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['spotFleetRequestId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      ExcessCapacityTerminationPolicy: excessCapacityTerminationPolicy,
      SpotFleetRequestId: spotFleetRequestId,
      TargetCapacity: targetCapacity,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Fleet', 'modifySpotFleetRequest', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifySpotFleetRequest'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies a subnet attribute. You can only modify one attribute at a time.
   *
   * @function modifySubnetAttribute
   * @param {string} assignIpv6AddressOnCreationValue - Describes a value for a resource attribute that is a Boolean value.
The attribute value. The valid values are  true  or  false .
   * @param {string} mapPublicIpOnLaunchValue - Describes a value for a resource attribute that is a Boolean value.
The attribute value. The valid values are  true  or  false .
   * @param {string} subnetId - The ID of the subnet.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifySubnetAttribute(assignIpv6AddressOnCreationValue, mapPublicIpOnLaunchValue, subnetId, callback) {
    const meth = 'adapter-modifySubnetAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifySubnetAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (subnetId === undefined || subnetId === null || subnetId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['subnetId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      AssignIpv6AddressOnCreationValue: assignIpv6AddressOnCreationValue,
      MapPublicIpOnLaunchValue: mapPublicIpOnLaunchValue,
      SubnetId: subnetId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Subnet', 'modifySubnetAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifySubnetAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the specified VPC attachment.
   *
   * @function modifyTransitGatewayVpcAttachment
   * @param {string} transitGatewayAttachmentId - The ID of the attachment.
   * @param {array} addSubnetIds - The IDs of one or more subnets to add. You can specify at most one subnet per Availability Zone.
   * @param {array} removeSubnetIds - The IDs of one or more subnets to remove.
   * @param {string} optionsDnsSupport - Describes the options for a VPC attachment.
Enable or disable DNS support. The default is  enable .
   * @param {string} optionsIpv6Support - Describes the options for a VPC attachment.
Enable or disable IPv6 support. The default is  enable .
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyTransitGatewayVpcAttachment(transitGatewayAttachmentId, addSubnetIds, removeSubnetIds, optionsDnsSupport, optionsIpv6Support, dryRun, callback) {
    const meth = 'adapter-modifyTransitGatewayVpcAttachment';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyTransitGatewayVpcAttachment';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayAttachmentId === undefined || transitGatewayAttachmentId === null || transitGatewayAttachmentId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayAttachmentId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      TransitGatewayAttachmentId: transitGatewayAttachmentId,
      OptionsDnsSupport: optionsDnsSupport,
      OptionsIpv6Support: optionsIpv6Support,
      Action: action,
      Version: version
    };
    if (addSubnetIds && addSubnetIds.length > 0) {
      processInArray(addSubnetIds, queryParamsAvailable, 'AddSubnetIds');
    }
    if (removeSubnetIds && removeSubnetIds.length > 0) {
      processInArray(removeSubnetIds, queryParamsAvailable, 'RemoveSubnetIds');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'modifyTransitGatewayVpcAttachment', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyTransitGatewayVpcAttachment'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary You can modify several parameters of an existing EBS volume, including volume size, volume type, and IOPS capacity. If your EBS volume is attached to a current-generation EC2 instance type, you may be able to apply these changes without stopping the instance or detaching the volume from it. For more information about modifying an EBS volume running Linux, see  Modifying the Size, IOPS, or Type of an EBS Volume on Linux . For more information about modifying an EBS volume running Windows, see  M...(description truncated)
   *
   * @function modifyVolume
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} volumeId - The ID of the volume.
   * @param {number} size - The target size of the volume, in GiB. The target volume size must be greater than or equal to than the existing size of the volume. For information about available EBS ...(description truncated)
   * @param {string} volumeType - The target EBS volume type of the volume.   Default: If no type is specified, the existing type is retained.
   * @param {number} iops - The target IOPS rate of the volume.   This is only valid for Provisioned IOPS SSD ( io1 ) volumes. For more information, see  Provisioned IOPS SSD (io1) Volumes .   Defa...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyVolume(dryRun, volumeId, size, volumeType = 'standard', iops, callback) {
    const meth = 'adapter-modifyVolume';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyVolume';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (volumeId === undefined || volumeId === null || volumeId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['volumeId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VolumeId: volumeId,
      Size: size,
      VolumeType: volumeType,
      Iops: iops,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Volume', 'modifyVolume', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyVolume'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies a volume attribute.   By default, all I/O operations for the volume are suspended when the data on the volume is determined to be potentially inconsistent, to prevent undetectable, latent data corruption. The I/O access to the volume can be resumed by first enabling I/O access and then checking the data consistency on your volume.   You can change the default behavior to resume I/O operations. We recommend that you change this only for boot volumes or for volumes that are stateless or ...(description truncated)
   *
   * @function modifyVolumeAttribute
   * @param {string} autoEnableIOValue - Describes a value for a resource attribute that is a Boolean value.
The attribute value. The valid values are  true  or  false .
   * @param {string} volumeId - The ID of the volume.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyVolumeAttribute(autoEnableIOValue, volumeId, dryRun, callback) {
    const meth = 'adapter-modifyVolumeAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyVolumeAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (volumeId === undefined || volumeId === null || volumeId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['volumeId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AutoEnableIOValue: autoEnableIOValue,
      VolumeId: volumeId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Volume', 'modifyVolumeAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyVolumeAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the specified attribute of the specified VPC.
   *
   * @function modifyVpcAttribute
   * @param {string} enableDnsHostnamesValue - Describes a value for a resource attribute that is a Boolean value.
The attribute value. The valid values are  true  or  false .
   * @param {string} enableDnsSupportValue - Describes a value for a resource attribute that is a Boolean value.
The attribute value. The valid values are  true  or  false .
   * @param {string} vpcId - The ID of the VPC.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyVpcAttribute(enableDnsHostnamesValue, enableDnsSupportValue, vpcId, callback) {
    const meth = 'adapter-modifyVpcAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyVpcAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      EnableDnsHostnamesValue: enableDnsHostnamesValue,
      EnableDnsSupportValue: enableDnsSupportValue,
      VpcId: vpcId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'modifyVpcAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyVpcAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies attributes of a specified VPC endpoint. The attributes that you can modify depend on the type of VPC endpoint (interface or gateway). For more information, see  VPC Endpoints  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function modifyVpcEndpoint
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} vpcEndpointId - The ID of the endpoint.
   * @param {boolean} resetPolicy - (Gateway endpoint) Specify  true  to reset the policy document to the default policy. The default policy allows full access to the service.
   * @param {string} policyDocument - A policy to attach to the endpoint that controls access to the service. The policy must be in valid JSON format. If this parameter is not specified, we attach a default p...(description truncated)
   * @param {array} addRouteTableId - (Gateway endpoint) One or more route tables IDs to associate with the endpoint.
   * @param {array} removeRouteTableId - (Gateway endpoint) One or more route table IDs to disassociate from the endpoint.
   * @param {array} addSubnetId - (Interface endpoint) One or more subnet IDs in which to serve the endpoint.
   * @param {array} removeSubnetId - (Interface endpoint) One or more subnets IDs in which to remove the endpoint.
   * @param {array} addSecurityGroupId - (Interface endpoint) One or more security group IDs to associate with the network interface.
   * @param {array} removeSecurityGroupId - (Interface endpoint) One or more security group IDs to disassociate from the network interface.
   * @param {boolean} privateDnsEnabled - (Interface endpoint) Indicate whether a private hosted zone is associated with the VPC.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyVpcEndpoint(dryRun, vpcEndpointId, resetPolicy, policyDocument, addRouteTableId, removeRouteTableId, addSubnetId, removeSubnetId, addSecurityGroupId, removeSecurityGroupId, privateDnsEnabled, callback) {
    const meth = 'adapter-modifyVpcEndpoint';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyVpcEndpoint';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcEndpointId === undefined || vpcEndpointId === null || vpcEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VpcEndpointId: vpcEndpointId,
      ResetPolicy: resetPolicy,
      PolicyDocument: policyDocument,
      PrivateDnsEnabled: privateDnsEnabled,
      Action: action,
      Version: version
    };
    if (addRouteTableId && addRouteTableId.length > 0) {
      processInArray(addRouteTableId, queryParamsAvailable, 'AddRouteTableId');
    }
    if (removeRouteTableId && removeRouteTableId.length > 0) {
      processInArray(removeRouteTableId, queryParamsAvailable, 'RemoveRouteTableId');
    }
    if (addSubnetId && addSubnetId.length > 0) {
      processInArray(addSubnetId, queryParamsAvailable, 'AddSubnetId');
    }
    if (removeSubnetId && removeSubnetId.length > 0) {
      processInArray(removeSubnetId, queryParamsAvailable, 'RemoveSubnetId');
    }
    if (addSecurityGroupId && addSecurityGroupId.length > 0) {
      processInArray(addSecurityGroupId, queryParamsAvailable, 'AddSecurityGroupId');
    }
    if (removeSecurityGroupId && removeSecurityGroupId.length > 0) {
      processInArray(removeSecurityGroupId, queryParamsAvailable, 'RemoveSecurityGroupId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'modifyVpcEndpoint', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyVpcEndpoint'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies a connection notification for VPC endpoint or VPC endpoint service. You can change the SNS topic for the notification, or the events for which to be notified.
   *
   * @function modifyVpcEndpointConnectionNotification
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} connectionNotificationId - The ID of the notification.
   * @param {string} connectionNotificationArn - The ARN for the SNS topic for the notification.
   * @param {array} connectionEvents - One or more events for the endpoint. Valid values are  Accept ,  Connect ,  Delete , and  Reject .
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyVpcEndpointConnectionNotification(dryRun, connectionNotificationId, connectionNotificationArn, connectionEvents, callback) {
    const meth = 'adapter-modifyVpcEndpointConnectionNotification';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyVpcEndpointConnectionNotification';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (connectionNotificationId === undefined || connectionNotificationId === null || connectionNotificationId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['connectionNotificationId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ConnectionNotificationId: connectionNotificationId,
      ConnectionNotificationArn: connectionNotificationArn,
      Action: action,
      Version: version
    };
    if (connectionEvents && connectionEvents.length > 0) {
      processInArray(connectionEvents, queryParamsAvailable, 'ConnectionEvents');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'modifyVpcEndpointConnectionNotification', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyVpcEndpointConnectionNotification'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the attributes of your VPC endpoint service configuration. You can change the Network Load Balancers for your service, and you can specify whether acceptance is required for requests to connect to your endpoint service through an interface VPC endpoint.
   *
   * @function modifyVpcEndpointServiceConfiguration
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} serviceId - The ID of the service.
   * @param {boolean} acceptanceRequired - Indicate whether requests to create an endpoint to your service must be accepted.
   * @param {array} addNetworkLoadBalancerArn - The Amazon Resource Names (ARNs) of Network Load Balancers to add to your service configuration.
   * @param {array} removeNetworkLoadBalancerArn - The Amazon Resource Names (ARNs) of Network Load Balancers to remove from your service configuration.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyVpcEndpointServiceConfiguration(dryRun, serviceId, acceptanceRequired, addNetworkLoadBalancerArn, removeNetworkLoadBalancerArn, callback) {
    const meth = 'adapter-modifyVpcEndpointServiceConfiguration';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyVpcEndpointServiceConfiguration';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (serviceId === undefined || serviceId === null || serviceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['serviceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ServiceId: serviceId,
      AcceptanceRequired: acceptanceRequired,
      Action: action,
      Version: version
    };
    if (addNetworkLoadBalancerArn && addNetworkLoadBalancerArn.length > 0) {
      processInArray(addNetworkLoadBalancerArn, queryParamsAvailable, 'AddNetworkLoadBalancerArn');
    }
    if (removeNetworkLoadBalancerArn && removeNetworkLoadBalancerArn.length > 0) {
      processInArray(removeNetworkLoadBalancerArn, queryParamsAvailable, 'RemoveNetworkLoadBalancerArn');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'modifyVpcEndpointServiceConfiguration', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyVpcEndpointServiceConfiguration'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the permissions for your  VPC endpoint service . You can add or remove permissions for service consumers (IAM users, IAM roles, and AWS accounts) to connect to your endpoint service.   If you grant permissions to all principals, the service is public. Any users who know the name of a public service can send a request to attach an endpoint. If the service does not require manual approval, attachments are automatically approved.
   *
   * @function modifyVpcEndpointServicePermissions
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} serviceId - The ID of the service.
   * @param {array} addAllowedPrincipals - The Amazon Resource Names (ARN) of one or more principals. Permissions are granted to the principals in this list. To grant permissions to all principals, specify an aste...(description truncated)
   * @param {array} removeAllowedPrincipals - The Amazon Resource Names (ARN) of one or more principals. Permissions are revoked for principals in this list.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyVpcEndpointServicePermissions(dryRun, serviceId, addAllowedPrincipals, removeAllowedPrincipals, callback) {
    const meth = 'adapter-modifyVpcEndpointServicePermissions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyVpcEndpointServicePermissions';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (serviceId === undefined || serviceId === null || serviceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['serviceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ServiceId: serviceId,
      Action: action,
      Version: version
    };
    if (addAllowedPrincipals && addAllowedPrincipals.length > 0) {
      processInArray(addAllowedPrincipals, queryParamsAvailable, 'AddAllowedPrincipals');
    }
    if (removeAllowedPrincipals && removeAllowedPrincipals.length > 0) {
      processInArray(removeAllowedPrincipals, queryParamsAvailable, 'RemoveAllowedPrincipals');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'modifyVpcEndpointServicePermissions', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyVpcEndpointServicePermissions'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the VPC peering connection options on one side of a VPC peering connection. You can do the following:       Enable/disable communication over the peering connection between an EC2-Classic instance that's linked to your VPC (using ClassicLink) and instances in the peer VPC.       Enable/disable communication over the peering connection between instances in your VPC and an EC2-Classic instance that's linked to the peer VPC.       Enable/disable the ability to resolve public DNS hostnames...(description truncated)
   *
   * @function modifyVpcPeeringConnectionOptions
   * @param {string} accepterPeeringConnectionOptionsAllowDnsResolutionFromRemoteVpc - The VPC peering connection options.
If true, enables a local VPC to resolve public DNS hostnames to private IP addresses when queried from instances in the peer VPC.
   * @param {string} accepterPeeringConnectionOptionsAllowEgressFromLocalClassicLinkToRemoteVpc - The VPC peering connection options.
If true, enables outbound communication from an EC2-Classic instance that's linked to a local VPC using ClassicLink to instances in a ...(description truncated)
   * @param {string} accepterPeeringConnectionOptionsAllowEgressFromLocalVpcToRemoteClassicLink - The VPC peering connection options.
If true, enables outbound communication from instances in a local VPC to an EC2-Classic instance that's linked to a peer VPC using Cla...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} requesterPeeringConnectionOptionsAllowDnsResolutionFromRemoteVpc - The VPC peering connection options.
If true, enables a local VPC to resolve public DNS hostnames to private IP addresses when queried from instances in the peer VPC.
   * @param {string} requesterPeeringConnectionOptionsAllowEgressFromLocalClassicLinkToRemoteVpc - The VPC peering connection options.
If true, enables outbound communication from an EC2-Classic instance that's linked to a local VPC using ClassicLink to instances in a ...(description truncated)
   * @param {string} requesterPeeringConnectionOptionsAllowEgressFromLocalVpcToRemoteClassicLink - The VPC peering connection options.
If true, enables outbound communication from instances in a local VPC to an EC2-Classic instance that's linked to a peer VPC using Cla...(description truncated)
   * @param {string} vpcPeeringConnectionId - The ID of the VPC peering connection.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyVpcPeeringConnectionOptions(accepterPeeringConnectionOptionsAllowDnsResolutionFromRemoteVpc, accepterPeeringConnectionOptionsAllowEgressFromLocalClassicLinkToRemoteVpc, accepterPeeringConnectionOptionsAllowEgressFromLocalVpcToRemoteClassicLink, dryRun, requesterPeeringConnectionOptionsAllowDnsResolutionFromRemoteVpc, requesterPeeringConnectionOptionsAllowEgressFromLocalClassicLinkToRemoteVpc, requesterPeeringConnectionOptionsAllowEgressFromLocalVpcToRemoteClassicLink, vpcPeeringConnectionId, callback) {
    const meth = 'adapter-modifyVpcPeeringConnectionOptions';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyVpcPeeringConnectionOptions';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcPeeringConnectionId === undefined || vpcPeeringConnectionId === null || vpcPeeringConnectionId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcPeeringConnectionId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AccepterPeeringConnectionOptionsAllowDnsResolutionFromRemoteVpc: accepterPeeringConnectionOptionsAllowDnsResolutionFromRemoteVpc,
      AccepterPeeringConnectionOptionsAllowEgressFromLocalClassicLinkToRemoteVpc: accepterPeeringConnectionOptionsAllowEgressFromLocalClassicLinkToRemoteVpc,
      AccepterPeeringConnectionOptionsAllowEgressFromLocalVpcToRemoteClassicLink: accepterPeeringConnectionOptionsAllowEgressFromLocalVpcToRemoteClassicLink,
      RequesterPeeringConnectionOptionsAllowDnsResolutionFromRemoteVpc: requesterPeeringConnectionOptionsAllowDnsResolutionFromRemoteVpc,
      RequesterPeeringConnectionOptionsAllowEgressFromLocalClassicLinkToRemoteVpc: requesterPeeringConnectionOptionsAllowEgressFromLocalClassicLinkToRemoteVpc,
      RequesterPeeringConnectionOptionsAllowEgressFromLocalVpcToRemoteClassicLink: requesterPeeringConnectionOptionsAllowEgressFromLocalVpcToRemoteClassicLink,
      VpcPeeringConnectionId: vpcPeeringConnectionId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'modifyVpcPeeringConnectionOptions', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyVpcPeeringConnectionOptions'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the instance tenancy attribute of the specified VPC. You can change the instance tenancy attribute of a VPC to  default  only. You cannot change the instance tenancy attribute to  dedicated .   After you modify the tenancy of the VPC, any new instances that you launch into the VPC have a tenancy of  default , unless you specify otherwise during launch. The tenancy of any existing instances in the VPC is not affected.   For more information, see  Dedicated Instances  in the  Amazon Elas...(description truncated)
   *
   * @function modifyVpcTenancy
   * @param {string} vpcId - The ID of the VPC.
   * @param {string} instanceTenancy - The instance tenancy attribute for the VPC.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyVpcTenancy(vpcId, instanceTenancy = 'default', dryRun, callback) {
    const meth = 'adapter-modifyVpcTenancy';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyVpcTenancy';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcId === undefined || vpcId === null || vpcId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (instanceTenancy === undefined || instanceTenancy === null || instanceTenancy === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceTenancy'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VpcId: vpcId,
      InstanceTenancy: instanceTenancy,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'modifyVpcTenancy', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyVpcTenancy'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Modifies the target gateway of a AWS Site-to-Site VPN connection. The following migration options are available:       An existing virtual private gateway to a new virtual private gateway       An existing virtual private gateway to a transit gateway       An existing transit gateway to a new transit gateway       An existing transit gateway to a virtual private gateway       Before you perform the migration to the new gateway, you must configure the new gateway. Use  CreateVpnGateway  to creat...(description truncated)
   *
   * @function modifyVpnConnection
   * @param {string} vpnConnectionId - The ID of the VPN connection.
   * @param {string} transitGatewayId - The ID of the transit gateway.
   * @param {string} vpnGatewayId - The ID of the virtual private gateway at the AWS side of the VPN connection.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  modifyVpnConnection(vpnConnectionId, transitGatewayId, vpnGatewayId, dryRun, callback) {
    const meth = 'adapter-modifyVpnConnection';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ModifyVpnConnection';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpnConnectionId === undefined || vpnConnectionId === null || vpnConnectionId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpnConnectionId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VpnConnectionId: vpnConnectionId,
      TransitGatewayId: transitGatewayId,
      VpnGatewayId: vpnGatewayId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'modifyVpnConnection', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['modifyVpnConnection'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Enables detailed monitoring for a running instance. Otherwise, basic monitoring is enabled. For more information, see  Monitoring Your Instances and Volumes  in the  Amazon Elastic Compute Cloud User Guide .   To disable detailed monitoring, see .
   *
   * @function monitorInstances
   * @param {array} instanceId - The IDs of the instances.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  monitorInstances(instanceId, dryRun, callback) {
    const meth = 'adapter-monitorInstances';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'MonitorInstances';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (instanceId && instanceId.length > 0) {
      processInArray(instanceId, queryParamsAvailable, 'InstanceId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'monitorInstances', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['monitorInstances'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Moves an Elastic IP address from the EC2-Classic platform to the EC2-VPC platform. The Elastic IP address must be allocated to your account for more than 24 hours, and it must not be associated with an instance. After the Elastic IP address is moved, it is no longer available for use in the EC2-Classic platform, unless you move it back using the  RestoreAddressToClassic  request. You cannot move an Elastic IP address that was originally allocated for use in the EC2-VPC platform to the EC2-Classi...(description truncated)
   *
   * @function moveAddressToVpc
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} publicIp - The Elastic IP address.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  moveAddressToVpc(dryRun, publicIp, callback) {
    const meth = 'adapter-moveAddressToVpc';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'MoveAddressToVpc';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (publicIp === undefined || publicIp === null || publicIp === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['publicIp'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      PublicIp: publicIp,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'moveAddressToVpc', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['moveAddressToVpc'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Provisions an address range for use with your AWS resources through bring your own IP addresses (BYOIP) and creates a corresponding address pool. After the address range is provisioned, it is ready to be advertised using  AdvertiseByoipCidr .   AWS verifies that you own the address range and are authorized to advertise it. You must ensure that the address range is registered to you and that you created an RPKI ROA to authorize Amazon ASNs 16509 and 14618 to advertise the address range. For more...(description truncated)
   *
   * @function provisionByoipCidr
   * @param {string} cidr - The public IPv4 address range, in CIDR notation. The most specific prefix that you can specify is /24. The address range cannot overlap with another address range that yo...(description truncated)
   * @param {string} cidrAuthorizationContextMessage - Provides authorization for Amazon to bring a specific IP address range to a specific AWS account using bring your own IP addresses (BYOIP). For more information, see  Pre...(description truncated)
   * @param {string} cidrAuthorizationContextSignature - Provides authorization for Amazon to bring a specific IP address range to a specific AWS account using bring your own IP addresses (BYOIP). For more information, see  Pre...(description truncated)
   * @param {string} description - A description for the address range and the address pool.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  provisionByoipCidr(cidr, cidrAuthorizationContextMessage, cidrAuthorizationContextSignature, description, dryRun, callback) {
    const meth = 'adapter-provisionByoipCidr';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ProvisionByoipCidr';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (cidr === undefined || cidr === null || cidr === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['cidr'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Cidr: cidr,
      CidrAuthorizationContextMessage: cidrAuthorizationContextMessage,
      CidrAuthorizationContextSignature: cidrAuthorizationContextSignature,
      Description: description,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'provisionByoipCidr', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['provisionByoipCidr'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Purchase a reservation with configurations that match those of your Dedicated Host. You must have active Dedicated Hosts in your account before you purchase a reservation. This action results in the specified reservation being purchased and charged to your account.
   *
   * @function purchaseHostReservation
   * @param {string} clientToken - Unique, case-sensitive identifier you provide to ensure idempotency of the request. For more information, see  How to Ensure Idempotency  in the  Amazon Elastic Compute C...(description truncated)
   * @param {string} currencyCode - The currency in which the  totalUpfrontPrice ,  LimitPrice , and  totalHourlyPrice  amounts are specified. At this time, the only supported currency is  USD .
   * @param {array} hostIdSet - The IDs of the Dedicated Hosts with which the reservation will be associated.
   * @param {string} limitPrice - The specified limit is checked against the total upfront cost of the reservation (calculated as the offering's upfront cost multiplied by the host count). If the total up...(description truncated)
   * @param {string} offeringId - The ID of the offering.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  purchaseHostReservation(clientToken, currencyCode = 'USD', hostIdSet, limitPrice, offeringId, callback) {
    const meth = 'adapter-purchaseHostReservation';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'PurchaseHostReservation';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (hostIdSet === undefined || hostIdSet === null || hostIdSet === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['hostIdSet'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (offeringId === undefined || offeringId === null || offeringId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['offeringId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      ClientToken: clientToken,
      CurrencyCode: currencyCode,
      LimitPrice: limitPrice,
      OfferingId: offeringId,
      Action: action,
      Version: version
    };
    if (hostIdSet && hostIdSet.length > 0) {
      processInArray(hostIdSet, queryParamsAvailable, 'HostIdSet');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Host', 'purchaseHostReservation', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['purchaseHostReservation'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Purchases a Reserved Instance for use with your account. With Reserved Instances, you pay a lower hourly rate compared to On-Demand instance pricing.   Use  DescribeReservedInstancesOfferings  to get a list of Reserved Instance offerings that match your specifications. After you've purchased a Reserved Instance, you can check for your new Reserved Instance with  DescribeReservedInstances .   For more information, see  Reserved Instances  and  Reserved Instance Marketplace  in the  Amazon Elasti...(description truncated)
   *
   * @function purchaseReservedInstancesOffering
   * @param {number} instanceCount - The number of Reserved Instances to purchase.
   * @param {string} reservedInstancesOfferingId - The ID of the Reserved Instance offering to purchase.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} limitPriceAmount - Describes the limit price of a Reserved Instance offering.
Used for Reserved Instance Marketplace offerings. Specifies the limit price on the total order (instanceCount *...(description truncated)
   * @param {string} limitPriceCurrencyCode - Describes the limit price of a Reserved Instance offering.
The currency in which the  limitPrice  amount is specified. At this time, the only supported currency is  USD .
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  purchaseReservedInstancesOffering(instanceCount, reservedInstancesOfferingId, dryRun, limitPriceAmount, limitPriceCurrencyCode, callback) {
    const meth = 'adapter-purchaseReservedInstancesOffering';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'PurchaseReservedInstancesOffering';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceCount === undefined || instanceCount === null || instanceCount === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceCount'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (reservedInstancesOfferingId === undefined || reservedInstancesOfferingId === null || reservedInstancesOfferingId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['reservedInstancesOfferingId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      InstanceCount: instanceCount,
      ReservedInstancesOfferingId: reservedInstancesOfferingId,
      LimitPriceAmount: limitPriceAmount,
      LimitPriceCurrencyCode: limitPriceCurrencyCode,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'purchaseReservedInstancesOffering', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['purchaseReservedInstancesOffering'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Purchases the Scheduled Instances with the specified schedule.   Scheduled Instances enable you to purchase Amazon EC2 compute capacity by the hour for a one-year term. Before you can purchase a Scheduled Instance, you must call  DescribeScheduledInstanceAvailability  to check for available schedules and obtain a purchase token. After you purchase a Scheduled Instance, you must call  RunScheduledInstances  during each scheduled time period.   After you purchase a Scheduled Instance, you can't c...(description truncated)
   *
   * @function purchaseScheduledInstances
   * @param {string} clientToken - Unique, case-sensitive identifier that ensures the idempotency of the request. For more information, see  Ensuring Idempotency .
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {array} purchaseRequest - The purchase requests.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  purchaseScheduledInstances(clientToken, dryRun, purchaseRequest, callback) {
    const meth = 'adapter-purchaseScheduledInstances';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'PurchaseScheduledInstances';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (purchaseRequest === undefined || purchaseRequest === null || purchaseRequest === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['purchaseRequest'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientToken: clientToken,
      Action: action,
      Version: version
    };
    if (purchaseRequest && purchaseRequest.length > 0) {
      processInArray(purchaseRequest, queryParamsAvailable, 'PurchaseRequest');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'purchaseScheduledInstances', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['purchaseScheduledInstances'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Requests a reboot of the specified instances. This operation is asynchronous; it only queues a request to reboot the specified instances. The operation succeeds if the instances are valid and belong to you. Requests to reboot terminated instances are ignored.   If an instance does not cleanly shut down within four minutes, Amazon EC2 performs a hard reboot.   For more information about troubleshooting, see  Getting Console Output and Rebooting Instances  in the  Amazon Elastic Compute Cloud Use...(description truncated)
   *
   * @function rebootInstances
   * @param {array} instanceId - The instance IDs.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  rebootInstances(instanceId, dryRun, callback) {
    const meth = 'adapter-rebootInstances';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'RebootInstances';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (instanceId && instanceId.length > 0) {
      processInArray(instanceId, queryParamsAvailable, 'InstanceId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'rebootInstances', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['rebootInstances'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Registers an AMI. When you're creating an AMI, this is the final step you must complete before you can launch an instance from the AMI. For more information about creating AMIs, see  Creating Your Own AMIs  in the  Amazon Elastic Compute Cloud User Guide .     For Amazon EBS-backed instances,  CreateImage  creates and registers the AMI in a single request, so you don't have to register the AMI yourself.     You can also use  RegisterImage  to create an Amazon EBS-backed Linux AMI from a snapsho...(description truncated)
   *
   * @function registerImage
   * @param {string} imageLocation - The full path to your AMI manifest in Amazon S3 storage.
   * @param {string} architecture - The architecture of the AMI.   Default: For Amazon EBS-backed AMIs,  i386 . For instance store-backed AMIs, the architecture specified in the manifest file.
   * @param {array} blockDeviceMapping - The block device mapping entries.
   * @param {string} description - A description for your AMI.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {boolean} enaSupport - Set to  true  to enable enhanced networking with ENA for the AMI and any instances that you launch from the AMI.   This option is supported only for HVM AMIs. Specifying...(description truncated)
   * @param {string} kernelId - The ID of the kernel.
   * @param {string} name - A name for your AMI.   Constraints: 3-128 alphanumeric characters, parentheses (()), square brackets ([]), spaces ( ), periods (.), slashes (/), dashes (-), single quote...(description truncated)
   * @param {array} billingProduct - The billing product codes. Your account must be authorized to specify billing product codes. Otherwise, you can use the AWS Marketplace to bill for the use of an AMI.
   * @param {string} ramdiskId - The ID of the RAM disk.
   * @param {string} rootDeviceName - The device name of the root device volume (for example,  /dev/sda1 ).
   * @param {string} sriovNetSupport - Set to  simple  to enable enhanced networking with the Intel 82599 Virtual Function interface for the AMI and any instances that you launch from the AMI.   There is no w...(description truncated)
   * @param {string} virtualizationType - The type of virtualization ( hvm  |  paravirtual ).   Default:  paravirtual
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  registerImage(imageLocation, architecture = 'i386', blockDeviceMapping, description, dryRun, enaSupport, kernelId, name, billingProduct, ramdiskId, rootDeviceName, sriovNetSupport, virtualizationType, callback) {
    const meth = 'adapter-registerImage';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'RegisterImage';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (name === undefined || name === null || name === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['name'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ImageLocation: imageLocation,
      Architecture: architecture,
      Description: description,
      EnaSupport: enaSupport,
      KernelId: kernelId,
      Name: name,
      RamdiskId: ramdiskId,
      RootDeviceName: rootDeviceName,
      SriovNetSupport: sriovNetSupport,
      VirtualizationType: virtualizationType,
      Action: action,
      Version: version
    };
    if (blockDeviceMapping && blockDeviceMapping.length > 0) {
      processInArray(blockDeviceMapping, queryParamsAvailable, 'BlockDeviceMapping');
    }
    if (billingProduct && billingProduct.length > 0) {
      processInArray(billingProduct, queryParamsAvailable, 'BillingProduct');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Image', 'registerImage', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['registerImage'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Rejects a request to attach a VPC to a transit gateway.   The VPC attachment must be in the  pendingAcceptance  state. Use  DescribeTransitGatewayVpcAttachments  to view your pending VPC attachment requests. Use  AcceptTransitGatewayVpcAttachment  to accept a VPC attachment request.
   *
   * @function rejectTransitGatewayVpcAttachment
   * @param {string} transitGatewayAttachmentId - The ID of the attachment.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  rejectTransitGatewayVpcAttachment(transitGatewayAttachmentId, dryRun, callback) {
    const meth = 'adapter-rejectTransitGatewayVpcAttachment';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'RejectTransitGatewayVpcAttachment';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayAttachmentId === undefined || transitGatewayAttachmentId === null || transitGatewayAttachmentId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayAttachmentId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      TransitGatewayAttachmentId: transitGatewayAttachmentId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'rejectTransitGatewayVpcAttachment', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['rejectTransitGatewayVpcAttachment'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Rejects one or more VPC endpoint connection requests to your VPC endpoint service.
   *
   * @function rejectVpcEndpointConnections
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} serviceId - The ID of the service.
   * @param {array} vpcEndpointId - The IDs of one or more VPC endpoints.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  rejectVpcEndpointConnections(dryRun, serviceId, vpcEndpointId, callback) {
    const meth = 'adapter-rejectVpcEndpointConnections';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'RejectVpcEndpointConnections';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (serviceId === undefined || serviceId === null || serviceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['serviceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (vpcEndpointId === undefined || vpcEndpointId === null || vpcEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ServiceId: serviceId,
      Action: action,
      Version: version
    };
    if (vpcEndpointId && vpcEndpointId.length > 0) {
      processInArray(vpcEndpointId, queryParamsAvailable, 'VpcEndpointId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'rejectVpcEndpointConnections', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['rejectVpcEndpointConnections'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Rejects a VPC peering connection request. The VPC peering connection must be in the  pending-acceptance  state. Use the  DescribeVpcPeeringConnections  request to view your outstanding VPC peering connection requests. To delete an active VPC peering connection, or to delete a VPC peering connection request that you initiated, use  DeleteVpcPeeringConnection .
   *
   * @function rejectVpcPeeringConnection
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} vpcPeeringConnectionId - The ID of the VPC peering connection.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  rejectVpcPeeringConnection(dryRun, vpcPeeringConnectionId, callback) {
    const meth = 'adapter-rejectVpcPeeringConnection';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'RejectVpcPeeringConnection';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (vpcPeeringConnectionId === undefined || vpcPeeringConnectionId === null || vpcPeeringConnectionId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['vpcPeeringConnectionId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      VpcPeeringConnectionId: vpcPeeringConnectionId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Vpc', 'rejectVpcPeeringConnection', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['rejectVpcPeeringConnection'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Releases the specified Elastic IP address.   [EC2-Classic, default VPC] Releasing an Elastic IP address automatically disassociates it from any instance that it's associated with. To disassociate an Elastic IP address without releasing it, use  DisassociateAddress .   [Nondefault VPC] You must use  DisassociateAddress  to disassociate the Elastic IP address before you can release it. Otherwise, Amazon EC2 returns an error ( InvalidIPAddress.InUse ).   After releasing an Elastic IP address, it i...(description truncated)
   *
   * @function releaseAddress
   * @param {string} allocationId - [EC2-VPC] The allocation ID. Required for EC2-VPC.
   * @param {string} publicIp - [EC2-Classic] The Elastic IP address. Required for EC2-Classic.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  releaseAddress(allocationId, publicIp, dryRun, callback) {
    const meth = 'adapter-releaseAddress';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ReleaseAddress';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AllocationId: allocationId,
      PublicIp: publicIp,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'releaseAddress', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['releaseAddress'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary When you no longer want to use an On-Demand Dedicated Host it can be released. On-Demand billing is stopped and the host goes into  released  state. The host ID of Dedicated Hosts that have been released can no longer be specified in another request, for example, to modify the host. You must stop or terminate all instances on a host before it can be released.   When Dedicated Hosts are released, it may take some time for them to stop counting toward your limit and you may receive capacity error...(description truncated)
   *
   * @function releaseHosts
   * @param {array} hostId - The IDs of the Dedicated Hosts to release.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  releaseHosts(hostId, callback) {
    const meth = 'adapter-releaseHosts';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ReleaseHosts';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (hostId === undefined || hostId === null || hostId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['hostId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      HostId: hostId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Host', 'releaseHosts', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['releaseHosts'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Replaces an IAM instance profile for the specified running instance. You can use this action to change the IAM instance profile that's associated with an instance without having to disassociate the existing IAM instance profile first.   Use  DescribeIamInstanceProfileAssociations  to get the association ID.
   *
   * @function replaceIamInstanceProfileAssociation
   * @param {string} iamInstanceProfileArn - Describes an IAM instance profile.
The Amazon Resource Name (ARN) of the instance profile.
   * @param {string} iamInstanceProfileName - Describes an IAM instance profile.
The name of the instance profile.
   * @param {string} associationId - The ID of the existing IAM instance profile association.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  replaceIamInstanceProfileAssociation(iamInstanceProfileArn, iamInstanceProfileName, associationId, callback) {
    const meth = 'adapter-replaceIamInstanceProfileAssociation';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ReplaceIamInstanceProfileAssociation';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (associationId === undefined || associationId === null || associationId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['associationId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      IamInstanceProfileArn: iamInstanceProfileArn,
      IamInstanceProfileName: iamInstanceProfileName,
      AssociationId: associationId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'replaceIamInstanceProfileAssociation', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['replaceIamInstanceProfileAssociation'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Changes which network ACL a subnet is associated with. By default when you create a subnet, it's automatically associated with the default network ACL. For more information, see  Network ACLs  in the  Amazon Virtual Private Cloud User Guide .   This is an idempotent operation.
   *
   * @function replaceNetworkAclAssociation
   * @param {string} associationId - The ID of the current association between the original network ACL and the subnet.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} networkAclId - The ID of the new network ACL to associate with the subnet.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  replaceNetworkAclAssociation(associationId, dryRun, networkAclId, callback) {
    const meth = 'adapter-replaceNetworkAclAssociation';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ReplaceNetworkAclAssociation';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (associationId === undefined || associationId === null || associationId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['associationId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (networkAclId === undefined || networkAclId === null || networkAclId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['networkAclId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AssociationId: associationId,
      NetworkAclId: networkAclId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkAcl', 'replaceNetworkAclAssociation', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['replaceNetworkAclAssociation'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Replaces an entry (rule) in a network ACL. For more information, see  Network ACLs  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function replaceNetworkAclEntry
   * @param {string} cidrBlock - The IPv4 network range to allow or deny, in CIDR notation (for example  172.16.0.0/24 ).
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {boolean} egress - Indicates whether to replace the egress rule.   Default: If no value is specified, we replace the ingress rule.
   * @param {string} icmpCode - Describes the ICMP type and code.
The ICMP code. A value of -1 means all codes for the specified ICMP type.
   * @param {string} icmpType - Describes the ICMP type and code.
The ICMP type. A value of -1 means all types.
   * @param {string} ipv6CidrBlock - The IPv6 network range to allow or deny, in CIDR notation (for example  2001:bd8:1234:1a00::/64 ).
   * @param {string} networkAclId - The ID of the ACL.
   * @param {string} portRangeFrom - Describes a range of ports.
The first port in the range.
   * @param {string} portRangeTo - Describes a range of ports.
The last port in the range.
   * @param {string} protocol - The protocol number. A value of "-1" means all protocols. If you specify "-1" or a protocol number other than "6" (TCP), "17" (UDP), or "1" (ICMP), traffic on all ports i...(description truncated)
   * @param {string} ruleAction - Indicates whether to allow or deny the traffic that matches the rule.
   * @param {number} ruleNumber - The rule number of the entry to replace.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  replaceNetworkAclEntry(cidrBlock, dryRun, egress, icmpCode, icmpType, ipv6CidrBlock, networkAclId, portRangeFrom, portRangeTo, protocol, ruleAction = 'allow', ruleNumber, callback) {
    const meth = 'adapter-replaceNetworkAclEntry';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ReplaceNetworkAclEntry';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (egress === undefined || egress === null || egress === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['egress'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (networkAclId === undefined || networkAclId === null || networkAclId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['networkAclId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (protocol === undefined || protocol === null || protocol === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['protocol'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (ruleAction === undefined || ruleAction === null || ruleAction === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['ruleAction'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (ruleNumber === undefined || ruleNumber === null || ruleNumber === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['ruleNumber'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      CidrBlock: cidrBlock,
      Egress: egress,
      IcmpCode: icmpCode,
      IcmpType: icmpType,
      Ipv6CidrBlock: ipv6CidrBlock,
      NetworkAclId: networkAclId,
      PortRangeFrom: portRangeFrom,
      PortRangeTo: portRangeTo,
      Protocol: protocol,
      RuleAction: ruleAction,
      RuleNumber: ruleNumber,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkAcl', 'replaceNetworkAclEntry', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['replaceNetworkAclEntry'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Replaces an existing route within a route table in a VPC. You must provide only one of the following: internet gateway or virtual private gateway, NAT instance, NAT gateway, VPC peering connection, network interface, or egress-only internet gateway.   For more information, see  Route Tables  in the  Amazon Virtual Private Cloud User Guide .
   *
   * @function replaceRoute
   * @param {string} destinationCidrBlock - The IPv4 CIDR address block used for the destination match. The value that you provide must match the CIDR of an existing route in the table.
   * @param {string} destinationIpv6CidrBlock - The IPv6 CIDR address block used for the destination match. The value that you provide must match the CIDR of an existing route in the table.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} egressOnlyInternetGatewayId - [IPv6 traffic only] The ID of an egress-only internet gateway.
   * @param {string} gatewayId - The ID of an internet gateway or virtual private gateway.
   * @param {string} instanceId - The ID of a NAT instance in your VPC.
   * @param {string} natGatewayId - [IPv4 traffic only] The ID of a NAT gateway.
   * @param {string} transitGatewayId - The ID of a transit gateway.
   * @param {string} networkInterfaceId - The ID of a network interface.
   * @param {string} routeTableId - The ID of the route table.
   * @param {string} vpcPeeringConnectionId - The ID of a VPC peering connection.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  replaceRoute(destinationCidrBlock, destinationIpv6CidrBlock, dryRun, egressOnlyInternetGatewayId, gatewayId, instanceId, natGatewayId, transitGatewayId, networkInterfaceId, routeTableId, vpcPeeringConnectionId, callback) {
    const meth = 'adapter-replaceRoute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ReplaceRoute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (routeTableId === undefined || routeTableId === null || routeTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['routeTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      DestinationCidrBlock: destinationCidrBlock,
      DestinationIpv6CidrBlock: destinationIpv6CidrBlock,
      EgressOnlyInternetGatewayId: egressOnlyInternetGatewayId,
      GatewayId: gatewayId,
      InstanceId: instanceId,
      NatGatewayId: natGatewayId,
      TransitGatewayId: transitGatewayId,
      NetworkInterfaceId: networkInterfaceId,
      RouteTableId: routeTableId,
      VpcPeeringConnectionId: vpcPeeringConnectionId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Route', 'replaceRoute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['replaceRoute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Changes the route table associated with a given subnet in a VPC. After the operation completes, the subnet uses the routes in the new route table it's associated with. For more information about route tables, see  Route Tables  in the  Amazon Virtual Private Cloud User Guide .   You can also use ReplaceRouteTableAssociation to change which table is the main route table in the VPC. You just specify the main route table's association ID and the route table to be the new main route table.
   *
   * @function replaceRouteTableAssociation
   * @param {string} associationId - The association ID.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} routeTableId - The ID of the new route table to associate with the subnet.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  replaceRouteTableAssociation(associationId, dryRun, routeTableId, callback) {
    const meth = 'adapter-replaceRouteTableAssociation';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ReplaceRouteTableAssociation';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (associationId === undefined || associationId === null || associationId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['associationId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (routeTableId === undefined || routeTableId === null || routeTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['routeTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AssociationId: associationId,
      RouteTableId: routeTableId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Route', 'replaceRouteTableAssociation', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['replaceRouteTableAssociation'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Replaces the specified route in the specified transit gateway route table.
   *
   * @function replaceTransitGatewayRoute
   * @param {string} destinationCidrBlock - The CIDR range used for the destination match. Routing decisions are based on the most specific match.
   * @param {string} transitGatewayRouteTableId - The ID of the route table.
   * @param {string} transitGatewayAttachmentId - The ID of the attachment.
   * @param {boolean} blackhole - Indicates whether traffic matching this route is to be dropped.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  replaceTransitGatewayRoute(destinationCidrBlock, transitGatewayRouteTableId, transitGatewayAttachmentId, blackhole, dryRun, callback) {
    const meth = 'adapter-replaceTransitGatewayRoute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ReplaceTransitGatewayRoute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (destinationCidrBlock === undefined || destinationCidrBlock === null || destinationCidrBlock === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['destinationCidrBlock'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (transitGatewayRouteTableId === undefined || transitGatewayRouteTableId === null || transitGatewayRouteTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayRouteTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      DestinationCidrBlock: destinationCidrBlock,
      TransitGatewayRouteTableId: transitGatewayRouteTableId,
      TransitGatewayAttachmentId: transitGatewayAttachmentId,
      Blackhole: blackhole,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'replaceTransitGatewayRoute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['replaceTransitGatewayRoute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Submits feedback about the status of an instance. The instance must be in the  running  state. If your experience with the instance differs from the instance status returned by  DescribeInstanceStatus , use  ReportInstanceStatus  to report your experience with the instance. Amazon EC2 collects this information to improve the accuracy of status checks.   Use of this action does not change the value returned by  DescribeInstanceStatus .
   *
   * @function reportInstanceStatus
   * @param {string} description - Descriptive text about the health state of your instance.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} endTime - The time at which the reported instance health state ended.
   * @param {array} instanceId - The instances.
   * @param {array} reasonCode - The reason codes that describe the health state of your instance.         instance-stuck-in-state : My instance is stuck in a state.         unresponsive : My instance i...(description truncated)
   * @param {string} startTime - The time at which the reported instance health state began.
   * @param {string} status - The status of all instances listed.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  reportInstanceStatus(description, dryRun, endTime, instanceId, reasonCode, startTime, status = 'ok', callback) {
    const meth = 'adapter-reportInstanceStatus';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ReportInstanceStatus';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (reasonCode === undefined || reasonCode === null || reasonCode === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['reasonCode'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (status === undefined || status === null || status === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['status'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Description: description,
      EndTime: endTime,
      StartTime: startTime,
      Status: status,
      Action: action,
      Version: version
    };
    if (instanceId && instanceId.length > 0) {
      processInArray(instanceId, queryParamsAvailable, 'InstanceId');
    }
    if (reasonCode && reasonCode.length > 0) {
      processInArray(reasonCode, queryParamsAvailable, 'ReasonCode');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'reportInstanceStatus', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['reportInstanceStatus'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a Spot Fleet request.   The Spot Fleet request specifies the total target capacity and the On-Demand target capacity. Amazon EC2 calculates the difference between the total capacity and On-Demand capacity, and launches the difference as Spot capacity.   You can submit a single request that includes multiple launch specifications that vary by instance type, AMI, Availability Zone, or subnet.   By default, the Spot Fleet requests Spot Instances in the Spot pool where the price per unit is...(description truncated)
   *
   * @function requestSpotFleet
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} spotFleetRequestConfigAllocationStrategy - Describes the configuration of a Spot Fleet request.
Indicates how to allocate the target capacity across the Spot pools specified by the Spot Fleet request. The default ...(description truncated)
   * @param {string} spotFleetRequestConfigOnDemandAllocationStrategy - Describes the configuration of a Spot Fleet request.
The order of the launch template overrides to use in fulfilling On-Demand capacity. If you specify  lowestPrice , Spo...(description truncated)
   * @param {string} spotFleetRequestConfigClientToken - Describes the configuration of a Spot Fleet request.
A unique, case-sensitive identifier that you provide to ensure the idempotency of your listings. This helps to avoid ...(description truncated)
   * @param {string} spotFleetRequestConfigExcessCapacityTerminationPolicy - Describes the configuration of a Spot Fleet request.
Indicates whether running Spot Instances should be terminated if you decrease the target capacity of the Spot Fleet r...(description truncated)
   * @param {string} spotFleetRequestConfigFulfilledCapacity - Describes the configuration of a Spot Fleet request.
The number of units fulfilled by this request compared to the set target capacity. You cannot set this value.
   * @param {string} spotFleetRequestConfigOnDemandFulfilledCapacity - Describes the configuration of a Spot Fleet request.
The number of On-Demand units fulfilled by this request compared to the set target On-Demand capacity.
   * @param {string} spotFleetRequestConfigIamFleetRole - Describes the configuration of a Spot Fleet request.
The Amazon Resource Name (ARN) of an AWS Identity and Access Management (IAM) role that grants the Spot Fleet the per...(description truncated)
   * @param {array} spotFleetRequestConfigLaunchSpecifications - Describes the configuration of a Spot Fleet request.
The launch specifications for the Spot Fleet request. If you specify  LaunchSpecifications , you can't specify  Launc...(description truncated)
   * @param {array} spotFleetRequestConfigLaunchTemplateConfigs - Describes the configuration of a Spot Fleet request.
The launch template and overrides. If you specify  LaunchTemplateConfigs , you can't specify  LaunchSpecifications . ...(description truncated)
   * @param {string} spotFleetRequestConfigSpotPrice - Describes the configuration of a Spot Fleet request.
The maximum price per unit hour that you are willing to pay for a Spot Instance. The default is the On-Demand price.
   * @param {string} spotFleetRequestConfigTargetCapacity - Describes the configuration of a Spot Fleet request.
The number of units to request for the Spot Fleet. You can choose to set the target capacity in terms of instances or...(description truncated)
   * @param {string} spotFleetRequestConfigOnDemandTargetCapacity - Describes the configuration of a Spot Fleet request.
The number of On-Demand units to request. You can choose to set the target capacity in terms of instances or a perfor...(description truncated)
   * @param {string} spotFleetRequestConfigTerminateInstancesWithExpiration - Describes the configuration of a Spot Fleet request.
Indicates whether running Spot Instances are terminated when the Spot Fleet request expires.
   * @param {string} spotFleetRequestConfigType - Describes the configuration of a Spot Fleet request.
The type of request. Indicates whether the Spot Fleet only requests the target capacity or also attempts to maintain ...(description truncated)
   * @param {string} spotFleetRequestConfigValidFrom - Describes the configuration of a Spot Fleet request.
The start date and time of the request, in UTC format ( YYYY - MM - DD T HH : MM : SS Z). By default, Amazon EC2 star...(description truncated)
   * @param {string} spotFleetRequestConfigValidUntil - Describes the configuration of a Spot Fleet request.
The end date and time of the request, in UTC format ( YYYY - MM - DD T HH : MM : SS Z). After the end date and time, ...(description truncated)
   * @param {string} spotFleetRequestConfigReplaceUnhealthyInstances - Describes the configuration of a Spot Fleet request.
Indicates whether Spot Fleet should replace unhealthy instances.
   * @param {string} spotFleetRequestConfigInstanceInterruptionBehavior - Describes the configuration of a Spot Fleet request.
The behavior when a Spot Instance is interrupted. The default is  terminate .
   * @param {string} spotFleetRequestConfigLoadBalancersConfig - Describes the configuration of a Spot Fleet request.
 One or more Classic Load Balancers and target groups to attach to the Spot Fleet request. Spot Fleet registers the r...(description truncated)
   * @param {string} spotFleetRequestConfigInstancePoolsToUseCount - Describes the configuration of a Spot Fleet request.
The number of Spot pools across which to allocate your target Spot capacity. Valid only when Spot  AllocationStrategy...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  requestSpotFleet(dryRun, spotFleetRequestConfigAllocationStrategy, spotFleetRequestConfigOnDemandAllocationStrategy, spotFleetRequestConfigClientToken, spotFleetRequestConfigExcessCapacityTerminationPolicy, spotFleetRequestConfigFulfilledCapacity, spotFleetRequestConfigOnDemandFulfilledCapacity, spotFleetRequestConfigIamFleetRole, spotFleetRequestConfigLaunchSpecifications, spotFleetRequestConfigLaunchTemplateConfigs, spotFleetRequestConfigSpotPrice, spotFleetRequestConfigTargetCapacity, spotFleetRequestConfigOnDemandTargetCapacity, spotFleetRequestConfigTerminateInstancesWithExpiration, spotFleetRequestConfigType, spotFleetRequestConfigValidFrom, spotFleetRequestConfigValidUntil, spotFleetRequestConfigReplaceUnhealthyInstances, spotFleetRequestConfigInstanceInterruptionBehavior, spotFleetRequestConfigLoadBalancersConfig, spotFleetRequestConfigInstancePoolsToUseCount, callback) {
    const meth = 'adapter-requestSpotFleet';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'RequestSpotFleet';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      SpotFleetRequestConfigAllocationStrategy: spotFleetRequestConfigAllocationStrategy,
      SpotFleetRequestConfigOnDemandAllocationStrategy: spotFleetRequestConfigOnDemandAllocationStrategy,
      SpotFleetRequestConfigClientToken: spotFleetRequestConfigClientToken,
      SpotFleetRequestConfigExcessCapacityTerminationPolicy: spotFleetRequestConfigExcessCapacityTerminationPolicy,
      SpotFleetRequestConfigFulfilledCapacity: spotFleetRequestConfigFulfilledCapacity,
      SpotFleetRequestConfigOnDemandFulfilledCapacity: spotFleetRequestConfigOnDemandFulfilledCapacity,
      SpotFleetRequestConfigIamFleetRole: spotFleetRequestConfigIamFleetRole,
      SpotFleetRequestConfigSpotPrice: spotFleetRequestConfigSpotPrice,
      SpotFleetRequestConfigTargetCapacity: spotFleetRequestConfigTargetCapacity,
      SpotFleetRequestConfigOnDemandTargetCapacity: spotFleetRequestConfigOnDemandTargetCapacity,
      SpotFleetRequestConfigTerminateInstancesWithExpiration: spotFleetRequestConfigTerminateInstancesWithExpiration,
      SpotFleetRequestConfigType: spotFleetRequestConfigType,
      SpotFleetRequestConfigValidFrom: spotFleetRequestConfigValidFrom,
      SpotFleetRequestConfigValidUntil: spotFleetRequestConfigValidUntil,
      SpotFleetRequestConfigReplaceUnhealthyInstances: spotFleetRequestConfigReplaceUnhealthyInstances,
      SpotFleetRequestConfigInstanceInterruptionBehavior: spotFleetRequestConfigInstanceInterruptionBehavior,
      SpotFleetRequestConfigLoadBalancersConfig: spotFleetRequestConfigLoadBalancersConfig,
      SpotFleetRequestConfigInstancePoolsToUseCount: spotFleetRequestConfigInstancePoolsToUseCount,
      Action: action,
      Version: version
    };
    if (spotFleetRequestConfigLaunchSpecifications && spotFleetRequestConfigLaunchSpecifications.length > 0) {
      processInArray(spotFleetRequestConfigLaunchSpecifications, queryParamsAvailable, 'SpotFleetRequestConfigLaunchSpecifications');
    }
    if (spotFleetRequestConfigLaunchTemplateConfigs && spotFleetRequestConfigLaunchTemplateConfigs.length > 0) {
      processInArray(spotFleetRequestConfigLaunchTemplateConfigs, queryParamsAvailable, 'SpotFleetRequestConfigLaunchTemplateConfigs');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Fleet', 'requestSpotFleet', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['requestSpotFleet'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Creates a Spot Instance request.   For more information, see  Spot Instance Requests  in the  Amazon EC2 User Guide for Linux Instances .
   *
   * @function requestSpotInstances
   * @param {string} availabilityZoneGroup - The user-specified name for a logical grouping of requests.   When you specify an Availability Zone group in a Spot Instance request, all Spot Instances in the request a...(description truncated)
   * @param {number} blockDurationMinutes - The required duration for the Spot Instances (also known as Spot blocks), in minutes. This value must be a multiple of 60 (60, 120, 180, 240, 300, or 360).   The duratio...(description truncated)
   * @param {string} clientToken - Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see  How to Ensure Idempotency  in the  Amazon EC2 User...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {number} instanceCount - The maximum number of Spot Instances to launch.   Default: 1
   * @param {string} launchGroup - The instance launch group. Launch groups are Spot Instances that launch together and terminate together.   Default: Instances are launched and terminated individually
   * @param {array} launchSpecificationSecurityGroupIds - Describes the launch specification for an instance.
One or more security group IDs.
   * @param {array} launchSpecificationSecurityGroups - Describes the launch specification for an instance.
One or more security groups. When requesting instances in a VPC, you must specify the IDs of the security groups. When...(description truncated)
   * @param {string} launchSpecificationAddressingType - Describes the launch specification for an instance.
Deprecated.
   * @param {array} launchSpecificationBlockDeviceMappings - Describes the launch specification for an instance.
One or more block device mapping entries. You can't specify both a snapshot ID and an encryption value. This is becaus...(description truncated)
   * @param {string} launchSpecificationEbsOptimized - Describes the launch specification for an instance.
 Indicates whether the instance is optimized for EBS I/O. This optimization provides dedicated throughput to Amazon EB...(description truncated)
   * @param {string} launchSpecificationIamInstanceProfile - Describes the launch specification for an instance.
The IAM instance profile.
   * @param {string} launchSpecificationImageId - Describes the launch specification for an instance.
The ID of the AMI.
   * @param {string} launchSpecificationInstanceType - Describes the launch specification for an instance.
The instance type.
   * @param {string} launchSpecificationKernelId - Describes the launch specification for an instance.
The ID of the kernel.
   * @param {string} launchSpecificationKeyName - Describes the launch specification for an instance.
The name of the key pair.
   * @param {string} launchSpecificationMonitoring - Describes the launch specification for an instance.
 Indicates whether basic or detailed monitoring is enabled for the instance.   Default: Disabled
   * @param {array} launchSpecificationNetworkInterfaces - Describes the launch specification for an instance.
One or more network interfaces. If you specify a network interface, you must specify subnet IDs and security group IDs...(description truncated)
   * @param {string} launchSpecificationPlacement - Describes the launch specification for an instance.
The placement information for the instance.
   * @param {string} launchSpecificationRamdiskId - Describes the launch specification for an instance.
The ID of the RAM disk.
   * @param {string} launchSpecificationSubnetId - Describes the launch specification for an instance.
The ID of the subnet in which to launch the instance.
   * @param {string} launchSpecificationUserData - Describes the launch specification for an instance.
The Base64-encoded user data for the instance. User data is limited to 16 KB.
   * @param {string} spotPrice - The maximum price per hour that you are willing to pay for a Spot Instance. The default is the On-Demand price.
   * @param {string} type - The Spot Instance request type.   Default:  one-time
   * @param {string} validFrom - The start date of the request. If this is a one-time request, the request becomes active at this date and time and remains active until all instances launch, the request ...(description truncated)
   * @param {string} validUntil - The end date of the request. If this is a one-time request, the request remains active until all instances launch, the request is canceled, or this date is reached. If th...(description truncated)
   * @param {string} instanceInterruptionBehavior - The behavior when a Spot Instance is interrupted. The default is  terminate .
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  requestSpotInstances(availabilityZoneGroup, blockDurationMinutes, clientToken, dryRun, instanceCount, launchGroup, launchSpecificationSecurityGroupIds, launchSpecificationSecurityGroups, launchSpecificationAddressingType, launchSpecificationBlockDeviceMappings, launchSpecificationEbsOptimized, launchSpecificationIamInstanceProfile, launchSpecificationImageId, launchSpecificationInstanceType, launchSpecificationKernelId, launchSpecificationKeyName, launchSpecificationMonitoring, launchSpecificationNetworkInterfaces, launchSpecificationPlacement, launchSpecificationRamdiskId, launchSpecificationSubnetId, launchSpecificationUserData, spotPrice, type = 'one-time', validFrom, validUntil, instanceInterruptionBehavior = 'hibernate', callback) {
    const meth = 'adapter-requestSpotInstances';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'RequestSpotInstances';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AvailabilityZoneGroup: availabilityZoneGroup,
      BlockDurationMinutes: blockDurationMinutes,
      ClientToken: clientToken,
      InstanceCount: instanceCount,
      LaunchGroup: launchGroup,
      LaunchSpecificationAddressingType: launchSpecificationAddressingType,
      LaunchSpecificationEbsOptimized: launchSpecificationEbsOptimized,
      LaunchSpecificationIamInstanceProfile: launchSpecificationIamInstanceProfile,
      LaunchSpecificationImageId: launchSpecificationImageId,
      LaunchSpecificationInstanceType: launchSpecificationInstanceType,
      LaunchSpecificationKernelId: launchSpecificationKernelId,
      LaunchSpecificationKeyName: launchSpecificationKeyName,
      LaunchSpecificationMonitoring: launchSpecificationMonitoring,
      LaunchSpecificationPlacement: launchSpecificationPlacement,
      LaunchSpecificationRamdiskId: launchSpecificationRamdiskId,
      LaunchSpecificationSubnetId: launchSpecificationSubnetId,
      LaunchSpecificationUserData: launchSpecificationUserData,
      SpotPrice: spotPrice,
      Type: type,
      ValidFrom: validFrom,
      ValidUntil: validUntil,
      InstanceInterruptionBehavior: instanceInterruptionBehavior,
      Action: action,
      Version: version
    };
    if (launchSpecificationSecurityGroupIds && launchSpecificationSecurityGroupIds.length > 0) {
      processInArray(launchSpecificationSecurityGroupIds, queryParamsAvailable, 'LaunchSpecificationSecurityGroupIds');
    }
    if (launchSpecificationSecurityGroups && launchSpecificationSecurityGroups.length > 0) {
      processInArray(launchSpecificationSecurityGroups, queryParamsAvailable, 'LaunchSpecificationSecurityGroups');
    }
    if (launchSpecificationBlockDeviceMappings && launchSpecificationBlockDeviceMappings.length > 0) {
      processInArray(launchSpecificationBlockDeviceMappings, queryParamsAvailable, 'LaunchSpecificationBlockDeviceMappings');
    }
    if (launchSpecificationNetworkInterfaces && launchSpecificationNetworkInterfaces.length > 0) {
      processInArray(launchSpecificationNetworkInterfaces, queryParamsAvailable, 'LaunchSpecificationNetworkInterfaces');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'requestSpotInstances', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['requestSpotInstances'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Resets the specified attribute of the specified Amazon FPGA Image (AFI) to its default value. You can only reset the load permission attribute.
   *
   * @function resetFpgaImageAttribute
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} fpgaImageId - The ID of the AFI.
   * @param {string} attribute - The attribute.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  resetFpgaImageAttribute(dryRun, fpgaImageId, attribute = 'loadPermission', callback) {
    const meth = 'adapter-resetFpgaImageAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ResetFpgaImageAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (fpgaImageId === undefined || fpgaImageId === null || fpgaImageId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['fpgaImageId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      FpgaImageId: fpgaImageId,
      Attribute: attribute,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Image', 'resetFpgaImageAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['resetFpgaImageAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Resets an attribute of an AMI to its default value.     The productCodes attribute can't be reset.
   *
   * @function resetImageAttribute
   * @param {string} attribute - The attribute to reset (currently you can only reset the launch permission attribute).
   * @param {string} imageId - The ID of the AMI.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  resetImageAttribute(attribute = 'launchPermission', imageId, dryRun, callback) {
    const meth = 'adapter-resetImageAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ResetImageAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (attribute === undefined || attribute === null || attribute === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['attribute'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (imageId === undefined || imageId === null || imageId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['imageId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Attribute: attribute,
      ImageId: imageId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Image', 'resetImageAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['resetImageAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Resets an attribute of an instance to its default value. To reset the  kernel  or  ramdisk , the instance must be in a stopped state. To reset the  sourceDestCheck , the instance can be either running or stopped.   The  sourceDestCheck  attribute controls whether source/destination checking is enabled. The default value is  true , which means checking is enabled. This value must be  false  for a NAT instance to perform NAT. For more information, see  NAT Instances  in the  Amazon Virtual Privat...(description truncated)
   *
   * @function resetInstanceAttribute
   * @param {string} attribute - The attribute to reset.     You can only reset the following attributes:  kernel  |  ramdisk  |  sourceDestCheck . To change an instance attribute, use  ModifyInstanceAt...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} instanceId - The ID of the instance.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  resetInstanceAttribute(attribute = 'instanceType', dryRun, instanceId, callback) {
    const meth = 'adapter-resetInstanceAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ResetInstanceAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (attribute === undefined || attribute === null || attribute === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['attribute'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Attribute: attribute,
      InstanceId: instanceId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'resetInstanceAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['resetInstanceAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Resets a network interface attribute. You can specify only one attribute at a time.
   *
   * @function resetNetworkInterfaceAttribute
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} networkInterfaceId - The ID of the network interface.
   * @param {string} sourceDestCheck - The source/destination checking attribute. Resets the value to  true .
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  resetNetworkInterfaceAttribute(dryRun, networkInterfaceId, sourceDestCheck, callback) {
    const meth = 'adapter-resetNetworkInterfaceAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ResetNetworkInterfaceAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (networkInterfaceId === undefined || networkInterfaceId === null || networkInterfaceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['networkInterfaceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      NetworkInterfaceId: networkInterfaceId,
      SourceDestCheck: sourceDestCheck,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('NetworkInterface', 'resetNetworkInterfaceAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['resetNetworkInterfaceAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Resets permission settings for the specified snapshot.   For more information about modifying snapshot permissions, see  Sharing Snapshots  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function resetSnapshotAttribute
   * @param {string} attribute - The attribute to reset. Currently, only the attribute for permission to create volumes can be reset.
   * @param {string} snapshotId - The ID of the snapshot.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  resetSnapshotAttribute(attribute = 'productCodes', snapshotId, dryRun, callback) {
    const meth = 'adapter-resetSnapshotAttribute';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'ResetSnapshotAttribute';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (attribute === undefined || attribute === null || attribute === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['attribute'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (snapshotId === undefined || snapshotId === null || snapshotId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['snapshotId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Attribute: attribute,
      SnapshotId: snapshotId,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Snapshot', 'resetSnapshotAttribute', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['resetSnapshotAttribute'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Restores an Elastic IP address that was previously moved to the EC2-VPC platform back to the EC2-Classic platform. You cannot move an Elastic IP address that was originally allocated for use in EC2-VPC. The Elastic IP address must not be associated with an instance or network interface.
   *
   * @function restoreAddressToClassic
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} publicIp - The Elastic IP address.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  restoreAddressToClassic(dryRun, publicIp, callback) {
    const meth = 'adapter-restoreAddressToClassic';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'RestoreAddressToClassic';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (publicIp === undefined || publicIp === null || publicIp === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['publicIp'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      PublicIp: publicIp,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'restoreAddressToClassic', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['restoreAddressToClassic'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Removes an ingress authorization rule from a Client VPN endpoint.
   *
   * @function revokeClientVpnIngress
   * @param {string} clientVpnEndpointId - The ID of the Client VPN endpoint with which the authorization rule is associated.
   * @param {string} targetNetworkCidr - The IPv4 address range, in CIDR notation, of the network for which access is being removed.
   * @param {string} accessGroupId - The ID of the Active Directory group for which to revoke access.
   * @param {boolean} revokeAllGroups - Indicates whether access should be revoked for all clients.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  revokeClientVpnIngress(clientVpnEndpointId, targetNetworkCidr, accessGroupId, revokeAllGroups, dryRun, callback) {
    const meth = 'adapter-revokeClientVpnIngress';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'RevokeClientVpnIngress';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointId === undefined || clientVpnEndpointId === null || clientVpnEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (targetNetworkCidr === undefined || targetNetworkCidr === null || targetNetworkCidr === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['targetNetworkCidr'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientVpnEndpointId: clientVpnEndpointId,
      TargetNetworkCidr: targetNetworkCidr,
      AccessGroupId: accessGroupId,
      RevokeAllGroups: revokeAllGroups,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'revokeClientVpnIngress', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['revokeClientVpnIngress'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary [VPC only] Removes the specified egress rules from a security group for EC2-VPC. This action doesn't apply to security groups for use in EC2-Classic. To remove a rule, the values that you specify (for example, ports) must match the existing rule's values exactly.   Each rule consists of the protocol and the IPv4 or IPv6 CIDR range or source security group. For the TCP and UDP protocols, you must also specify the destination port or range of ports. For the ICMP protocol, you must also specify th...(description truncated)
   *
   * @function revokeSecurityGroupEgress
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} groupId - The ID of the security group.
   * @param {array} ipPermissions - The sets of IP permissions. You can't specify a destination security group and a CIDR IP address range in the same set of permissions.
   * @param {string} cidrIp - Not supported. Use a set of IP permissions to specify the CIDR.
   * @param {number} fromPort - Not supported. Use a set of IP permissions to specify the port.
   * @param {string} ipProtocol - Not supported. Use a set of IP permissions to specify the protocol name or number.
   * @param {number} toPort - Not supported. Use a set of IP permissions to specify the port.
   * @param {string} sourceSecurityGroupName - Not supported. Use a set of IP permissions to specify a destination security group.
   * @param {string} sourceSecurityGroupOwnerId - Not supported. Use a set of IP permissions to specify a destination security group.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  revokeSecurityGroupEgress(dryRun, groupId, ipPermissions, cidrIp, fromPort, ipProtocol, toPort, sourceSecurityGroupName, sourceSecurityGroupOwnerId, callback) {
    const meth = 'adapter-revokeSecurityGroupEgress';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'RevokeSecurityGroupEgress';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (groupId === undefined || groupId === null || groupId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['groupId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      GroupId: groupId,
      CidrIp: cidrIp,
      FromPort: fromPort,
      IpProtocol: ipProtocol,
      ToPort: toPort,
      SourceSecurityGroupName: sourceSecurityGroupName,
      SourceSecurityGroupOwnerId: sourceSecurityGroupOwnerId,
      Action: action,
      Version: version
    };
    if (ipPermissions && ipPermissions.length > 0) {
      processInArray(ipPermissions, queryParamsAvailable, 'IpPermissions');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('SecurityGroup', 'revokeSecurityGroupEgress', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['revokeSecurityGroupEgress'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Removes the specified ingress rules from a security group. To remove a rule, the values that you specify (for example, ports) must match the existing rule's values exactly.     [EC2-Classic only] If the values you specify do not match the existing rule's values, no error is returned. Use  DescribeSecurityGroups  to verify that the rule has been removed.     Each rule consists of the protocol and the CIDR range or source security group. For the TCP and UDP protocols, you must also specify the de...(description truncated)
   *
   * @function revokeSecurityGroupIngress
   * @param {string} cidrIp - The CIDR IP address range. You can't specify this parameter when specifying a source security group.
   * @param {number} fromPort - The start of port range for the TCP and UDP protocols, or an ICMP type number. For the ICMP type number, use  -1  to specify all ICMP types.
   * @param {string} groupId - The ID of the security group. You must specify either the security group ID or the security group name in the request. For security groups in a nondefault VPC, you must s...(description truncated)
   * @param {string} groupName - [EC2-Classic, default VPC] The name of the security group. You must specify either the security group ID or the security group name in the request.
   * @param {array} ipPermissions - The sets of IP permissions. You can't specify a source security group and a CIDR IP address range in the same set of permissions.
   * @param {string} ipProtocol - The IP protocol name ( tcp ,  udp ,  icmp ) or number (see  Protocol Numbers ). Use  -1  to specify all.
   * @param {string} sourceSecurityGroupName - [EC2-Classic, default VPC] The name of the source security group. You can't specify this parameter in combination with the following parameters: the CIDR IP address range...(description truncated)
   * @param {string} sourceSecurityGroupOwnerId - [EC2-Classic] The AWS account ID of the source security group, if the source security group is in a different account. You can't specify this parameter in combination wit...(description truncated)
   * @param {number} toPort - The end of port range for the TCP and UDP protocols, or an ICMP code number. For the ICMP code number, use  -1  to specify all ICMP codes for the ICMP type.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  revokeSecurityGroupIngress(cidrIp, fromPort, groupId, groupName, ipPermissions, ipProtocol, sourceSecurityGroupName, sourceSecurityGroupOwnerId, toPort, dryRun, callback) {
    const meth = 'adapter-revokeSecurityGroupIngress';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'RevokeSecurityGroupIngress';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      CidrIp: cidrIp,
      FromPort: fromPort,
      GroupId: groupId,
      GroupName: groupName,
      IpProtocol: ipProtocol,
      SourceSecurityGroupName: sourceSecurityGroupName,
      SourceSecurityGroupOwnerId: sourceSecurityGroupOwnerId,
      ToPort: toPort,
      Action: action,
      Version: version
    };
    if (ipPermissions && ipPermissions.length > 0) {
      processInArray(ipPermissions, queryParamsAvailable, 'IpPermissions');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('SecurityGroup', 'revokeSecurityGroupIngress', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['revokeSecurityGroupIngress'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Launches the specified number of instances using an AMI for which you have permissions.    You can specify a number of options, or leave the default options. The following rules apply:       [EC2-VPC] If you don't specify a subnet ID, we choose a default subnet from your default VPC for you. If you don't have a default VPC, you must specify a subnet ID in the request.       [EC2-Classic] If don't specify an Availability Zone, we choose one for you.       Some instance types must be launched int...(description truncated)
   *
   * @function runInstances
   * @param {array} blockDeviceMapping - The block device mapping entries. You can't specify both a snapshot ID and an encryption value. This is because only blank volumes can be encrypted on creation. If a snap...(description truncated)
   * @param {string} imageId - The ID of the AMI. An AMI is required to launch an instance and must be specified here or in a launch template.
   * @param {string} instanceType - The instance type. For more information, see  Instance Types  in the  Amazon Elastic Compute Cloud User Guide .   Default:  m1.small
   * @param {number} ipv6AddressCount - [EC2-VPC] The number of IPv6 addresses to associate with the primary network interface. Amazon EC2 chooses the IPv6 addresses from the range of your subnet. You cannot s...(description truncated)
   * @param {array} ipv6Address - [EC2-VPC] The IPv6 addresses from the range of the subnet to associate with the primary network interface. You cannot specify this option and the option to assign a numb...(description truncated)
   * @param {string} kernelId - The ID of the kernel.     We recommend that you use PV-GRUB instead of kernels and RAM disks. For more information, see   PV-GRUB  in the  Amazon Elastic Compute Cloud U...(description truncated)
   * @param {string} keyName - The name of the key pair. You can create a key pair using  CreateKeyPair  or  ImportKeyPair .     If you do not specify a key pair, you can't connect to the instance unl...(description truncated)
   * @param {number} maxCount - The maximum number of instances to launch. If you specify more instances than Amazon EC2 can launch in the target Availability Zone, Amazon EC2 launches the largest poss...(description truncated)
   * @param {number} minCount - The minimum number of instances to launch. If you specify a minimum that is more instances than Amazon EC2 can launch in the target Availability Zone, Amazon EC2 launche...(description truncated)
   * @param {string} monitoringEnabled - Describes the monitoring of an instance.
Indicates whether detailed monitoring is enabled. Otherwise, basic monitoring is enabled.
   * @param {string} placementAvailabilityZone - Describes the placement of an instance.
 The Availability Zone of the instance.   If not specified, an Availability Zone will be automatically chosen for you based on the...(description truncated)
   * @param {string} placementAffinity - Describes the placement of an instance.
The affinity setting for the instance on the Dedicated Host. This parameter is not supported for the  ImportInstance  command.
   * @param {string} placementGroupName - Describes the placement of an instance.
The name of the placement group the instance is in.
   * @param {string} placementPartitionNumber - Describes the placement of an instance.
The number of the partition the instance is in. Valid only if the placement group strategy is set to  partition .
   * @param {string} placementHostId - Describes the placement of an instance.
The ID of the Dedicated Host on which the instance resides. This parameter is not supported for the  ImportInstance  command.
   * @param {string} placementTenancy - Describes the placement of an instance.
The tenancy of the instance (if the instance is running in a VPC). An instance with a tenancy of  dedicated  runs on single-tenant...(description truncated)
   * @param {string} placementSpreadDomain - Describes the placement of an instance.
Reserved for future use.
   * @param {string} ramdiskId - The ID of the RAM disk to select. Some kernels require additional drivers at launch. Check the kernel requirements for information about whether you need to specify a RA...(description truncated)
   * @param {array} securityGroupId - The IDs of the security groups. You can create a security group using  CreateSecurityGroup .   If you specify a network interface, you must specify any security groups a...(description truncated)
   * @param {array} securityGroup - [EC2-Classic, default VPC] The names of the security groups. For a nondefault VPC, you must use security group IDs instead.   If you specify a network interface, you mus...(description truncated)
   * @param {string} subnetId - [EC2-VPC] The ID of the subnet to launch the instance into.   You cannot specify this option and the network interfaces option in the same request.
   * @param {string} userData - The user data to make available to the instance. For more information, see  Running Commands on Your Linux Instance at Launch  (Linux) and  Adding User Data  (Windows). I...(description truncated)
   * @param {string} additionalInfo - Reserved.
   * @param {string} clientToken - Unique, case-sensitive identifier you provide to ensure the idempotency of the request. For more information, see  Ensuring Idempotency .   Constraints: Maximum 64 ASCII...(description truncated)
   * @param {boolean} disableApiTermination - If you set this parameter to  true , you can't terminate the instance using the Amazon EC2 console, CLI, or API; otherwise, you can. To change this attribute to  false  ...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {boolean} ebsOptimized - Indicates whether the instance is optimized for Amazon EBS I/O. This optimization provides dedicated throughput to Amazon EBS and an optimized configuration stack to pro...(description truncated)
   * @param {string} iamInstanceProfileArn - Describes an IAM instance profile.
The Amazon Resource Name (ARN) of the instance profile.
   * @param {string} iamInstanceProfileName - Describes an IAM instance profile.
The name of the instance profile.
   * @param {string} instanceInitiatedShutdownBehavior - Indicates whether an instance stops or terminates when you initiate shutdown from the instance (using the operating system command for system shutdown).   Default:  stop...(description truncated)
   * @param {array} networkInterface - The network interfaces to associate with the instance. If you specify a network interface, you must specify any security groups as part of the network interface.
   * @param {string} privateIpAddress - [EC2-VPC] The primary IPv4 address. You must specify a value from the IPv4 address range of the subnet.   Only one private IP address can be designated as primary. You c...(description truncated)
   * @param {array} elasticGpuSpecification - An elastic GPU to associate with the instance. An Elastic GPU is a GPU resource that you can attach to your Windows instance to accelerate the graphics performance of you...(description truncated)
   * @param {array} elasticInferenceAccelerator - An elastic inference accelerator to associate with the instance. Elastic inference accelerators are a resource you can attach to your Amazon EC2 instances to accelerate y...(description truncated)
   * @param {array} tagSpecification - The tags to apply to the resources during launch. You can only tag instances and volumes on launch. The specified tags are applied to all instances or volumes that are cr...(description truncated)
   * @param {string} launchTemplateLaunchTemplateId - The launch template to use. You must specify either the launch template ID or launch template name in the request, but not both.
The ID of the launch template.
   * @param {string} launchTemplateLaunchTemplateName - The launch template to use. You must specify either the launch template ID or launch template name in the request, but not both.
The name of the launch template.
   * @param {string} launchTemplateVersion - The launch template to use. You must specify either the launch template ID or launch template name in the request, but not both.
 The version number of the launch templat...(description truncated)
   * @param {string} instanceMarketOptionsMarketType - Describes the market (purchasing) option for the instances.
The market type.
   * @param {string} instanceMarketOptionsSpotOptions - Describes the market (purchasing) option for the instances.
The options for Spot Instances.
   * @param {string} creditSpecificationCpuCredits - The credit option for CPU usage of a T2 or T3 instance.
The credit option for CPU usage of a T2 or T3 instance. Valid values are  standard  and  unlimited .
   * @param {string} cpuOptionsCoreCount - The CPU options for the instance. Both the core count and threads per core must be specified in the request.
The number of CPU cores for the instance.
   * @param {string} cpuOptionsThreadsPerCore - The CPU options for the instance. Both the core count and threads per core must be specified in the request.
The number of threads per CPU core. To disable Intel Hyper-Th...(description truncated)
   * @param {string} capacityReservationSpecificationCapacityReservationPreference - Describes an instance's Capacity Reservation targeting option. You can specify only one parameter at a time. If you specify  CapacityReservationPreference  and  Capacity...(description truncated)
   * @param {string} capacityReservationSpecificationCapacityReservationTarget - Describes an instance's Capacity Reservation targeting option. You can specify only one parameter at a time. If you specify  CapacityReservationPreference  and  Capacity...(description truncated)
   * @param {string} hibernationOptionsConfigured - Indicates whether your instance is configured for hibernation. This parameter is valid only if the instance meets the  hibernation prerequisites . Hibernation is currentl...(description truncated)
   * @param {array} licenseSpecification - The license configurations.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  runInstances(blockDeviceMapping, imageId, instanceType = 't1.micro', ipv6AddressCount, ipv6Address, kernelId, keyName, maxCount, minCount, monitoringEnabled, placementAvailabilityZone, placementAffinity, placementGroupName, placementPartitionNumber, placementHostId, placementTenancy, placementSpreadDomain, ramdiskId, securityGroupId, securityGroup, subnetId, userData, additionalInfo, clientToken, disableApiTermination, dryRun, ebsOptimized, iamInstanceProfileArn, iamInstanceProfileName, instanceInitiatedShutdownBehavior = 'stop', networkInterface, privateIpAddress, elasticGpuSpecification, elasticInferenceAccelerator, tagSpecification, launchTemplateLaunchTemplateId, launchTemplateLaunchTemplateName, launchTemplateVersion, instanceMarketOptionsMarketType, instanceMarketOptionsSpotOptions, creditSpecificationCpuCredits, cpuOptionsCoreCount, cpuOptionsThreadsPerCore, capacityReservationSpecificationCapacityReservationPreference, capacityReservationSpecificationCapacityReservationTarget, hibernationOptionsConfigured, licenseSpecification, callback) {
    const meth = 'adapter-runInstances';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'RunInstances';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (maxCount === undefined || maxCount === null || maxCount === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['maxCount'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (minCount === undefined || minCount === null || minCount === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['minCount'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ImageId: imageId,
      InstanceType: instanceType,
      Ipv6AddressCount: ipv6AddressCount,
      KernelId: kernelId,
      KeyName: keyName,
      MaxCount: maxCount,
      MinCount: minCount,
      MonitoringEnabled: monitoringEnabled,
      PlacementAvailabilityZone: placementAvailabilityZone,
      PlacementAffinity: placementAffinity,
      PlacementGroupName: placementGroupName,
      PlacementPartitionNumber: placementPartitionNumber,
      PlacementHostId: placementHostId,
      PlacementTenancy: placementTenancy,
      PlacementSpreadDomain: placementSpreadDomain,
      RamdiskId: ramdiskId,
      SubnetId: subnetId,
      UserData: userData,
      AdditionalInfo: additionalInfo,
      ClientToken: clientToken,
      DisableApiTermination: disableApiTermination,
      EbsOptimized: ebsOptimized,
      IamInstanceProfileArn: iamInstanceProfileArn,
      IamInstanceProfileName: iamInstanceProfileName,
      InstanceInitiatedShutdownBehavior: instanceInitiatedShutdownBehavior,
      PrivateIpAddress: privateIpAddress,
      LaunchTemplateLaunchTemplateId: launchTemplateLaunchTemplateId,
      LaunchTemplateLaunchTemplateName: launchTemplateLaunchTemplateName,
      LaunchTemplateVersion: launchTemplateVersion,
      InstanceMarketOptionsMarketType: instanceMarketOptionsMarketType,
      InstanceMarketOptionsSpotOptions: instanceMarketOptionsSpotOptions,
      CreditSpecificationCpuCredits: creditSpecificationCpuCredits,
      CpuOptionsCoreCount: cpuOptionsCoreCount,
      CpuOptionsThreadsPerCore: cpuOptionsThreadsPerCore,
      CapacityReservationSpecificationCapacityReservationPreference: capacityReservationSpecificationCapacityReservationPreference,
      CapacityReservationSpecificationCapacityReservationTarget: capacityReservationSpecificationCapacityReservationTarget,
      HibernationOptionsConfigured: hibernationOptionsConfigured,
      Action: action,
      Version: version
    };
    if (blockDeviceMapping && blockDeviceMapping.length > 0) {
      processInArray(blockDeviceMapping, queryParamsAvailable, 'BlockDeviceMapping');
    }
    if (ipv6Address && ipv6Address.length > 0) {
      processInArray(ipv6Address, queryParamsAvailable, 'Ipv6Address');
    }
    if (securityGroupId && securityGroupId.length > 0) {
      processInArray(securityGroupId, queryParamsAvailable, 'SecurityGroupId');
    }
    if (securityGroup && securityGroup.length > 0) {
      processInArray(securityGroup, queryParamsAvailable, 'SecurityGroup');
    }
    if (networkInterface && networkInterface.length > 0) {
      processInArray(networkInterface, queryParamsAvailable, 'NetworkInterface');
    }
    if (elasticGpuSpecification && elasticGpuSpecification.length > 0) {
      processInArray(elasticGpuSpecification, queryParamsAvailable, 'ElasticGpuSpecification');
    }
    if (elasticInferenceAccelerator && elasticInferenceAccelerator.length > 0) {
      processInArray(elasticInferenceAccelerator, queryParamsAvailable, 'ElasticInferenceAccelerator');
    }
    if (tagSpecification && tagSpecification.length > 0) {
      processInArray(tagSpecification, queryParamsAvailable, 'TagSpecification');
    }
    if (licenseSpecification && licenseSpecification.length > 0) {
      processInArray(licenseSpecification, queryParamsAvailable, 'LicenseSpecification');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'runInstances', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['runInstances'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Launches the specified Scheduled Instances.   Before you can launch a Scheduled Instance, you must purchase it and obtain an identifier using  PurchaseScheduledInstances .   You must launch a Scheduled Instance during its scheduled time period. You can't stop or reboot a Scheduled Instance, but you can terminate it as needed. If you terminate a Scheduled Instance before the current scheduled time period ends, you can launch it again after a few minutes. For more information, see  Scheduled Inst...(description truncated)
   *
   * @function runScheduledInstances
   * @param {string} clientToken - Unique, case-sensitive identifier that ensures the idempotency of the request. For more information, see  Ensuring Idempotency .
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {number} instanceCount - The number of instances.   Default: 1
   * @param {array} launchSpecificationBlockDeviceMappings - Describes the launch specification for a Scheduled Instance.   If you are launching the Scheduled Instance in EC2-VPC, you must specify the ID of the subnet. You can spe...(description truncated)
   * @param {string} launchSpecificationEbsOptimized - Describes the launch specification for a Scheduled Instance.   If you are launching the Scheduled Instance in EC2-VPC, you must specify the ID of the subnet. You can spe...(description truncated)
   * @param {string} launchSpecificationIamInstanceProfile - Describes the launch specification for a Scheduled Instance.   If you are launching the Scheduled Instance in EC2-VPC, you must specify the ID of the subnet. You can spe...(description truncated)
   * @param {string} launchSpecificationImageId - Describes the launch specification for a Scheduled Instance.   If you are launching the Scheduled Instance in EC2-VPC, you must specify the ID of the subnet. You can spe...(description truncated)
   * @param {string} launchSpecificationInstanceType - Describes the launch specification for a Scheduled Instance.   If you are launching the Scheduled Instance in EC2-VPC, you must specify the ID of the subnet. You can spe...(description truncated)
   * @param {string} launchSpecificationKernelId - Describes the launch specification for a Scheduled Instance.   If you are launching the Scheduled Instance in EC2-VPC, you must specify the ID of the subnet. You can spe...(description truncated)
   * @param {string} launchSpecificationKeyName - Describes the launch specification for a Scheduled Instance.   If you are launching the Scheduled Instance in EC2-VPC, you must specify the ID of the subnet. You can spe...(description truncated)
   * @param {string} launchSpecificationMonitoring - Describes the launch specification for a Scheduled Instance.   If you are launching the Scheduled Instance in EC2-VPC, you must specify the ID of the subnet. You can spe...(description truncated)
   * @param {array} launchSpecificationNetworkInterfaces - Describes the launch specification for a Scheduled Instance.   If you are launching the Scheduled Instance in EC2-VPC, you must specify the ID of the subnet. You can spe...(description truncated)
   * @param {string} launchSpecificationPlacement - Describes the launch specification for a Scheduled Instance.   If you are launching the Scheduled Instance in EC2-VPC, you must specify the ID of the subnet. You can spe...(description truncated)
   * @param {string} launchSpecificationRamdiskId - Describes the launch specification for a Scheduled Instance.   If you are launching the Scheduled Instance in EC2-VPC, you must specify the ID of the subnet. You can spe...(description truncated)
   * @param {array} launchSpecificationSecurityGroupIds - Describes the launch specification for a Scheduled Instance.   If you are launching the Scheduled Instance in EC2-VPC, you must specify the ID of the subnet. You can spe...(description truncated)
   * @param {string} launchSpecificationSubnetId - Describes the launch specification for a Scheduled Instance.   If you are launching the Scheduled Instance in EC2-VPC, you must specify the ID of the subnet. You can spe...(description truncated)
   * @param {string} launchSpecificationUserData - Describes the launch specification for a Scheduled Instance.   If you are launching the Scheduled Instance in EC2-VPC, you must specify the ID of the subnet. You can spe...(description truncated)
   * @param {string} scheduledInstanceId - The Scheduled Instance ID.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  runScheduledInstances(clientToken, dryRun, instanceCount, launchSpecificationBlockDeviceMappings, launchSpecificationEbsOptimized, launchSpecificationIamInstanceProfile, launchSpecificationImageId, launchSpecificationInstanceType, launchSpecificationKernelId, launchSpecificationKeyName, launchSpecificationMonitoring, launchSpecificationNetworkInterfaces, launchSpecificationPlacement, launchSpecificationRamdiskId, launchSpecificationSecurityGroupIds, launchSpecificationSubnetId, launchSpecificationUserData, scheduledInstanceId, callback) {
    const meth = 'adapter-runScheduledInstances';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'RunScheduledInstances';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (scheduledInstanceId === undefined || scheduledInstanceId === null || scheduledInstanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['scheduledInstanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientToken: clientToken,
      InstanceCount: instanceCount,
      LaunchSpecificationEbsOptimized: launchSpecificationEbsOptimized,
      LaunchSpecificationIamInstanceProfile: launchSpecificationIamInstanceProfile,
      LaunchSpecificationImageId: launchSpecificationImageId,
      LaunchSpecificationInstanceType: launchSpecificationInstanceType,
      LaunchSpecificationKernelId: launchSpecificationKernelId,
      LaunchSpecificationKeyName: launchSpecificationKeyName,
      LaunchSpecificationMonitoring: launchSpecificationMonitoring,
      LaunchSpecificationPlacement: launchSpecificationPlacement,
      LaunchSpecificationRamdiskId: launchSpecificationRamdiskId,
      LaunchSpecificationSubnetId: launchSpecificationSubnetId,
      LaunchSpecificationUserData: launchSpecificationUserData,
      ScheduledInstanceId: scheduledInstanceId,
      Action: action,
      Version: version
    };
    if (launchSpecificationBlockDeviceMappings && launchSpecificationBlockDeviceMappings.length > 0) {
      processInArray(launchSpecificationBlockDeviceMappings, queryParamsAvailable, 'LaunchSpecificationBlockDeviceMappings');
    }
    if (launchSpecificationNetworkInterfaces && launchSpecificationNetworkInterfaces.length > 0) {
      processInArray(launchSpecificationNetworkInterfaces, queryParamsAvailable, 'LaunchSpecificationNetworkInterfaces');
    }
    if (launchSpecificationSecurityGroupIds && launchSpecificationSecurityGroupIds.length > 0) {
      processInArray(launchSpecificationSecurityGroupIds, queryParamsAvailable, 'LaunchSpecificationSecurityGroupIds');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'runScheduledInstances', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['runScheduledInstances'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Searches for routes in the specified transit gateway route table.
   *
   * @function searchTransitGatewayRoutes
   * @param {string} transitGatewayRouteTableId - The ID of the transit gateway route table.
   * @param {array} filter - One or more filters. The possible values are:         attachment.transit-gateway-attachment-id - The id of the transit gateway attachment.         attachment.resource-id...(description truncated)
   * @param {number} maxResults - The maximum number of routes to return.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  searchTransitGatewayRoutes(transitGatewayRouteTableId, filter, maxResults, dryRun, callback) {
    const meth = 'adapter-searchTransitGatewayRoutes';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'SearchTransitGatewayRoutes';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (transitGatewayRouteTableId === undefined || transitGatewayRouteTableId === null || transitGatewayRouteTableId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['transitGatewayRouteTableId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (filter === undefined || filter === null || filter === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['filter'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      MaxResults: maxResults,
      TransitGatewayRouteTableId: transitGatewayRouteTableId,
      Action: action,
      Version: version
    };
    if (filter && filter.length > 0) {
      processInArray(filter, queryParamsAvailable, 'Filter');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('TransitGateway', 'searchTransitGatewayRoutes', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['searchTransitGatewayRoutes'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Starts an Amazon EBS-backed instance that you've previously stopped.   Instances that use Amazon EBS volumes as their root devices can be quickly stopped and started. When an instance is stopped, the compute resources are released and you are not billed for instance usage. However, your root partition Amazon EBS volume remains and continues to persist your data, and you are charged for Amazon EBS volume usage. You can restart your instance at any time. Every time you start your Windows instance...(description truncated)
   *
   * @function startInstances
   * @param {array} instanceId - The IDs of the instances.
   * @param {string} additionalInfo - Reserved.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  startInstances(instanceId, additionalInfo, dryRun, callback) {
    const meth = 'adapter-startInstances';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'StartInstances';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      AdditionalInfo: additionalInfo,
      Action: action,
      Version: version
    };
    if (instanceId && instanceId.length > 0) {
      processInArray(instanceId, queryParamsAvailable, 'InstanceId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'startInstances', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['startInstances'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Stops an Amazon EBS-backed instance.   You can use the Stop action to hibernate an instance if the instance is  enabled for hibernation  and it meets the  hibernation prerequisites . For more information, see  Hibernate Your Instance  in the  Amazon Elastic Compute Cloud User Guide .   We don't charge usage for a stopped instance, or data transfer fees; however, your root partition Amazon EBS volume remains and continues to persist your data, and you are charged for Amazon EBS volume usage. Eve...(description truncated)
   *
   * @function stopInstances
   * @param {array} instanceId - The IDs of the instances.
   * @param {boolean} hibernate - Hibernates the instance if the instance was enabled for hibernation at launch. If the instance cannot hibernate successfully, a normal shutdown occurs. For more informat...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {boolean} force - Forces the instances to stop. The instances do not have an opportunity to flush file system caches or file system metadata. If you use this option, you must perform file...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  stopInstances(instanceId, hibernate, dryRun, force, callback) {
    const meth = 'adapter-stopInstances';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'StopInstances';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Hibernate: hibernate,
      Force: force,
      Action: action,
      Version: version
    };
    if (instanceId && instanceId.length > 0) {
      processInArray(instanceId, queryParamsAvailable, 'InstanceId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'stopInstances', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['stopInstances'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Terminates active Client VPN endpoint connections. This action can be used to terminate a specific client connection, or up to five connections established by a specific user.
   *
   * @function terminateClientVpnConnections
   * @param {string} clientVpnEndpointId - The ID of the Client VPN endpoint to which the client is connected.
   * @param {string} connectionId - The ID of the client connection to be terminated.
   * @param {string} username - The name of the user who initiated the connection. Use this option to terminate all active connections for the specified user. This option can only be used if the user ha...(description truncated)
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  terminateClientVpnConnections(clientVpnEndpointId, connectionId, username, dryRun, callback) {
    const meth = 'adapter-terminateClientVpnConnections';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'TerminateClientVpnConnections';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (clientVpnEndpointId === undefined || clientVpnEndpointId === null || clientVpnEndpointId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['clientVpnEndpointId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      ClientVpnEndpointId: clientVpnEndpointId,
      ConnectionId: connectionId,
      Username: username,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('VPN', 'terminateClientVpnConnections', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['terminateClientVpnConnections'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Shuts down the specified instances. This operation is idempotent; if you terminate an instance more than once, each call succeeds.    If you specify multiple instances and the request fails (for example, because of a single incorrect instance ID), none of the instances are terminated.   Terminated instances remain visible after termination (for approximately one hour).   By default, Amazon EC2 deletes all EBS volumes that were attached when the instance launched. Volumes attached after instance...(description truncated)
   *
   * @function terminateInstances
   * @param {array} instanceId - The IDs of the instances.   Constraints: Up to 1000 instance IDs. We recommend breaking up this request into smaller batches.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  terminateInstances(instanceId, dryRun, callback) {
    const meth = 'adapter-terminateInstances';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'TerminateInstances';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (instanceId && instanceId.length > 0) {
      processInArray(instanceId, queryParamsAvailable, 'InstanceId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'terminateInstances', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['terminateInstances'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Unassigns one or more IPv6 addresses from a network interface.
   *
   * @function unassignIpv6Addresses
   * @param {array} ipv6Addresses - The IPv6 addresses to unassign from the network interface.
   * @param {string} networkInterfaceId - The ID of the network interface.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  unassignIpv6Addresses(ipv6Addresses, networkInterfaceId, callback) {
    const meth = 'adapter-unassignIpv6Addresses';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'UnassignIpv6Addresses';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (ipv6Addresses === undefined || ipv6Addresses === null || ipv6Addresses === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['ipv6Addresses'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (networkInterfaceId === undefined || networkInterfaceId === null || networkInterfaceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['networkInterfaceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      NetworkInterfaceId: networkInterfaceId,
      Action: action,
      Version: version
    };
    if (ipv6Addresses && ipv6Addresses.length > 0) {
      processInArray(ipv6Addresses, queryParamsAvailable, 'Ipv6Addresses');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'unassignIpv6Addresses', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['unassignIpv6Addresses'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Unassigns one or more secondary private IP addresses from a network interface.
   *
   * @function unassignPrivateIpAddresses
   * @param {string} networkInterfaceId - The ID of the network interface.
   * @param {array} privateIpAddress - The secondary private IP addresses to unassign from the network interface. You can specify this option multiple times to unassign more than one IP address.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  unassignPrivateIpAddresses(networkInterfaceId, privateIpAddress, callback) {
    const meth = 'adapter-unassignPrivateIpAddresses';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'UnassignPrivateIpAddresses';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (networkInterfaceId === undefined || networkInterfaceId === null || networkInterfaceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['networkInterfaceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
    if (privateIpAddress === undefined || privateIpAddress === null || privateIpAddress === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['privateIpAddress'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      NetworkInterfaceId: networkInterfaceId,
      Action: action,
      Version: version
    };
    if (privateIpAddress && privateIpAddress.length > 0) {
      processInArray(privateIpAddress, queryParamsAvailable, 'PrivateIpAddress');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'unassignPrivateIpAddresses', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['unassignPrivateIpAddresses'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Disables detailed monitoring for a running instance. For more information, see  Monitoring Your Instances and Volumes  in the  Amazon Elastic Compute Cloud User Guide .
   *
   * @function unmonitorInstances
   * @param {array} instanceId - The IDs of the instances.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  unmonitorInstances(instanceId, dryRun, callback) {
    const meth = 'adapter-unmonitorInstances';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'UnmonitorInstances';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (instanceId === undefined || instanceId === null || instanceId === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['instanceId'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Action: action,
      Version: version
    };
    if (instanceId && instanceId.length > 0) {
      processInArray(instanceId, queryParamsAvailable, 'InstanceId');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Instance', 'unmonitorInstances', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['unmonitorInstances'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary [VPC only] Updates the description of an egress (outbound) security group rule. You can replace an existing description, or add a description to a rule that did not have one previously.   You specify the description as part of the IP permissions structure. You can remove a description for a security group rule by omitting the description parameter in the request.
   *
   * @function updateSecurityGroupRuleDescriptionsEgress
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} groupId - The ID of the security group. You must specify either the security group ID or the security group name in the request. For security groups in a nondefault VPC, you must s...(description truncated)
   * @param {string} groupName - [Default VPC] The name of the security group. You must specify either the security group ID or the security group name in the request.
   * @param {array} ipPermissions - The IP permissions for the security group rule.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  updateSecurityGroupRuleDescriptionsEgress(dryRun, groupId, groupName, ipPermissions, callback) {
    const meth = 'adapter-updateSecurityGroupRuleDescriptionsEgress';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'UpdateSecurityGroupRuleDescriptionsEgress';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (ipPermissions === undefined || ipPermissions === null || ipPermissions === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['ipPermissions'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      GroupId: groupId,
      GroupName: groupName,
      Action: action,
      Version: version
    };
    if (ipPermissions && ipPermissions.length > 0) {
      processInArray(ipPermissions, queryParamsAvailable, 'IpPermissions');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('SecurityGroup', 'updateSecurityGroupRuleDescriptionsEgress', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['updateSecurityGroupRuleDescriptionsEgress'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Updates the description of an ingress (inbound) security group rule. You can replace an existing description, or add a description to a rule that did not have one previously.   You specify the description as part of the IP permissions structure. You can remove a description for a security group rule by omitting the description parameter in the request.
   *
   * @function updateSecurityGroupRuleDescriptionsIngress
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {string} groupId - The ID of the security group. You must specify either the security group ID or the security group name in the request. For security groups in a nondefault VPC, you must s...(description truncated)
   * @param {string} groupName - [EC2-Classic, default VPC] The name of the security group. You must specify either the security group ID or the security group name in the request.
   * @param {array} ipPermissions - The IP permissions for the security group rule.
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  updateSecurityGroupRuleDescriptionsIngress(dryRun, groupId, groupName, ipPermissions, callback) {
    const meth = 'adapter-updateSecurityGroupRuleDescriptionsIngress';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'UpdateSecurityGroupRuleDescriptionsIngress';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (ipPermissions === undefined || ipPermissions === null || ipPermissions === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['ipPermissions'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      GroupId: groupId,
      GroupName: groupName,
      Action: action,
      Version: version
    };
    if (ipPermissions && ipPermissions.length > 0) {
      processInArray(ipPermissions, queryParamsAvailable, 'IpPermissions');
    }

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('SecurityGroup', 'updateSecurityGroupRuleDescriptionsIngress', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['updateSecurityGroupRuleDescriptionsIngress'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }

  /**
   * @summary Stops advertising an IPv4 address range that is provisioned as an address pool.   You can perform this operation at most once every 10 seconds, even if you specify different address ranges each time.   It can take a few minutes before traffic to the specified addresses stops routing to AWS because of BGP propagation delays.
   *
   * @function withdrawByoipCidr
   * @param {string} cidr - The public IPv4 address range, in CIDR notation.
   * @param {boolean} dryRun - Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions...(description truncated)
   * @param {getCallback} callback - a callback function to return the result
   */
  /* YOU CAN CHANGE THE PARAMETERS YOU TAKE IN HERE AND IN THE pronghorn.json FILE */
  withdrawByoipCidr(cidr, dryRun, callback) {
    const meth = 'adapter-withdrawByoipCidr';
    const origin = `${this.id}-${meth}`;
    log.trace(origin);
    const action = 'WithdrawByoipCidr';
    const version = '2016-11-15';

    /* HERE IS WHERE YOU VALIDATE DATA */
    if (cidr === undefined || cidr === null || cidr === '') {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Missing Data', ['cidr'], null, null, null);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }

    /* HERE IS WHERE YOU SET THE DATA TO PASS INTO REQUEST */
    const queryParamsAvailable = {
      DryRun: dryRun,
      Cidr: cidr,
      Action: action,
      Version: version
    };

    const queryParams = {};
    const pathVars = [];
    const bodyVars = {};

    // loop in template. long callback arg name to avoid identifier conflicts
    Object.keys(queryParamsAvailable).forEach((thisKeyInQueryParamsAvailable) => {
      if (queryParamsAvailable[thisKeyInQueryParamsAvailable] !== undefined && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== null
          && queryParamsAvailable[thisKeyInQueryParamsAvailable] !== '') {
        queryParams[thisKeyInQueryParamsAvailable] = queryParamsAvailable[thisKeyInQueryParamsAvailable];
      }
    });

    // if you want to expose addlHeaders to workflow, add it to the method signature here and in pronghorn.json
    let thisHeaderData = null;
    // if the additional headers was passed in as a string parse the json into an object
    if (thisHeaderData !== null && thisHeaderData.constructor === String) {
      try {
        // parse the additional headers object that was passed in
        thisHeaderData = JSON.parse(thisHeaderData);
      } catch (err) {
        const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'addlHeaders string must be a stringified JSON', [], null, null, null);
        log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
        return callback(null, errorObj);
      }
    } else if (thisHeaderData === null) {
      thisHeaderData = { xAmzContentSha256: '', xAmzDate: '', xAmzAlgorithm: '', xAmzCredential: '', xAmzSecurityToken: '', xAmzSignature: '', xAmzSignedHeaders: '' };
    }

    // set up the request object - payload, uriPathVars, uriQuery, uriOptions, addlHeaders
    Object.assign(thisHeaderData, this.getAuthorization('GET', `/?${querystring.stringify(queryParams)}`));
    const reqObj = {
      payload: bodyVars,
      uriPathVars: pathVars,
      uriQuery: queryParams,
      addlHeaders: thisHeaderData
    };

    try {
      // Make the call -
      // identifyRequest(entity, action, requestObj, returnDataFlag, callback)
      return this.requestHandlerInst.identifyRequest('Address', 'withdrawByoipCidr', reqObj, true, (irReturnData, irReturnError) => {
        // if we received an error or their is no response on the results
        // return an error
        if (irReturnError) {
          /* HERE IS WHERE YOU CAN ALTER THE ERROR MESSAGE */
          return callback(null, irReturnError);
        }
        if (!Object.hasOwnProperty.call(irReturnData, 'response')) {
          const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Invalid Response', ['withdrawByoipCidr'], null, null, null);
          log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
          return callback(null, errorObj);
        }

        /* HERE IS WHERE YOU CAN ALTER THE RETURN DATA */
        // return the response
        return callback(irReturnData, null);
      });
    } catch (ex) {
      const errorObj = this.requestHandlerInst.formatErrorObject(this.id, meth, 'Caught Exception', null, null, null, ex);
      log.error(`${origin}: ${errorObj.IAPerror.displayString}`);
      return callback(null, errorObj);
    }
  }
}

module.exports = Awsec2;
